/****************************/
/* THIS IS OPEN SOURCE CODE */
/****************************/

/*
* File:    linux-ppc64.h
* Author:  Maynard Johnson
*          maynardj@us.ibm.com
* Mods:    <your name here>
*          <your email address>
*/

#ifndef _LINUX_PPC64_H		 /* _LINUX_PPC64_H */
#define _LINUX_PPC64_H


#include <signal.h>
#include <malloc.h>
#include <assert.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/ucontext.h>

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <string.h>
#include <libgen.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <ctype.h>
#include <dlfcn.h>
#include <sys/time.h>
#include <sys/times.h>
#include <stddef.h>

#define POWER_MAX_COUNTERS MAX_COUNTERS
#define MAX_COUNTER_TERMS MAX_COUNTERS

#include "linux-context.h"


#endif /* _LINUX_PPC64_H */
