/****************************/
/* THIS IS OPEN SOURCE CODE */
/****************************/

/* 
* File:    map-p4.h
* CVS:     $Id$
* Author:  Harald Servat
*          redcrash@gmail.com
*/

#ifndef FreeBSD_MAP_P4
#define FreeBSD_MAP_P4

enum NativeEvent_Value_P4Processor {
	PNE_P4_128BIT_MMX_UOP = PAPI_NATIVE_MASK,
	PNE_P4_64BIT_MMX_UOP,
	PNE_P4_B2B_CYCLES,
	PNE_P4_BNR,
	PNE_P4_BPU_FETCH_REQUEST,
	PNE_P4_BRANH_RETIRED,
	PNE_P4_BSQ_ACTIVE_ENTRIES,
	PNE_P4_BSQ_ALLOCATION,
	PNE_P4_BSQ_CACHE_REFERENCE,
	PNE_P4_EXECUTION_EVENT,
	PNE_P4_FRONT_END_EVENT,
	PNE_P4_FSB_DATA_ACTIVITY,
	PNE_P4_GLOBAL_POWER_EVENTS,
	PNE_P4_INSTR_RETIRED,
	PNE_P4_IOQ_ACTIVE_ENTRIES,
	PNE_P4_IOQ_ALLOCATION,
	PNE_P4_ITLB_REFERENCE,
	PNE_P4_LOAD_PORT_REPLAY,
	PNE_P4_MISPRED_BRANCH_RETIRED,
	PNE_P4_MACHINE_CLEAR,
	PNE_P4_MEMORY_CANCEL,
	PNE_P4_MEMORY_COMPLETE,
	PNE_P4_MOB_LOAD_REPLAY,
	PNE_P4_PACKED_DP_UOP,
	PNE_P4_PACKED_SP_UOP,
	PNE_P4_PAGE_WALK_TYPE,
	PNE_P4_REPLAY_EVENT,
	PNE_P4_RESOURCE_STALL,
	PNE_P4_RESPONSE,
	PNE_P4_RETIRED_BRANCH_TYPE,
	PNE_P4_RETIRED_MISPRED_BRANCH_TYPE,
	PNE_P4_SCALAR_DP_UOP,
	PNE_P4_SCALAR_SP_UOP,
	PNE_P4_SNOOP,
	PNE_P4_SSE_INPUT_ASSIST,
	PNE_P4_STORE_PORT_REPLAY,
	PNE_P4_TC_DELIVER_MODE,
	PNE_P4_TC_MS_XFER,
	PNE_P4_UOP_QUEUE_WRITES,
	PNE_P4_UOP_TYPE,
	PNE_P4_UOPS_RETIRED,
	PNE_P4_WC_BUFFER,
	PNE_P4_X87_ASSIST,
	PNE_P4_X87_FP_UOP,
	PNE_P4_X87_SIMD_MOVES_UOP,
	/* Special counters */
	PNE_P4_UOP_QUEUE_WRITES_TC_BUILD_DELIVER,
	PNE_P4_PAGE_WALK_TYPE_D,
	PNE_P4_PAGE_WALK_TYPE_I,
	PNE_P4_INSTR_RETIRED_NON_BOGUS,
	PNE_P4_BRANCH_RETIRED_NOT_TAKEN,
	PNE_P4_BRANCH_RETIRED_TAKEN,
	PNE_P4_BRANCH_RETIRED_PREDICTED,
	PNE_P4_BRANCH_RETIRED_MISPREDICTED,
	PNE_P4_BSQ_CACHE_REFERENCE_2L_MISSES,
	PNE_P4_BSQ_CACHE_REFERENCE_2L_ACCESSES,
	PNE_P4_BSQ_CACHE_REFERENCE_2L_HITS,
	PNE_P4_BSQ_CACHE_REFERENCE_3L_MISSES,
	PNE_P4_BSQ_CACHE_REFERENCE_3L_ACCESSES,
	PNE_P4_BSQ_CACHE_REFERENCE_3L_HITS,
	PNE_P4_NATNAME_GUARD
};

extern Native_Event_LabelDescription_t P4Processor_info[];
extern hwi_search_t P4Processor_map[];

#endif
