/****************************/
/* THIS IS OPEN SOURCE CODE */
/****************************/

#ifndef _PAPI_PPC970_EVENTS_H
#define _PAPI_PPC970_EVENTS_H

/* 
* File:    ppc970_events.h
* CVS:     
* Author:  Maynard Johnson
*          maynardj@us.ibm.com
* Mods:    <your name here>
*          <your email address>
*          
* This file MUST be kept synchronised with the events file.
*
*/
#include "papiStdEventDefs.h"

#define GROUP_INTS 2
#define PAPI_MAX_NATIVE_EVENTS 256
#define MAX_GROUPS (GROUP_INTS * 32)
#define MAX_NATNAME_MAP_INDEX 215


enum native_name
{
	PNE_PM_BRQ_FULL_CYC = PAPI_NATIVE_MASK,
	PNE_PM_CR_MAP_FULL_CYC,
	PNE_PM_CYC,
	PNE_PM_DATA_FROM_L2,
	PNE_PM_DATA_TABLEWALK_CYC,
	PNE_PM_DSLB_MISS,
	PNE_PM_DTLB_MISS,
	PNE_PM_FPR_MAP_FULL_CYC,
	PNE_PM_FPU0_ALL,
	PNE_PM_FPU0_DENORM,
	PNE_PM_FPU0_FDIV,
	PNE_PM_FPU0_FMA,
	PNE_PM_FPU0_FSQRT,
	PNE_PM_FPU0_FULL_CYC,
	PNE_PM_FPU0_SINGLE,
	PNE_PM_FPU0_STALL3,
	PNE_PM_FPU0_STF,
	PNE_PM_FPU1_ALL,
	PNE_PM_FPU1_DENORM,
	PNE_PM_FPU1_FDIV,
	PNE_PM_FPU1_FMA,
	PNE_PM_FPU1_FSQRT,
	PNE_PM_FPU1_FULL_CYC,
	PNE_PM_FPU1_SINGLE,
	PNE_PM_FPU1_STALL3,
	PNE_PM_FPU1_STF,
	PNE_PM_FPU_DENORM,
	PNE_PM_FPU_FDIV,
	PNE_PM_GCT_EMPTY_CYC,
	PNE_PM_GCT_FULL_CYC,
	PNE_PM_GRP_BR_MPRED,
	PNE_PM_GRP_BR_REDIR,
	PNE_PM_GRP_DISP_REJECT,
	PNE_PM_GRP_DISP_VALID,
	PNE_PM_IC_PREF_INSTALL,
	PNE_PM_IC_PREF_REQ,
	PNE_PM_IERAT_XLATE_WR,
	PNE_PM_INST_CMPL,
	PNE_PM_INST_DISP,
	PNE_PM_INST_FROM_L1,
	PNE_PM_INST_FROM_L2,
	PNE_PM_ISLB_MISS,
	PNE_PM_ITLB_MISS,
	PNE_PM_LARX_LSU0,
	PNE_PM_LR_CTR_MAP_FULL_CYC,
	PNE_PM_LSU0_DERAT_MISS,
	PNE_PM_LSU0_FLUSH_LRQ,
	PNE_PM_LSU0_FLUSH_SRQ,
	PNE_PM_LSU0_FLUSH_ULD,
	PNE_PM_LSU0_FLUSH_UST,
	PNE_PM_LSU0_REJECT_ERAT_MISS,
	PNE_PM_LSU0_REJECT_LMQ_FULL,
	PNE_PM_LSU0_REJECT_RELOAD_CDF,
	PNE_PM_LSU0_REJECT_SRQ,
	PNE_PM_LSU0_SRQ_STFWD,
	PNE_PM_LSU1_DERAT_MISS,
	PNE_PM_LSU1_FLUSH_LRQ,
	PNE_PM_LSU1_FLUSH_SRQ,
	PNE_PM_LSU1_FLUSH_ULD,
	PNE_PM_LSU1_FLUSH_UST,
	PNE_PM_LSU1_REJECT_ERAT_MISS,
	PNE_PM_LSU1_REJECT_LMQ_FULL,
	PNE_PM_LSU1_REJECT_RELOAD_CDF,
	PNE_PM_LSU1_REJECT_SRQ,
	PNE_PM_LSU1_SRQ_STFWD,
	PNE_PM_LSU_FLUSH_ULD,
	PNE_PM_LSU_LRQ_S0_ALLOC,
	PNE_PM_LSU_LRQ_S0_VALID,
	PNE_PM_LSU_REJECT_SRQ,
	PNE_PM_LSU_SRQ_S0_ALLOC,
	PNE_PM_LSU_SRQ_S0_VALID,
	PNE_PM_LSU_SRQ_STFWD,
	PNE_PM_MRK_DATA_FROM_L2,
	PNE_PM_MRK_GRP_DISP,
	PNE_PM_MRK_IMR_RELOAD,
	PNE_PM_MRK_LD_MISS_L1,
	PNE_PM_MRK_LD_MISS_L1_LSU0,
	PNE_PM_MRK_LD_MISS_L1_LSU1,
	PNE_PM_MRK_STCX_FAIL,
	PNE_PM_MRK_ST_CMPL,
	PNE_PM_MRK_ST_MISS_L1,
	PNE_PM_PMC8_OVERFLOW,
	PNE_PM_RUN_CYC,
	PNE_PM_SNOOP_TLBIE,
	PNE_PM_STCX_FAIL,
	PNE_PM_STCX_PASS,
	PNE_PM_ST_MISS_L1,
	PNE_PM_SUSPENDED,
	PNE_PM_XER_MAP_FULL_CYC,
	PNE_PM_FPU_FMA,
	PNE_PM_FPU_STALL3,
	PNE_PM_GCT_EMPTY_SRQ_FULL,
	PNE_PM_GRP_DISP,
	PNE_PM_INST_FROM_MEM,
	PNE_PM_LSU_FLUSH_UST,
	PNE_PM_LSU_LMQ_SRQ_EMPTY_CYC,
	PNE_PM_LSU_REJECT_LMQ_FULL,
	PNE_PM_MRK_BRU_FIN,
	PNE_PM_PMC1_OVERFLOW,
	PNE_PM_THRESH_TIMEO,
	PNE_PM_WORK_HELD,
	PNE_PM_BR_ISSUED,
	PNE_PM_BR_MPRED_CR,
	PNE_PM_BR_MPRED_TA,
	PNE_PM_CRQ_FULL_CYC,
	PNE_PM_DATA_FROM_MEM,
	PNE_PM_DC_INV_L2,
	PNE_PM_DC_PREF_OUT_OF_STREAMS,
	PNE_PM_DC_PREF_STREAM_ALLOC,
	PNE_PM_EE_OFF,
	PNE_PM_EE_OFF_EXT_INT,
	PNE_PM_FLUSH_BR_MPRED,
	PNE_PM_FLUSH_LSU_BR_MPRED,
	PNE_PM_FPU0_FEST,
	PNE_PM_FPU0_FIN,
	PNE_PM_FPU0_FMOV_FEST,
	PNE_PM_FPU0_FPSCR,
	PNE_PM_FPU0_FRSP_FCONV,
	PNE_PM_FPU1_FEST,
	PNE_PM_FPU1_FIN,
	PNE_PM_FPU1_FMOV_FEST,
	PNE_PM_FPU1_FRSP_FCONV,
	PNE_PM_FPU_FEST,
	PNE_PM_FXLS0_FULL_CYC,
	PNE_PM_FXLS1_FULL_CYC,
	PNE_PM_FXU0_FIN,
	PNE_PM_FXU1_FIN,
	PNE_PM_FXU_FIN,
	PNE_PM_GPR_MAP_FULL_CYC,
	PNE_PM_GRP_DISP_BLK_SB_CYC,
	PNE_PM_HV_CYC,
	PNE_PM_INST_FROM_PREF,
	PNE_PM_L1_DCACHE_RELOAD_VALID,
	PNE_PM_L1_PREF,
	PNE_PM_L1_WRITE_CYC,
	PNE_PM_L2_PREF,
	PNE_PM_LD_MISS_L1,
	PNE_PM_LD_MISS_L1_LSU0,
	PNE_PM_LD_MISS_L1_LSU1,
	PNE_PM_LD_REF_L1_LSU0,
	PNE_PM_LD_REF_L1_LSU1,
	PNE_PM_LSU0_LDF,
	PNE_PM_LSU1_LDF,
	PNE_PM_LSU_FLUSH,
	PNE_PM_LSU_LMQ_FULL_CYC,
	PNE_PM_LSU_LMQ_LHR_MERGE,
	PNE_PM_LSU_LMQ_S0_ALLOC,
	PNE_PM_LSU_LMQ_S0_VALID,
	PNE_PM_LSU_LRQ_FULL_CYC,
	PNE_PM_LSU_SRQ_FULL_CYC,
	PNE_PM_LSU_SRQ_SYNC_CYC,
	PNE_PM_MRK_DATA_FROM_MEM,
	PNE_PM_MRK_L1_RELOAD_VALID,
	PNE_PM_MRK_LSU0_FLUSH_LRQ,
	PNE_PM_MRK_LSU0_FLUSH_SRQ,
	PNE_PM_MRK_LSU0_FLUSH_ULD,
	PNE_PM_MRK_LSU0_FLUSH_UST,
	PNE_PM_MRK_LSU1_FLUSH_LRQ,
	PNE_PM_MRK_LSU1_FLUSH_SRQ,
	PNE_PM_MRK_LSU1_FLUSH_ULD,
	PNE_PM_MRK_LSU1_FLUSH_UST,
	PNE_PM_MRK_LSU_SRQ_INST_VALID,
	PNE_PM_MRK_ST_CMPL_INT,
	PNE_PM_MRK_VMX_FIN,
	PNE_PM_PMC2_OVERFLOW,
	PNE_PM_STOP_COMPLETION,
	PNE_PM_ST_REF_L1_LSU0,
	PNE_PM_ST_REF_L1_LSU1,
	PNE_PM_0INST_FETCH,
	PNE_PM_FPU_FIN,
	PNE_PM_FXU1_BUSY_FXU0_IDLE,
	PNE_PM_LSU_SRQ_EMPTY_CYC,
	PNE_PM_MRK_CRU_FIN,
	PNE_PM_MRK_GRP_CMPL,
	PNE_PM_PMC3_OVERFLOW,
	PNE_PM_1PLUS_PPC_CMPL,
	PNE_PM_DATA_FROM_L25_SHR,
	PNE_PM_FPU_ALL,
	PNE_PM_FPU_SINGLE,
	PNE_PM_FXU_IDLE,
	PNE_PM_GRP_DISP_SUCCESS,
	PNE_PM_GRP_MRK,
	PNE_PM_INST_FROM_L25_SHR,
	PNE_PM_LSU_FLUSH_SRQ,
	PNE_PM_LSU_REJECT_ERAT_MISS,
	PNE_PM_MRK_DATA_FROM_L25_SHR,
	PNE_PM_MRK_GRP_TIMEO,
	PNE_PM_PMC4_OVERFLOW,
	PNE_PM_DATA_FROM_L25_MOD,
	PNE_PM_FPU_FSQRT,
	PNE_PM_FPU_STF,
	PNE_PM_FXU_BUSY,
	PNE_PM_INST_FROM_L25_MOD,
	PNE_PM_LSU_DERAT_MISS,
	PNE_PM_LSU_FLUSH_LRQ,
	PNE_PM_LSU_REJECT_RELOAD_CDF,
	PNE_PM_MRK_DATA_FROM_L25_MOD,
	PNE_PM_MRK_FXU_FIN,
	PNE_PM_MRK_GRP_ISSUED,
	PNE_PM_MRK_ST_GPS,
	PNE_PM_PMC5_OVERFLOW,
	PNE_PM_FPU_FRSP_FCONV,
	PNE_PM_FXU0_BUSY_FXU1_IDLE,
	PNE_PM_GRP_CMPL,
	PNE_PM_MRK_FPU_FIN,
	PNE_PM_MRK_INST_FIN,
	PNE_PM_PMC6_OVERFLOW,
	PNE_PM_ST_REF_L1,
	PNE_PM_EXT_INT,
	PNE_PM_FPU_FMOV_FEST,
	PNE_PM_LD_REF_L1,
	PNE_PM_LSU_LDF,
	PNE_PM_MRK_LSU_FIN,
	PNE_PM_PMC7_OVERFLOW,
	PNE_PM_TB_BIT_TRANS,
	NATNAME_GUARD,
};
#endif //_PAPI_PPC970_EVENTS_H
