/****************************/
/* THIS IS OPEN SOURCE CODE */
/****************************/

#ifndef __PFMLIB_POWER6_PRIV_H__
#define __PFMLIB_POWER6_PRIV_H__

/*
* File:    pfmlib_power6_priv.h
* CVS:
* Author:  Corey Ashford
*          cjashfor@us.ibm.com
* Mods:    <your name here>
*          <your email address>
*
* (C) Copyright IBM Corporation, 2007.  All Rights Reserved.
* Contributed by Corey Ashford <cjashfor.ibm.com>
*
* Note: This code was automatically generated and should not be modified by
* hand.
*
*/

#define POWER6_NUM_EVENT_COUNTERS 6
#define POWER6_NUM_GROUP_VEC 4
#define POWER6_NUM_CONTROL_REGS 3


#endif

