/****************************/
/* THIS IS OPEN SOURCE CODE */
/****************************/

#ifndef _PAPI_POWER7_EVENTS_H
#define _PAPI_POWER7_EVENTS_H

/*
* File:    power7_events.h
* CVS:
* Author:  Maynard Johnson
*          maynardj@us.ibm.com
* Mods:    <your name here>
*          <your email address>
*
* (C) Copyright IBM Corporation, 2010.  All Rights Reserved.
* Contributed by Maynard Johnson <maynardj@us.ibm.com>
*
*
*/
#include "papiStdEventDefs.h"

#define GROUP_INTS 9
#define PAPI_MAX_NATIVE_EVENTS 1024
#define MAX_GROUPS (GROUP_INTS * 32)
#define MAX_NATNAME_MAP_INDEX 545


enum native_name {
PNE_PM_1PLUS_PPC_CMPL = PAPI_NATIVE_MASK,
	PNE_PM_ANY_THRD_RUN_CYC,
	PNE_PM_BC_PLUS_8_CONV,
	PNE_PM_BC_PLUS_8_RSLV_TAKEN,
	PNE_PM_BR_MPRED_CCACHE,
	PNE_PM_BR_MPRED_CR,
	PNE_PM_BR_MPRED_CR_TA,
	PNE_PM_BR_MPRED_LSTACK,
	PNE_PM_BR_MPRED_TA,
	PNE_PM_BR_PRED,
	PNE_PM_BR_PRED_CCACHE,
	PNE_PM_BR_PRED_CR,
	PNE_PM_BR_PRED_CR_TA,
	PNE_PM_BR_PRED_LSTACK,
	PNE_PM_BR_PRED_TA,
	PNE_PM_BR_UNCOND,
	PNE_PM_BRU_FIN,
	PNE_PM_BTAC_HIT,
	PNE_PM_BTAC_MISS,
	PNE_PM_CMPLU_STALL_END_GCT_NOSLOT,
	PNE_PM_CMPLU_STALL_THRD,
	PNE_PM_CYC,
	PNE_PM_DATA_FROM_DMEM,
	PNE_PM_DATA_FROM_L2,
	PNE_PM_DATA_FROM_L3,
	PNE_PM_DATA_FROM_L31_MOD,
	PNE_PM_DATA_FROM_L31_SHR,
	PNE_PM_DATA_FROM_RL2L3_MOD,
	PNE_PM_DATA_FROM_RL2L3_SHR,
	PNE_PM_DC_PREF_DST,
	PNE_PM_DERAT_MISS_4K,
	PNE_PM_DISP_CLB_HELD,
	PNE_PM_DISP_CLB_HELD_BAL,
	PNE_PM_DISP_CLB_HELD_RES,
	PNE_PM_DISP_CLB_HELD_SB,
	PNE_PM_DISP_CLB_HELD_SYNC,
	PNE_PM_DISP_CLB_HELD_TLBIE,
	PNE_PM_DISP_HELD,
	PNE_PM_DSEG,
	PNE_PM_DSLB_MISS,
	PNE_PM_DTLB_MISS_16G,
	PNE_PM_EE_OFF_EXT_INT,
	PNE_PM_FLOP,
	PNE_PM_FLUSH_BR_MPRED,
	PNE_PM_FLUSH_DISP,
	PNE_PM_FLUSH_DISP_SB,
	PNE_PM_FLUSH_DISP_SYNC,
	PNE_PM_FLUSH_DISP_TLBIE,
	PNE_PM_FLUSH_PARTIAL,
	PNE_PM_FXU_IDLE,
	PNE_PM_FXU0_FIN,
	PNE_PM_GCT_FULL_CYC,
	PNE_PM_GCT_NOSLOT_CYC,
	PNE_PM_GCT_UTIL_1_TO_2_SLOTS,
	PNE_PM_GCT_UTIL_11_PLUS_SLOTS,
	PNE_PM_GCT_UTIL_3_TO_6_SLOTS,
	PNE_PM_GCT_UTIL_7_TO_10_SLOTS,
	PNE_PM_GRP_BR_MPRED_NONSPEC,
	PNE_PM_GRP_IC_MISS_NONSPEC,
	PNE_PM_GRP_MRK,
	PNE_PM_GRP_MRK_CYC,
	PNE_PM_IBUF_FULL_CYC,
	PNE_PM_IC_BANK_CONFLICT,
	PNE_PM_IC_DEMAND_CYC,
	PNE_PM_IC_DEMAND_L2_BHT_REDIRECT,
	PNE_PM_IC_DEMAND_L2_BR_ALL,
	PNE_PM_IC_DEMAND_L2_BR_REDIRECT,
	PNE_PM_IC_DEMAND_REQ,
	PNE_PM_IC_PREF_CANCEL_ALL,
	PNE_PM_IC_PREF_CANCEL_HIT,
	PNE_PM_IC_PREF_CANCEL_L2,
	PNE_PM_IC_PREF_CANCEL_PAGE,
	PNE_PM_IC_PREF_REQ,
	PNE_PM_IC_PREF_WRITE,
	PNE_PM_IC_RELOAD_SHR,
	PNE_PM_IC_REQ_ALL,
	PNE_PM_IC_WRITE_ALL,
	PNE_PM_IERAT_MISS,
	PNE_PM_IERAT_WR_64K,
	PNE_PM_IERAT_XLATE_WR_16MPLUS,
	PNE_PM_INST_CMPL,
	PNE_PM_INST_FROM_DMEM,
	PNE_PM_INST_FROM_L1,
	PNE_PM_INST_FROM_L2,
	PNE_PM_INST_FROM_L3,
	PNE_PM_INST_FROM_L31_MOD,
	PNE_PM_INST_FROM_L31_SHR,
	PNE_PM_INST_FROM_PREF,
	PNE_PM_INST_FROM_RL2L3_MOD,
	PNE_PM_INST_FROM_RL2L3_SHR,
	PNE_PM_INST_IMC_MATCH_CMPL,
	PNE_PM_INST_PTEG_FROM_L2,
	PNE_PM_INST_PTEG_FROM_L31_MOD,
	PNE_PM_INST_PTEG_FROM_RL2L3_MOD,
	PNE_PM_IOPS_CMPL,
	PNE_PM_ISEG,
	PNE_PM_ISLB_MISS,
	PNE_PM_L1_DEMAND_WRITE,
	PNE_PM_L1_PREF,
	PNE_PM_L2_CASTOUT_MOD,
	PNE_PM_L2_CASTOUT_SHR,
	PNE_PM_L2_CO_FAIL_BUSY,
	PNE_PM_L2_GLOB_GUESS_CORRECT,
	PNE_PM_L2_LD,
	PNE_PM_L2_LDST,
	PNE_PM_L2_LOC_GUESS_CORRECT,
	PNE_PM_L2_RCLD_DISP,
	PNE_PM_L2_RCLD_DISP_FAIL_ADDR,
	PNE_PM_L2_ST,
	PNE_PM_L3_HIT,
	PNE_PM_L3_MISS,
	PNE_PM_L3_PREF_LD,
	PNE_PM_L3_PREF_LDST,
	PNE_PM_L3_PREF_ST,
	PNE_PM_LARX_LSU,
	PNE_PM_LARX_LSU0,
	PNE_PM_LARX_LSU1,
	PNE_PM_LD_REF_L1,
	PNE_PM_LD_REF_L1_LSU0,
	PNE_PM_LD_REF_L1_LSU1,
	PNE_PM_LSU_DC_PREF_STREAM_ALLOC,
	PNE_PM_LSU_DC_PREF_STREAM_CONFIRM,
	PNE_PM_LSU_DC_PREF_STRIDED_STREAM_CONFIRM,
	PNE_PM_LSU_DCACHE_RELOAD_VALID,
	PNE_PM_LSU_FLUSH,
	PNE_PM_LSU_FLUSH_LRQ,
	PNE_PM_LSU_FLUSH_SRQ,
	PNE_PM_LSU_FLUSH_ULD,
	PNE_PM_LSU_FLUSH_UST,
	PNE_PM_LSU_FX_FIN,
	PNE_PM_LSU_LDF,
	PNE_PM_LSU_LDX,
	PNE_PM_LSU_LMQ_FULL_CYC,
	PNE_PM_LSU_LMQ_S0_ALLOC,
	PNE_PM_LSU_LMQ_S0_VALID,
	PNE_PM_LSU_LRQ_S0_ALLOC,
	PNE_PM_LSU_LRQ_S0_VALID,
	PNE_PM_LSU_NCLD,
	PNE_PM_LSU_NCST,
	PNE_PM_LSU_PARTIAL_CDF,
	PNE_PM_LSU_REJECT,
	PNE_PM_LSU_REJECT_LHS,
	PNE_PM_LSU_REJECT_LMQ_FULL,
	PNE_PM_LSU_REJECT_SET_MPRED,
	PNE_PM_LSU_SET_MPRED,
	PNE_PM_LSU_SRQ_FULL_CYC,
	PNE_PM_LSU_SRQ_S0_ALLOC,
	PNE_PM_LSU_SRQ_S0_VALID,
	PNE_PM_LSU_SRQ_STFWD,
	PNE_PM_LSU_SRQ_SYNC_COUNT,
	PNE_PM_LSU_SRQ_SYNC_CYC,
	PNE_PM_LSU_TWO_TABLEWALK_CYC,
	PNE_PM_LSU0_DC_PREF_STREAM_ALLOC,
	PNE_PM_LSU0_DC_PREF_STREAM_CONFIRM,
	PNE_PM_LSU0_DC_PREF_STREAM_CONFIRM_STRIDE,
	PNE_PM_LSU0_FLUSH_LRQ,
	PNE_PM_LSU0_FLUSH_SRQ,
	PNE_PM_LSU0_FLUSH_ULD,
	PNE_PM_LSU0_FLUSH_UST,
	PNE_PM_LSU0_L1_PREF,
	PNE_PM_LSU0_L1_SW_PREF,
	PNE_PM_LSU0_LDF,
	PNE_PM_LSU0_LDX,
	PNE_PM_LSU0_LMQ_LHR_MERGE,
	PNE_PM_LSU0_NCLD,
	PNE_PM_LSU0_REJECT_LHS,
	PNE_PM_LSU0_REJECT_LMQ_FULL,
	PNE_PM_LSU0_SRQ_STFWD,
	PNE_PM_LSU1_DC_PREF_STREAM_ALLOC,
	PNE_PM_LSU1_DC_PREF_STREAM_CONFIRM,
	PNE_PM_LSU1_DC_PREF_STREAM_CONFIRM_STRIDE,
	PNE_PM_LSU1_FLUSH_LRQ,
	PNE_PM_LSU1_FLUSH_SRQ,
	PNE_PM_LSU1_FLUSH_ULD,
	PNE_PM_LSU1_FLUSH_UST,
	PNE_PM_LSU1_L1_PREF,
	PNE_PM_LSU1_L1_SW_PREF,
	PNE_PM_LSU1_LDF,
	PNE_PM_LSU1_LDX,
	PNE_PM_LSU1_LMQ_LHR_MERGE,
	PNE_PM_LSU1_NCLD,
	PNE_PM_LSU1_REJECT_LHS,
	PNE_PM_LSU1_REJECT_LMQ_FULL,
	PNE_PM_LSU1_SRQ_STFWD,
	PNE_PM_LWSYNC,
	PNE_PM_LWSYNC_HELD,
	PNE_PM_MRK_BR_TAKEN,
	PNE_PM_MRK_DATA_FROM_DMEM,
	PNE_PM_MRK_DATA_FROM_L2,
	PNE_PM_MRK_DATA_FROM_L3,
	PNE_PM_MRK_DATA_FROM_L31_MOD,
	PNE_PM_MRK_DATA_FROM_L31_SHR,
	PNE_PM_MRK_DATA_FROM_RL2L3_MOD,
	PNE_PM_MRK_DATA_FROM_RL2L3_SHR,
	PNE_PM_MRK_DERAT_MISS_4K,
	PNE_PM_MRK_DTLB_MISS_16G,
	PNE_PM_MRK_FIN_STALL_CYC,
	PNE_PM_MRK_FIN_STALL_CYC_COUNT,
	PNE_PM_MRK_INST_ISSUED,
	PNE_PM_MRK_LD_MISS_EXPOSED_CYC,
	PNE_PM_MRK_LD_MISS_EXPOSED_CYC_COUNT,
	PNE_PM_MRK_LSU_FLUSH,
	PNE_PM_MRK_LSU_FLUSH_LRQ,
	PNE_PM_MRK_LSU_FLUSH_SRQ,
	PNE_PM_MRK_LSU_FLUSH_ULD,
	PNE_PM_MRK_LSU_FLUSH_UST,
	PNE_PM_MRK_LSU_PARTIAL_CDF,
	PNE_PM_MRK_LSU_REJECT_LHS,
	PNE_PM_MRK_PTEG_FROM_L2,
	PNE_PM_MRK_PTEG_FROM_L31_MOD,
	PNE_PM_MRK_PTEG_FROM_RL2L3_MOD,
	PNE_PM_MRK_ST_CMPL,
	PNE_PM_MRK_STCX_FAIL,
	PNE_PM_PB_NODE_PUMP,
	PNE_PM_MEM0_RQ_DISP,
	PNE_PM_NEST_PAIR0_ADD,
	PNE_PM_NEST_PAIR0_AND,
	PNE_PM_PMC2_SAVED,
	PNE_PM_PMC4_OVERFLOW,
	PNE_PM_PMC4_REWIND,
	PNE_PM_PMC5_OVERFLOW,
	PNE_PM_POWER_EVENT1,
	PNE_PM_PTEG_FROM_L2,
	PNE_PM_PTEG_FROM_L31_MOD,
	PNE_PM_PTEG_FROM_RL2L3_MOD,
	PNE_PM_RUN_SPURR,
	PNE_PM_SEG_EXCEPTION,
	PNE_PM_SHL_CREATED,
	PNE_PM_SHL_DEALLOCATED,
	PNE_PM_SHL_MATCH,
	PNE_PM_SHL_MERGED,
	PNE_PM_SLB_MISS,
	PNE_PM_SNOOP_TLBIE,
	PNE_PM_STCX_CMPL,
	PNE_PM_STCX_FAIL,
	PNE_PM_SUSPENDED,
	PNE_PM_TABLEWALK_CYC,
	PNE_PM_THERMAL_WARN,
	PNE_PM_THRD_1_RUN_CYC,
	PNE_PM_THRD_3_CONC_RUN_INST,
	PNE_PM_THRD_GRP_CMPL_BOTH_CYC,
	PNE_PM_THRD_PRIO_0_1_CYC,
	PNE_PM_THRD_PRIO_2_3_CYC,
	PNE_PM_THRD_PRIO_4_5_CYC,
	PNE_PM_THRD_PRIO_6_7_CYC,
	PNE_PM_THRESH_TIMEO,
	PNE_PM_VMX_RESULT_SAT_1,
	PNE_PM_VSU_1FLOP,
	PNE_PM_VSU_2FLOP,
	PNE_PM_VSU_2FLOP_DOUBLE,
	PNE_PM_VSU_4FLOP,
	PNE_PM_VSU_8FLOP,
	PNE_PM_VSU_DENORM,
	PNE_PM_VSU_FCONV,
	PNE_PM_VSU_FEST,
	PNE_PM_VSU_FIN,
	PNE_PM_VSU_FMA,
	PNE_PM_VSU_FMA_DOUBLE,
	PNE_PM_VSU_FRSP,
	PNE_PM_VSU_FSQRT_FDIV,
	PNE_PM_VSU_FSQRT_FDIV_DOUBLE,
	PNE_PM_VSU_SCALAR_DOUBLE_ISSUED,
	PNE_PM_VSU_SCALAR_SINGLE_ISSUED,
	PNE_PM_VSU_SIMPLE_ISSUED,
	PNE_PM_VSU_SINGLE,
	PNE_PM_VSU_STF,
	PNE_PM_VSU_VECTOR_DOUBLE_ISSUED,
	PNE_PM_VSU_VECTOR_SINGLE_ISSUED,
	PNE_PM_VSU0_16FLOP,
	PNE_PM_VSU0_1FLOP,
	PNE_PM_VSU0_2FLOP,
	PNE_PM_VSU0_2FLOP_DOUBLE,
	PNE_PM_VSU0_4FLOP,
	PNE_PM_VSU0_8FLOP,
	PNE_PM_VSU0_COMPLEX_ISSUED,
	PNE_PM_VSU0_DENORM,
	PNE_PM_VSU0_FCONV,
	PNE_PM_VSU0_FEST,
	PNE_PM_VSU0_FIN,
	PNE_PM_VSU0_FMA,
	PNE_PM_VSU0_FMA_DOUBLE,
	PNE_PM_VSU0_FPSCR,
	PNE_PM_VSU0_FRSP,
	PNE_PM_VSU0_FSQRT_FDIV,
	PNE_PM_VSU0_FSQRT_FDIV_DOUBLE,
	PNE_PM_VSU0_SCAL_DOUBLE_ISSUED,
	PNE_PM_VSU0_SCAL_SINGLE_ISSUED,
	PNE_PM_VSU0_SIMPLE_ISSUED,
	PNE_PM_VSU0_SINGLE,
	PNE_PM_VSU0_STF,
	PNE_PM_VSU0_VECT_DOUBLE_ISSUED,
	PNE_PM_VSU0_VECTOR_SP_ISSUED,
	PNE_PM_VSU1_1FLOP,
	PNE_PM_VSU1_2FLOP,
	PNE_PM_VSU1_2FLOP_DOUBLE,
	PNE_PM_VSU1_4FLOP,
	PNE_PM_VSU1_8FLOP,
	PNE_PM_VSU1_DD_ISSUED,
	PNE_PM_VSU1_DENORM,
	PNE_PM_VSU1_DQ_ISSUED,
	PNE_PM_VSU1_FCONV,
	PNE_PM_VSU1_FEST,
	PNE_PM_VSU1_FIN,
	PNE_PM_VSU1_FMA,
	PNE_PM_VSU1_FMA_DOUBLE,
	PNE_PM_VSU1_FRSP,
	PNE_PM_VSU1_FSQRT_FDIV,
	PNE_PM_VSU1_FSQRT_FDIV_DOUBLE,
	PNE_PM_VSU1_PERMUTE_ISSUED,
	PNE_PM_VSU1_SCAL_DOUBLE_ISSUED,
	PNE_PM_VSU1_SCAL_SINGLE_ISSUED,
	PNE_PM_VSU1_SINGLE,
	PNE_PM_VSU1_SQ,
	PNE_PM_VSU1_STF,
	PNE_PM_VSU1_VECT_DOUBLE_ISSUED,
	PNE_PM_BR_TAKEN,
	PNE_PM_CMPLU_STALL_DCACHE_MISS,
	PNE_PM_CMPLU_STALL_DFU,
	PNE_PM_CMPLU_STALL_FXU,
	PNE_PM_CMPLU_STALL_LSU,
	PNE_PM_CMPLU_STALL_SCALAR_LONG,
	PNE_PM_CMPLU_STALL_STORE,
	PNE_PM_CMPLU_STALL_VECTOR,
	PNE_PM_DATA_FROM_L2MISS,
	PNE_PM_DATA_FROM_L3MISS,
	PNE_PM_DERAT_MISS_64K,
	PNE_PM_DPU_HELD_POWER,
	PNE_PM_DTLB_MISS_4K,
	PNE_PM_EXT_INT,
	PNE_PM_FXU_BUSY,
	PNE_PM_GCT_EMPTY_CYC,
	PNE_PM_GCT_NOSLOT_IC_MISS,
	PNE_PM_HV_CYC,
	PNE_PM_INST_DISP,
	PNE_PM_INST_FROM_L3MISS,
	PNE_PM_INST_PTEG_FROM_DMEM,
	PNE_PM_INST_PTEG_FROM_L3,
	PNE_PM_INST_PTEG_FROM_L31_SHR,
	PNE_PM_INST_PTEG_FROM_L3MISS,
	PNE_PM_INST_PTEG_FROM_RL2L3_SHR,
	PNE_PM_L1_ICACHE_MISS,
	PNE_PM_L2_DC_INV,
	PNE_PM_L2_GLOB_GUESS_WRONG,
	PNE_PM_L2_IC_INV,
	PNE_PM_L2_LD_MISS,
	PNE_PM_L2_LDST_MISS,
	PNE_PM_L2_LOC_GUESS_WRONG,
	PNE_PM_L2_RCLD_DISP_FAIL_OTHER,
	PNE_PM_L2_RCST_BUSY_RC_FULL,
	PNE_PM_L2_ST_MISS,
	PNE_PM_L3_LD_HIT,
	PNE_PM_L3_LD_MISS,
	PNE_PM_LSU_DERAT_MISS,
	PNE_PM_LSU_LMQ_SRQ_EMPTY_CYC,
	PNE_PM_LSU_REJECT_ERAT_MISS,
	PNE_PM_MRK_BRU_FIN,
	PNE_PM_MRK_DATA_FROM_DL2L3_SHR_CYC,
	PNE_PM_MRK_DATA_FROM_DMEM_CYC,
	PNE_PM_MRK_DATA_FROM_L2_CYC,
	PNE_PM_MRK_DATA_FROM_L21_SHR_CYC,
	PNE_PM_MRK_DATA_FROM_L31_SHR_CYC,
	PNE_PM_MRK_DATA_FROM_L3MISS,
	PNE_PM_MRK_DATA_FROM_LMEM_CYC,
	PNE_PM_MRK_DATA_FROM_RL2L3_SHR_CYC,
	PNE_PM_MRK_DERAT_MISS_64K,
	PNE_PM_MRK_DFU_FIN,
	PNE_PM_MRK_DTLB_MISS_4K,
	PNE_PM_MRK_FXU_FIN,
	PNE_PM_MRK_INST_DISP,
	PNE_PM_MRK_LD_MISS_L1,
	PNE_PM_MRK_PTEG_FROM_DMEM,
	PNE_PM_MRK_PTEG_FROM_L3,
	PNE_PM_MRK_PTEG_FROM_L31_SHR,
	PNE_PM_MRK_PTEG_FROM_L3MISS,
	PNE_PM_MRK_PTEG_FROM_RL2L3_SHR,
	PNE_PM_MRK_ST_NEST,
	PNE_PM_PB_SYS_PUMP,
	PNE_PM_MEM0_PREFETCH_DISP,
	PNE_PM_NEST_PAIR1_ADD,
	PNE_PM_NEST_PAIR1_AND,
	PNE_PM_PMC1_OVERFLOW,
	PNE_PM_POWER_EVENT2,
	PNE_PM_PTEG_FROM_DMEM,
	PNE_PM_PTEG_FROM_L3,
	PNE_PM_PTEG_FROM_L31_SHR,
	PNE_PM_PTEG_FROM_L3MISS,
	PNE_PM_PTEG_FROM_RL2L3_SHR,
	PNE_PM_RUN_CYC,
	PNE_PM_ST_FIN,
	PNE_PM_THRD_2_RUN_CYC,
	PNE_PM_THRD_4_CONC_RUN_INST,
	PNE_PM_THRD_ALL_RUN_CYC,
	PNE_PM_TLB_MISS,
	PNE_PM_1THRD_CON_RUN_INSTR,
	PNE_PM_DATA_FROM_DL2L3_MOD,
	PNE_PM_DATA_FROM_DL2L3_SHR,
	PNE_PM_DATA_FROM_L21_MOD,
	PNE_PM_DATA_FROM_L21_SHR,
	PNE_PM_DATA_FROM_LMEM,
	PNE_PM_DATA_FROM_RMEM,
	PNE_PM_DATA_TABLEWALK_CYC,
	PNE_PM_DERAT_MISS_16M,
	PNE_PM_DISP_HELD_THERMAL,
	PNE_PM_DISP_WT,
	PNE_PM_DTLB_MISS,
	PNE_PM_DTLB_MISS_64K,
	PNE_PM_FLUSH_COMPLETION,
	PNE_PM_FREQ_DOWN,
	PNE_PM_FXU0_BUSY_FXU1_IDLE,
	PNE_PM_GRP_CMPL,
	PNE_PM_GRP_DISP,
	PNE_PM_INST_FROM_DL2L3_MOD,
	PNE_PM_INST_FROM_DL2L3_SHR,
	PNE_PM_INST_FROM_L21_MOD,
	PNE_PM_INST_FROM_L21_SHR,
	PNE_PM_INST_FROM_LMEM,
	PNE_PM_INST_FROM_RMEM,
	PNE_PM_INST_IMC_MATCH_DISP,
	PNE_PM_INST_PTEG_FROM_L21_MOD,
	PNE_PM_INST_PTEG_FROM_DL2L3_SHR,
	PNE_PM_INST_PTEG_FROM_RMEM,
	PNE_PM_IOPS_DISP,
	PNE_PM_L1_DCACHE_RELOAD_VALID,
	PNE_PM_L2_INST,
	PNE_PM_L2_INST_MISS,
	PNE_PM_L2_LD_DISP,
	PNE_PM_L2_LD_HIT,
	PNE_PM_L2_NODE_PUMP,
	PNE_PM_L2_RC_ST_DONE,
	PNE_PM_L2_RCST_DISP,
	PNE_PM_L2_RCST_DISP_FAIL_ADDR,
	PNE_PM_L2_SN_SX_I_DONE,
	PNE_PM_L2_SYS_PUMP,
	PNE_PM_L3_PREF_HIT,
	PNE_PM_L3_PREF_MISS,
	PNE_PM_LSU_FIN,
	PNE_PM_LSU_LMQ_SRQ_EMPTY_ALL_CYC,
	PNE_PM_MRK_BR_MPRED,
	PNE_PM_MRK_DATA_FROM_DL2L3_MOD,
	PNE_PM_MRK_DATA_FROM_DL2L3_SHR,
	PNE_PM_MRK_DATA_FROM_L21_MOD,
	PNE_PM_MRK_DATA_FROM_L21_SHR,
	PNE_PM_MRK_DATA_FROM_LMEM,
	PNE_PM_MRK_DATA_FROM_RMEM,
	PNE_PM_MRK_DERAT_MISS_16M,
	PNE_PM_MRK_DTLB_MISS_64K,
	PNE_PM_MRK_IFU_FIN,
	PNE_PM_MRK_INST_FIN,
	PNE_PM_MRK_LSU_DERAT_MISS,
	PNE_PM_MRK_LSU_REJECT_ERAT_MISS,
	PNE_PM_MRK_PTEG_FROM_L21_MOD,
	PNE_PM_MRK_PTEG_FROM_DL2L3_SHR,
	PNE_PM_MRK_PTEG_FROM_RMEM,
	PNE_PM_MRK_ST_CMPL_INT,
	PNE_PM_MRK_STALL_CMPLU_CYC,
	PNE_PM_MRK_STALL_CMPLU_CYC_COUNT,
	PNE_PM_MRK_VSU_FIN,
	PNE_PM_PB_RETRY_NODE_PUMP,
	PNE_PM_MEM0_RD_CANCEL_TOTAL,
	PNE_PM_NEST_PAIR2_ADD,
	PNE_PM_NEST_PAIR2_AND,
	PNE_PM_PMC2_OVERFLOW,
	PNE_PM_PMC2_REWIND,
	PNE_PM_PMC4_SAVED,
	PNE_PM_PMC6_OVERFLOW,
	PNE_PM_POWER_EVENT3,
	PNE_PM_PTEG_FROM_L21_MOD,
	PNE_PM_PTEG_FROM_DL2L3_SHR,
	PNE_PM_PTEG_FROM_RMEM,
	PNE_PM_ST_MISS_L1,
	PNE_PM_TB_BIT_TRANS,
	PNE_PM_THRD_3_RUN_CYC,
	PNE_PM_THRD_CONC_RUN_INST,
	PNE_PM_1PLUS_PPC_DISP,
	PNE_PM_BR_MPRED,
	PNE_PM_CMPLU_STALL,
	PNE_PM_CMPLU_STALL_BRU,
	PNE_PM_CMPLU_STALL_DIV,
	PNE_PM_CMPLU_STALL_ERAT_MISS,
	PNE_PM_CMPLU_STALL_IFU,
	PNE_PM_CMPLU_STALL_REJECT,
	PNE_PM_CMPLU_STALL_SCALAR,
	PNE_PM_CMPLU_STALL_VECTOR_LONG,
	PNE_PM_DERAT_MISS_16G,
	PNE_PM_DTLB_MISS_16M,
	PNE_PM_FLUSH,
	PNE_PM_FREQ_UP,
	PNE_PM_FXU1_BUSY_FXU0_IDLE,
	PNE_PM_FXU1_FIN,
	PNE_PM_GCT_NOSLOT_BR_MPRED,
	PNE_PM_GCT_NOSLOT_BR_MPRED_IC_MISS,
	PNE_PM_IFU_FIN,
	PNE_PM_INST_FROM_L2MISS,
	PNE_PM_INST_PTEG_FROM_DL2L3_MOD,
	PNE_PM_INST_PTEG_FROM_L21_SHR,
	PNE_PM_INST_PTEG_FROM_L2MISS,
	PNE_PM_INST_PTEG_FROM_LMEM,
	PNE_PM_ITLB_MISS,
	PNE_PM_L2_DISP_ALL,
	PNE_PM_L2_RCLD_BUSY_RC_FULL,
	PNE_PM_L2_RCST_DISP_FAIL_OTHER,
	PNE_PM_L2_SN_M_RD_DONE,
	PNE_PM_L2_SN_M_WR_DONE,
	PNE_PM_L2_ST_DISP,
	PNE_PM_L2_ST_HIT,
	PNE_PM_L3_CO_MEM,
	PNE_PM_L3_CO_L31,
	PNE_PM_L3_PREF_BUSY,
	PNE_PM_L3_RD_BUSY,
	PNE_PM_LD_MISS_L1,
	PNE_PM_LSU_SRQ_EMPTY_CYC,
	PNE_PM_MRK_DATA_FROM_DL2L3_MOD_CYC,
	PNE_PM_MRK_DATA_FROM_L21_MOD_CYC,
	PNE_PM_MRK_DATA_FROM_L2MISS,
	PNE_PM_MRK_DATA_FROM_L3_CYC,
	PNE_PM_MRK_DATA_FROM_L31_MOD_CYC,
	PNE_PM_MRK_DATA_FROM_RL2L3_MOD_CYC,
	PNE_PM_MRK_DATA_FROM_RMEM_CYC,
	PNE_PM_MRK_DERAT_MISS_16G,
	PNE_PM_MRK_DTLB_MISS_16M,
	PNE_PM_MRK_GRP_CMPL,
	PNE_PM_MRK_GRP_IC_MISS,
	PNE_PM_MRK_INST_TIMEO,
	PNE_PM_MRK_LD_MISS_L1_CYC,
	PNE_PM_MRK_LSU_FIN,
	PNE_PM_MRK_LSU_REJECT,
	PNE_PM_MRK_PTEG_FROM_DL2L3_MOD,
	PNE_PM_MRK_PTEG_FROM_L21_SHR,
	PNE_PM_MRK_PTEG_FROM_L2MISS,
	PNE_PM_MRK_PTEG_FROM_LMEM,
	PNE_PM_PB_RETRY_SYS_PUMP,
	PNE_PM_MEM0_WQ_DISP,
	PNE_PM_NEST_PAIR3_ADD,
	PNE_PM_NEST_PAIR3_AND,
	PNE_PM_PMC3_OVERFLOW,
	PNE_PM_POWER_EVENT4,
	PNE_PM_PTEG_FROM_DL2L3_MOD,
	PNE_PM_PTEG_FROM_L21_SHR,
	PNE_PM_PTEG_FROM_L2MISS,
	PNE_PM_PTEG_FROM_LMEM,
	PNE_PM_RUN_INST_CMPL,
	PNE_PM_RUN_PURR,
	PNE_PM_THERMAL_MAX,
	PNE_PM_THRD_2_CONC_RUN_INSTR,
	PNE_PM_THRD_4_RUN_CYC,
	NATNAME_GUARD,
};

#endif
