#ifndef _LINUX_BGP_NATIVE_EVENTS_H
#define _LINUX_BGP_NATIVE_EVENTS_H

/* BEGIN PYTHON GENERATION 1 (Do not modify or move this comment) */
/*
 * Native BG/P Performance Counter Events
 */
typedef enum _papi_hwd_bgp_native_event_id
{

	/* User Mode 0 */
	PNE_BGP_PU0_JPIPE_INSTRUCTIONS = 0x40000000,
	PNE_BGP_PU0_JPIPE_ADD_SUB,
	PNE_BGP_PU0_JPIPE_LOGICAL_OPS,
	PNE_BGP_PU0_JPIPE_SHROTMK,
	PNE_BGP_PU0_IPIPE_INSTRUCTIONS,
	PNE_BGP_PU0_IPIPE_MULT_DIV,
	PNE_BGP_PU0_IPIPE_ADD_SUB,
	PNE_BGP_PU0_IPIPE_LOGICAL_OPS,
	PNE_BGP_PU0_IPIPE_SHROTMK,
	PNE_BGP_PU0_IPIPE_BRANCHES,
	PNE_BGP_PU0_IPIPE_TLB_OPS,
	PNE_BGP_PU0_IPIPE_PROCESS_CONTROL,
	PNE_BGP_PU0_IPIPE_OTHER,
	PNE_BGP_PU0_DCACHE_LINEFILLINPROG,
	PNE_BGP_PU0_ICACHE_LINEFILLINPROG,
	PNE_BGP_PU0_DCACHE_MISS,
	PNE_BGP_PU0_DCACHE_HIT,
	PNE_BGP_PU0_DATA_LOADS,
	PNE_BGP_PU0_DATA_STORES,
	PNE_BGP_PU0_DCACHE_OPS,
	PNE_BGP_PU0_ICACHE_MISS,
	PNE_BGP_PU0_ICACHE_HIT,
	PNE_BGP_PU0_FPU_ADD_SUB_1,
	PNE_BGP_PU0_FPU_MULT_1,
	PNE_BGP_PU0_FPU_FMA_2,
	PNE_BGP_PU0_FPU_DIV_1,
	PNE_BGP_PU0_FPU_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU0_FPU_ADD_SUB_2,
	PNE_BGP_PU0_FPU_MULT_2,
	PNE_BGP_PU0_FPU_FMA_4,
	PNE_BGP_PU0_FPU_DUAL_PIPE_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU0_FPU_QUADWORD_LOADS,
	PNE_BGP_PU0_FPU_OTHER_LOADS,
	PNE_BGP_PU0_FPU_QUADWORD_STORES,
	PNE_BGP_PU0_FPU_OTHER_STORES,
	PNE_BGP_PU1_JPIPE_INSTRUCTIONS,
	PNE_BGP_PU1_JPIPE_ADD_SUB,
	PNE_BGP_PU1_JPIPE_LOGICAL_OPS,
	PNE_BGP_PU1_JPIPE_SHROTMK,
	PNE_BGP_PU1_IPIPE_INSTRUCTIONS,
	PNE_BGP_PU1_IPIPE_MULT_DIV,
	PNE_BGP_PU1_IPIPE_ADD_SUB,
	PNE_BGP_PU1_IPIPE_LOGICAL_OPS,
	PNE_BGP_PU1_IPIPE_SHROTMK,
	PNE_BGP_PU1_IPIPE_BRANCHES,
	PNE_BGP_PU1_IPIPE_TLB_OPS,
	PNE_BGP_PU1_IPIPE_PROCESS_CONTROL,
	PNE_BGP_PU1_IPIPE_OTHER,
	PNE_BGP_PU1_DCACHE_LINEFILLINPROG,
	PNE_BGP_PU1_ICACHE_LINEFILLINPROG,
	PNE_BGP_PU1_DCACHE_MISS,
	PNE_BGP_PU1_DCACHE_HIT,
	PNE_BGP_PU1_DATA_LOADS,
	PNE_BGP_PU1_DATA_STORES,
	PNE_BGP_PU1_DCACHE_OPS,
	PNE_BGP_PU1_ICACHE_MISS,
	PNE_BGP_PU1_ICACHE_HIT,
	PNE_BGP_PU1_FPU_ADD_SUB_1,
	PNE_BGP_PU1_FPU_MULT_1,
	PNE_BGP_PU1_FPU_FMA_2,
	PNE_BGP_PU1_FPU_DIV_1,
	PNE_BGP_PU1_FPU_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU1_FPU_ADD_SUB_2,
	PNE_BGP_PU1_FPU_MULT_2,
	PNE_BGP_PU1_FPU_FMA_4,
	PNE_BGP_PU1_FPU_DUAL_PIPE_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU1_FPU_QUADWORD_LOADS,
	PNE_BGP_PU1_FPU_OTHER_LOADS,
	PNE_BGP_PU1_FPU_QUADWORD_STORES,
	PNE_BGP_PU1_FPU_OTHER_STORES,
	PNE_BGP_PU0_L1_INVALIDATION_REQUESTS,
	PNE_BGP_PU1_L1_INVALIDATION_REQUESTS,
	PNE_BGP_PU0_L2_VALID_PREFETCH_REQUESTS,
	PNE_BGP_PU0_L2_PREFETCH_HITS_IN_FILTER,
	PNE_BGP_PU0_L2_PREFETCH_HITS_IN_STREAM,
	PNE_BGP_PU0_L2_CYCLES_PREFETCH_PENDING,
	PNE_BGP_PU0_L2_PAGE_ALREADY_IN_L2,
	PNE_BGP_PU0_L2_PREFETCH_SNOOP_HIT_SAME_CORE,
	PNE_BGP_PU0_L2_PREFETCH_SNOOP_HIT_OTHER_CORE,
	PNE_BGP_PU0_L2_PREFETCH_SNOOP_HIT_PLB,
	PNE_BGP_PU0_L2_CYCLES_READ_REQUEST_PENDING,
	PNE_BGP_PU0_L2_READ_REQUESTS,
	PNE_BGP_PU0_L2_DEVBUS_READ_REQUESTS,
	PNE_BGP_PU0_L2_L3_READ_REQUESTS,
	PNE_BGP_PU0_L2_NETBUS_READ_REQUESTS,
	PNE_BGP_PU0_L2_BLIND_DEV_READ_REQUESTS,
	PNE_BGP_PU0_L2_PREFETCHABLE_REQUESTS,
	PNE_BGP_PU0_L2_HIT,
	PNE_BGP_PU0_L2_SAME_CORE_SNOOPS,
	PNE_BGP_PU0_L2_OTHER_CORE_SNOOPS,
	PNE_BGP_PU0_L2_OTHER_DP_PU0_SNOOPS,
	PNE_BGP_PU0_L2_OTHER_DP_PU1_SNOOPS,
	PNE_BGP_PU0_L2_MEMORY_WRITES = 0x40000065,
	PNE_BGP_PU0_L2_NETWORK_WRITES,
	PNE_BGP_PU0_L2_DEVBUS_WRITES,
	PNE_BGP_PU1_L2_VALID_PREFETCH_REQUESTS,
	PNE_BGP_PU1_L2_PREFETCH_HITS_IN_FILTER,
	PNE_BGP_PU1_L2_PREFETCH_HITS_IN_STREAM,
	PNE_BGP_PU1_L2_CYCLES_PREFETCH_PENDING,
	PNE_BGP_PU1_L2_PAGE_ALREADY_IN_L2,
	PNE_BGP_PU1_L2_PREFETCH_SNOOP_HIT_SAME_CORE,
	PNE_BGP_PU1_L2_PREFETCH_SNOOP_HIT_OTHER_CORE,
	PNE_BGP_PU1_L2_PREFETCH_SNOOP_HIT_PLB,
	PNE_BGP_PU1_L2_CYCLES_READ_REQUEST_PENDING,
	PNE_BGP_PU1_L2_READ_REQUESTS,
	PNE_BGP_PU1_L2_DEVBUS_READ_REQUESTS,
	PNE_BGP_PU1_L2_L3_READ_REQUESTS,
	PNE_BGP_PU1_L2_NETBUS_READ_REQUESTS,
	PNE_BGP_PU1_L2_BLIND_DEV_READ_REQUESTS,
	PNE_BGP_PU1_L2_PREFETCHABLE_REQUESTS,
	PNE_BGP_PU1_L2_HIT,
	PNE_BGP_PU1_L2_SAME_CORE_SNOOPS,
	PNE_BGP_PU1_L2_OTHER_CORE_SNOOPS,
	PNE_BGP_PU1_L2_OTHER_DP_PU0_SNOOPS,
	PNE_BGP_PU1_L2_OTHER_DP_PU1_SNOOPS,
	PNE_BGP_PU1_L2_MEMORY_WRITES = 0x40000085,
	PNE_BGP_PU1_L2_NETWORK_WRITES,
	PNE_BGP_PU1_L2_DEVBUS_WRITES,
	PNE_BGP_L3_M0_RD0_SINGLE_LINE_DELIVERED_L2,
	PNE_BGP_L3_M0_RD0_BURST_DELIVERED_L2,
	PNE_BGP_L3_M0_RD0_READ_RETURN_COLLISION,
	PNE_BGP_L3_M0_RD0_DIR0_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M0_RD0_DIR0_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M0_RD0_DIR1_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M0_RD0_DIR1_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M0_RD1_SINGLE_LINE_DELIVERED_L2,
	PNE_BGP_L3_M0_RD1_BURST_DELIVERED_L2,
	PNE_BGP_L3_M0_RD1_READ_RETURN_COLLISION,
	PNE_BGP_L3_M0_RD1_DIR0_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M0_RD1_DIR0_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M0_RD1_DIR1_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M0_RD1_DIR1_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M0_DIR0_LOOKUPS,
	PNE_BGP_L3_M0_DIR0_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M0_DIR1_LOOKUPS,
	PNE_BGP_L3_M0_DIR1_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M0_MH_DDR_STORES,
	PNE_BGP_L3_M0_MH_DDR_FETCHES,
	PNE_BGP_L3_M1_RD0_SINGLE_LINE_DELIVERED_L2,
	PNE_BGP_L3_M1_RD0_BURST_DELIVERED_L2,
	PNE_BGP_L3_M1_RD0_READ_RETURN_COLLISION,
	PNE_BGP_L3_M1_RD0_DIR0_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M1_RD0_DIR0_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M1_RD0_DIR1_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M1_RD0_DIR1_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M1_RD1_SINGLE_LINE_DELIVERED_L2,
	PNE_BGP_L3_M1_RD1_BURST_DELIVERED_L2,
	PNE_BGP_L3_M1_RD1_READ_RETURN_COLLISION,
	PNE_BGP_L3_M1_RD1_DIR0_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M1_RD1_DIR0_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M1_RD1_DIR1_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M1_RD1_DIR1_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M1_DIR0_LOOKUPS,
	PNE_BGP_L3_M1_DIR0_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M1_DIR1_LOOKUPS,
	PNE_BGP_L3_M1_DIR1_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M1_MH_DDR_STORES,
	PNE_BGP_L3_M1_MH_DDR_FETCHES,
	PNE_BGP_PU0_SNOOP_PORT0_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU0_SNOOP_PORT1_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU0_SNOOP_PORT2_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU0_SNOOP_PORT3_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU0_SNOOP_PORT0_REJECTED_REQUESTS,
	PNE_BGP_PU0_SNOOP_PORT1_REJECTED_REQUESTS,
	PNE_BGP_PU0_SNOOP_PORT2_REJECTED_REQUESTS,
	PNE_BGP_PU0_SNOOP_PORT3_REJECTED_REQUESTS,
	PNE_BGP_PU0_SNOOP_L1_CACHE_WRAP,
	PNE_BGP_PU1_SNOOP_PORT0_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU1_SNOOP_PORT1_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU1_SNOOP_PORT2_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU1_SNOOP_PORT3_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU1_SNOOP_PORT0_REJECTED_REQUESTS,
	PNE_BGP_PU1_SNOOP_PORT1_REJECTED_REQUESTS,
	PNE_BGP_PU1_SNOOP_PORT2_REJECTED_REQUESTS,
	PNE_BGP_PU1_SNOOP_PORT3_REJECTED_REQUESTS,
	PNE_BGP_PU1_SNOOP_L1_CACHE_WRAP,
	PNE_BGP_TORUS_XP_PACKETS,
	PNE_BGP_TORUS_XP_32BCHUNKS,
	PNE_BGP_TORUS_XM_PACKETS,
	PNE_BGP_TORUS_XM_32BCHUNKS,
	PNE_BGP_TORUS_YP_PACKETS,
	PNE_BGP_TORUS_YP_32BCHUNKS,
	PNE_BGP_TORUS_YM_PACKETS,
	PNE_BGP_TORUS_YM_32BCHUNKS,
	PNE_BGP_TORUS_ZP_PACKETS,
	PNE_BGP_TORUS_ZP_32BCHUNKS,
	PNE_BGP_TORUS_ZM_PACKETS,
	PNE_BGP_TORUS_ZM_32BCHUNKS,
	PNE_BGP_DMA_PACKETS_INJECTED,
	PNE_BGP_DMA_DESCRIPTORS_READ_FROM_L3,
	PNE_BGP_DMA_FIFO_PACKETS_RECEIVED,
	PNE_BGP_DMA_COUNTER_PACKETS_RECEIVED,
	PNE_BGP_DMA_REMOTE_GET_PACKETS_RECEIVED,
	PNE_BGP_DMA_IDPU_READ_REQUESTS_TO_L3,
	PNE_BGP_DMA_READ_VALID_RETURNED,
	PNE_BGP_DMA_ACKED_READ_REQUESTS,
	PNE_BGP_DMA_CYCLES_RDPU_WRITE_ACTIVE,
	PNE_BGP_DMA_WRITE_REQUESTS_TO_L3,
	PNE_BGP_COL_AC_CH2_VC0_MATURE = 0x400000DE,
	PNE_BGP_COL_AC_CH1_VC0_MATURE,
	PNE_BGP_COL_AC_CH0_VC0_MATURE,
	PNE_BGP_COL_AC_INJECT_VC0_MATURE,
	PNE_BGP_COL_AC_CH2_VC1_MATURE,
	PNE_BGP_COL_AC_CH1_VC1_MATURE,
	PNE_BGP_COL_AC_CH0_VC1_MATURE,
	PNE_BGP_COL_AC_INJECT_VC1_MATURE,
	PNE_BGP_COL_AC_PENDING_REQUESTS,
	PNE_BGP_COL_AC_WAITING_REQUESTS,
	PNE_BGP_COL_AR2_PACKET_TAKEN,
	PNE_BGP_COL_AR1_PACKET_TAKEN,
	PNE_BGP_COL_AR0_PACKET_TAKEN,
	PNE_BGP_COL_ALC_PACKET_TAKEN,
	PNE_BGP_COL_AR0_VC0_DATA_PACKETS_RECEIVED,
	PNE_BGP_COL_AR0_VC1_DATA_PACKETS_RECEIVED,
	PNE_BGP_COL_AR1_VC0_DATA_PACKETS_RECEIVED,
	PNE_BGP_COL_AR1_VC1_DATA_PACKETS_RECEIVED,
	PNE_BGP_COL_AR2_VC0_DATA_PACKETS_RECEIVED,
	PNE_BGP_COL_AR2_VC1_DATA_PACKETS_RECEIVED,
	PNE_BGP_COL_AS0_VC0_DATA_PACKETS_SENT,
	PNE_BGP_COL_AS0_VC1_DATA_PACKETS_SENT,
	PNE_BGP_COL_AS1_VC0_DATA_PACKETS_SENT,
	PNE_BGP_COL_AS1_VC1_DATA_PACKETS_SENT,
	PNE_BGP_COL_AS2_VC0_DATA_PACKETS_SENT,
	PNE_BGP_COL_AS2_VC1_DATA_PACKETS_SENT,
	PNE_BGP_COL_INJECT_VC0_HEADER,
	PNE_BGP_COL_INJECT_VC1_HEADER,
	PNE_BGP_COL_RECEPTION_VC0_PACKET_ADDED,
	PNE_BGP_COL_RECEPTION_VC1_PACKET_ADDED,
	PNE_BGP_IC_TIMESTAMP,
	PNE_BGP_MISC_ELAPSED_TIME = 0x400000FF,

	/* User Mode 1 */
	PNE_BGP_PU2_JPIPE_INSTRUCTIONS = 0x40000100,
	PNE_BGP_PU2_JPIPE_ADD_SUB,
	PNE_BGP_PU2_JPIPE_LOGICAL_OPS,
	PNE_BGP_PU2_JPIPE_SHROTMK,
	PNE_BGP_PU2_IPIPE_INSTRUCTIONS,
	PNE_BGP_PU2_IPIPE_MULT_DIV,
	PNE_BGP_PU2_IPIPE_ADD_SUB,
	PNE_BGP_PU2_IPIPE_LOGICAL_OPS,
	PNE_BGP_PU2_IPIPE_SHROTMK,
	PNE_BGP_PU2_IPIPE_BRANCHES,
	PNE_BGP_PU2_IPIPE_TLB_OPS,
	PNE_BGP_PU2_IPIPE_PROCESS_CONTROL,
	PNE_BGP_PU2_IPIPE_OTHER,
	PNE_BGP_PU2_DCACHE_LINEFILLINPROG,
	PNE_BGP_PU2_ICACHE_LINEFILLINPROG,
	PNE_BGP_PU2_DCACHE_MISS,
	PNE_BGP_PU2_DCACHE_HIT,
	PNE_BGP_PU2_DATA_LOADS,
	PNE_BGP_PU2_DATA_STORES,
	PNE_BGP_PU2_DCACHE_OPS,
	PNE_BGP_PU2_ICACHE_MISS,
	PNE_BGP_PU2_ICACHE_HIT,
	PNE_BGP_PU2_FPU_ADD_SUB_1,
	PNE_BGP_PU2_FPU_MULT_1,
	PNE_BGP_PU2_FPU_FMA_2,
	PNE_BGP_PU2_FPU_DIV_1,
	PNE_BGP_PU2_FPU_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU2_FPU_ADD_SUB_2,
	PNE_BGP_PU2_FPU_MULT_2,
	PNE_BGP_PU2_FPU_FMA_4,
	PNE_BGP_PU2_FPU_DUAL_PIPE_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU2_FPU_QUADWORD_LOADS,
	PNE_BGP_PU2_FPU_OTHER_LOADS,
	PNE_BGP_PU2_FPU_QUADWORD_STORES,
	PNE_BGP_PU2_FPU_OTHER_STORES,
	PNE_BGP_PU3_JPIPE_INSTRUCTIONS,
	PNE_BGP_PU3_JPIPE_ADD_SUB,
	PNE_BGP_PU3_JPIPE_LOGICAL_OPS,
	PNE_BGP_PU3_JPIPE_SHROTMK,
	PNE_BGP_PU3_IPIPE_INSTRUCTIONS,
	PNE_BGP_PU3_IPIPE_MULT_DIV,
	PNE_BGP_PU3_IPIPE_ADD_SUB,
	PNE_BGP_PU3_IPIPE_LOGICAL_OPS,
	PNE_BGP_PU3_IPIPE_SHROTMK,
	PNE_BGP_PU3_IPIPE_BRANCHES,
	PNE_BGP_PU3_IPIPE_TLB_OPS,
	PNE_BGP_PU3_IPIPE_PROCESS_CONTROL,
	PNE_BGP_PU3_IPIPE_OTHER,
	PNE_BGP_PU3_DCACHE_LINEFILLINPROG,
	PNE_BGP_PU3_ICACHE_LINEFILLINPROG,
	PNE_BGP_PU3_DCACHE_MISS,
	PNE_BGP_PU3_DCACHE_HIT,
	PNE_BGP_PU3_DATA_LOADS,
	PNE_BGP_PU3_DATA_STORES,
	PNE_BGP_PU3_DCACHE_OPS,
	PNE_BGP_PU3_ICACHE_MISS,
	PNE_BGP_PU3_ICACHE_HIT,
	PNE_BGP_PU3_FPU_ADD_SUB_1,
	PNE_BGP_PU3_FPU_MULT_1,
	PNE_BGP_PU3_FPU_FMA_2,
	PNE_BGP_PU3_FPU_DIV_1,
	PNE_BGP_PU3_FPU_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU3_FPU_ADD_SUB_2,
	PNE_BGP_PU3_FPU_MULT_2,
	PNE_BGP_PU3_FPU_FMA_4,
	PNE_BGP_PU3_FPU_DUAL_PIPE_OTHER_NON_STORAGE_OPS,
	PNE_BGP_PU3_FPU_QUADWORD_LOADS,
	PNE_BGP_PU3_FPU_OTHER_LOADS,
	PNE_BGP_PU3_FPU_QUADWORD_STORES,
	PNE_BGP_PU3_FPU_OTHER_STORES,
	PNE_BGP_PU2_L1_INVALIDATION_REQUESTS,
	PNE_BGP_PU3_L1_INVALIDATION_REQUESTS,
	PNE_BGP_COL_AC_CH2_VC0_MATURE_UM1,
	PNE_BGP_COL_AC_CH1_VC0_MATURE_UM1,
	PNE_BGP_COL_AC_CH0_VC0_MATURE_UM1,
	PNE_BGP_COL_AC_INJECT_VC0_MATURE_UM1,
	PNE_BGP_COL_AC_CH2_VC1_MATURE_UM1,
	PNE_BGP_COL_AC_CH1_VC1_MATURE_UM1,
	PNE_BGP_COL_AC_CH0_VC1_MATURE_UM1,
	PNE_BGP_COL_AC_INJECT_VC1_MATURE_UM1,
	PNE_BGP_COL_AR0_VC0_EMPTY_PACKET,
	PNE_BGP_COL_AR0_VC1_EMPTY_PACKET,
	PNE_BGP_COL_AR0_IDLE_PACKET,
	PNE_BGP_COL_AR0_BAD_PACKET_MARKER,
	PNE_BGP_COL_AR0_VC0_CUT_THROUGH,
	PNE_BGP_COL_AR0_VC1_CUT_THROUGH,
	PNE_BGP_COL_AR0_HEADER_PARITY_ERROR,
	PNE_BGP_COL_AR0_UNEXPECTED_HEADER_ERROR,
	PNE_BGP_COL_AR0_RESYNC,
	PNE_BGP_COL_AR1_VC0_EMPTY_PACKET,
	PNE_BGP_COL_AR1_VC1_EMPTY_PACKET,
	PNE_BGP_COL_AR1_IDLE_PACKET,
	PNE_BGP_COL_AR1_BAD_PACKET_MARKER,
	PNE_BGP_COL_AR1_VC0_CUT_THROUGH,
	PNE_BGP_COL_AR1_VC1_CUT_THROUGH,
	PNE_BGP_COL_AR1_HEADER_PARITY_ERROR,
	PNE_BGP_COL_AR1_UNEXPECTED_HEADER_ERROR,
	PNE_BGP_COL_AR1_RESYNC,
	PNE_BGP_COL_AR2_VC0_EMPTY_PACKET,
	PNE_BGP_COL_AR2_VC1_EMPTY_PACKET,
	PNE_BGP_COL_AR2_IDLE_PACKET,
	PNE_BGP_COL_AR2_BAD_PACKET_MARKER,
	PNE_BGP_COL_AR2_VC0_CUT_THROUGH,
	PNE_BGP_COL_AR2_VC1_CUT_THROUGH,
	PNE_BGP_COL_AR2_HEADER_PARITY_ERROR,
	PNE_BGP_COL_AR2_UNEXPECTED_HEADER_ERROR,
	PNE_BGP_COL_AR2_RESYNC,
	PNE_BGP_COL_AS0_VC0_CUT_THROUGH,
	PNE_BGP_COL_AS0_VC1_CUT_THROUGH,
	PNE_BGP_COL_AS0_VC0_PACKETS_SENT,
	PNE_BGP_COL_AS0_VC1_PACKETS_SENT,
	PNE_BGP_COL_AS0_IDLE_PACKETS_SENT,
	PNE_BGP_COL_AS1_VC0_CUT_THROUGH,
	PNE_BGP_COL_AS1_VC1_CUT_THROUGH,
	PNE_BGP_COL_AS1_VC0_PACKETS_SENT,
	PNE_BGP_COL_AS1_VC1_PACKETS_SENT,
	PNE_BGP_COL_AS1_IDLE_PACKETS_SENT,
	PNE_BGP_COL_AS2_VC0_CUT_THROUGH,
	PNE_BGP_COL_AS2_VC1_CUT_THROUGH,
	PNE_BGP_COL_AS2_VC0_PACKETS_SENT,
	PNE_BGP_COL_AS2_VC1_PACKETS_SENT,
	PNE_BGP_COL_AS2_IDLE_PACKETS_SENT,
	PNE_BGP_COL_INJECT_VC0_PAYLOAD_ADDED,
	PNE_BGP_COL_INJECT_VC1_PAYLOAD_ADDED,
	PNE_BGP_COL_INJECT_VC0_PACKET_TAKEN,
	PNE_BGP_COL_INJECT_VC1_PACKET_TAKEN,
	PNE_BGP_COL_RECEPTION_VC0_HEADER_TAKEN,
	PNE_BGP_COL_RECEPTION_VC1_HEADER_TAKEN,
	PNE_BGP_COL_RECEPTION_VC0_PAYLOAD_TAKEN,
	PNE_BGP_COL_RECEPTION_VC1_PAYLOAD_TAKEN,
	PNE_BGP_COL_RECEPTION_VC0_PACKET_DISCARDED,
	PNE_BGP_COL_RECEPTION_VC1_PACKET_DISCARDED,
	PNE_BGP_PU2_L2_VALID_PREFETCH_REQUESTS,
	PNE_BGP_PU2_L2_PREFETCH_HITS_IN_FILTER,
	PNE_BGP_PU2_L2_PREFETCH_HITS_IN_STREAM,
	PNE_BGP_PU2_L2_CYCLES_PREFETCH_PENDING,
	PNE_BGP_PU2_L2_PAGE_ALREADY_IN_L2,
	PNE_BGP_PU2_L2_PREFETCH_SNOOP_HIT_SAME_CORE,
	PNE_BGP_PU2_L2_PREFETCH_SNOOP_HIT_OTHER_CORE,
	PNE_BGP_PU2_L2_PREFETCH_SNOOP_HIT_PLB,
	PNE_BGP_PU2_L2_CYCLES_READ_REQUEST_PENDING,
	PNE_BGP_PU2_L2_READ_REQUESTS,
	PNE_BGP_PU2_L2_DEVBUS_READ_REQUESTS,
	PNE_BGP_PU2_L2_L3_READ_REQUESTS,
	PNE_BGP_PU2_L2_NETBUS_READ_REQUESTS,
	PNE_BGP_PU2_L2_BLIND_DEV_READ_REQUESTS,
	PNE_BGP_PU2_L2_PREFETCHABLE_REQUESTS,
	PNE_BGP_PU2_L2_HIT,
	PNE_BGP_PU2_L2_SAME_CORE_SNOOPS,
	PNE_BGP_PU2_L2_OTHER_CORE_SNOOPS,
	PNE_BGP_PU2_L2_OTHER_DP_PU0_SNOOPS,
	PNE_BGP_PU2_L2_OTHER_DP_PU1_SNOOPS,
	PNE_BGP_PU2_L2_MEMORY_WRITES = 0x400001A1,
	PNE_BGP_PU2_L2_NETWORK_WRITES,
	PNE_BGP_PU2_L2_DEVBUS_WRITES,
	PNE_BGP_PU3_L2_VALID_PREFETCH_REQUESTS,
	PNE_BGP_PU3_L2_PREFETCH_HITS_IN_FILTER,
	PNE_BGP_PU3_L2_PREFETCH_HITS_IN_STREAM,
	PNE_BGP_PU3_L2_CYCLES_PREFETCH_PENDING,
	PNE_BGP_PU3_L2_PAGE_ALREADY_IN_L2,
	PNE_BGP_PU3_L2_PREFETCH_SNOOP_HIT_SAME_CORE,
	PNE_BGP_PU3_L2_PREFETCH_SNOOP_HIT_OTHER_CORE,
	PNE_BGP_PU3_L2_PREFETCH_SNOOP_HIT_PLB,
	PNE_BGP_PU3_L2_CYCLES_READ_REQUEST_PENDING,
	PNE_BGP_PU3_L2_READ_REQUESTS,
	PNE_BGP_PU3_L2_DEVBUS_READ_REQUESTS,
	PNE_BGP_PU3_L2_L3_READ_REQUESTS,
	PNE_BGP_PU3_L2_NETBUS_READ_REQUESTS,
	PNE_BGP_PU3_L2_BLIND_DEV_READ_REQUESTS,
	PNE_BGP_PU3_L2_PREFETCHABLE_REQUESTS,
	PNE_BGP_PU3_L2_HIT,
	PNE_BGP_PU3_L2_SAME_CORE_SNOOPS,
	PNE_BGP_PU3_L2_OTHER_CORE_SNOOPS,
	PNE_BGP_PU3_L2_OTHER_DP_PU0_SNOOPS,
	PNE_BGP_PU3_L2_OTHER_DP_PU1_SNOOPS,
	PNE_BGP_PU3_L2_MEMORY_WRITES = 0x400001C1,
	PNE_BGP_PU3_L2_NETWORK_WRITES,
	PNE_BGP_PU3_L2_DEVBUS_WRITES,
	PNE_BGP_L3_M0_R2_SINGLE_LINE_DELIVERED_L2,
	PNE_BGP_L3_M0_R2_BURST_DELIVERED_L2,
	PNE_BGP_L3_M0_R2_READ_RETURN_COLLISION,
	PNE_BGP_L3_M0_R2_DIR0_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M0_R2_DIR0_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M0_R2_DIR1_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M0_R2_DIR1_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M0_W0_DEPOSIT_REQUESTS,
	PNE_BGP_L3_M0_W0_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M0_W1_DEPOSIT_REQUESTS,
	PNE_BGP_L3_M0_W1_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M0_MH_ALLOCATION_REQUESTS,
	PNE_BGP_L3_M0_MH_CYCLES_ALLOCATION_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M0_PF_PREFETCH_INTO_EDRAM,
	PNE_BGP_L3_M1_R2_SINGLE_LINE_DELIVERED_L2 = 0x400001D8,
	PNE_BGP_L3_M1_R2_BURST_DELIVERED_L2,
	PNE_BGP_L3_M1_R2_READ_RETURN_COLLISION,
	PNE_BGP_L3_M1_R2_DIR0_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M1_R2_DIR0_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M1_R2_DIR1_HIT_OR_INFLIGHT,
	PNE_BGP_L3_M1_R2_DIR1_MISS_OR_LOCKDOWN,
	PNE_BGP_L3_M1_W0_DEPOSIT_REQUESTS,
	PNE_BGP_L3_M1_W0_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M1_W1_DEPOSIT_REQUESTS,
	PNE_BGP_L3_M1_W1_CYCLES_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M1_MH_ALLOCATION_REQUESTS,
	PNE_BGP_L3_M1_MH_CYCLES_ALLOCATION_REQUESTS_NOT_TAKEN,
	PNE_BGP_L3_M1_PF_PREFETCH_INTO_EDRAM,
	PNE_BGP_PU2_SNOOP_PORT0_REMOTE_SOURCE_REQUESTS = 0x400001EC,
	PNE_BGP_PU2_SNOOP_PORT1_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU2_SNOOP_PORT2_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU2_SNOOP_PORT3_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU2_SNOOP_PORT0_REJECTED_REQUESTS,
	PNE_BGP_PU2_SNOOP_PORT1_REJECTED_REQUESTS,
	PNE_BGP_PU2_SNOOP_PORT2_REJECTED_REQUESTS,
	PNE_BGP_PU2_SNOOP_PORT3_REJECTED_REQUESTS,
	PNE_BGP_PU2_SNOOP_L1_CACHE_WRAP,
	PNE_BGP_PU3_SNOOP_PORT0_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU3_SNOOP_PORT1_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU3_SNOOP_PORT2_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU3_SNOOP_PORT3_REMOTE_SOURCE_REQUESTS,
	PNE_BGP_PU3_SNOOP_PORT0_REJECTED_REQUESTS,
	PNE_BGP_PU3_SNOOP_PORT1_REJECTED_REQUESTS,
	PNE_BGP_PU3_SNOOP_PORT2_REJECTED_REQUESTS,
	PNE_BGP_PU3_SNOOP_PORT3_REJECTED_REQUESTS,
	PNE_BGP_PU3_SNOOP_L1_CACHE_WRAP,
	PNE_BGP_MISC_ELAPSED_TIME_UM1 = 0x400001FF
} _papi_hwd_bgp_native_event_id_t;
/* END PYTHON GENERATION 1 (Do not modify or move this comment) */

#endif
