/****************************/
/* THIS IS OPEN SOURCE CODE */
/****************************/

#ifndef __POWER6_EVENTS_H__
#define __POWER6_EVENTS_H__

/*
* File:    power6_events.h
* CVS:
* Author:  Corey Ashford
*          cjashfor@us.ibm.com
* Mods:    <your name here>
*          <your email address>
*
* (C) Copyright IBM Corporation, 2007.  All Rights Reserved.
* Contributed by Corey Ashford <cjashfor.ibm.com>
*
* Note: This code was automatically generated and should not be modified by
* hand.
*
*/
#define POWER6_PME_PM_LSU_REJECT_STQ_FULL 0
#define POWER6_PME_PM_DPU_HELD_FXU_MULTI 1
#define POWER6_PME_PM_VMX1_STALL 2
#define POWER6_PME_PM_PMC2_SAVED 3
#define POWER6_PME_PM_L2SB_IC_INV 4
#define POWER6_PME_PM_IERAT_MISS_64K 5
#define POWER6_PME_PM_THRD_PRIO_DIFF_3or4_CYC 6
#define POWER6_PME_PM_LD_REF_L1_BOTH 7
#define POWER6_PME_PM_FPU1_FCONV 8
#define POWER6_PME_PM_IBUF_FULL_COUNT 9
#define POWER6_PME_PM_MRK_LSU_DERAT_MISS 10
#define POWER6_PME_PM_MRK_ST_CMPL 11
#define POWER6_PME_PM_L2_CASTOUT_MOD 12
#define POWER6_PME_PM_FPU1_ST_FOLDED 13
#define POWER6_PME_PM_MRK_INST_TIMEO 14
#define POWER6_PME_PM_DPU_WT 15
#define POWER6_PME_PM_DPU_HELD_RESTART 16
#define POWER6_PME_PM_IERAT_MISS 17
#define POWER6_PME_PM_FPU_SINGLE 18
#define POWER6_PME_PM_MRK_PTEG_FROM_LMEM 19
#define POWER6_PME_PM_HV_COUNT 20
#define POWER6_PME_PM_L2SA_ST_HIT 21
#define POWER6_PME_PM_L2_LD_MISS_INST 22
#define POWER6_PME_PM_EXT_INT 23
#define POWER6_PME_PM_LSU1_LDF 24
#define POWER6_PME_PM_FAB_CMD_ISSUED 25
#define POWER6_PME_PM_PTEG_FROM_L21 26
#define POWER6_PME_PM_L2SA_MISS 27
#define POWER6_PME_PM_PTEG_FROM_RL2L3_MOD 28
#define POWER6_PME_PM_DPU_WT_COUNT 29
#define POWER6_PME_PM_MRK_PTEG_FROM_L25_MOD 30
#define POWER6_PME_PM_LD_HIT_L2 31
#define POWER6_PME_PM_PTEG_FROM_DL2L3_SHR 32
#define POWER6_PME_PM_MEM_DP_RQ_GLOB_LOC 33
#define POWER6_PME_PM_L3SA_MISS 34
#define POWER6_PME_PM_NO_ITAG_COUNT 35
#define POWER6_PME_PM_DSLB_MISS 36
#define POWER6_PME_PM_LSU_FLUSH_ALIGN 37
#define POWER6_PME_PM_DPU_HELD_FPU_CR 38
#define POWER6_PME_PM_PTEG_FROM_L2MISS 39
#define POWER6_PME_PM_MRK_DATA_FROM_DMEM 40
#define POWER6_PME_PM_PTEG_FROM_LMEM 41
#define POWER6_PME_PM_MRK_DERAT_REF_64K 42
#define POWER6_PME_PM_L2SA_LD_REQ_INST 43
#define POWER6_PME_PM_MRK_DERAT_MISS_16M 44
#define POWER6_PME_PM_DATA_FROM_DL2L3_MOD 45
#define POWER6_PME_PM_FPU0_FXMULT 46
#define POWER6_PME_PM_L3SB_MISS 47
#define POWER6_PME_PM_STCX_CANCEL 48
#define POWER6_PME_PM_L2SA_LD_MISS_DATA 49
#define POWER6_PME_PM_IC_INV_L2 50
#define POWER6_PME_PM_DPU_HELD 51
#define POWER6_PME_PM_PMC1_OVERFLOW 52
#define POWER6_PME_PM_THRD_PRIO_6_CYC 53
#define POWER6_PME_PM_MRK_PTEG_FROM_L3MISS 54
#define POWER6_PME_PM_MRK_LSU0_REJECT_UST 55
#define POWER6_PME_PM_MRK_INST_DISP 56
#define POWER6_PME_PM_LARX 57
#define POWER6_PME_PM_INST_CMPL 58
#define POWER6_PME_PM_FXU_IDLE 59
#define POWER6_PME_PM_MRK_DATA_FROM_DL2L3_MOD 60
#define POWER6_PME_PM_L2_LD_REQ_DATA 61
#define POWER6_PME_PM_LSU_DERAT_MISS_CYC 62
#define POWER6_PME_PM_DPU_HELD_POWER_COUNT 63
#define POWER6_PME_PM_INST_FROM_RL2L3_MOD 64
#define POWER6_PME_PM_DATA_FROM_DMEM_CYC 65
#define POWER6_PME_PM_DATA_FROM_DMEM 66
#define POWER6_PME_PM_LSU_REJECT_PARTIAL_SECTOR 67
#define POWER6_PME_PM_LSU_REJECT_DERAT_MPRED 68
#define POWER6_PME_PM_LSU1_REJECT_ULD 69
#define POWER6_PME_PM_DATA_FROM_L3_CYC 70
#define POWER6_PME_PM_FXU1_BUSY_FXU0_IDLE 71
#define POWER6_PME_PM_INST_FROM_MEM_DP 72
#define POWER6_PME_PM_LSU_FLUSH_DSI 73
#define POWER6_PME_PM_MRK_DERAT_REF_16G 74
#define POWER6_PME_PM_LSU_LDF_BOTH 75
#define POWER6_PME_PM_FPU1_1FLOP 76
#define POWER6_PME_PM_DATA_FROM_RMEM_CYC 77
#define POWER6_PME_PM_INST_PTEG_SECONDARY 78
#define POWER6_PME_PM_L1_ICACHE_MISS 79
#define POWER6_PME_PM_INST_DISP_LLA 80
#define POWER6_PME_PM_THRD_BOTH_RUN_CYC 81
#define POWER6_PME_PM_LSU_ST_CHAINED 82
#define POWER6_PME_PM_FPU1_FXDIV 83
#define POWER6_PME_PM_FREQ_UP 84
#define POWER6_PME_PM_FAB_RETRY_SYS_PUMP 85
#define POWER6_PME_PM_DATA_FROM_LMEM 86
#define POWER6_PME_PM_PMC3_OVERFLOW 87
#define POWER6_PME_PM_LSU0_REJECT_SET_MPRED 88
#define POWER6_PME_PM_LSU0_REJECT_DERAT_MPRED 89
#define POWER6_PME_PM_LSU1_REJECT_STQ_FULL 90
#define POWER6_PME_PM_MRK_BR_MPRED 91
#define POWER6_PME_PM_L2SA_ST_MISS 92
#define POWER6_PME_PM_LSU0_REJECT_EXTERN 93
#define POWER6_PME_PM_MRK_BR_TAKEN 94
#define POWER6_PME_PM_ISLB_MISS 95
#define POWER6_PME_PM_CYC 96
#define POWER6_PME_PM_FPU_FXDIV 97
#define POWER6_PME_PM_DPU_HELD_LLA_END 98
#define POWER6_PME_PM_MEM0_DP_CL_WR_LOC 99
#define POWER6_PME_PM_MRK_LSU_REJECT_ULD 100
#define POWER6_PME_PM_1PLUS_PPC_CMPL 101
#define POWER6_PME_PM_PTEG_FROM_DMEM 102
#define POWER6_PME_PM_DPU_WT_BR_MPRED_COUNT 103
#define POWER6_PME_PM_GCT_FULL_CYC 104
#define POWER6_PME_PM_INST_FROM_L25_SHR 105
#define POWER6_PME_PM_MRK_DERAT_MISS_4K 106
#define POWER6_PME_PM_DC_PREF_STREAM_ALLOC 107
#define POWER6_PME_PM_FPU1_FIN 108
#define POWER6_PME_PM_BR_MPRED_TA 109
#define POWER6_PME_PM_DPU_HELD_POWER 110
#define POWER6_PME_PM_RUN_INST_CMPL 111
#define POWER6_PME_PM_GCT_EMPTY_CYC 112
#define POWER6_PME_PM_LLA_COUNT 113
#define POWER6_PME_PM_LSU0_REJECT_NO_SCRATCH 114
#define POWER6_PME_PM_DPU_WT_IC_MISS 115
#define POWER6_PME_PM_DATA_FROM_L3MISS 116
#define POWER6_PME_PM_FPU_FPSCR 117
#define POWER6_PME_PM_VMX1_INST_ISSUED 118
#define POWER6_PME_PM_FLUSH 119
#define POWER6_PME_PM_ST_HIT_L2 120
#define POWER6_PME_PM_SYNC_CYC 121
#define POWER6_PME_PM_FAB_SYS_PUMP 122
#define POWER6_PME_PM_IC_PREF_REQ 123
#define POWER6_PME_PM_MEM0_DP_RQ_GLOB_LOC 124
#define POWER6_PME_PM_FPU_ISSUE_0 125
#define POWER6_PME_PM_THRD_PRIO_2_CYC 126
#define POWER6_PME_PM_VMX_SIMPLE_ISSUED 127
#define POWER6_PME_PM_MRK_FPU1_FIN 128
#define POWER6_PME_PM_DPU_HELD_CW 129
#define POWER6_PME_PM_L3SA_REF 130
#define POWER6_PME_PM_STCX 131
#define POWER6_PME_PM_L2SB_MISS 132
#define POWER6_PME_PM_LSU0_REJECT 133
#define POWER6_PME_PM_TB_BIT_TRANS 134
#define POWER6_PME_PM_THERMAL_MAX 135
#define POWER6_PME_PM_FPU0_STF 136
#define POWER6_PME_PM_FPU1_FMA 137
#define POWER6_PME_PM_LSU1_REJECT_LHS 138
#define POWER6_PME_PM_DPU_HELD_INT 139
#define POWER6_PME_PM_THRD_LLA_BOTH_CYC 140
#define POWER6_PME_PM_DPU_HELD_THERMAL_COUNT 141
#define POWER6_PME_PM_PMC4_REWIND 142
#define POWER6_PME_PM_DERAT_REF_16M 143
#define POWER6_PME_PM_FPU0_FCONV 144
#define POWER6_PME_PM_L2SA_LD_REQ_DATA 145
#define POWER6_PME_PM_DATA_FROM_MEM_DP 146
#define POWER6_PME_PM_MRK_VMX_FLOAT_ISSUED 147
#define POWER6_PME_PM_MRK_PTEG_FROM_L2MISS 148
#define POWER6_PME_PM_THRD_PRIO_DIFF_1or2_CYC 149
#define POWER6_PME_PM_VMX0_STALL 150
#define POWER6_PME_PM_IC_DEMAND_L2_BHT_REDIRECT 151
#define POWER6_PME_PM_LSU_DERAT_MISS 152
#define POWER6_PME_PM_FPU0_SINGLE 153
#define POWER6_PME_PM_FPU_ISSUE_STEERING 154
#define POWER6_PME_PM_THRD_PRIO_1_CYC 155
#define POWER6_PME_PM_VMX_COMPLEX_ISSUED 156
#define POWER6_PME_PM_FPU_ISSUE_ST_FOLDED 157
#define POWER6_PME_PM_DFU_FIN 158
#define POWER6_PME_PM_BR_PRED_CCACHE 159
#define POWER6_PME_PM_MRK_ST_CMPL_INT 160
#define POWER6_PME_PM_FAB_MMIO 161
#define POWER6_PME_PM_MRK_VMX_SIMPLE_ISSUED 162
#define POWER6_PME_PM_FPU_STF 163
#define POWER6_PME_PM_MEM1_DP_CL_WR_GLOB 164
#define POWER6_PME_PM_MRK_DATA_FROM_L3MISS 165
#define POWER6_PME_PM_GCT_NOSLOT_CYC 166
#define POWER6_PME_PM_L2_ST_REQ_DATA 167
#define POWER6_PME_PM_INST_TABLEWALK_COUNT 168
#define POWER6_PME_PM_PTEG_FROM_L35_SHR 169
#define POWER6_PME_PM_DPU_HELD_ISYNC 170
#define POWER6_PME_PM_MRK_DATA_FROM_L25_SHR 171
#define POWER6_PME_PM_L3SA_HIT 172
#define POWER6_PME_PM_DERAT_MISS_16G 173
#define POWER6_PME_PM_DATA_PTEG_2ND_HALF 174
#define POWER6_PME_PM_L2SA_ST_REQ 175
#define POWER6_PME_PM_INST_FROM_LMEM 176
#define POWER6_PME_PM_IC_DEMAND_L2_BR_REDIRECT 177
#define POWER6_PME_PM_PTEG_FROM_L2 178
#define POWER6_PME_PM_DATA_PTEG_1ST_HALF 179
#define POWER6_PME_PM_BR_MPRED_COUNT 180
#define POWER6_PME_PM_IERAT_MISS_4K 181
#define POWER6_PME_PM_THRD_BOTH_RUN_COUNT 182
#define POWER6_PME_PM_LSU_REJECT_ULD 183
#define POWER6_PME_PM_DATA_FROM_DL2L3_MOD_CYC 184
#define POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_MOD 185
#define POWER6_PME_PM_FPU0_FLOP 186
#define POWER6_PME_PM_FPU0_FEST 187
#define POWER6_PME_PM_MRK_LSU0_REJECT_LHS 188
#define POWER6_PME_PM_VMX_RESULT_SAT_1 189
#define POWER6_PME_PM_NO_ITAG_CYC 190
#define POWER6_PME_PM_LSU1_REJECT_NO_SCRATCH 191
#define POWER6_PME_PM_0INST_FETCH 192
#define POWER6_PME_PM_DPU_WT_BR_MPRED 193
#define POWER6_PME_PM_L1_PREF 194
#define POWER6_PME_PM_VMX_FLOAT_MULTICYCLE 195
#define POWER6_PME_PM_DATA_FROM_L25_SHR_CYC 196
#define POWER6_PME_PM_DATA_FROM_L3 197
#define POWER6_PME_PM_PMC2_OVERFLOW 198
#define POWER6_PME_PM_VMX0_LD_WRBACK 199
#define POWER6_PME_PM_FPU0_DENORM 200
#define POWER6_PME_PM_INST_FETCH_CYC 201
#define POWER6_PME_PM_LSU_LDF 202
#define POWER6_PME_PM_LSU_REJECT_L2_CORR 203
#define POWER6_PME_PM_DERAT_REF_64K 204
#define POWER6_PME_PM_THRD_PRIO_3_CYC 205
#define POWER6_PME_PM_FPU_FMA 206
#define POWER6_PME_PM_INST_FROM_L35_MOD 207
#define POWER6_PME_PM_DFU_CONV 208
#define POWER6_PME_PM_INST_FROM_L25_MOD 209
#define POWER6_PME_PM_PTEG_FROM_L35_MOD 210
#define POWER6_PME_PM_MRK_VMX_ST_ISSUED 211
#define POWER6_PME_PM_VMX_FLOAT_ISSUED 212
#define POWER6_PME_PM_LSU0_REJECT_L2_CORR 213
#define POWER6_PME_PM_THRD_L2MISS 214
#define POWER6_PME_PM_FPU_FCONV 215
#define POWER6_PME_PM_FPU_FXMULT 216
#define POWER6_PME_PM_FPU1_FRSP 217
#define POWER6_PME_PM_MRK_DERAT_REF_16M 218
#define POWER6_PME_PM_L2SB_CASTOUT_SHR 219
#define POWER6_PME_PM_THRD_ONE_RUN_COUNT 220
#define POWER6_PME_PM_INST_FROM_RMEM 221
#define POWER6_PME_PM_LSU_BOTH_BUS 222
#define POWER6_PME_PM_FPU1_FSQRT_FDIV 223
#define POWER6_PME_PM_L2_LD_REQ_INST 224
#define POWER6_PME_PM_MRK_PTEG_FROM_L35_SHR 225
#define POWER6_PME_PM_BR_PRED_CR 226
#define POWER6_PME_PM_MRK_LSU0_REJECT_ULD 227
#define POWER6_PME_PM_LSU_REJECT 228
#define POWER6_PME_PM_LSU_REJECT_LHS_BOTH 229
#define POWER6_PME_PM_GXO_ADDR_CYC_BUSY 230
#define POWER6_PME_PM_LSU_SRQ_EMPTY_COUNT 231
#define POWER6_PME_PM_PTEG_FROM_L3 232
#define POWER6_PME_PM_VMX0_LD_ISSUED 233
#define POWER6_PME_PM_FXU_PIPELINED_MULT_DIV 234
#define POWER6_PME_PM_FPU1_STF 235
#define POWER6_PME_PM_DFU_ADD 236
#define POWER6_PME_PM_MEM_DP_CL_WR_GLOB 237
#define POWER6_PME_PM_MRK_LSU1_REJECT_ULD 238
#define POWER6_PME_PM_ITLB_REF 239
#define POWER6_PME_PM_LSU0_REJECT_L2MISS 240
#define POWER6_PME_PM_DATA_FROM_L35_SHR 241
#define POWER6_PME_PM_MRK_DATA_FROM_RL2L3_MOD 242
#define POWER6_PME_PM_FPU0_FPSCR 243
#define POWER6_PME_PM_DATA_FROM_L2 244
#define POWER6_PME_PM_DPU_HELD_XER 245
#define POWER6_PME_PM_FAB_NODE_PUMP 246
#define POWER6_PME_PM_VMX_RESULT_SAT_0_1 247
#define POWER6_PME_PM_LD_REF_L1 248
#define POWER6_PME_PM_TLB_REF 249
#define POWER6_PME_PM_DC_PREF_OUT_OF_STREAMS 250
#define POWER6_PME_PM_FLUSH_FPU 251
#define POWER6_PME_PM_MEM1_DP_CL_WR_LOC 252
#define POWER6_PME_PM_L2SB_LD_HIT 253
#define POWER6_PME_PM_FAB_DCLAIM 254
#define POWER6_PME_PM_MEM_DP_CL_WR_LOC 255
#define POWER6_PME_PM_BR_MPRED_CR 256
#define POWER6_PME_PM_LSU_REJECT_EXTERN 257
#define POWER6_PME_PM_DATA_FROM_RL2L3_MOD 258
#define POWER6_PME_PM_DPU_HELD_RU_WQ 259
#define POWER6_PME_PM_LD_MISS_L1 260
#define POWER6_PME_PM_DC_INV_L2 261
#define POWER6_PME_PM_MRK_PTEG_FROM_RMEM 262
#define POWER6_PME_PM_FPU_FIN 263
#define POWER6_PME_PM_FXU0_FIN 264
#define POWER6_PME_PM_DPU_HELD_FPQ 265
#define POWER6_PME_PM_GX_DMA_READ 266
#define POWER6_PME_PM_LSU1_REJECT_PARTIAL_SECTOR 267
#define POWER6_PME_PM_0INST_FETCH_COUNT 268
#define POWER6_PME_PM_PMC5_OVERFLOW 269
#define POWER6_PME_PM_L2SB_LD_REQ 270
#define POWER6_PME_PM_THRD_PRIO_DIFF_0_CYC 271
#define POWER6_PME_PM_DATA_FROM_RMEM 272
#define POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_CYC 273
#define POWER6_PME_PM_ST_REF_L1_BOTH 274
#define POWER6_PME_PM_VMX_PERMUTE_ISSUED 275
#define POWER6_PME_PM_BR_TAKEN 276
#define POWER6_PME_PM_FAB_DMA 277
#define POWER6_PME_PM_GCT_EMPTY_COUNT 278
#define POWER6_PME_PM_FPU1_SINGLE 279
#define POWER6_PME_PM_L2SA_CASTOUT_SHR 280
#define POWER6_PME_PM_L3SB_REF 281
#define POWER6_PME_PM_FPU0_FRSP 282
#define POWER6_PME_PM_PMC4_SAVED 283
#define POWER6_PME_PM_L2SA_DC_INV 284
#define POWER6_PME_PM_GXI_ADDR_CYC_BUSY 285
#define POWER6_PME_PM_FPU0_FMA 286
#define POWER6_PME_PM_SLB_MISS 287
#define POWER6_PME_PM_MRK_ST_GPS 288
#define POWER6_PME_PM_DERAT_REF_4K 289
#define POWER6_PME_PM_L2_CASTOUT_SHR 290
#define POWER6_PME_PM_DPU_HELD_STCX_CR 291
#define POWER6_PME_PM_FPU0_ST_FOLDED 292
#define POWER6_PME_PM_MRK_DATA_FROM_L21 293
#define POWER6_PME_PM_THRD_PRIO_DIFF_minus3or4_CYC 294
#define POWER6_PME_PM_DATA_FROM_L35_MOD 295
#define POWER6_PME_PM_DATA_FROM_DL2L3_SHR 296
#define POWER6_PME_PM_GXI_DATA_CYC_BUSY 297
#define POWER6_PME_PM_LSU_REJECT_STEAL 298
#define POWER6_PME_PM_ST_FIN 299
#define POWER6_PME_PM_DPU_HELD_CR_LOGICAL 300
#define POWER6_PME_PM_THRD_SEL_T0 301
#define POWER6_PME_PM_PTEG_RELOAD_VALID 302
#define POWER6_PME_PM_L2_PREF_ST 303
#define POWER6_PME_PM_MRK_STCX_FAIL 304
#define POWER6_PME_PM_LSU0_REJECT_LHS 305
#define POWER6_PME_PM_DFU_EXP_EQ 306
#define POWER6_PME_PM_DPU_HELD_FP_FX_MULT 307
#define POWER6_PME_PM_L2_LD_MISS_DATA 308
#define POWER6_PME_PM_DATA_FROM_L35_MOD_CYC 309
#define POWER6_PME_PM_FLUSH_FXU 310
#define POWER6_PME_PM_FPU_ISSUE_1 311
#define POWER6_PME_PM_DATA_FROM_LMEM_CYC 312
#define POWER6_PME_PM_DPU_HELD_LSU_SOPS 313
#define POWER6_PME_PM_INST_PTEG_2ND_HALF 314
#define POWER6_PME_PM_THRESH_TIMEO 315
#define POWER6_PME_PM_LSU_REJECT_UST_BOTH 316
#define POWER6_PME_PM_LSU_REJECT_FAST 317
#define POWER6_PME_PM_DPU_HELD_THRD_PRIO 318
#define POWER6_PME_PM_L2_PREF_LD 319
#define POWER6_PME_PM_FPU_FEST 320
#define POWER6_PME_PM_MRK_DATA_FROM_RMEM 321
#define POWER6_PME_PM_LD_MISS_L1_CYC 322
#define POWER6_PME_PM_DERAT_MISS_4K 323
#define POWER6_PME_PM_DPU_HELD_COMPLETION 324
#define POWER6_PME_PM_FPU_ISSUE_STALL_ST 325
#define POWER6_PME_PM_L2SB_DC_INV 326
#define POWER6_PME_PM_PTEG_FROM_L25_SHR 327
#define POWER6_PME_PM_PTEG_FROM_DL2L3_MOD 328
#define POWER6_PME_PM_FAB_CMD_RETRIED 329
#define POWER6_PME_PM_BR_PRED_LSTACK 330
#define POWER6_PME_PM_GXO_DATA_CYC_BUSY 331
#define POWER6_PME_PM_DFU_SUBNORM 332
#define POWER6_PME_PM_FPU_ISSUE_OOO 333
#define POWER6_PME_PM_LSU_REJECT_ULD_BOTH 334
#define POWER6_PME_PM_L2SB_ST_MISS 335
#define POWER6_PME_PM_DATA_FROM_L25_MOD_CYC 336
#define POWER6_PME_PM_INST_PTEG_1ST_HALF 337
#define POWER6_PME_PM_DERAT_MISS_16M 338
#define POWER6_PME_PM_GX_DMA_WRITE 339
#define POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_MOD 340
#define POWER6_PME_PM_MEM1_DP_RQ_GLOB_LOC 341
#define POWER6_PME_PM_L2SB_LD_REQ_DATA 342
#define POWER6_PME_PM_L2SA_LD_MISS_INST 343
#define POWER6_PME_PM_MRK_LSU0_REJECT_L2MISS 344
#define POWER6_PME_PM_MRK_IFU_FIN 345
#define POWER6_PME_PM_INST_FROM_L3 346
#define POWER6_PME_PM_FXU1_FIN 347
#define POWER6_PME_PM_THRD_PRIO_4_CYC 348
#define POWER6_PME_PM_MRK_DATA_FROM_L35_MOD 349
#define POWER6_PME_PM_LSU_REJECT_SET_MPRED 350
#define POWER6_PME_PM_MRK_DERAT_MISS_16G 351
#define POWER6_PME_PM_FPU0_FXDIV 352
#define POWER6_PME_PM_MRK_LSU1_REJECT_UST 353
#define POWER6_PME_PM_FPU_ISSUE_DIV_SQRT_OVERLAP 354
#define POWER6_PME_PM_INST_FROM_L35_SHR 355
#define POWER6_PME_PM_MRK_LSU_REJECT_LHS 356
#define POWER6_PME_PM_LSU_LMQ_FULL_CYC 357
#define POWER6_PME_PM_SYNC_COUNT 358
#define POWER6_PME_PM_MEM0_DP_RQ_LOC_GLOB 359
#define POWER6_PME_PM_L2SA_CASTOUT_MOD 360
#define POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_COUNT 361
#define POWER6_PME_PM_PTEG_FROM_MEM_DP 362
#define POWER6_PME_PM_LSU_REJECT_SLOW 363
#define POWER6_PME_PM_PTEG_FROM_L25_MOD 364
#define POWER6_PME_PM_THRD_PRIO_7_CYC 365
#define POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_SHR 366
#define POWER6_PME_PM_ST_REQ_L2 367
#define POWER6_PME_PM_ST_REF_L1 368
#define POWER6_PME_PM_FPU_ISSUE_STALL_THRD 369
#define POWER6_PME_PM_RUN_COUNT 370
#define POWER6_PME_PM_RUN_CYC 371
#define POWER6_PME_PM_PTEG_FROM_RMEM 372
#define POWER6_PME_PM_LSU0_LDF 373
#define POWER6_PME_PM_ST_MISS_L1 374
#define POWER6_PME_PM_INST_FROM_DL2L3_SHR 375
#define POWER6_PME_PM_L2SA_IC_INV 376
#define POWER6_PME_PM_THRD_ONE_RUN_CYC 377
#define POWER6_PME_PM_L2SB_LD_REQ_INST 378
#define POWER6_PME_PM_MRK_DATA_FROM_L25_MOD 379
#define POWER6_PME_PM_DPU_HELD_XTHRD 380
#define POWER6_PME_PM_L2SB_ST_REQ 381
#define POWER6_PME_PM_INST_FROM_L21 382
#define POWER6_PME_PM_INST_FROM_L3MISS 383
#define POWER6_PME_PM_L3SB_HIT 384
#define POWER6_PME_PM_EE_OFF_EXT_INT 385
#define POWER6_PME_PM_INST_FROM_DL2L3_MOD 386
#define POWER6_PME_PM_PMC6_OVERFLOW 387
#define POWER6_PME_PM_FPU_FLOP 388
#define POWER6_PME_PM_FXU_BUSY 389
#define POWER6_PME_PM_FPU1_FLOP 390
#define POWER6_PME_PM_IC_RELOAD_SHR 391
#define POWER6_PME_PM_INST_TABLEWALK_CYC 392
#define POWER6_PME_PM_DATA_FROM_RL2L3_MOD_CYC 393
#define POWER6_PME_PM_THRD_PRIO_DIFF_5or6_CYC 394
#define POWER6_PME_PM_IBUF_FULL_CYC 395
#define POWER6_PME_PM_L2SA_LD_REQ 396
#define POWER6_PME_PM_VMX1_LD_WRBACK 397
#define POWER6_PME_PM_MRK_FPU_FIN 398
#define POWER6_PME_PM_THRD_PRIO_5_CYC 399
#define POWER6_PME_PM_DFU_BACK2BACK 400
#define POWER6_PME_PM_MRK_DATA_FROM_LMEM 401
#define POWER6_PME_PM_LSU_REJECT_LHS 402
#define POWER6_PME_PM_DPU_HELD_SPR 403
#define POWER6_PME_PM_FREQ_DOWN 404
#define POWER6_PME_PM_DFU_ENC_BCD_DPD 405
#define POWER6_PME_PM_DPU_HELD_GPR 406
#define POWER6_PME_PM_LSU0_NCST 407
#define POWER6_PME_PM_MRK_INST_ISSUED 408
#define POWER6_PME_PM_INST_FROM_RL2L3_SHR 409
#define POWER6_PME_PM_FPU_DENORM 410
#define POWER6_PME_PM_PTEG_FROM_L3MISS 411
#define POWER6_PME_PM_RUN_PURR 412
#define POWER6_PME_PM_MRK_VMX0_LD_WRBACK 413
#define POWER6_PME_PM_L2_MISS 414
#define POWER6_PME_PM_MRK_DATA_FROM_L3 415
#define POWER6_PME_PM_MRK_LSU1_REJECT_LHS 416
#define POWER6_PME_PM_L2SB_LD_MISS_INST 417
#define POWER6_PME_PM_PTEG_FROM_RL2L3_SHR 418
#define POWER6_PME_PM_MRK_DERAT_MISS_64K 419
#define POWER6_PME_PM_LWSYNC 420
#define POWER6_PME_PM_FPU1_FXMULT 421
#define POWER6_PME_PM_MEM0_DP_CL_WR_GLOB 422
#define POWER6_PME_PM_LSU0_REJECT_PARTIAL_SECTOR 423
#define POWER6_PME_PM_INST_IMC_MATCH_CMPL 424
#define POWER6_PME_PM_DPU_HELD_THERMAL 425
#define POWER6_PME_PM_FPU_FRSP 426
#define POWER6_PME_PM_MRK_INST_FIN 427
#define POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_SHR 428
#define POWER6_PME_PM_MRK_DTLB_REF 429
#define POWER6_PME_PM_MRK_PTEG_FROM_L25_SHR 430
#define POWER6_PME_PM_DPU_HELD_LSU 431
#define POWER6_PME_PM_FPU_FSQRT_FDIV 432
#define POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_COUNT 433
#define POWER6_PME_PM_DATA_PTEG_SECONDARY 434
#define POWER6_PME_PM_FPU1_FEST 435
#define POWER6_PME_PM_L2SA_LD_HIT 436
#define POWER6_PME_PM_DATA_FROM_MEM_DP_CYC 437
#define POWER6_PME_PM_BR_MPRED_CCACHE 438
#define POWER6_PME_PM_DPU_HELD_COUNT 439
#define POWER6_PME_PM_LSU1_REJECT_SET_MPRED 440
#define POWER6_PME_PM_FPU_ISSUE_2 441
#define POWER6_PME_PM_LSU1_REJECT_L2_CORR 442
#define POWER6_PME_PM_MRK_PTEG_FROM_DMEM 443
#define POWER6_PME_PM_MEM1_DP_RQ_LOC_GLOB 444
#define POWER6_PME_PM_THRD_PRIO_DIFF_minus1or2_CYC 445
#define POWER6_PME_PM_THRD_PRIO_0_CYC 446
#define POWER6_PME_PM_FXU0_BUSY_FXU1_IDLE 447
#define POWER6_PME_PM_LSU1_REJECT_DERAT_MPRED 448
#define POWER6_PME_PM_MRK_VMX1_LD_WRBACK 449
#define POWER6_PME_PM_DATA_FROM_RL2L3_SHR_CYC 450
#define POWER6_PME_PM_IERAT_MISS_16M 451
#define POWER6_PME_PM_MRK_DATA_FROM_MEM_DP 452
#define POWER6_PME_PM_LARX_L1HIT 453
#define POWER6_PME_PM_L2_ST_MISS_DATA 454
#define POWER6_PME_PM_FPU_ST_FOLDED 455
#define POWER6_PME_PM_MRK_DATA_FROM_L35_SHR 456
#define POWER6_PME_PM_DPU_HELD_MULT_GPR 457
#define POWER6_PME_PM_FPU0_1FLOP 458
#define POWER6_PME_PM_IERAT_MISS_16G 459
#define POWER6_PME_PM_IC_PREF_WRITE 460
#define POWER6_PME_PM_THRD_PRIO_DIFF_minus5or6_CYC 461
#define POWER6_PME_PM_FPU0_FIN 462
#define POWER6_PME_PM_DATA_FROM_L2_CYC 463
#define POWER6_PME_PM_DERAT_REF_16G 464
#define POWER6_PME_PM_BR_PRED 465
#define POWER6_PME_PM_VMX1_LD_ISSUED 466
#define POWER6_PME_PM_L2SB_CASTOUT_MOD 467
#define POWER6_PME_PM_INST_FROM_DMEM 468
#define POWER6_PME_PM_DATA_FROM_L35_SHR_CYC 469
#define POWER6_PME_PM_LSU0_NCLD 470
#define POWER6_PME_PM_FAB_RETRY_NODE_PUMP 471
#define POWER6_PME_PM_VMX0_INST_ISSUED 472
#define POWER6_PME_PM_DATA_FROM_L25_MOD 473
#define POWER6_PME_PM_DPU_HELD_ITLB_ISLB 474
#define POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_CYC 475
#define POWER6_PME_PM_THRD_CONC_RUN_INST 476
#define POWER6_PME_PM_MRK_PTEG_FROM_L2 477
#define POWER6_PME_PM_PURR 478
#define POWER6_PME_PM_DERAT_MISS_64K 479
#define POWER6_PME_PM_PMC2_REWIND 480
#define POWER6_PME_PM_INST_FROM_L2 481
#define POWER6_PME_PM_INST_DISP 482
#define POWER6_PME_PM_DATA_FROM_L25_SHR 483
#define POWER6_PME_PM_L1_DCACHE_RELOAD_VALID 484
#define POWER6_PME_PM_LSU1_REJECT_UST 485
#define POWER6_PME_PM_FAB_ADDR_COLLISION 486
#define POWER6_PME_PM_MRK_FXU_FIN 487
#define POWER6_PME_PM_LSU0_REJECT_UST 488
#define POWER6_PME_PM_PMC4_OVERFLOW 489
#define POWER6_PME_PM_MRK_PTEG_FROM_L3 490
#define POWER6_PME_PM_INST_FROM_L2MISS 491
#define POWER6_PME_PM_L2SB_ST_HIT 492
#define POWER6_PME_PM_DPU_WT_IC_MISS_COUNT 493
#define POWER6_PME_PM_MRK_DATA_FROM_DL2L3_SHR 494
#define POWER6_PME_PM_MRK_PTEG_FROM_L35_MOD 495
#define POWER6_PME_PM_FPU1_FPSCR 496
#define POWER6_PME_PM_LSU_REJECT_UST 497
#define POWER6_PME_PM_LSU0_DERAT_MISS 498
#define POWER6_PME_PM_MRK_PTEG_FROM_MEM_DP 499
#define POWER6_PME_PM_MRK_DATA_FROM_L2 500
#define POWER6_PME_PM_FPU0_FSQRT_FDIV 501
#define POWER6_PME_PM_DPU_HELD_FXU_SOPS 502
#define POWER6_PME_PM_MRK_FPU0_FIN 503
#define POWER6_PME_PM_L2SB_LD_MISS_DATA 504
#define POWER6_PME_PM_LSU_SRQ_EMPTY_CYC 505
#define POWER6_PME_PM_1PLUS_PPC_DISP 506
#define POWER6_PME_PM_VMX_ST_ISSUED 507
#define POWER6_PME_PM_DATA_FROM_L2MISS 508
#define POWER6_PME_PM_LSU0_REJECT_ULD 509
#define POWER6_PME_PM_SUSPENDED 510
#define POWER6_PME_PM_DFU_ADD_SHIFTED_BOTH 511
#define POWER6_PME_PM_LSU_REJECT_NO_SCRATCH 512
#define POWER6_PME_PM_STCX_FAIL 513
#define POWER6_PME_PM_FPU1_DENORM 514
#define POWER6_PME_PM_GCT_NOSLOT_COUNT 515
#define POWER6_PME_PM_DATA_FROM_DL2L3_SHR_CYC 516
#define POWER6_PME_PM_DATA_FROM_L21 517
#define POWER6_PME_PM_FPU_1FLOP 518
#define POWER6_PME_PM_LSU1_REJECT 519
#define POWER6_PME_PM_IC_REQ 520
#define POWER6_PME_PM_MRK_DFU_FIN 521
#define POWER6_PME_PM_NOT_LLA_CYC 522
#define POWER6_PME_PM_INST_FROM_L1 523
#define POWER6_PME_PM_MRK_VMX_COMPLEX_ISSUED 524
#define POWER6_PME_PM_BRU_FIN 525
#define POWER6_PME_PM_LSU1_REJECT_EXTERN 526
#define POWER6_PME_PM_DATA_FROM_L21_CYC 527
#define POWER6_PME_PM_GXI_CYC_BUSY 528
#define POWER6_PME_PM_MRK_LD_MISS_L1 529
#define POWER6_PME_PM_L1_WRITE_CYC 530
#define POWER6_PME_PM_LLA_CYC 531
#define POWER6_PME_PM_MRK_DATA_FROM_L2MISS 532
#define POWER6_PME_PM_GCT_FULL_COUNT 533
#define POWER6_PME_PM_MEM_DP_RQ_LOC_GLOB 534
#define POWER6_PME_PM_DATA_FROM_RL2L3_SHR 535
#define POWER6_PME_PM_MRK_LSU_REJECT_UST 536
#define POWER6_PME_PM_MRK_VMX_PERMUTE_ISSUED 537
#define POWER6_PME_PM_MRK_PTEG_FROM_L21 538
#define POWER6_PME_PM_THRD_GRP_CMPL_BOTH_CYC 539
#define POWER6_PME_PM_BR_MPRED 540
#define POWER6_PME_PM_LD_REQ_L2 541
#define POWER6_PME_PM_FLUSH_ASYNC 542
#define POWER6_PME_PM_HV_CYC 543
#define POWER6_PME_PM_LSU1_DERAT_MISS 544
#define POWER6_PME_PM_DPU_HELD_SMT 545
#define POWER6_PME_PM_MRK_LSU_FIN 546
#define POWER6_PME_PM_MRK_DATA_FROM_RL2L3_SHR 547
#define POWER6_PME_PM_LSU0_REJECT_STQ_FULL 548
#define POWER6_PME_PM_MRK_DERAT_REF_4K 549
#define POWER6_PME_PM_FPU_ISSUE_STALL_FPR 550
#define POWER6_PME_PM_IFU_FIN 551
#define POWER6_PME_PM_GXO_CYC_BUSY 552


static const int power6_event_ids[][POWER6_NUM_EVENT_COUNTERS] = {
	[ POWER6_PME_PM_LSU_REJECT_STQ_FULL ] = { 243, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_FXU_MULTI ] = { 37, 45, 36, 44, -1, -1 },
	[ POWER6_PME_PM_VMX1_STALL ] = { 328, 335, 322, 320, -1, -1 },
	[ POWER6_PME_PM_PMC2_SAVED ] = { 291, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2SB_IC_INV ] = { 174, 183, 174, 180, -1, -1 },
	[ POWER6_PME_PM_IERAT_MISS_64K ] = { -1, -1, 344, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_DIFF_3or4_CYC ] = { -1, -1, 310, -1, -1, -1 },
	[ POWER6_PME_PM_LD_REF_L1_BOTH ] = { 202, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU1_FCONV ] = { 88, 97, 86, 94, -1, -1 },
	[ POWER6_PME_PM_IBUF_FULL_COUNT ] = { 338, 345, 332, 330, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU_DERAT_MISS ] = { -1, -1, -1, 271, -1, -1 },
	[ POWER6_PME_PM_MRK_ST_CMPL ] = { 282, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2_CASTOUT_MOD ] = { 185, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU1_ST_FOLDED ] = { 100, 109, 98, 106, -1, -1 },
	[ POWER6_PME_PM_MRK_INST_TIMEO ] = { -1, -1, -1, 263, -1, -1 },
	[ POWER6_PME_PM_DPU_WT ] = { -1, -1, 54, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_RESTART ] = { 47, 56, 46, 54, -1, -1 },
	[ POWER6_PME_PM_IERAT_MISS ] = { 137, 146, 136, 143, -1, -1 },
	[ POWER6_PME_PM_FPU_SINGLE ] = { -1, -1, -1, 122, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_LMEM ] = { -1, -1, -1, 278, -1, -1 },
	[ POWER6_PME_PM_HV_COUNT ] = { -1, 351, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2SA_ST_HIT ] = { 168, 177, 168, 174, -1, -1 },
	[ POWER6_PME_PM_L2_LD_MISS_INST ] = { -1, 196, -1, -1, -1, -1 },
	[ POWER6_PME_PM_EXT_INT ] = { -1, 67, 57, -1, -1, -1 },
	[ POWER6_PME_PM_LSU1_LDF ] = { 221, 230, 216, 221, -1, -1 },
	[ POWER6_PME_PM_FAB_CMD_ISSUED ] = { 59, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L21 ] = { -1, 305, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2SA_MISS ] = { 167, 176, 167, 173, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_RL2L3_MOD ] = { 299, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_WT_COUNT ] = { -1, -1, 340, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L25_MOD ] = { -1, -1, 272, -1, -1, -1 },
	[ POWER6_PME_PM_LD_HIT_L2 ] = { -1, 209, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_DL2L3_SHR ] = { -1, -1, 290, -1, -1, -1 },
	[ POWER6_PME_PM_MEM_DP_RQ_GLOB_LOC ] = { 257, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L3SA_MISS ] = { 192, 202, 190, 195, -1, -1 },
	[ POWER6_PME_PM_NO_ITAG_COUNT ] = { 340, 347, 334, 332, -1, -1 },
	[ POWER6_PME_PM_DSLB_MISS ] = { 56, 65, 55, 63, -1, -1 },
	[ POWER6_PME_PM_LSU_FLUSH_ALIGN ] = { 235, 244, 229, 235, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_FPU_CR ] = { 35, 43, 34, 42, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L2MISS ] = { 296, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_DMEM ] = { -1, 269, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_LMEM ] = { -1, -1, -1, 291, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_REF_64K ] = { 353, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2SA_LD_REQ_INST ] = { 166, 175, 166, 172, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_MISS_16M ] = { -1, -1, 346, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_DL2L3_MOD ] = { -1, -1, -1, 13, -1, -1 },
	[ POWER6_PME_PM_FPU0_FXMULT ] = { 82, 91, 80, 88, -1, -1 },
	[ POWER6_PME_PM_L3SB_MISS ] = { 195, 205, 193, 198, -1, -1 },
	[ POWER6_PME_PM_STCX_CANCEL ] = { 305, 311, 297, 296, -1, -1 },
	[ POWER6_PME_PM_L2SA_LD_MISS_DATA ] = { 162, 171, 162, 168, -1, -1 },
	[ POWER6_PME_PM_IC_INV_L2 ] = { -1, 141, 131, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD ] = { -1, 38, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PMC1_OVERFLOW ] = { -1, 303, -1, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_6_CYC ] = { -1, 323, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L3MISS ] = { -1, -1, 274, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU0_REJECT_UST ] = { 272, 284, 266, 267, -1, -1 },
	[ POWER6_PME_PM_MRK_INST_DISP ] = { 267, 279, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LARX ] = { 197, 207, 195, 200, -1, -1 },
	[ POWER6_PME_PM_INST_CMPL ] = { 139, 148, 138, 145, -1, -1 },
	[ POWER6_PME_PM_FXU_IDLE ] = { 117, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_DL2L3_MOD ] = { -1, -1, -1, 256, -1, -1 },
	[ POWER6_PME_PM_L2_LD_REQ_DATA ] = { 186, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_DERAT_MISS_CYC ] = { 234, -1, -1, 234, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_POWER_COUNT ] = { -1, 356, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_RL2L3_MOD ] = { 146, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_DMEM_CYC ] = { -1, 14, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_DMEM ] = { -1, 13, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_PARTIAL_SECTOR ] = { 241, -1, -1, 242, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_DERAT_MPRED ] = { -1, 249, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_ULD ] = { 231, 240, 226, 231, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L3_CYC ] = { -1, 21, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FXU1_BUSY_FXU0_IDLE ] = { -1, -1, -1, 124, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_MEM_DP ] = { 145, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_FLUSH_DSI ] = { 236, 245, 230, 236, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_REF_16G ] = { -1, -1, -1, 345, -1, -1 },
	[ POWER6_PME_PM_LSU_LDF_BOTH ] = { 237, -1, 232, -1, -1, -1 },
	[ POWER6_PME_PM_FPU1_1FLOP ] = { 86, 95, 84, 92, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_RMEM_CYC ] = { -1, -1, -1, 23, -1, -1 },
	[ POWER6_PME_PM_INST_PTEG_SECONDARY ] = { 150, 159, 150, 156, -1, -1 },
	[ POWER6_PME_PM_L1_ICACHE_MISS ] = { 154, 163, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_DISP_LLA ] = { 140, 150, 140, 146, -1, -1 },
	[ POWER6_PME_PM_THRD_BOTH_RUN_CYC ] = { -1, -1, -1, 304, -1, -1 },
	[ POWER6_PME_PM_LSU_ST_CHAINED ] = { 246, 257, 240, 245, -1, -1 },
	[ POWER6_PME_PM_FPU1_FXDIV ] = { 96, 105, 94, 102, -1, -1 },
	[ POWER6_PME_PM_FREQ_UP ] = { -1, -1, -1, 123, -1, -1 },
	[ POWER6_PME_PM_FAB_RETRY_SYS_PUMP ] = { 65, 75, 64, 71, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_LMEM ] = { -1, -1, -1, 20, -1, -1 },
	[ POWER6_PME_PM_PMC3_OVERFLOW ] = { -1, -1, -1, 288, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_SET_MPRED ] = { 216, 225, 211, 216, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_DERAT_MPRED ] = { 209, 218, 204, 209, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_STQ_FULL ] = { 230, 239, 225, 230, -1, -1 },
	[ POWER6_PME_PM_MRK_BR_MPRED ] = { -1, -1, 251, -1, -1, -1 },
	[ POWER6_PME_PM_L2SA_ST_MISS ] = { 169, 178, 169, 175, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_EXTERN ] = { 210, 219, 205, 210, -1, -1 },
	[ POWER6_PME_PM_MRK_BR_TAKEN ] = { 258, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_ISLB_MISS ] = { 152, 161, 152, 158, -1, -1 },
	[ POWER6_PME_PM_CYC ] = { 12, 11, 10, 12, -1, -1 },
	[ POWER6_PME_PM_FPU_FXDIV ] = { 105, -1, -1, 110, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_LLA_END ] = { 43, 51, 42, 50, -1, -1 },
	[ POWER6_PME_PM_MEM0_DP_CL_WR_LOC ] = { 249, 260, 243, 248, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU_REJECT_ULD ] = { 276, -1, -1, 274, -1, -1 },
	[ POWER6_PME_PM_1PLUS_PPC_CMPL ] = { 1, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_DMEM ] = { -1, 304, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_WT_BR_MPRED_COUNT ] = { -1, -1, -1, 340, -1, -1 },
	[ POWER6_PME_PM_GCT_FULL_CYC ] = { 120, 128, 119, 127, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L25_SHR ] = { -1, -1, -1, 150, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_MISS_4K ] = { -1, 364, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DC_PREF_STREAM_ALLOC ] = { 22, 29, 21, 29, -1, -1 },
	[ POWER6_PME_PM_FPU1_FIN ] = { 90, 99, 88, 96, -1, -1 },
	[ POWER6_PME_PM_BR_MPRED_TA ] = { 7, 5, 5, 7, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_POWER ] = { -1, 55, -1, -1, -1, -1 },
	[ POWER6_PME_PM_RUN_INST_CMPL ] = { -1, -1, -1, -1, 0, -1 },
	[ POWER6_PME_PM_GCT_EMPTY_CYC ] = { 119, 127, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LLA_COUNT ] = { 347, 354, 339, 339, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_NO_SCRATCH ] = { 214, 223, 209, 214, -1, -1 },
	[ POWER6_PME_PM_DPU_WT_IC_MISS ] = { -1, 64, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L3MISS ] = { -1, -1, 15, 19, -1, -1 },
	[ POWER6_PME_PM_FPU_FPSCR ] = { -1, 112, 100, -1, -1, -1 },
	[ POWER6_PME_PM_VMX1_INST_ISSUED ] = { 325, 332, 319, 317, -1, -1 },
	[ POWER6_PME_PM_FLUSH ] = { 67, -1, -1, 73, -1, -1 },
	[ POWER6_PME_PM_ST_HIT_L2 ] = { 308, -1, -1, 298, -1, -1 },
	[ POWER6_PME_PM_SYNC_CYC ] = { 312, 319, 303, 303, -1, -1 },
	[ POWER6_PME_PM_FAB_SYS_PUMP ] = { 66, 76, 65, 72, -1, -1 },
	[ POWER6_PME_PM_IC_PREF_REQ ] = { 133, 142, 132, 139, -1, -1 },
	[ POWER6_PME_PM_MEM0_DP_RQ_GLOB_LOC ] = { 250, 261, 244, 249, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_0 ] = { 107, 115, 103, 112, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_2_CYC ] = { -1, -1, 308, -1, -1, -1 },
	[ POWER6_PME_PM_VMX_SIMPLE_ISSUED ] = { 335, 342, 329, 327, -1, -1 },
	[ POWER6_PME_PM_MRK_FPU1_FIN ] = { 266, 275, 260, 261, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_CW ] = { 33, 41, 32, 40, -1, -1 },
	[ POWER6_PME_PM_L3SA_REF ] = { 193, 203, 191, 196, -1, -1 },
	[ POWER6_PME_PM_STCX ] = { 304, 310, 296, 295, -1, -1 },
	[ POWER6_PME_PM_L2SB_MISS ] = { 181, 190, 181, 187, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT ] = { 208, 217, 203, 208, -1, -1 },
	[ POWER6_PME_PM_TB_BIT_TRANS ] = { 313, -1, 304, -1, -1, -1 },
	[ POWER6_PME_PM_THERMAL_MAX ] = { -1, -1, 305, -1, -1, -1 },
	[ POWER6_PME_PM_FPU0_STF ] = { 84, 93, 82, 90, -1, -1 },
	[ POWER6_PME_PM_FPU1_FMA ] = { 92, 101, 90, 98, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_LHS ] = { 226, 235, 221, 226, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_INT ] = { 40, 48, 39, 47, -1, -1 },
	[ POWER6_PME_PM_THRD_LLA_BOTH_CYC ] = { -1, -1, -1, 306, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_THERMAL_COUNT ] = { 348, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PMC4_REWIND ] = { 293, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DERAT_REF_16M ] = { -1, -1, 342, -1, -1, -1 },
	[ POWER6_PME_PM_FPU0_FCONV ] = { 73, 82, 71, 79, -1, -1 },
	[ POWER6_PME_PM_L2SA_LD_REQ_DATA ] = { 165, 174, 165, 171, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_MEM_DP ] = { 15, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_VMX_FLOAT_ISSUED ] = { 286, 298, 281, 283, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L2MISS ] = { -1, -1, -1, 277, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_DIFF_1or2_CYC ] = { -1, 324, -1, -1, -1, -1 },
	[ POWER6_PME_PM_VMX0_STALL ] = { 324, 331, 318, 316, -1, -1 },
	[ POWER6_PME_PM_IC_DEMAND_L2_BHT_REDIRECT ] = { 131, 139, 129, 137, -1, -1 },
	[ POWER6_PME_PM_LSU_DERAT_MISS ] = { -1, 243, -1, 335, -1, -1 },
	[ POWER6_PME_PM_FPU0_SINGLE ] = { 83, 92, 81, 89, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_STEERING ] = { 115, 123, 111, 120, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_1_CYC ] = { -1, 322, -1, -1, -1, -1 },
	[ POWER6_PME_PM_VMX_COMPLEX_ISSUED ] = { 329, 336, 323, 321, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_ST_FOLDED ] = { 116, 124, 112, 121, -1, -1 },
	[ POWER6_PME_PM_DFU_FIN ] = { 29, 36, 28, 36, -1, -1 },
	[ POWER6_PME_PM_BR_PRED_CCACHE ] = { 9, 7, 7, 9, -1, -1 },
	[ POWER6_PME_PM_MRK_ST_CMPL_INT ] = { -1, -1, 277, -1, -1, -1 },
	[ POWER6_PME_PM_FAB_MMIO ] = { 62, 72, 61, 68, -1, -1 },
	[ POWER6_PME_PM_MRK_VMX_SIMPLE_ISSUED ] = { 288, 300, 283, 285, -1, -1 },
	[ POWER6_PME_PM_FPU_STF ] = { -1, -1, 113, -1, -1, -1 },
	[ POWER6_PME_PM_MEM1_DP_CL_WR_GLOB ] = { 252, 263, 246, 251, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L3MISS ] = { -1, -1, 255, -1, -1, -1 },
	[ POWER6_PME_PM_GCT_NOSLOT_CYC ] = { 121, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2_ST_REQ_DATA ] = { -1, 200, 188, -1, -1, -1 },
	[ POWER6_PME_PM_INST_TABLEWALK_COUNT ] = { 341, 348, 335, 333, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L35_SHR ] = { -1, 306, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_ISYNC ] = { 41, 49, 40, 48, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L25_SHR ] = { -1, -1, -1, 257, -1, -1 },
	[ POWER6_PME_PM_L3SA_HIT ] = { 191, 201, 189, 194, -1, -1 },
	[ POWER6_PME_PM_DERAT_MISS_16G ] = { -1, -1, -1, 343, -1, -1 },
	[ POWER6_PME_PM_DATA_PTEG_2ND_HALF ] = { 18, 26, 18, 25, -1, -1 },
	[ POWER6_PME_PM_L2SA_ST_REQ ] = { 170, 179, 170, 176, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_LMEM ] = { -1, -1, -1, 152, -1, -1 },
	[ POWER6_PME_PM_IC_DEMAND_L2_BR_REDIRECT ] = { 132, 140, 130, 138, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L2 ] = { 295, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_PTEG_1ST_HALF ] = { 17, 25, 17, 24, -1, -1 },
	[ POWER6_PME_PM_BR_MPRED_COUNT ] = { 5, 3, 3, 5, -1, -1 },
	[ POWER6_PME_PM_IERAT_MISS_4K ] = { -1, -1, -1, 344, -1, -1 },
	[ POWER6_PME_PM_THRD_BOTH_RUN_COUNT ] = { -1, -1, -1, 336, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_ULD ] = { 244, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_DL2L3_MOD_CYC ] = { -1, -1, -1, 14, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_MOD ] = { 280, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU0_FLOP ] = { 76, 85, 74, 82, -1, -1 },
	[ POWER6_PME_PM_FPU0_FEST ] = { 74, 83, 72, 80, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU0_REJECT_LHS ] = { 270, 282, 264, 265, -1, -1 },
	[ POWER6_PME_PM_VMX_RESULT_SAT_1 ] = { 334, 341, 328, 326, -1, -1 },
	[ POWER6_PME_PM_NO_ITAG_CYC ] = { 290, 302, 285, 287, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_NO_SCRATCH ] = { 227, 236, 222, 227, -1, -1 },
	[ POWER6_PME_PM_0INST_FETCH ] = { 0, 0, 0, 0, -1, -1 },
	[ POWER6_PME_PM_DPU_WT_BR_MPRED ] = { -1, -1, -1, 62, -1, -1 },
	[ POWER6_PME_PM_L1_PREF ] = { 155, 164, 155, 161, -1, -1 },
	[ POWER6_PME_PM_VMX_FLOAT_MULTICYCLE ] = { 331, 338, 325, 323, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L25_SHR_CYC ] = { -1, 16, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L3 ] = { -1, -1, 14, -1, -1, -1 },
	[ POWER6_PME_PM_PMC2_OVERFLOW ] = { -1, -1, 286, -1, -1, -1 },
	[ POWER6_PME_PM_VMX0_LD_WRBACK ] = { 323, 330, 317, 315, -1, -1 },
	[ POWER6_PME_PM_FPU0_DENORM ] = { 72, 81, 70, 78, -1, -1 },
	[ POWER6_PME_PM_INST_FETCH_CYC ] = { 141, 151, 141, 147, -1, -1 },
	[ POWER6_PME_PM_LSU_LDF ] = { -1, 246, 231, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_L2_CORR ] = { 239, -1, -1, 239, -1, -1 },
	[ POWER6_PME_PM_DERAT_REF_64K ] = { -1, 360, -1, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_3_CYC ] = { -1, -1, -1, 307, -1, -1 },
	[ POWER6_PME_PM_FPU_FMA ] = { -1, 111, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L35_MOD ] = { 144, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DFU_CONV ] = { 26, 33, 25, 33, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L25_MOD ] = { -1, -1, 144, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L35_MOD ] = { 297, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_VMX_ST_ISSUED ] = { 289, 301, 284, 286, -1, -1 },
	[ POWER6_PME_PM_VMX_FLOAT_ISSUED ] = { 330, 337, 324, 322, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_L2_CORR ] = { 212, 221, 207, 212, -1, -1 },
	[ POWER6_PME_PM_THRD_L2MISS ] = { 314, 321, 307, 305, -1, -1 },
	[ POWER6_PME_PM_FPU_FCONV ] = { 102, -1, -1, 107, -1, -1 },
	[ POWER6_PME_PM_FPU_FXMULT ] = { 106, -1, -1, 111, -1, -1 },
	[ POWER6_PME_PM_FPU1_FRSP ] = { 94, 103, 92, 100, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_REF_16M ] = { -1, -1, 345, -1, -1, -1 },
	[ POWER6_PME_PM_L2SB_CASTOUT_SHR ] = { 172, 181, 172, 178, -1, -1 },
	[ POWER6_PME_PM_THRD_ONE_RUN_COUNT ] = { 344, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_RMEM ] = { -1, -1, 147, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_BOTH_BUS ] = { 233, 242, 228, 233, -1, -1 },
	[ POWER6_PME_PM_FPU1_FSQRT_FDIV ] = { 95, 104, 93, 101, -1, -1 },
	[ POWER6_PME_PM_L2_LD_REQ_INST ] = { 187, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L35_SHR ] = { -1, 291, -1, -1, -1, -1 },
	[ POWER6_PME_PM_BR_PRED_CR ] = { 10, 8, 8, 10, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU0_REJECT_ULD ] = { 271, 283, 265, 266, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT ] = { -1, -1, -1, 238, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_LHS_BOTH ] = { -1, 250, -1, 241, -1, -1 },
	[ POWER6_PME_PM_GXO_ADDR_CYC_BUSY ] = { 125, 132, 123, 131, -1, -1 },
	[ POWER6_PME_PM_LSU_SRQ_EMPTY_COUNT ] = { -1, -1, -1, 341, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L3 ] = { -1, -1, 292, -1, -1, -1 },
	[ POWER6_PME_PM_VMX0_LD_ISSUED ] = { 322, 329, 316, 314, -1, -1 },
	[ POWER6_PME_PM_FXU_PIPELINED_MULT_DIV ] = { 118, 126, 118, 126, -1, -1 },
	[ POWER6_PME_PM_FPU1_STF ] = { 99, 108, 97, 105, -1, -1 },
	[ POWER6_PME_PM_DFU_ADD ] = { 23, 30, 22, 30, -1, -1 },
	[ POWER6_PME_PM_MEM_DP_CL_WR_GLOB ] = { -1, 267, 250, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU1_REJECT_ULD ] = { 274, 286, 268, 269, -1, -1 },
	[ POWER6_PME_PM_ITLB_REF ] = { 153, 162, 153, 159, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_L2MISS ] = { 211, 220, 206, 211, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L35_SHR ] = { -1, 19, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_RL2L3_MOD ] = { 263, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU0_FPSCR ] = { 78, 87, 76, 84, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L2 ] = { 13, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_XER ] = { 54, 62, 52, 60, -1, -1 },
	[ POWER6_PME_PM_FAB_NODE_PUMP ] = { 63, 73, 62, 69, -1, -1 },
	[ POWER6_PME_PM_VMX_RESULT_SAT_0_1 ] = { 333, 340, 327, 325, -1, -1 },
	[ POWER6_PME_PM_LD_REF_L1 ] = { 201, 212, 198, 203, -1, -1 },
	[ POWER6_PME_PM_TLB_REF ] = { 320, 327, 314, 312, -1, -1 },
	[ POWER6_PME_PM_DC_PREF_OUT_OF_STREAMS ] = { 21, 28, 20, 28, -1, -1 },
	[ POWER6_PME_PM_FLUSH_FPU ] = { 69, 78, 67, 75, -1, -1 },
	[ POWER6_PME_PM_MEM1_DP_CL_WR_LOC ] = { 253, 264, 247, 252, -1, -1 },
	[ POWER6_PME_PM_L2SB_LD_HIT ] = { 175, 184, 175, 181, -1, -1 },
	[ POWER6_PME_PM_FAB_DCLAIM ] = { 60, 70, 59, 66, -1, -1 },
	[ POWER6_PME_PM_MEM_DP_CL_WR_LOC ] = { 256, -1, -1, 255, -1, -1 },
	[ POWER6_PME_PM_BR_MPRED_CR ] = { 6, 4, 4, 6, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_EXTERN ] = { -1, -1, 235, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_RL2L3_MOD ] = { 16, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_RU_WQ ] = { 48, 57, 47, 55, -1, -1 },
	[ POWER6_PME_PM_LD_MISS_L1 ] = { 199, 210, 197, 202, -1, -1 },
	[ POWER6_PME_PM_DC_INV_L2 ] = { 20, -1, -1, 27, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_RMEM ] = { -1, -1, 275, -1, -1, -1 },
	[ POWER6_PME_PM_FPU_FIN ] = { 103, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FXU0_FIN ] = { -1, -1, 117, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_FPQ ] = { 34, 42, 33, 41, -1, -1 },
	[ POWER6_PME_PM_GX_DMA_READ ] = { 128, 135, 126, 134, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_PARTIAL_SECTOR ] = { 228, 237, 223, 228, -1, -1 },
	[ POWER6_PME_PM_0INST_FETCH_COUNT ] = { 337, 344, 331, 329, -1, -1 },
	[ POWER6_PME_PM_PMC5_OVERFLOW ] = { 294, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2SB_LD_REQ ] = { 178, 187, 178, 184, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_DIFF_0_CYC ] = { 318, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_RMEM ] = { -1, -1, 16, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_CYC ] = { -1, -1, 234, -1, -1, -1 },
	[ POWER6_PME_PM_ST_REF_L1_BOTH ] = { -1, 316, -1, 301, -1, -1 },
	[ POWER6_PME_PM_VMX_PERMUTE_ISSUED ] = { 332, 339, 326, 324, -1, -1 },
	[ POWER6_PME_PM_BR_TAKEN ] = { -1, 10, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FAB_DMA ] = { 61, 71, 60, 67, -1, -1 },
	[ POWER6_PME_PM_GCT_EMPTY_COUNT ] = { -1, 358, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU1_SINGLE ] = { 98, 107, 96, 104, -1, -1 },
	[ POWER6_PME_PM_L2SA_CASTOUT_SHR ] = { 158, 167, 158, 164, -1, -1 },
	[ POWER6_PME_PM_L3SB_REF ] = { 196, 206, 194, 199, -1, -1 },
	[ POWER6_PME_PM_FPU0_FRSP ] = { 79, 88, 77, 85, -1, -1 },
	[ POWER6_PME_PM_PMC4_SAVED ] = { -1, -1, 288, -1, -1, -1 },
	[ POWER6_PME_PM_L2SA_DC_INV ] = { 159, 168, 159, 165, -1, -1 },
	[ POWER6_PME_PM_GXI_ADDR_CYC_BUSY ] = { 122, 129, 120, 128, -1, -1 },
	[ POWER6_PME_PM_FPU0_FMA ] = { 77, 86, 75, 83, -1, -1 },
	[ POWER6_PME_PM_SLB_MISS ] = { 303, -1, -1, 294, -1, -1 },
	[ POWER6_PME_PM_MRK_ST_GPS ] = { -1, 294, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DERAT_REF_4K ] = { 350, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2_CASTOUT_SHR ] = { -1, 194, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_STCX_CR ] = { 51, 60, 50, 58, -1, -1 },
	[ POWER6_PME_PM_FPU0_ST_FOLDED ] = { 85, 94, 83, 91, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L21 ] = { -1, 270, -1, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus3or4_CYC ] = { -1, -1, 311, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L35_MOD ] = { 14, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_DL2L3_SHR ] = { -1, -1, 11, -1, -1, -1 },
	[ POWER6_PME_PM_GXI_DATA_CYC_BUSY ] = { 124, 131, 122, 130, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_STEAL ] = { 242, 254, 239, 243, -1, -1 },
	[ POWER6_PME_PM_ST_FIN ] = { 307, 313, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_CR_LOGICAL ] = { 32, 40, 31, 39, -1, -1 },
	[ POWER6_PME_PM_THRD_SEL_T0 ] = { 319, 326, 312, 311, -1, -1 },
	[ POWER6_PME_PM_PTEG_RELOAD_VALID ] = { 300, 308, 295, 292, -1, -1 },
	[ POWER6_PME_PM_L2_PREF_ST ] = { 189, 199, 187, 192, -1, -1 },
	[ POWER6_PME_PM_MRK_STCX_FAIL ] = { 281, 293, 276, 279, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_LHS ] = { 213, 222, 208, 213, -1, -1 },
	[ POWER6_PME_PM_DFU_EXP_EQ ] = { 28, 35, 27, 35, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_FP_FX_MULT ] = { 36, 44, 35, 43, -1, -1 },
	[ POWER6_PME_PM_L2_LD_MISS_DATA ] = { -1, 195, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L35_MOD_CYC ] = { -1, -1, -1, 18, -1, -1 },
	[ POWER6_PME_PM_FLUSH_FXU ] = { 70, 79, 68, 76, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_1 ] = { 108, 116, 104, 113, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_LMEM_CYC ] = { -1, 22, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_LSU_SOPS ] = { 45, 53, 44, 52, -1, -1 },
	[ POWER6_PME_PM_INST_PTEG_2ND_HALF ] = { 149, 158, 149, 155, -1, -1 },
	[ POWER6_PME_PM_THRESH_TIMEO ] = { -1, -1, 313, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_UST_BOTH ] = { 245, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_FAST ] = { -1, -1, 236, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_THRD_PRIO ] = { 53, 61, 51, 59, -1, -1 },
	[ POWER6_PME_PM_L2_PREF_LD ] = { 188, 198, 186, 191, -1, -1 },
	[ POWER6_PME_PM_FPU_FEST ] = { -1, -1, -1, 108, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_RMEM ] = { -1, -1, 256, -1, -1, -1 },
	[ POWER6_PME_PM_LD_MISS_L1_CYC ] = { 200, 211, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DERAT_MISS_4K ] = { 351, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_COMPLETION ] = { 31, 39, 30, 38, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_STALL_ST ] = { 113, 121, 109, 118, -1, -1 },
	[ POWER6_PME_PM_L2SB_DC_INV ] = { 173, 182, 173, 179, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L25_SHR ] = { -1, -1, -1, 290, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_DL2L3_MOD ] = { -1, -1, -1, 289, -1, -1 },
	[ POWER6_PME_PM_FAB_CMD_RETRIED ] = { -1, 69, -1, -1, -1, -1 },
	[ POWER6_PME_PM_BR_PRED_LSTACK ] = { 11, 9, 9, 11, -1, -1 },
	[ POWER6_PME_PM_GXO_DATA_CYC_BUSY ] = { 127, 134, 125, 133, -1, -1 },
	[ POWER6_PME_PM_DFU_SUBNORM ] = { 30, 37, 29, 37, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_OOO ] = { 111, 119, 107, 116, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_ULD_BOTH ] = { -1, 255, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2SB_ST_MISS ] = { 183, 192, 183, 189, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L25_MOD_CYC ] = { -1, -1, -1, 17, -1, -1 },
	[ POWER6_PME_PM_INST_PTEG_1ST_HALF ] = { 148, 157, 148, 154, -1, -1 },
	[ POWER6_PME_PM_DERAT_MISS_16M ] = { -1, -1, 343, -1, -1, -1 },
	[ POWER6_PME_PM_GX_DMA_WRITE ] = { 129, 136, 127, 135, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_MOD ] = { -1, -1, -1, 275, -1, -1 },
	[ POWER6_PME_PM_MEM1_DP_RQ_GLOB_LOC ] = { 254, 265, 248, 253, -1, -1 },
	[ POWER6_PME_PM_L2SB_LD_REQ_DATA ] = { 179, 188, 179, 185, -1, -1 },
	[ POWER6_PME_PM_L2SA_LD_MISS_INST ] = { 163, 172, 163, 169, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU0_REJECT_L2MISS ] = { 269, 281, 263, 264, -1, -1 },
	[ POWER6_PME_PM_MRK_IFU_FIN ] = { -1, 278, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L3 ] = { -1, -1, 145, -1, -1, -1 },
	[ POWER6_PME_PM_FXU1_FIN ] = { -1, -1, -1, 125, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_4_CYC ] = { -1, -1, -1, 308, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L35_MOD ] = { 261, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_SET_MPRED ] = { -1, 252, 238, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_MISS_16G ] = { -1, -1, -1, 346, -1, -1 },
	[ POWER6_PME_PM_FPU0_FXDIV ] = { 81, 90, 79, 87, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU1_REJECT_UST ] = { 275, 287, 269, 270, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_DIV_SQRT_OVERLAP ] = { 110, 118, 106, 115, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L35_SHR ] = { -1, 155, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU_REJECT_LHS ] = { -1, -1, -1, 273, -1, -1 },
	[ POWER6_PME_PM_LSU_LMQ_FULL_CYC ] = { 238, 247, 233, 237, -1, -1 },
	[ POWER6_PME_PM_SYNC_COUNT ] = { 342, 349, 336, 334, -1, -1 },
	[ POWER6_PME_PM_MEM0_DP_RQ_LOC_GLOB ] = { 251, 262, 245, 250, -1, -1 },
	[ POWER6_PME_PM_L2SA_CASTOUT_MOD ] = { 157, 166, 157, 163, -1, -1 },
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_COUNT ] = { -1, -1, 341, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_MEM_DP ] = { 298, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_SLOW ] = { -1, 253, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L25_MOD ] = { -1, -1, 291, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_7_CYC ] = { 317, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_SHR ] = { -1, 292, -1, -1, -1, -1 },
	[ POWER6_PME_PM_ST_REQ_L2 ] = { -1, 317, 301, -1, -1, -1 },
	[ POWER6_PME_PM_ST_REF_L1 ] = { 310, 315, 300, 300, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_STALL_THRD ] = { 114, 122, 110, 119, -1, -1 },
	[ POWER6_PME_PM_RUN_COUNT ] = { 343, 350, -1, -1, -1, -1 },
	[ POWER6_PME_PM_RUN_CYC ] = { 302, 309, -1, -1, -1, 0 },
	[ POWER6_PME_PM_PTEG_FROM_RMEM ] = { -1, -1, 294, -1, -1, -1 },
	[ POWER6_PME_PM_LSU0_LDF ] = { 205, 214, 200, 205, -1, -1 },
	[ POWER6_PME_PM_ST_MISS_L1 ] = { 309, 314, 299, 299, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_DL2L3_SHR ] = { -1, -1, 142, -1, -1, -1 },
	[ POWER6_PME_PM_L2SA_IC_INV ] = { 160, 169, 160, 166, -1, -1 },
	[ POWER6_PME_PM_THRD_ONE_RUN_CYC ] = { 315, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L2SB_LD_REQ_INST ] = { 180, 189, 180, 186, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L25_MOD ] = { -1, -1, 253, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_XTHRD ] = { 55, 63, 53, 61, -1, -1 },
	[ POWER6_PME_PM_L2SB_ST_REQ ] = { 184, 193, 184, 190, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L21 ] = { -1, 154, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L3MISS ] = { -1, -1, 146, -1, -1, -1 },
	[ POWER6_PME_PM_L3SB_HIT ] = { 194, 204, 192, 197, -1, -1 },
	[ POWER6_PME_PM_EE_OFF_EXT_INT ] = { 57, 66, 56, 64, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_DL2L3_MOD ] = { -1, -1, -1, 148, -1, -1 },
	[ POWER6_PME_PM_PMC6_OVERFLOW ] = { -1, -1, 289, -1, -1, -1 },
	[ POWER6_PME_PM_FPU_FLOP ] = { 104, -1, -1, 109, -1, -1 },
	[ POWER6_PME_PM_FXU_BUSY ] = { -1, 125, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU1_FLOP ] = { 91, 100, 89, 97, -1, -1 },
	[ POWER6_PME_PM_IC_RELOAD_SHR ] = { 135, 144, 134, 141, -1, -1 },
	[ POWER6_PME_PM_INST_TABLEWALK_CYC ] = { 151, 160, 151, 157, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_RL2L3_MOD_CYC ] = { -1, -1, -1, 22, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_DIFF_5or6_CYC ] = { -1, -1, -1, 309, -1, -1 },
	[ POWER6_PME_PM_IBUF_FULL_CYC ] = { 130, 138, 128, 136, -1, -1 },
	[ POWER6_PME_PM_L2SA_LD_REQ ] = { 164, 173, 164, 170, -1, -1 },
	[ POWER6_PME_PM_VMX1_LD_WRBACK ] = { 327, 334, 321, 319, -1, -1 },
	[ POWER6_PME_PM_MRK_FPU_FIN ] = { -1, 276, 261, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_5_CYC ] = { -1, -1, 309, -1, -1, -1 },
	[ POWER6_PME_PM_DFU_BACK2BACK ] = { 25, 32, 24, 32, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_LMEM ] = { -1, -1, -1, 258, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_LHS ] = { 240, -1, -1, 240, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_SPR ] = { 50, 59, 49, 57, -1, -1 },
	[ POWER6_PME_PM_FREQ_DOWN ] = { -1, -1, 115, -1, -1, -1 },
	[ POWER6_PME_PM_DFU_ENC_BCD_DPD ] = { 27, 34, 26, 34, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_GPR ] = { 39, 47, 38, 46, -1, -1 },
	[ POWER6_PME_PM_LSU0_NCST ] = { 207, 216, 202, 207, -1, -1 },
	[ POWER6_PME_PM_MRK_INST_ISSUED ] = { 268, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_RL2L3_SHR ] = { -1, 156, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU_DENORM ] = { -1, 110, 99, -1, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_L3MISS ] = { -1, -1, 293, -1, -1, -1 },
	[ POWER6_PME_PM_RUN_PURR ] = { -1, -1, -1, 347, -1, -1 },
	[ POWER6_PME_PM_MRK_VMX0_LD_WRBACK ] = { 283, 295, 278, 280, -1, -1 },
	[ POWER6_PME_PM_L2_MISS ] = { -1, 197, 185, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L3 ] = { -1, -1, 254, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU1_REJECT_LHS ] = { 273, 285, 267, 268, -1, -1 },
	[ POWER6_PME_PM_L2SB_LD_MISS_INST ] = { 177, 186, 177, 183, -1, -1 },
	[ POWER6_PME_PM_PTEG_FROM_RL2L3_SHR ] = { -1, 307, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_MISS_64K ] = { 354, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LWSYNC ] = { 247, 258, 241, 246, -1, -1 },
	[ POWER6_PME_PM_FPU1_FXMULT ] = { 97, 106, 95, 103, -1, -1 },
	[ POWER6_PME_PM_MEM0_DP_CL_WR_GLOB ] = { 248, 259, 242, 247, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_PARTIAL_SECTOR ] = { 215, 224, 210, 215, -1, -1 },
	[ POWER6_PME_PM_INST_IMC_MATCH_CMPL ] = { 147, -1, -1, 153, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_THERMAL ] = { 52, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU_FRSP ] = { -1, 113, 101, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_INST_FIN ] = { -1, -1, 262, 262, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_SHR ] = { -1, -1, 271, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DTLB_REF ] = { 264, 273, 258, 259, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L25_SHR ] = { -1, -1, -1, 276, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_LSU ] = { 44, 52, 43, 51, -1, -1 },
	[ POWER6_PME_PM_FPU_FSQRT_FDIV ] = { -1, 114, 102, -1, -1, -1 },
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_COUNT ] = { -1, 359, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_PTEG_SECONDARY ] = { 19, 27, 19, 26, -1, -1 },
	[ POWER6_PME_PM_FPU1_FEST ] = { 89, 98, 87, 95, -1, -1 },
	[ POWER6_PME_PM_L2SA_LD_HIT ] = { 161, 170, 161, 167, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_MEM_DP_CYC ] = { -1, -1, -1, 21, -1, -1 },
	[ POWER6_PME_PM_BR_MPRED_CCACHE ] = { 4, 2, 2, 4, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_COUNT ] = { -1, 355, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_SET_MPRED ] = { 229, 238, 224, 229, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_2 ] = { 109, 117, 105, 114, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_L2_CORR ] = { 225, 234, 220, 225, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_DMEM ] = { -1, 289, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MEM1_DP_RQ_LOC_GLOB ] = { 255, 266, 249, 254, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus1or2_CYC ] = { -1, 325, -1, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_0_CYC ] = { 316, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FXU0_BUSY_FXU1_IDLE ] = { -1, -1, 116, -1, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_DERAT_MPRED ] = { 223, 232, 218, 223, -1, -1 },
	[ POWER6_PME_PM_MRK_VMX1_LD_WRBACK ] = { 284, 296, 279, 281, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_RL2L3_SHR_CYC ] = { -1, 24, -1, -1, -1, -1 },
	[ POWER6_PME_PM_IERAT_MISS_16M ] = { -1, 362, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_MEM_DP ] = { 262, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LARX_L1HIT ] = { 198, 208, 196, 201, -1, -1 },
	[ POWER6_PME_PM_L2_ST_MISS_DATA ] = { 190, -1, -1, 193, -1, -1 },
	[ POWER6_PME_PM_FPU_ST_FOLDED ] = { -1, -1, 114, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L35_SHR ] = { -1, 271, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_MULT_GPR ] = { 46, 54, 45, 53, -1, -1 },
	[ POWER6_PME_PM_FPU0_1FLOP ] = { 71, 80, 69, 77, -1, -1 },
	[ POWER6_PME_PM_IERAT_MISS_16G ] = { 352, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_IC_PREF_WRITE ] = { 134, 143, 133, 140, -1, -1 },
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus5or6_CYC ] = { -1, -1, -1, 310, -1, -1 },
	[ POWER6_PME_PM_FPU0_FIN ] = { 75, 84, 73, 81, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L2_CYC ] = { -1, 18, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DERAT_REF_16G ] = { -1, -1, -1, 342, -1, -1 },
	[ POWER6_PME_PM_BR_PRED ] = { 8, 6, 6, 8, -1, -1 },
	[ POWER6_PME_PM_VMX1_LD_ISSUED ] = { 326, 333, 320, 318, -1, -1 },
	[ POWER6_PME_PM_L2SB_CASTOUT_MOD ] = { 171, 180, 171, 177, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_DMEM ] = { -1, 152, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L35_SHR_CYC ] = { -1, 20, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU0_NCLD ] = { 206, 215, 201, 206, -1, -1 },
	[ POWER6_PME_PM_FAB_RETRY_NODE_PUMP ] = { 64, 74, 63, 70, -1, -1 },
	[ POWER6_PME_PM_VMX0_INST_ISSUED ] = { 321, 328, 315, 313, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L25_MOD ] = { -1, -1, 12, -1, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_ITLB_ISLB ] = { 42, 50, 41, 49, -1, -1 },
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_CYC ] = { -1, 248, -1, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_CONC_RUN_INST ] = { -1, -1, 306, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L2 ] = { 277, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PURR ] = { 301, -1, -1, 293, -1, -1 },
	[ POWER6_PME_PM_DERAT_MISS_64K ] = { -1, 361, -1, -1, -1, -1 },
	[ POWER6_PME_PM_PMC2_REWIND ] = { -1, -1, 287, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L2 ] = { 143, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_INST_DISP ] = { -1, 149, 139, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L25_SHR ] = { -1, -1, -1, 16, -1, -1 },
	[ POWER6_PME_PM_L1_DCACHE_RELOAD_VALID ] = { -1, -1, 154, 160, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_UST ] = { 232, 241, 227, 232, -1, -1 },
	[ POWER6_PME_PM_FAB_ADDR_COLLISION ] = { 58, 68, 58, 65, -1, -1 },
	[ POWER6_PME_PM_MRK_FXU_FIN ] = { -1, 277, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_UST ] = { 219, 228, 214, 219, -1, -1 },
	[ POWER6_PME_PM_PMC4_OVERFLOW ] = { 292, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L3 ] = { -1, -1, 273, -1, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L2MISS ] = { -1, -1, -1, 151, -1, -1 },
	[ POWER6_PME_PM_L2SB_ST_HIT ] = { 182, 191, 182, 188, -1, -1 },
	[ POWER6_PME_PM_DPU_WT_IC_MISS_COUNT ] = { -1, 357, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_DL2L3_SHR ] = { -1, -1, 252, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L35_MOD ] = { 278, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU1_FPSCR ] = { 93, 102, 91, 99, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_UST ] = { -1, 256, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU0_DERAT_MISS ] = { 204, 213, 199, 204, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_MEM_DP ] = { 279, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L2 ] = { 259, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU0_FSQRT_FDIV ] = { 80, 89, 78, 86, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_FXU_SOPS ] = { 38, 46, 37, 45, -1, -1 },
	[ POWER6_PME_PM_MRK_FPU0_FIN ] = { 265, 274, 259, 260, -1, -1 },
	[ POWER6_PME_PM_L2SB_LD_MISS_DATA ] = { 176, 185, 176, 182, -1, -1 },
	[ POWER6_PME_PM_LSU_SRQ_EMPTY_CYC ] = { -1, -1, -1, 244, -1, -1 },
	[ POWER6_PME_PM_1PLUS_PPC_DISP ] = { 2, -1, -1, 1, -1, -1 },
	[ POWER6_PME_PM_VMX_ST_ISSUED ] = { 336, 343, 330, 328, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L2MISS ] = { -1, 17, 13, -1, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_ULD ] = { 218, 227, 213, 218, -1, -1 },
	[ POWER6_PME_PM_SUSPENDED ] = { 311, 318, 302, 302, -1, -1 },
	[ POWER6_PME_PM_DFU_ADD_SHIFTED_BOTH ] = { 24, 31, 23, 31, -1, -1 },
	[ POWER6_PME_PM_LSU_REJECT_NO_SCRATCH ] = { -1, 251, 237, -1, -1, -1 },
	[ POWER6_PME_PM_STCX_FAIL ] = { 306, 312, 298, 297, -1, -1 },
	[ POWER6_PME_PM_FPU1_DENORM ] = { 87, 96, 85, 93, -1, -1 },
	[ POWER6_PME_PM_GCT_NOSLOT_COUNT ] = { 349, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_DL2L3_SHR_CYC ] = { -1, 12, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L21 ] = { -1, 15, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU_1FLOP ] = { 101, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT ] = { 222, 231, 217, 222, -1, -1 },
	[ POWER6_PME_PM_IC_REQ ] = { 136, 145, 135, 142, -1, -1 },
	[ POWER6_PME_PM_MRK_DFU_FIN ] = { -1, -1, 257, -1, -1, -1 },
	[ POWER6_PME_PM_NOT_LLA_CYC ] = { 346, 353, 338, 338, -1, -1 },
	[ POWER6_PME_PM_INST_FROM_L1 ] = { 142, 153, 143, 149, -1, -1 },
	[ POWER6_PME_PM_MRK_VMX_COMPLEX_ISSUED ] = { 285, 297, 280, 282, -1, -1 },
	[ POWER6_PME_PM_BRU_FIN ] = { 3, 1, 1, 2, -1, -1 },
	[ POWER6_PME_PM_LSU1_REJECT_EXTERN ] = { 224, 233, 219, 224, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_L21_CYC ] = { -1, -1, -1, 15, -1, -1 },
	[ POWER6_PME_PM_GXI_CYC_BUSY ] = { 123, 130, 121, 129, -1, -1 },
	[ POWER6_PME_PM_MRK_LD_MISS_L1 ] = { -1, 280, -1, -1, -1, -1 },
	[ POWER6_PME_PM_L1_WRITE_CYC ] = { 156, 165, 156, 162, -1, -1 },
	[ POWER6_PME_PM_LLA_CYC ] = { 345, 352, 337, 337, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_L2MISS ] = { 260, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_GCT_FULL_COUNT ] = { 339, 346, 333, 331, -1, -1 },
	[ POWER6_PME_PM_MEM_DP_RQ_LOC_GLOB ] = { -1, 268, -1, -1, -1, -1 },
	[ POWER6_PME_PM_DATA_FROM_RL2L3_SHR ] = { -1, 23, -1, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU_REJECT_UST ] = { -1, 288, 270, -1, -1, -1 },
	[ POWER6_PME_PM_MRK_VMX_PERMUTE_ISSUED ] = { 287, 299, 282, 284, -1, -1 },
	[ POWER6_PME_PM_MRK_PTEG_FROM_L21 ] = { -1, 290, -1, -1, -1, -1 },
	[ POWER6_PME_PM_THRD_GRP_CMPL_BOTH_CYC ] = { -1, 320, -1, -1, -1, -1 },
	[ POWER6_PME_PM_BR_MPRED ] = { -1, -1, -1, 3, -1, -1 },
	[ POWER6_PME_PM_LD_REQ_L2 ] = { 203, -1, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FLUSH_ASYNC ] = { 68, 77, 66, 74, -1, -1 },
	[ POWER6_PME_PM_HV_CYC ] = { -1, 137, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU1_DERAT_MISS ] = { 220, 229, 215, 220, -1, -1 },
	[ POWER6_PME_PM_DPU_HELD_SMT ] = { 49, 58, 48, 56, -1, -1 },
	[ POWER6_PME_PM_MRK_LSU_FIN ] = { -1, -1, -1, 272, -1, -1 },
	[ POWER6_PME_PM_MRK_DATA_FROM_RL2L3_SHR ] = { -1, 272, -1, -1, -1, -1 },
	[ POWER6_PME_PM_LSU0_REJECT_STQ_FULL ] = { 217, 226, 212, 217, -1, -1 },
	[ POWER6_PME_PM_MRK_DERAT_REF_4K ] = { -1, 363, -1, -1, -1, -1 },
	[ POWER6_PME_PM_FPU_ISSUE_STALL_FPR ] = { 112, 120, 108, 117, -1, -1 },
	[ POWER6_PME_PM_IFU_FIN ] = { 138, 147, 137, 144, -1, -1 },
	[ POWER6_PME_PM_GXO_CYC_BUSY ] = { 126, 133, 124, 132, -1, -1 }
};

static const unsigned long long power6_group_vecs[][POWER6_NUM_GROUP_VEC] = {
	[ POWER6_PME_PM_LSU_REJECT_STQ_FULL ] = {
		0x2000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_FXU_MULTI ] = {
		0x0000008000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX1_STALL ] = {
		0x0000000000000000ULL,
		0x0000000001000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC2_SAVED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_IC_INV ] = {
		0x0000000000000000ULL,
		0x0000400000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IERAT_MISS_64K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_3or4_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000010000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LD_REF_L1_BOTH ] = {
		0x0000400000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FCONV ] = {
		0x0000000000000000ULL,
		0x4000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IBUF_FULL_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000100000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU_DERAT_MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000800000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_ST_CMPL ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000002000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_CASTOUT_MOD ] = {
		0x0000000000000000ULL,
		0x0000008000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_ST_FOLDED ] = {
		0x0000000000000000ULL,
		0x4000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_INST_TIMEO ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000400000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_WT ] = {
		0x0000000800000000ULL,
		0x0000000000000000ULL,
		0x0004000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_RESTART ] = {
		0x0000020000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IERAT_MISS ] = {
		0x0400000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_SINGLE ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000002ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_LMEM ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000040000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_HV_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000800000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_ST_HIT ] = {
		0x0000000000000000ULL,
		0x0001000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_LD_MISS_INST ] = {
		0x0000000000000000ULL,
		0x0000004000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_EXT_INT ] = {
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_LDF ] = {
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_CMD_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000028000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L21 ] = {
		0x0000000018000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_MISS ] = {
		0x0000000000000000ULL,
		0x0000100000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_RL2L3_MOD ] = {
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_WT_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0002040000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L25_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000008000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LD_HIT_L2 ] = {
		0x0000000000000000ULL,
		0x0000010000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_DL2L3_SHR ] = {
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM_DP_RQ_GLOB_LOC ] = {
		0x0000000000000000ULL,
		0x0000000040000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L3SA_MISS ] = {
		0x0000000000000000ULL,
		0x0004000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_NO_ITAG_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000400000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DSLB_MISS ] = {
		0x0600000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_FLUSH_ALIGN ] = {
		0x0000000000000000ULL,
		0x0000000000000800ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_FPU_CR ] = {
		0x0000188000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L2MISS ] = {
		0x0000000010000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_DMEM ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000400ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_LMEM ] = {
		0x0000000060000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_64K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000001000000000ULL,
		0x0000000000000010ULL
	},
	[ POWER6_PME_PM_L2SA_LD_REQ_INST ] = {
		0x0000000000000000ULL,
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_16M ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000002000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_MOD ] = {
		0x0000000000000400ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FXMULT ] = {
		0x0000000000000000ULL,
		0x0200000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L3SB_MISS ] = {
		0x0000000000000000ULL,
		0x0008000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_STCX_CANCEL ] = {
		0x0008000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_LD_MISS_DATA ] = {
		0x0000000000000000ULL,
		0x0000020000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IC_INV_L2 ] = {
		0x0000000000000000ULL,
		0x0000008000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD ] = {
		0x0000002000000000ULL,
		0x0000000000000000ULL,
		0x0004000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC1_OVERFLOW ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_6_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000008000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L3MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000040000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_UST ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000020000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_INST_DISP ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000080000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LARX ] = {
		0x0018000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_CMPL ] = {
		0x0100000000006001ULL,
		0x0000000000000000ULL,
		0x3800003ffffffe18ULL,
		0x0000000000000038ULL
	},
	[ POWER6_PME_PM_FXU_IDLE ] = {
		0x0000000000000000ULL,
		0x0000000000040000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_DL2L3_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000400ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_LD_REQ_DATA ] = {
		0x0000000000000000ULL,
		0x0000002000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_DERAT_MISS_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000000800ULL,
		0x0040000000000020ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_POWER_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0100020000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_RL2L3_MOD ] = {
		0x0000000001000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_DMEM_CYC ] = {
		0x0000000000004000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_DMEM ] = {
		0x0000000000000800ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_PARTIAL_SECTOR ] = {
		0x2000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_DERAT_MPRED ] = {
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_ULD ] = {
		0x8000000000000000ULL,
		0x0000000000000002ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L3_CYC ] = {
		0x0000000000010000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FXU1_BUSY_FXU0_IDLE ] = {
		0x0000000000000000ULL,
		0x0000000000040000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_MEM_DP ] = {
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_FLUSH_DSI ] = {
		0x0000000000000000ULL,
		0x0000000000000800ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_16G ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000010ULL
	},
	[ POWER6_PME_PM_LSU_LDF_BOTH ] = {
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_1FLOP ] = {
		0x0000000000000000ULL,
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_RMEM_CYC ] = {
		0x0000000000004000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_PTEG_SECONDARY ] = {
		0x0000000200000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L1_ICACHE_MISS ] = {
		0x0000000800000000ULL,
		0x0000000000000000ULL,
		0x0000040000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_DISP_LLA ] = {
		0x0000000000000000ULL,
		0x0000000000001000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_BOTH_RUN_CYC ] = {
		0x0020000000000000ULL,
		0x0000000000000000ULL,
		0x0200000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_ST_CHAINED ] = {
		0x0000000000000000ULL,
		0x0000000000000200ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FXDIV ] = {
		0x0000000000000000ULL,
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FREQ_UP ] = {
		0x0000000400000000ULL,
		0x0000000000000000ULL,
		0x0000020000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_RETRY_SYS_PUMP ] = {
		0x0000000000000000ULL,
		0x0000000010000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_LMEM ] = {
		0x0000000000100800ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC3_OVERFLOW ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_SET_MPRED ] = {
		0x0000000000000000ULL,
		0x0000000000000001ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_DERAT_MPRED ] = {
		0x0000000000000000ULL,
		0x0000000000000084ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_STQ_FULL ] = {
		0x0000000000000000ULL,
		0x0000000000000040ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_BR_MPRED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000200ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_ST_MISS ] = {
		0x0000000000000000ULL,
		0x0000080000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_EXTERN ] = {
		0x0000000000000000ULL,
		0x0000000000000008ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_BR_TAKEN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000200ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ISLB_MISS ] = {
		0x0600000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_CYC ] = {
		0x0100200000283003ULL,
		0x0000000000000000ULL,
		0x5c00000481000018ULL,
		0x0000000000000005ULL
	},
	[ POWER6_PME_PM_FPU_FXDIV ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000002ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_LLA_END ] = {
		0x0000020000000000ULL,
		0x0000000000001000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM0_DP_CL_WR_LOC ] = {
		0x0000000000000000ULL,
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU_REJECT_ULD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000210000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_1PLUS_PPC_CMPL ] = {
		0x0040000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_DMEM ] = {
		0x0000000040000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_WT_BR_MPRED_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0002040000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GCT_FULL_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000002000ULL,
		0x0000400000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L25_SHR ] = {
		0x0000000000800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_4K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000002000000000ULL,
		0x0000000000000020ULL
	},
	[ POWER6_PME_PM_DC_PREF_STREAM_ALLOC ] = {
		0x0001000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FIN ] = {
		0x0000000000000000ULL,
		0x2000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_MPRED_TA ] = {
		0x0000000000000028ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_POWER ] = {
		0x0000001400000000ULL,
		0x0000000000000000ULL,
		0x0200000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_RUN_INST_CMPL ] = {
		0xffffffffffffffffULL,
		0xffffffffffffffffULL,
		0xffffffffffffffffULL,
		0x000000000000003fULL
	},
	[ POWER6_PME_PM_GCT_EMPTY_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000002000ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LLA_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000080000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_NO_SCRATCH ] = {
		0x0000000000000000ULL,
		0x0000000000000010ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_WT_IC_MISS ] = {
		0x0000000800000000ULL,
		0x0000000000000000ULL,
		0x0000080000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L3MISS ] = {
		0x0000000000000180ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000008ULL
	},
	[ POWER6_PME_PM_FPU_FPSCR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000001ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX1_INST_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000200000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FLUSH ] = {
		0x0002000000080000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ST_HIT_L2 ] = {
		0x0000000000000000ULL,
		0x0000010000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_SYNC_CYC ] = {
		0x0000200000000000ULL,
		0x0000000000000000ULL,
		0x0061800000000010ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_SYS_PUMP ] = {
		0x0000000000000000ULL,
		0x0000000010000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IC_PREF_REQ ] = {
		0x0004000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM0_DP_RQ_GLOB_LOC ] = {
		0x0000000000000000ULL,
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_0 ] = {
		0x0000000000000000ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_2_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000004000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_SIMPLE_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000100000ULL,
		0x0000000400000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_FPU1_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000800100000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_CW ] = {
		0x0000002000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L3SA_REF ] = {
		0x0000000000000000ULL,
		0x0004000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_STCX ] = {
		0x0018000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_MISS ] = {
		0x0000000000000000ULL,
		0x0000100000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT ] = {
		0x0000000000000000ULL,
		0x0000000000000044ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_TB_BIT_TRANS ] = {
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THERMAL_MAX ] = {
		0x0000001000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_STF ] = {
		0x0000000000000000ULL,
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FMA ] = {
		0x0000000000000000ULL,
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_LHS ] = {
		0x0000000000000000ULL,
		0x0000000000000020ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_INT ] = {
		0x0000020000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_LLA_BOTH_CYC ] = {
		0x0040000000000000ULL,
		0x0000000000001000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_THERMAL_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0002020000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC4_REWIND ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_REF_16M ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000080ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FCONV ] = {
		0x0000000000000000ULL,
		0x0400000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_LD_REQ_DATA ] = {
		0x0000000000000000ULL,
		0x0000020000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_MEM_DP ] = {
		0x0000000000020800ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_VMX_FLOAT_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000100000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L2MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000020000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_1or2_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000010000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX0_STALL ] = {
		0x0000000000000000ULL,
		0x0000000001000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IC_DEMAND_L2_BHT_REDIRECT ] = {
		0x0000000000000000ULL,
		0x0002000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_DERAT_MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000800ULL,
		0x0020004000000020ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_SINGLE ] = {
		0x0000000000000000ULL,
		0x0100000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_STEERING ] = {
		0x0000000000000000ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_1_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000004000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_COMPLEX_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000100000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_ST_FOLDED ] = {
		0x0000000000000000ULL,
		0x0020000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000004000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_PRED_CCACHE ] = {
		0x0000000000000018ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_ST_CMPL_INT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000002000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_MMIO ] = {
		0x0000000000000000ULL,
		0x0000000020000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_VMX_SIMPLE_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000400000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_STF ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0800000000000002ULL,
		0x0000000000000002ULL
	},
	[ POWER6_PME_PM_MEM1_DP_CL_WR_GLOB ] = {
		0x0000000000000000ULL,
		0x0000000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L3MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000002000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GCT_NOSLOT_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000002000ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_ST_REQ_DATA ] = {
		0x0000000000000000ULL,
		0x0000002000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_TABLEWALK_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000200000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L35_SHR ] = {
		0x0000000020000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_ISYNC ] = {
		0x0000184000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L25_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000001000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L3SA_HIT ] = {
		0x0000000000000000ULL,
		0x0004000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_MISS_16G ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000040ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_PTEG_2ND_HALF ] = {
		0x0000000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_ST_REQ ] = {
		0x0000000000000000ULL,
		0x0001080000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_LMEM ] = {
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IC_DEMAND_L2_BR_REDIRECT ] = {
		0x0000000000000000ULL,
		0x0002000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L2 ] = {
		0x0000000008000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_PTEG_1ST_HALF ] = {
		0x0000000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_MPRED_COUNT ] = {
		0x0000000000000024ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IERAT_MISS_4K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_BOTH_RUN_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0100000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_ULD ] = {
		0x8000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_MOD_CYC ] = {
		0x0000000000002000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000040000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FLOP ] = {
		0x0000000000000000ULL,
		0x0100000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FEST ] = {
		0x0000000000000000ULL,
		0x0600000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_LHS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000020000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_RESULT_SAT_1 ] = {
		0x0000000000000000ULL,
		0x0000000000800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_NO_ITAG_CYC ] = {
		0x0000000000000000ULL,
		0x0000001000000000ULL,
		0x0000400000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_NO_SCRATCH ] = {
		0x0000000000000000ULL,
		0x0000000000000014ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_0INST_FETCH ] = {
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000100000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_WT_BR_MPRED ] = {
		0x0000000800000000ULL,
		0x0000000000000000ULL,
		0x0004000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L1_PREF ] = {
		0x0001000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_FLOAT_MULTICYCLE ] = {
		0x0000000000000000ULL,
		0x0000000001800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L25_SHR_CYC ] = {
		0x0000000000020000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L3 ] = {
		0x0000000000010100ULL,
		0x000c000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000001ULL
	},
	[ POWER6_PME_PM_PMC2_OVERFLOW ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX0_LD_WRBACK ] = {
		0x0000000000000000ULL,
		0x0000000000400000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_DENORM ] = {
		0x0000000000000000ULL,
		0x0100000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FETCH_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000002000ULL,
		0x0000010000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_LDF ] = {
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0800000000000000ULL,
		0x0000000000000004ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_L2_CORR ] = {
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_REF_64K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000080ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_3_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000004000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FMA ] = {
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0400000000000000ULL,
		0x0000000000000002ULL
	},
	[ POWER6_PME_PM_INST_FROM_L35_MOD ] = {
		0x0000000000800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_CONV ] = {
		0x0000000000000000ULL,
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L25_MOD ] = {
		0x0000000000400000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L35_MOD ] = {
		0x0000000020000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_VMX_ST_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000800000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_FLOAT_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000100000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_L2_CORR ] = {
		0x0000000000000000ULL,
		0x0000000000000008ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_L2MISS ] = {
		0x0100000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FCONV ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000004ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FXMULT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000001ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FRSP ] = {
		0x0000000000000000ULL,
		0x4000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_16M ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000001000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_CASTOUT_SHR ] = {
		0x0000000000000000ULL,
		0x0000200000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_ONE_RUN_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000800000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_RMEM ] = {
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_BOTH_BUS ] = {
		0x0000000000000000ULL,
		0x0000000000000200ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FSQRT_FDIV ] = {
		0x0000000000000000ULL,
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_LD_REQ_INST ] = {
		0x0000000000000000ULL,
		0x0000004000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L35_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000010000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_PRED_CR ] = {
		0x0000000000000014ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_ULD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000020000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT ] = {
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_LHS_BOTH ] = {
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GXO_ADDR_CYC_BUSY ] = {
		0x0000000000000000ULL,
		0x0000000200000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_SRQ_EMPTY_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0008000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L3 ] = {
		0x0000000010000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX0_LD_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000600000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FXU_PIPELINED_MULT_DIV ] = {
		0x0000000000000000ULL,
		0x0000000000080000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_STF ] = {
		0x0000000000000000ULL,
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_ADD ] = {
		0x0000000000000000ULL,
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM_DP_CL_WR_GLOB ] = {
		0x0000000000000000ULL,
		0x0000000040000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU1_REJECT_ULD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000040000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ITLB_REF ] = {
		0x0200000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_L2MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000400ULL,
		0x0000008000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L35_SHR ] = {
		0x0000000000000300ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000001ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_RL2L3_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000008000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FPSCR ] = {
		0x0000000000000000ULL,
		0x0200000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L2 ] = {
		0x0000000000040080ULL,
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_XER ] = {
		0x0000004000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_NODE_PUMP ] = {
		0x0000000000000000ULL,
		0x0000000010000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_RESULT_SAT_0_1 ] = {
		0x0000000000000000ULL,
		0x0000000000800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LD_REF_L1 ] = {
		0x0000c00000000000ULL,
		0x0000000000000000ULL,
		0x2080000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_TLB_REF ] = {
		0x0200000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DC_PREF_OUT_OF_STREAMS ] = {
		0x0001000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FLUSH_FPU ] = {
		0x0002000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM1_DP_CL_WR_LOC ] = {
		0x0000000000000000ULL,
		0x0000000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_LD_HIT ] = {
		0x0000000000000000ULL,
		0x0000800000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_DCLAIM ] = {
		0x0000000000000000ULL,
		0x0000000008000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM_DP_CL_WR_LOC ] = {
		0x0000000000000000ULL,
		0x0000000040000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_MPRED_CR ] = {
		0x0000000000000024ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_EXTERN ] = {
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_MOD ] = {
		0x0000000000010400ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_RU_WQ ] = {
		0x0000084000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LD_MISS_L1 ] = {
		0x0000800000000000ULL,
		0x0000000000000000ULL,
		0x1080084000000020ULL,
		0x0000000000000006ULL
	},
	[ POWER6_PME_PM_DC_INV_L2 ] = {
		0x0000000000000000ULL,
		0x0000008000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_RMEM ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000080000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x4000000000000001ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FXU0_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000080000ULL,
		0x4000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_FPQ ] = {
		0x0000002000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GX_DMA_READ ] = {
		0x0000000000000000ULL,
		0x0000000800000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_PARTIAL_SECTOR ] = {
		0x0000000000000000ULL,
		0x0000000000000010ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_0INST_FETCH_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000100000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC5_OVERFLOW ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_LD_REQ ] = {
		0x0000000000000000ULL,
		0x0000800000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_0_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000010000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_RMEM ] = {
		0x0000000000004800ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000000400ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ST_REF_L1_BOTH ] = {
		0x0000400000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_PERMUTE_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000100000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_TAKEN ] = {
		0x0000000000000040ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_DMA ] = {
		0x0000000000000000ULL,
		0x0000000008000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GCT_EMPTY_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0008010000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_SINGLE ] = {
		0x0000000000000000ULL,
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_CASTOUT_SHR ] = {
		0x0000000000000000ULL,
		0x0000200000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L3SB_REF ] = {
		0x0000000000000000ULL,
		0x0008000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FRSP ] = {
		0x0000000000000000ULL,
		0x0400000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC4_SAVED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_DC_INV ] = {
		0x0000000000000000ULL,
		0x0000400000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GXI_ADDR_CYC_BUSY ] = {
		0x0000000000000000ULL,
		0x0000000400000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FMA ] = {
		0x0000000000000000ULL,
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_SLB_MISS ] = {
		0x0400000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_ST_GPS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000002000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_REF_4K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000080ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_CASTOUT_SHR ] = {
		0x0000000000000000ULL,
		0x0000008000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_STCX_CR ] = {
		0x0000084000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_ST_FOLDED ] = {
		0x0000000000000000ULL,
		0x0400000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L21 ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000800ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus3or4_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000020000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L35_MOD ] = {
		0x0000000000208300ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000001ULL
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_SHR ] = {
		0x0000000000008400ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GXI_DATA_CYC_BUSY ] = {
		0x0000000000000000ULL,
		0x0000000400000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_STEAL ] = {
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ST_FIN ] = {
		0x0100000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_CR_LOGICAL ] = {
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0300000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_SEL_T0 ] = {
		0x0000000000000000ULL,
		0x0000000000020000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_RELOAD_VALID ] = {
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_PREF_ST ] = {
		0x0000000000000000ULL,
		0x0002000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_STCX_FAIL ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000400000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_LHS ] = {
		0x0000000000000000ULL,
		0x0000000000000020ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_EXP_EQ ] = {
		0x0000000000000000ULL,
		0x0000000004000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_FP_FX_MULT ] = {
		0x0000010000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_LD_MISS_DATA ] = {
		0x0000000000000000ULL,
		0x0000002000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L35_MOD_CYC ] = {
		0x0000000000208000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FLUSH_FXU ] = {
		0x0002000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_1 ] = {
		0x0000000000000000ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_LMEM_CYC ] = {
		0x0000000000102000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_LSU_SOPS ] = {
		0x0000200000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_PTEG_2ND_HALF ] = {
		0x0000000200000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRESH_TIMEO ] = {
		0x0040000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_UST_BOTH ] = {
		0x4000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_FAST ] = {
		0x0800000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_THRD_PRIO ] = {
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_PREF_LD ] = {
		0x0000000000000000ULL,
		0x0002004000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FEST ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000004ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_RMEM ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000008000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LD_MISS_L1_CYC ] = {
		0x0000000000001000ULL,
		0x0000000000000000ULL,
		0x0000000000000020ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_MISS_4K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000040ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_COMPLETION ] = {
		0x0000110000000000ULL,
		0x0000000000000000ULL,
		0x0300000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_STALL_ST ] = {
		0x0000000000000000ULL,
		0x0060000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_DC_INV ] = {
		0x0000000000000000ULL,
		0x0000400000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L25_SHR ] = {
		0x0000000008000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_DL2L3_MOD ] = {
		0x0000000010000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_CMD_RETRIED ] = {
		0x0000000000000000ULL,
		0x0000000028000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_PRED_LSTACK ] = {
		0x0000000000000018ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GXO_DATA_CYC_BUSY ] = {
		0x0000000000000000ULL,
		0x0000000200000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_SUBNORM ] = {
		0x0000000000000000ULL,
		0x0000000004000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_OOO ] = {
		0x0000000000000000ULL,
		0x0020000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_ULD_BOTH ] = {
		0x8000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_ST_MISS ] = {
		0x0000000000000000ULL,
		0x0000080000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L25_MOD_CYC ] = {
		0x0000000000001000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_PTEG_1ST_HALF ] = {
		0x0000000200000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_MISS_16M ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000040ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GX_DMA_WRITE ] = {
		0x0000000000000000ULL,
		0x0000000800000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000008000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM1_DP_RQ_GLOB_LOC ] = {
		0x0000000000000000ULL,
		0x0000000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_LD_REQ_DATA ] = {
		0x0000000000000000ULL,
		0x0000020000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_LD_MISS_INST ] = {
		0x0000000000000000ULL,
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_L2MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000800000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_IFU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000400000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L3 ] = {
		0x0000000000800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FXU1_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000080000ULL,
		0x4000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_4_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000008000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L35_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000002000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_SET_MPRED ] = {
		0x0000000000000000ULL,
		0x0000000000000001ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_16G ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000020ULL
	},
	[ POWER6_PME_PM_FPU0_FXDIV ] = {
		0x0000000000000000ULL,
		0x0100000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU1_REJECT_UST ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000040000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_DIV_SQRT_OVERLAP ] = {
		0x0000000000000000ULL,
		0x0060000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L35_SHR ] = {
		0x0000000000800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU_REJECT_LHS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000010000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_LMQ_FULL_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000000400ULL,
		0x0000008000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_SYNC_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0061800000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM0_DP_RQ_LOC_GLOB ] = {
		0x0000000000000000ULL,
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_CASTOUT_MOD ] = {
		0x0000000000000000ULL,
		0x0000200000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0008008000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_MEM_DP ] = {
		0x0000000040000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_SLOW ] = {
		0x2000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L25_MOD ] = {
		0x0000000008000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_7_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000008000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000080000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ST_REQ_L2 ] = {
		0x0000000000000000ULL,
		0x0000010000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ST_REF_L1 ] = {
		0x0000c00000000000ULL,
		0x0000000000000000ULL,
		0x2000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_STALL_THRD ] = {
		0x0000000000000000ULL,
		0x0040000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_RUN_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000200000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_RUN_CYC ] = {
		0xffffffffffffffffULL,
		0xffffffffffffffffULL,
		0xffffffffffffffffULL,
		0x000000000000003fULL
	},
	[ POWER6_PME_PM_PTEG_FROM_RMEM ] = {
		0x0000000040000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_LDF ] = {
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_ST_MISS_L1 ] = {
		0x0000800000000000ULL,
		0x0000000000000000ULL,
		0x1080000000000000ULL,
		0x0000000000000004ULL
	},
	[ POWER6_PME_PM_INST_FROM_DL2L3_SHR ] = {
		0x0000000001000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_IC_INV ] = {
		0x0000000000000000ULL,
		0x0000400000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_ONE_RUN_CYC ] = {
		0x0020000000000002ULL,
		0x0000000000000000ULL,
		0x0001000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_LD_REQ_INST ] = {
		0x0000000000000000ULL,
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L25_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000800ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_XTHRD ] = {
		0x0000020000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_ST_REQ ] = {
		0x0000000000000000ULL,
		0x0001080000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L21 ] = {
		0x0000000004400000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L3MISS ] = {
		0x0000000004000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000008ULL
	},
	[ POWER6_PME_PM_L3SB_HIT ] = {
		0x0000000000000000ULL,
		0x0008000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_EE_OFF_EXT_INT ] = {
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_DL2L3_MOD ] = {
		0x0000000001000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC6_OVERFLOW ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FLOP ] = {
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FXU_BUSY ] = {
		0x0000000000000000ULL,
		0x0000000000040000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FLOP ] = {
		0x0000000000000000ULL,
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IC_RELOAD_SHR ] = {
		0x0004000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_TABLEWALK_CYC ] = {
		0x0000000200000000ULL,
		0x0000000000000000ULL,
		0x0000200000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_MOD_CYC ] = {
		0x0000000000010000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_5or6_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000010000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IBUF_FULL_CYC ] = {
		0x0001000000000000ULL,
		0x0000000000000000ULL,
		0x0000100000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_LD_REQ ] = {
		0x0000000000000000ULL,
		0x0000800000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX1_LD_WRBACK ] = {
		0x0000000000000000ULL,
		0x0000000000400000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_FPU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000800100000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_5_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000008000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_BACK2BACK ] = {
		0x0000000000000000ULL,
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_LMEM ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000004000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_LHS ] = {
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_SPR ] = {
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FREQ_DOWN ] = {
		0x0000000400000000ULL,
		0x0000000000000000ULL,
		0x0000020000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_ENC_BCD_DPD ] = {
		0x0000000000000000ULL,
		0x0000000004000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_GPR ] = {
		0x0000012000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_NCST ] = {
		0x0000000000000000ULL,
		0x0000000000000200ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_INST_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000080000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_RL2L3_SHR ] = {
		0x0000000001000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_DENORM ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000002ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_L3MISS ] = {
		0x0000000020000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_RUN_PURR ] = {
		0x0000000000000002ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_VMX0_LD_WRBACK ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000200000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_MISS ] = {
		0x0000000000000000ULL,
		0x0000104000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000008ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L3 ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000001000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU1_REJECT_LHS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000040000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_LD_MISS_INST ] = {
		0x0000000000000000ULL,
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PTEG_FROM_RL2L3_SHR ] = {
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_64K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000002000000000ULL,
		0x0000000000000020ULL
	},
	[ POWER6_PME_PM_LWSYNC ] = {
		0x0000200000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FXMULT ] = {
		0x0000000000000000ULL,
		0x2000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM0_DP_CL_WR_GLOB ] = {
		0x0000000000000000ULL,
		0x0000000080000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_PARTIAL_SECTOR ] = {
		0x0000000000000000ULL,
		0x0000000000000010ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_IMC_MATCH_CMPL ] = {
		0x0000000000000000ULL,
		0x0000001000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_THERMAL ] = {
		0x0000001400000000ULL,
		0x0000000000000000ULL,
		0x0004000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FRSP ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000005ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_INST_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000080000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000004000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DTLB_REF ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000200000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L25_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000010000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_LSU ] = {
		0x0000008000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_FSQRT_FDIV ] = {
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0400000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0020008000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_PTEG_SECONDARY ] = {
		0x0000000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FEST ] = {
		0x0000000000000000ULL,
		0x6000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SA_LD_HIT ] = {
		0x0000000000000000ULL,
		0x0000800000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_MEM_DP_CYC ] = {
		0x0000000000020000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_MPRED_CCACHE ] = {
		0x0000000000000028ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0002000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_SET_MPRED ] = {
		0x0000000000000000ULL,
		0x0000000000000001ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_2 ] = {
		0x0000000000000000ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_L2_CORR ] = {
		0x0000000000000000ULL,
		0x0000000000000008ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_DMEM ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000004000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM1_DP_RQ_LOC_GLOB ] = {
		0x0000000000000000ULL,
		0x0000000100000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus1or2_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000020000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_0_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000004000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FXU0_BUSY_FXU1_IDLE ] = {
		0x0000000000000000ULL,
		0x0000000000040000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_DERAT_MPRED ] = {
		0x0000000000000000ULL,
		0x0000000000000080ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_VMX1_LD_WRBACK ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000200000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_SHR_CYC ] = {
		0x0000000000001000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IERAT_MISS_16M ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_MEM_DP ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000004000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LARX_L1HIT ] = {
		0x0010000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2_ST_MISS_DATA ] = {
		0x0000000000000000ULL,
		0x0000002000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_ST_FOLDED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000004ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L35_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000002000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_MULT_GPR ] = {
		0x0000110000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_1FLOP ] = {
		0x0000000000000000ULL,
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IERAT_MISS_16G ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000100ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IC_PREF_WRITE ] = {
		0x0004000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus5or6_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000020000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FIN ] = {
		0x0000000000000000ULL,
		0x0200000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L2_CYC ] = {
		0x0000000000040000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_REF_16G ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000080ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_PRED ] = {
		0x0000000000000054ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX1_LD_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000600000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_CASTOUT_MOD ] = {
		0x0000000000000000ULL,
		0x0000200000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_DMEM ] = {
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L35_SHR_CYC ] = {
		0x0000000000200000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_NCLD ] = {
		0x0000000000000000ULL,
		0x0000000000000200ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_RETRY_NODE_PUMP ] = {
		0x0000000000000000ULL,
		0x0000000010000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX0_INST_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000200000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L25_MOD ] = {
		0x0000000000020200ULL,
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_ITLB_ISLB ] = {
		0x0000008000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000000400ULL,
		0x0040000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_CONC_RUN_INST ] = {
		0x0020000000000002ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L2 ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000004000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PURR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000008ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DERAT_MISS_64K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000040ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC2_REWIND ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L2 ] = {
		0x0000000004400000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_DISP ] = {
		0x0000000000140001ULL,
		0x0000000000001000ULL,
		0x2000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L25_SHR ] = {
		0x0000000000000200ULL,
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L1_DCACHE_RELOAD_VALID ] = {
		0x0000000000040000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_UST ] = {
		0x4000000000000000ULL,
		0x0000000000000002ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FAB_ADDR_COLLISION ] = {
		0x0000000000000000ULL,
		0x0000000020000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_FXU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000200000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_UST ] = {
		0x4000000000000000ULL,
		0x0000000000000002ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_PMC4_OVERFLOW ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L3 ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000020000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L2MISS ] = {
		0x0000000004400000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_ST_HIT ] = {
		0x0000000000000000ULL,
		0x0001000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_WT_IC_MISS_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0080040000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_DL2L3_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000400ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L35_MOD ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000010000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_FPSCR ] = {
		0x0000000000000000ULL,
		0x2000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_UST ] = {
		0x4000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_DERAT_MISS ] = {
		0x0000000000000000ULL,
		0x00000000000000a0ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_MEM_DP ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000020000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L2 ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000800ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU0_FSQRT_FDIV ] = {
		0x0000000000000000ULL,
		0x0080000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_FXU_SOPS ] = {
		0x0000040000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_FPU0_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000800100000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L2SB_LD_MISS_DATA ] = {
		0x0000000000000000ULL,
		0x0000020000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_SRQ_EMPTY_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000000001ULL,
		0x0010000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_1PLUS_PPC_DISP ] = {
		0x0000000000100000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_VMX_ST_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000001800000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L2MISS ] = {
		0x0000000000000080ULL,
		0x0000100000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_ULD ] = {
		0x8000000000000000ULL,
		0x0000000000000002ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_SUSPENDED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000010ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DFU_ADD_SHIFTED_BOTH ] = {
		0x0000000000000000ULL,
		0x0000000002000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU_REJECT_NO_SCRATCH ] = {
		0x2000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_STCX_FAIL ] = {
		0x0018000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU1_DENORM ] = {
		0x0000000000000000ULL,
		0x1000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GCT_NOSLOT_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0008010000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_SHR_CYC ] = {
		0x0000000000008000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L21 ] = {
		0x0000000000080080ULL,
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FPU_1FLOP ] = {
		0x0000000000000000ULL,
		0x8000000000000000ULL,
		0x0400000000000000ULL,
		0x0000000000000002ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT ] = {
		0x0000000000000000ULL,
		0x0000000000000044ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IC_REQ ] = {
		0x0004000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DFU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000200000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_NOT_LLA_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_INST_FROM_L1 ] = {
		0x0000000000000000ULL,
		0x0000001000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_VMX_COMPLEX_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000100000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BRU_FIN ] = {
		0x0000000000000040ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_REJECT_EXTERN ] = {
		0x0000000000000000ULL,
		0x0000000000000008ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_L21_CYC ] = {
		0x0000000000080000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GXI_CYC_BUSY ] = {
		0x0000000000000000ULL,
		0x0000000e00000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LD_MISS_L1 ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000200ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_L1_WRITE_CYC ] = {
		0x0000000000000000ULL,
		0x0000001000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LLA_CYC ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000080000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L2MISS ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000001000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GCT_FULL_COUNT ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000410000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MEM_DP_RQ_LOC_GLOB ] = {
		0x0000000000000000ULL,
		0x0000000040000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_SHR ] = {
		0x0000000000000400ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU_REJECT_UST ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000010000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_VMX_PERMUTE_ISSUED ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000100000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L21 ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000008000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_THRD_GRP_CMPL_BOTH_CYC ] = {
		0x0020000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_BR_MPRED ] = {
		0x0000000000000040ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LD_REQ_L2 ] = {
		0x0000000000000000ULL,
		0x0000010000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_FLUSH_ASYNC ] = {
		0x0002000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_HV_CYC ] = {
		0x0040000000000000ULL,
		0x0000000000000000ULL,
		0x0001000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU1_DERAT_MISS ] = {
		0x0000000000000000ULL,
		0x00000000000000a0ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_DPU_HELD_SMT ] = {
		0x0000001000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_LSU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000000001000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_RL2L3_SHR ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x000000000000c000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_LSU0_REJECT_STQ_FULL ] = {
		0x0000000000000000ULL,
		0x0000000000000040ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_4K ] = {
		0x0000000000000000ULL,
		0x0000000000000000ULL,
		0x0000001000000000ULL,
		0x0000000000000010ULL
	},
	[ POWER6_PME_PM_FPU_ISSUE_STALL_FPR ] = {
		0x0000000000000000ULL,
		0x0040000000000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_IFU_FIN ] = {
		0x0000000000000000ULL,
		0x0000000000080000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	},
	[ POWER6_PME_PM_GXO_CYC_BUSY ] = {
		0x0000000000000000ULL,
		0x0000000e00000000ULL,
		0x0000000000000000ULL,
		0x0000000000000000ULL
	}
};

static const pme_power_entry_t power6_pe[] = {
	[ POWER6_PME_PM_LSU_REJECT_STQ_FULL ] = {
		.pme_name = "PM_LSU_REJECT_STQ_FULL",
		.pme_code = 0x1a0030,
		.pme_short_desc = "LSU reject due to store queue full",
		.pme_long_desc = "LSU reject due to store queue full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_STQ_FULL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_STQ_FULL]
	},
	[ POWER6_PME_PM_DPU_HELD_FXU_MULTI ] = {
		.pme_name = "PM_DPU_HELD_FXU_MULTI",
		.pme_code = 0x210a6,
		.pme_short_desc = "DISP unit held due to FXU multicycle",
		.pme_long_desc = "DISP unit held due to FXU multicycle",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_FXU_MULTI],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_FXU_MULTI]
	},
	[ POWER6_PME_PM_VMX1_STALL ] = {
		.pme_name = "PM_VMX1_STALL",
		.pme_code = 0xb008c,
		.pme_short_desc = "VMX1 stall",
		.pme_long_desc = "VMX1 stall",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX1_STALL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX1_STALL]
	},
	[ POWER6_PME_PM_PMC2_SAVED ] = {
		.pme_name = "PM_PMC2_SAVED",
		.pme_code = 0x100022,
		.pme_short_desc = "PMC2 rewind value saved",
		.pme_long_desc = "PMC2 rewind value saved",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC2_SAVED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC2_SAVED]
	},
	[ POWER6_PME_PM_L2SB_IC_INV ] = {
		.pme_name = "PM_L2SB_IC_INV",
		.pme_code = 0x5068c,
		.pme_short_desc = "L2 slice B I cache invalidate",
		.pme_long_desc = "L2 slice B I cache invalidate",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_IC_INV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_IC_INV]
	},
	[ POWER6_PME_PM_IERAT_MISS_64K ] = {
		.pme_name = "PM_IERAT_MISS_64K",
		.pme_code = 0x392076,
		.pme_short_desc = "IERAT misses for 64K page",
		.pme_long_desc = "IERAT misses for 64K page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IERAT_MISS_64K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IERAT_MISS_64K]
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_3or4_CYC ] = {
		.pme_name = "PM_THRD_PRIO_DIFF_3or4_CYC",
		.pme_code = 0x323040,
		.pme_short_desc = "Cycles thread priority difference is 3 or 4",
		.pme_long_desc = "Cycles thread priority difference is 3 or 4",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_DIFF_3or4_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_DIFF_3or4_CYC]
	},
	[ POWER6_PME_PM_LD_REF_L1_BOTH ] = {
		.pme_name = "PM_LD_REF_L1_BOTH",
		.pme_code = 0x180036,
		.pme_short_desc = "Both units L1 D cache load reference",
		.pme_long_desc = "Both units L1 D cache load reference",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LD_REF_L1_BOTH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LD_REF_L1_BOTH]
	},
	[ POWER6_PME_PM_FPU1_FCONV ] = {
		.pme_name = "PM_FPU1_FCONV",
		.pme_code = 0xd10a8,
		.pme_short_desc = "FPU1 executed FCONV instruction",
		.pme_long_desc = "FPU1 executed FCONV instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FCONV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FCONV]
	},
	[ POWER6_PME_PM_IBUF_FULL_COUNT ] = {
		.pme_name = "PM_IBUF_FULL_COUNT",
		.pme_code = 0x40085,
		.pme_short_desc = "Periods instruction buffer full",
		.pme_long_desc = "Periods instruction buffer full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IBUF_FULL_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IBUF_FULL_COUNT]
	},
	[ POWER6_PME_PM_MRK_LSU_DERAT_MISS ] = {
		.pme_name = "PM_MRK_LSU_DERAT_MISS",
		.pme_code = 0x400012,
		.pme_short_desc = "Marked DERAT miss",
		.pme_long_desc = "Marked DERAT miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU_DERAT_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU_DERAT_MISS]
	},
	[ POWER6_PME_PM_MRK_ST_CMPL ] = {
		.pme_name = "PM_MRK_ST_CMPL",
		.pme_code = 0x100006,
		.pme_short_desc = "Marked store instruction completed",
		.pme_long_desc = "A sampled store has completed (data home)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_ST_CMPL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_ST_CMPL]
	},
	[ POWER6_PME_PM_L2_CASTOUT_MOD ] = {
		.pme_name = "PM_L2_CASTOUT_MOD",
		.pme_code = 0x150630,
		.pme_short_desc = "L2 castouts - Modified (M",
		.pme_long_desc = " Mu",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_CASTOUT_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_CASTOUT_MOD]
	},
	[ POWER6_PME_PM_FPU1_ST_FOLDED ] = {
		.pme_name = "PM_FPU1_ST_FOLDED",
		.pme_code = 0xd10ac,
		.pme_short_desc = "FPU1 folded store",
		.pme_long_desc = "FPU1 folded store",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_ST_FOLDED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_ST_FOLDED]
	},
	[ POWER6_PME_PM_MRK_INST_TIMEO ] = {
		.pme_name = "PM_MRK_INST_TIMEO",
		.pme_code = 0x40003e,
		.pme_short_desc = "Marked Instruction finish timeout ",
		.pme_long_desc = "Marked Instruction finish timeout ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_INST_TIMEO],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_INST_TIMEO]
	},
	[ POWER6_PME_PM_DPU_WT ] = {
		.pme_name = "PM_DPU_WT",
		.pme_code = 0x300004,
		.pme_short_desc = "Cycles DISP unit is stalled waiting for instructions",
		.pme_long_desc = "Cycles DISP unit is stalled waiting for instructions",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_WT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_WT]
	},
	[ POWER6_PME_PM_DPU_HELD_RESTART ] = {
		.pme_name = "PM_DPU_HELD_RESTART",
		.pme_code = 0x30086,
		.pme_short_desc = "DISP unit held after restart coming",
		.pme_long_desc = "DISP unit held after restart coming",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_RESTART],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_RESTART]
	},
	[ POWER6_PME_PM_IERAT_MISS ] = {
		.pme_name = "PM_IERAT_MISS",
		.pme_code = 0x420ce,
		.pme_short_desc = "IERAT miss count",
		.pme_long_desc = "IERAT miss count",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IERAT_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IERAT_MISS]
	},
	[ POWER6_PME_PM_FPU_SINGLE ] = {
		.pme_name = "PM_FPU_SINGLE",
		.pme_code = 0x4c1030,
		.pme_short_desc = "FPU executed single precision instruction",
		.pme_long_desc = "FPU is executing single precision instruction. Combined Unit 0 + Unit 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_SINGLE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_SINGLE]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_LMEM ] = {
		.pme_name = "PM_MRK_PTEG_FROM_LMEM",
		.pme_code = 0x412042,
		.pme_short_desc = "Marked PTEG loaded from local memory",
		.pme_long_desc = "Marked PTEG loaded from local memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_LMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_LMEM]
	},
	[ POWER6_PME_PM_HV_COUNT ] = {
		.pme_name = "PM_HV_COUNT",
		.pme_code = 0x200017,
		.pme_short_desc = "Hypervisor Periods",
		.pme_long_desc = "Periods when the processor is executing in Hypervisor (MSR[HV] = 1 and MSR[PR]=0)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_HV_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_HV_COUNT]
	},
	[ POWER6_PME_PM_L2SA_ST_HIT ] = {
		.pme_name = "PM_L2SA_ST_HIT",
		.pme_code = 0x50786,
		.pme_short_desc = "L2 slice A store hits",
		.pme_long_desc = "A store request made from the core hit in the L2 directory.  This event is provided on each of the three L2 slices A,B, and C.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_ST_HIT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_ST_HIT]
	},
	[ POWER6_PME_PM_L2_LD_MISS_INST ] = {
		.pme_name = "PM_L2_LD_MISS_INST",
		.pme_code = 0x250530,
		.pme_short_desc = "L2 instruction load misses",
		.pme_long_desc = "L2 instruction load misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_LD_MISS_INST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_LD_MISS_INST]
	},
	[ POWER6_PME_PM_EXT_INT ] = {
		.pme_name = "PM_EXT_INT",
		.pme_code = 0x2000f8,
		.pme_short_desc = "External interrupts",
		.pme_long_desc = "An external interrupt occurred",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_EXT_INT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_EXT_INT]
	},
	[ POWER6_PME_PM_LSU1_LDF ] = {
		.pme_name = "PM_LSU1_LDF",
		.pme_code = 0x8008c,
		.pme_short_desc = "LSU1 executed Floating Point load instruction",
		.pme_long_desc = "A floating point load was executed from LSU unit 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_LDF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_LDF]
	},
	[ POWER6_PME_PM_FAB_CMD_ISSUED ] = {
		.pme_name = "PM_FAB_CMD_ISSUED",
		.pme_code = 0x150130,
		.pme_short_desc = "Fabric command issued",
		.pme_long_desc = "Fabric command issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_CMD_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_CMD_ISSUED]
	},
	[ POWER6_PME_PM_PTEG_FROM_L21 ] = {
		.pme_name = "PM_PTEG_FROM_L21",
		.pme_code = 0x213048,
		.pme_short_desc = "PTEG loaded from private L2 other core",
		.pme_long_desc = "PTEG loaded from private L2 other core",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L21],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L21]
	},
	[ POWER6_PME_PM_L2SA_MISS ] = {
		.pme_name = "PM_L2SA_MISS",
		.pme_code = 0x50584,
		.pme_short_desc = "L2 slice A misses",
		.pme_long_desc = "L2 slice A misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_MISS]
	},
	[ POWER6_PME_PM_PTEG_FROM_RL2L3_MOD ] = {
		.pme_name = "PM_PTEG_FROM_RL2L3_MOD",
		.pme_code = 0x11304c,
		.pme_short_desc = "PTEG loaded from remote L2 or L3 modified",
		.pme_long_desc = "PTEG loaded from remote L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_RL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_RL2L3_MOD]
	},
	[ POWER6_PME_PM_DPU_WT_COUNT ] = {
		.pme_name = "PM_DPU_WT_COUNT",
		.pme_code = 0x300005,
		.pme_short_desc = "Periods DISP unit is stalled waiting for instructions",
		.pme_long_desc = "Periods DISP unit is stalled waiting for instructions",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_WT_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_WT_COUNT]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L25_MOD ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L25_MOD",
		.pme_code = 0x312046,
		.pme_short_desc = "Marked PTEG loaded from L2.5 modified",
		.pme_long_desc = "Marked PTEG loaded from L2.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L25_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L25_MOD]
	},
	[ POWER6_PME_PM_LD_HIT_L2 ] = {
		.pme_name = "PM_LD_HIT_L2",
		.pme_code = 0x250730,
		.pme_short_desc = "L2 D cache load hits",
		.pme_long_desc = "L2 D cache load hits",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LD_HIT_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LD_HIT_L2]
	},
	[ POWER6_PME_PM_PTEG_FROM_DL2L3_SHR ] = {
		.pme_name = "PM_PTEG_FROM_DL2L3_SHR",
		.pme_code = 0x31304c,
		.pme_short_desc = "PTEG loaded from distant L2 or L3 shared",
		.pme_long_desc = "PTEG loaded from distant L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_DL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_DL2L3_SHR]
	},
	[ POWER6_PME_PM_MEM_DP_RQ_GLOB_LOC ] = {
		.pme_name = "PM_MEM_DP_RQ_GLOB_LOC",
		.pme_code = 0x150230,
		.pme_short_desc = "Memory read queue marking cache line double pump state from global to local",
		.pme_long_desc = "Memory read queue marking cache line double pump state from global to local",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM_DP_RQ_GLOB_LOC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM_DP_RQ_GLOB_LOC]
	},
	[ POWER6_PME_PM_L3SA_MISS ] = {
		.pme_name = "PM_L3SA_MISS",
		.pme_code = 0x50084,
		.pme_short_desc = "L3 slice A misses",
		.pme_long_desc = "L3 slice A misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L3SA_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L3SA_MISS]
	},
	[ POWER6_PME_PM_NO_ITAG_COUNT ] = {
		.pme_name = "PM_NO_ITAG_COUNT",
		.pme_code = 0x40089,
		.pme_short_desc = "Periods no ITAG available",
		.pme_long_desc = "Periods no ITAG available",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_NO_ITAG_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_NO_ITAG_COUNT]
	},
	[ POWER6_PME_PM_DSLB_MISS ] = {
		.pme_name = "PM_DSLB_MISS",
		.pme_code = 0x830e8,
		.pme_short_desc = "Data SLB misses",
		.pme_long_desc = "A SLB miss for a data request occurred. SLB misses trap to the operating system to resolve",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DSLB_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DSLB_MISS]
	},
	[ POWER6_PME_PM_LSU_FLUSH_ALIGN ] = {
		.pme_name = "PM_LSU_FLUSH_ALIGN",
		.pme_code = 0x220cc,
		.pme_short_desc = "Flush caused by alignement exception",
		.pme_long_desc = "Flush caused by alignement exception",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_FLUSH_ALIGN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_FLUSH_ALIGN]
	},
	[ POWER6_PME_PM_DPU_HELD_FPU_CR ] = {
		.pme_name = "PM_DPU_HELD_FPU_CR",
		.pme_code = 0x210a0,
		.pme_short_desc = "DISP unit held due to FPU updating CR",
		.pme_long_desc = "DISP unit held due to FPU updating CR",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_FPU_CR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_FPU_CR]
	},
	[ POWER6_PME_PM_PTEG_FROM_L2MISS ] = {
		.pme_name = "PM_PTEG_FROM_L2MISS",
		.pme_code = 0x113028,
		.pme_short_desc = "PTEG loaded from L2 miss",
		.pme_long_desc = "PTEG loaded from L2 miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L2MISS]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_DMEM ] = {
		.pme_name = "PM_MRK_DATA_FROM_DMEM",
		.pme_code = 0x20304a,
		.pme_short_desc = "Marked data loaded from distant memory",
		.pme_long_desc = "Marked data loaded from distant memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_DMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_DMEM]
	},
	[ POWER6_PME_PM_PTEG_FROM_LMEM ] = {
		.pme_name = "PM_PTEG_FROM_LMEM",
		.pme_code = 0x41304a,
		.pme_short_desc = "PTEG loaded from local memory",
		.pme_long_desc = "PTEG loaded from local memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_LMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_LMEM]
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_64K ] = {
		.pme_name = "PM_MRK_DERAT_REF_64K",
		.pme_code = 0x182044,
		.pme_short_desc = "Marked DERAT reference for 64K page",
		.pme_long_desc = "Marked DERAT reference for 64K page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_REF_64K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_REF_64K]
	},
	[ POWER6_PME_PM_L2SA_LD_REQ_INST ] = {
		.pme_name = "PM_L2SA_LD_REQ_INST",
		.pme_code = 0x50580,
		.pme_short_desc = "L2 slice A instruction load requests",
		.pme_long_desc = "L2 slice A instruction load requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_LD_REQ_INST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_LD_REQ_INST]
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_16M ] = {
		.pme_name = "PM_MRK_DERAT_MISS_16M",
		.pme_code = 0x392044,
		.pme_short_desc = "Marked DERAT misses for 16M page",
		.pme_long_desc = "A marked data request (load or store) missed the ERAT for 16M page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_MISS_16M],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_MISS_16M]
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_MOD ] = {
		.pme_name = "PM_DATA_FROM_DL2L3_MOD",
		.pme_code = 0x40005c,
		.pme_short_desc = "Data loaded from distant L2 or L3 modified",
		.pme_long_desc = "Data loaded from distant L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_DL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_DL2L3_MOD]
	},
	[ POWER6_PME_PM_FPU0_FXMULT ] = {
		.pme_name = "PM_FPU0_FXMULT",
		.pme_code = 0xd0086,
		.pme_short_desc = "FPU0 executed fixed point multiplication",
		.pme_long_desc = "FPU0 executed fixed point multiplication",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FXMULT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FXMULT]
	},
	[ POWER6_PME_PM_L3SB_MISS ] = {
		.pme_name = "PM_L3SB_MISS",
		.pme_code = 0x5008c,
		.pme_short_desc = "L3 slice B misses",
		.pme_long_desc = "L3 slice B misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L3SB_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L3SB_MISS]
	},
	[ POWER6_PME_PM_STCX_CANCEL ] = {
		.pme_name = "PM_STCX_CANCEL",
		.pme_code = 0x830ec,
		.pme_short_desc = "stcx cancel by core",
		.pme_long_desc = "stcx cancel by core",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_STCX_CANCEL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_STCX_CANCEL]
	},
	[ POWER6_PME_PM_L2SA_LD_MISS_DATA ] = {
		.pme_name = "PM_L2SA_LD_MISS_DATA",
		.pme_code = 0x50482,
		.pme_short_desc = "L2 slice A data load misses",
		.pme_long_desc = "L2 slice A data load misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_LD_MISS_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_LD_MISS_DATA]
	},
	[ POWER6_PME_PM_IC_INV_L2 ] = {
		.pme_name = "PM_IC_INV_L2",
		.pme_code = 0x250632,
		.pme_short_desc = "L1 I cache entries invalidated from L2",
		.pme_long_desc = "L1 I cache entries invalidated from L2",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IC_INV_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IC_INV_L2]
	},
	[ POWER6_PME_PM_DPU_HELD ] = {
		.pme_name = "PM_DPU_HELD",
		.pme_code = 0x200004,
		.pme_short_desc = "DISP unit held",
		.pme_long_desc = "DISP unit held",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD]
	},
	[ POWER6_PME_PM_PMC1_OVERFLOW ] = {
		.pme_name = "PM_PMC1_OVERFLOW",
		.pme_code = 0x200014,
		.pme_short_desc = "PMC1 Overflow",
		.pme_long_desc = "PMC1 Overflow",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC1_OVERFLOW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC1_OVERFLOW]
	},
	[ POWER6_PME_PM_THRD_PRIO_6_CYC ] = {
		.pme_name = "PM_THRD_PRIO_6_CYC",
		.pme_code = 0x222046,
		.pme_short_desc = "Cycles thread running at priority level 6",
		.pme_long_desc = "Cycles thread running at priority level 6",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_6_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_6_CYC]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L3MISS ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L3MISS",
		.pme_code = 0x312054,
		.pme_short_desc = "Marked PTEG loaded from L3 miss",
		.pme_long_desc = "Marked PTEG loaded from L3 miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L3MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L3MISS]
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_UST ] = {
		.pme_name = "PM_MRK_LSU0_REJECT_UST",
		.pme_code = 0x930e2,
		.pme_short_desc = "LSU0 marked unaligned store reject",
		.pme_long_desc = "LSU0 marked unaligned store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU0_REJECT_UST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU0_REJECT_UST]
	},
	[ POWER6_PME_PM_MRK_INST_DISP ] = {
		.pme_name = "PM_MRK_INST_DISP",
		.pme_code = 0x10001a,
		.pme_short_desc = "Marked instruction dispatched",
		.pme_long_desc = "Marked instruction dispatched",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_INST_DISP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_INST_DISP]
	},
	[ POWER6_PME_PM_LARX ] = {
		.pme_name = "PM_LARX",
		.pme_code = 0x830ea,
		.pme_short_desc = "Larx executed",
		.pme_long_desc = "Larx executed",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LARX],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LARX]
	},
	[ POWER6_PME_PM_INST_CMPL ] = {
		.pme_name = "PM_INST_CMPL",
		.pme_code = 0x2,
		.pme_short_desc = "Instructions completed",
		.pme_long_desc = "Number of PPC instructions completed. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_CMPL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_CMPL]
	},
	[ POWER6_PME_PM_FXU_IDLE ] = {
		.pme_name = "PM_FXU_IDLE",
		.pme_code = 0x100050,
		.pme_short_desc = "FXU idle",
		.pme_long_desc = "FXU0 and FXU1 are both idle",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FXU_IDLE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FXU_IDLE]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_DL2L3_MOD ] = {
		.pme_name = "PM_MRK_DATA_FROM_DL2L3_MOD",
		.pme_code = 0x40304c,
		.pme_short_desc = "Marked data loaded from distant L2 or L3 modified",
		.pme_long_desc = "Marked data loaded from distant L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_DL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_DL2L3_MOD]
	},
	[ POWER6_PME_PM_L2_LD_REQ_DATA ] = {
		.pme_name = "PM_L2_LD_REQ_DATA",
		.pme_code = 0x150430,
		.pme_short_desc = "L2 data load requests",
		.pme_long_desc = "L2 data load requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_LD_REQ_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_LD_REQ_DATA]
	},
	[ POWER6_PME_PM_LSU_DERAT_MISS_CYC ] = {
		.pme_name = "PM_LSU_DERAT_MISS_CYC",
		.pme_code = 0x1000fc,
		.pme_short_desc = "DERAT miss latency",
		.pme_long_desc = "DERAT miss latency",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_DERAT_MISS_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_DERAT_MISS_CYC]
	},
	[ POWER6_PME_PM_DPU_HELD_POWER_COUNT ] = {
		.pme_name = "PM_DPU_HELD_POWER_COUNT",
		.pme_code = 0x20003d,
		.pme_short_desc = "Periods DISP unit held due to Power Management",
		.pme_long_desc = "Periods DISP unit held due to Power Management",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_POWER_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_POWER_COUNT]
	},
	[ POWER6_PME_PM_INST_FROM_RL2L3_MOD ] = {
		.pme_name = "PM_INST_FROM_RL2L3_MOD",
		.pme_code = 0x142044,
		.pme_short_desc = "Instruction fetched from remote L2 or L3 modified",
		.pme_long_desc = "Instruction fetched from remote L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_RL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_RL2L3_MOD]
	},
	[ POWER6_PME_PM_DATA_FROM_DMEM_CYC ] = {
		.pme_name = "PM_DATA_FROM_DMEM_CYC",
		.pme_code = 0x20002e,
		.pme_short_desc = "Load latency from distant memory",
		.pme_long_desc = "Load latency from distant memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_DMEM_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_DMEM_CYC]
	},
	[ POWER6_PME_PM_DATA_FROM_DMEM ] = {
		.pme_name = "PM_DATA_FROM_DMEM",
		.pme_code = 0x20005e,
		.pme_short_desc = "Data loaded from distant memory",
		.pme_long_desc = "Data loaded from distant memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_DMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_DMEM]
	},
	[ POWER6_PME_PM_LSU_REJECT_PARTIAL_SECTOR ] = {
		.pme_name = "PM_LSU_REJECT_PARTIAL_SECTOR",
		.pme_code = 0x1a0032,
		.pme_short_desc = "LSU reject due to partial sector valid",
		.pme_long_desc = "LSU reject due to partial sector valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_PARTIAL_SECTOR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_PARTIAL_SECTOR]
	},
	[ POWER6_PME_PM_LSU_REJECT_DERAT_MPRED ] = {
		.pme_name = "PM_LSU_REJECT_DERAT_MPRED",
		.pme_code = 0x2a0030,
		.pme_short_desc = "LSU reject due to mispredicted DERAT",
		.pme_long_desc = "LSU reject due to mispredicted DERAT",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_DERAT_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_DERAT_MPRED]
	},
	[ POWER6_PME_PM_LSU1_REJECT_ULD ] = {
		.pme_name = "PM_LSU1_REJECT_ULD",
		.pme_code = 0x90088,
		.pme_short_desc = "LSU1 unaligned load reject",
		.pme_long_desc = "LSU1 unaligned load reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_ULD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_ULD]
	},
	[ POWER6_PME_PM_DATA_FROM_L3_CYC ] = {
		.pme_name = "PM_DATA_FROM_L3_CYC",
		.pme_code = 0x200022,
		.pme_short_desc = "Load latency from L3",
		.pme_long_desc = "Load latency from L3",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L3_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L3_CYC]
	},
	[ POWER6_PME_PM_FXU1_BUSY_FXU0_IDLE ] = {
		.pme_name = "PM_FXU1_BUSY_FXU0_IDLE",
		.pme_code = 0x400050,
		.pme_short_desc = "FXU1 busy FXU0 idle",
		.pme_long_desc = "FXU0 was idle while FXU1 was busy",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FXU1_BUSY_FXU0_IDLE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FXU1_BUSY_FXU0_IDLE]
	},
	[ POWER6_PME_PM_INST_FROM_MEM_DP ] = {
		.pme_name = "PM_INST_FROM_MEM_DP",
		.pme_code = 0x142042,
		.pme_short_desc = "Instruction fetched from double pump memory",
		.pme_long_desc = "Instruction fetched from double pump memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_MEM_DP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_MEM_DP]
	},
	[ POWER6_PME_PM_LSU_FLUSH_DSI ] = {
		.pme_name = "PM_LSU_FLUSH_DSI",
		.pme_code = 0x220ce,
		.pme_short_desc = "Flush caused by DSI",
		.pme_long_desc = "Flush caused by DSI",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_FLUSH_DSI],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_FLUSH_DSI]
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_16G ] = {
		.pme_name = "PM_MRK_DERAT_REF_16G",
		.pme_code = 0x482044,
		.pme_short_desc = "Marked DERAT reference for 16G page",
		.pme_long_desc = "Marked DERAT reference for 16G page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_REF_16G],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_REF_16G]
	},
	[ POWER6_PME_PM_LSU_LDF_BOTH ] = {
		.pme_name = "PM_LSU_LDF_BOTH",
		.pme_code = 0x180038,
		.pme_short_desc = "Both LSU units executed Floating Point load instruction",
		.pme_long_desc = "Both LSU units executed Floating Point load instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_LDF_BOTH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_LDF_BOTH]
	},
	[ POWER6_PME_PM_FPU1_1FLOP ] = {
		.pme_name = "PM_FPU1_1FLOP",
		.pme_code = 0xc0088,
		.pme_short_desc = "FPU1 executed add",
		.pme_long_desc = " mult",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_1FLOP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_1FLOP]
	},
	[ POWER6_PME_PM_DATA_FROM_RMEM_CYC ] = {
		.pme_name = "PM_DATA_FROM_RMEM_CYC",
		.pme_code = 0x40002c,
		.pme_short_desc = "Load latency from remote memory",
		.pme_long_desc = "Load latency from remote memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_RMEM_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_RMEM_CYC]
	},
	[ POWER6_PME_PM_INST_PTEG_SECONDARY ] = {
		.pme_name = "PM_INST_PTEG_SECONDARY",
		.pme_code = 0x910ac,
		.pme_short_desc = "Instruction table walk matched in secondary PTEG",
		.pme_long_desc = "Instruction table walk matched in secondary PTEG",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_PTEG_SECONDARY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_PTEG_SECONDARY]
	},
	[ POWER6_PME_PM_L1_ICACHE_MISS ] = {
		.pme_name = "PM_L1_ICACHE_MISS",
		.pme_code = 0x100056,
		.pme_short_desc = "L1 I cache miss count",
		.pme_long_desc = "L1 I cache miss count",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L1_ICACHE_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L1_ICACHE_MISS]
	},
	[ POWER6_PME_PM_INST_DISP_LLA ] = {
		.pme_name = "PM_INST_DISP_LLA",
		.pme_code = 0x310a2,
		.pme_short_desc = "Instruction dispatched under load look ahead",
		.pme_long_desc = "Instruction dispatched under load look ahead",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_DISP_LLA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_DISP_LLA]
	},
	[ POWER6_PME_PM_THRD_BOTH_RUN_CYC ] = {
		.pme_name = "PM_THRD_BOTH_RUN_CYC",
		.pme_code = 0x400004,
		.pme_short_desc = "Both threads in run cycles",
		.pme_long_desc = "Both threads in run cycles",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_BOTH_RUN_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_BOTH_RUN_CYC]
	},
	[ POWER6_PME_PM_LSU_ST_CHAINED ] = {
		.pme_name = "PM_LSU_ST_CHAINED",
		.pme_code = 0x820ce,
		.pme_short_desc = "number of chained stores",
		.pme_long_desc = "number of chained stores",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_ST_CHAINED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_ST_CHAINED]
	},
	[ POWER6_PME_PM_FPU1_FXDIV ] = {
		.pme_name = "PM_FPU1_FXDIV",
		.pme_code = 0xc10a8,
		.pme_short_desc = "FPU1 executed fixed point division",
		.pme_long_desc = "FPU1 executed fixed point division",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FXDIV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FXDIV]
	},
	[ POWER6_PME_PM_FREQ_UP ] = {
		.pme_name = "PM_FREQ_UP",
		.pme_code = 0x40003c,
		.pme_short_desc = "Frequency is being slewed up due to Power Management",
		.pme_long_desc = "Frequency is being slewed up due to Power Management",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FREQ_UP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FREQ_UP]
	},
	[ POWER6_PME_PM_FAB_RETRY_SYS_PUMP ] = {
		.pme_name = "PM_FAB_RETRY_SYS_PUMP",
		.pme_code = 0x50182,
		.pme_short_desc = "Retry of a system pump",
		.pme_long_desc = " locally mastered  ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_RETRY_SYS_PUMP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_RETRY_SYS_PUMP]
	},
	[ POWER6_PME_PM_DATA_FROM_LMEM ] = {
		.pme_name = "PM_DATA_FROM_LMEM",
		.pme_code = 0x40005e,
		.pme_short_desc = "Data loaded from local memory",
		.pme_long_desc = "Data loaded from local memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_LMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_LMEM]
	},
	[ POWER6_PME_PM_PMC3_OVERFLOW ] = {
		.pme_name = "PM_PMC3_OVERFLOW",
		.pme_code = 0x400014,
		.pme_short_desc = "PMC3 Overflow",
		.pme_long_desc = "PMC3 Overflow",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC3_OVERFLOW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC3_OVERFLOW]
	},
	[ POWER6_PME_PM_LSU0_REJECT_SET_MPRED ] = {
		.pme_name = "PM_LSU0_REJECT_SET_MPRED",
		.pme_code = 0xa0084,
		.pme_short_desc = "LSU0 reject due to mispredicted set",
		.pme_long_desc = "LSU0 reject due to mispredicted set",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_SET_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_SET_MPRED]
	},
	[ POWER6_PME_PM_LSU0_REJECT_DERAT_MPRED ] = {
		.pme_name = "PM_LSU0_REJECT_DERAT_MPRED",
		.pme_code = 0xa0082,
		.pme_short_desc = "LSU0 reject due to mispredicted DERAT",
		.pme_long_desc = "LSU0 reject due to mispredicted DERAT",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_DERAT_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_DERAT_MPRED]
	},
	[ POWER6_PME_PM_LSU1_REJECT_STQ_FULL ] = {
		.pme_name = "PM_LSU1_REJECT_STQ_FULL",
		.pme_code = 0xa0088,
		.pme_short_desc = "LSU1 reject due to store queue full",
		.pme_long_desc = "LSU1 reject due to store queue full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_STQ_FULL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_STQ_FULL]
	},
	[ POWER6_PME_PM_MRK_BR_MPRED ] = {
		.pme_name = "PM_MRK_BR_MPRED",
		.pme_code = 0x300052,
		.pme_short_desc = "Marked branch mispredicted",
		.pme_long_desc = "Marked branch mispredicted",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_BR_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_BR_MPRED]
	},
	[ POWER6_PME_PM_L2SA_ST_MISS ] = {
		.pme_name = "PM_L2SA_ST_MISS",
		.pme_code = 0x50486,
		.pme_short_desc = "L2 slice A store misses",
		.pme_long_desc = "L2 slice A store misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_ST_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_ST_MISS]
	},
	[ POWER6_PME_PM_LSU0_REJECT_EXTERN ] = {
		.pme_name = "PM_LSU0_REJECT_EXTERN",
		.pme_code = 0xa10a4,
		.pme_short_desc = "LSU0 external reject request ",
		.pme_long_desc = "LSU0 external reject request ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_EXTERN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_EXTERN]
	},
	[ POWER6_PME_PM_MRK_BR_TAKEN ] = {
		.pme_name = "PM_MRK_BR_TAKEN",
		.pme_code = 0x100052,
		.pme_short_desc = "Marked branch taken",
		.pme_long_desc = "Marked branch taken",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_BR_TAKEN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_BR_TAKEN]
	},
	[ POWER6_PME_PM_ISLB_MISS ] = {
		.pme_name = "PM_ISLB_MISS",
		.pme_code = 0x830e0,
		.pme_short_desc = "Instruction SLB misses",
		.pme_long_desc = "A SLB miss for an instruction fetch as occurred",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ISLB_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ISLB_MISS]
	},
	[ POWER6_PME_PM_CYC ] = {
		.pme_name = "PM_CYC",
		.pme_code = 0x1e,
		.pme_short_desc = "Processor cycles",
		.pme_long_desc = "Processor cycles",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_CYC]
	},
	[ POWER6_PME_PM_FPU_FXDIV ] = {
		.pme_name = "PM_FPU_FXDIV",
		.pme_code = 0x1c1034,
		.pme_short_desc = "FPU executed fixed point division",
		.pme_long_desc = "FPU executed fixed point division",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FXDIV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FXDIV]
	},
	[ POWER6_PME_PM_DPU_HELD_LLA_END ] = {
		.pme_name = "PM_DPU_HELD_LLA_END",
		.pme_code = 0x30084,
		.pme_short_desc = "DISP unit held due to load look ahead ended",
		.pme_long_desc = "DISP unit held due to load look ahead ended",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_LLA_END],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_LLA_END]
	},
	[ POWER6_PME_PM_MEM0_DP_CL_WR_LOC ] = {
		.pme_name = "PM_MEM0_DP_CL_WR_LOC",
		.pme_code = 0x50286,
		.pme_short_desc = "cacheline write setting dp to local side 0",
		.pme_long_desc = "cacheline write setting dp to local side 0",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM0_DP_CL_WR_LOC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM0_DP_CL_WR_LOC]
	},
	[ POWER6_PME_PM_MRK_LSU_REJECT_ULD ] = {
		.pme_name = "PM_MRK_LSU_REJECT_ULD",
		.pme_code = 0x193034,
		.pme_short_desc = "Marked unaligned load reject",
		.pme_long_desc = "Marked unaligned load reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU_REJECT_ULD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU_REJECT_ULD]
	},
	[ POWER6_PME_PM_1PLUS_PPC_CMPL ] = {
		.pme_name = "PM_1PLUS_PPC_CMPL",
		.pme_code = 0x100004,
		.pme_short_desc = "One or more PPC instruction completed",
		.pme_long_desc = "A group containing at least one PPC instruction completed. For microcoded instructions that span multiple groups, this will only occur once.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_1PLUS_PPC_CMPL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_1PLUS_PPC_CMPL]
	},
	[ POWER6_PME_PM_PTEG_FROM_DMEM ] = {
		.pme_name = "PM_PTEG_FROM_DMEM",
		.pme_code = 0x21304a,
		.pme_short_desc = "PTEG loaded from distant memory",
		.pme_long_desc = "PTEG loaded from distant memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_DMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_DMEM]
	},
	[ POWER6_PME_PM_DPU_WT_BR_MPRED_COUNT ] = {
		.pme_name = "PM_DPU_WT_BR_MPRED_COUNT",
		.pme_code = 0x40000d,
		.pme_short_desc = "Periods DISP unit is stalled due to branch misprediction",
		.pme_long_desc = "Periods DISP unit is stalled due to branch misprediction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_WT_BR_MPRED_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_WT_BR_MPRED_COUNT]
	},
	[ POWER6_PME_PM_GCT_FULL_CYC ] = {
		.pme_name = "PM_GCT_FULL_CYC",
		.pme_code = 0x40086,
		.pme_short_desc = "Cycles GCT full",
		.pme_long_desc = "The ISU sends a signal indicating the gct is full. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GCT_FULL_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GCT_FULL_CYC]
	},
	[ POWER6_PME_PM_INST_FROM_L25_SHR ] = {
		.pme_name = "PM_INST_FROM_L25_SHR",
		.pme_code = 0x442046,
		.pme_short_desc = "Instruction fetched from L2.5 shared",
		.pme_long_desc = "Instruction fetched from L2.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L25_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L25_SHR]
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_4K ] = {
		.pme_name = "PM_MRK_DERAT_MISS_4K",
		.pme_code = 0x292044,
		.pme_short_desc = "Marked DERAT misses for 4K page",
		.pme_long_desc = "A marked data request (load or store) missed the ERAT for 4K page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_MISS_4K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_MISS_4K]
	},
	[ POWER6_PME_PM_DC_PREF_STREAM_ALLOC ] = {
		.pme_name = "PM_DC_PREF_STREAM_ALLOC",
		.pme_code = 0x810a2,
		.pme_short_desc = "D cache new prefetch stream allocated",
		.pme_long_desc = "A new Prefetch Stream was allocated",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DC_PREF_STREAM_ALLOC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DC_PREF_STREAM_ALLOC]
	},
	[ POWER6_PME_PM_FPU1_FIN ] = {
		.pme_name = "PM_FPU1_FIN",
		.pme_code = 0xd0088,
		.pme_short_desc = "FPU1 produced a result",
		.pme_long_desc = "fp1 finished, produced a result. This only indicates finish, not completion. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FIN]
	},
	[ POWER6_PME_PM_BR_MPRED_TA ] = {
		.pme_name = "PM_BR_MPRED_TA",
		.pme_code = 0x410ac,
		.pme_short_desc = "Branch mispredictions due to target address",
		.pme_long_desc = "branch miss predict due to a target address prediction. This signal will be asserted each time the branch execution unit detects an incorrect target address prediction. This signal will be asserted after a valid branch execution unit issue and will cause a branch mispredict flush unless a flush is detected from an older instruction.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_MPRED_TA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_MPRED_TA]
	},
	[ POWER6_PME_PM_DPU_HELD_POWER ] = {
		.pme_name = "PM_DPU_HELD_POWER",
		.pme_code = 0x20003c,
		.pme_short_desc = "DISP unit held due to Power Management",
		.pme_long_desc = "DISP unit held due to Power Management",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_POWER],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_POWER]
	},
	[ POWER6_PME_PM_RUN_INST_CMPL ] = {
		.pme_name = "PM_RUN_INST_CMPL",
		.pme_code = 0x500009,
		.pme_short_desc = "Run instructions completed",
		.pme_long_desc = "Number of run instructions completed. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_RUN_INST_CMPL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_RUN_INST_CMPL]
	},
	[ POWER6_PME_PM_GCT_EMPTY_CYC ] = {
		.pme_name = "PM_GCT_EMPTY_CYC",
		.pme_code = 0x1000f8,
		.pme_short_desc = "Cycles GCT empty",
		.pme_long_desc = "The Global Completion Table is completely empty",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GCT_EMPTY_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GCT_EMPTY_CYC]
	},
	[ POWER6_PME_PM_LLA_COUNT ] = {
		.pme_name = "PM_LLA_COUNT",
		.pme_code = 0xc01f,
		.pme_short_desc = "Transitions into Load Look Ahead mode",
		.pme_long_desc = "Transitions into Load Look Ahead mode",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LLA_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LLA_COUNT]
	},
	[ POWER6_PME_PM_LSU0_REJECT_NO_SCRATCH ] = {
		.pme_name = "PM_LSU0_REJECT_NO_SCRATCH",
		.pme_code = 0xa10a2,
		.pme_short_desc = "LSU0 reject due to scratch register not available",
		.pme_long_desc = "LSU0 reject due to scratch register not available",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_NO_SCRATCH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_NO_SCRATCH]
	},
	[ POWER6_PME_PM_DPU_WT_IC_MISS ] = {
		.pme_name = "PM_DPU_WT_IC_MISS",
		.pme_code = 0x20000c,
		.pme_short_desc = "Cycles DISP unit is stalled due to I cache miss",
		.pme_long_desc = "Cycles DISP unit is stalled due to I cache miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_WT_IC_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_WT_IC_MISS]
	},
	[ POWER6_PME_PM_DATA_FROM_L3MISS ] = {
		.pme_name = "PM_DATA_FROM_L3MISS",
		.pme_code = 0x3000fe,
		.pme_short_desc = "Data loaded from private L3 miss",
		.pme_long_desc = "Data loaded from private L3 miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L3MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L3MISS]
	},
	[ POWER6_PME_PM_FPU_FPSCR ] = {
		.pme_name = "PM_FPU_FPSCR",
		.pme_code = 0x2d0032,
		.pme_short_desc = "FPU executed FPSCR instruction",
		.pme_long_desc = "FPU executed FPSCR instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FPSCR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FPSCR]
	},
	[ POWER6_PME_PM_VMX1_INST_ISSUED ] = {
		.pme_name = "PM_VMX1_INST_ISSUED",
		.pme_code = 0x60088,
		.pme_short_desc = "VMX1 instruction issued",
		.pme_long_desc = "VMX1 instruction issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX1_INST_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX1_INST_ISSUED]
	},
	[ POWER6_PME_PM_FLUSH ] = {
		.pme_name = "PM_FLUSH",
		.pme_code = 0x100010,
		.pme_short_desc = "Flushes",
		.pme_long_desc = "Flushes",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FLUSH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FLUSH]
	},
	[ POWER6_PME_PM_ST_HIT_L2 ] = {
		.pme_name = "PM_ST_HIT_L2",
		.pme_code = 0x150732,
		.pme_short_desc = "L2 D cache store hits",
		.pme_long_desc = "L2 D cache store hits",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ST_HIT_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ST_HIT_L2]
	},
	[ POWER6_PME_PM_SYNC_CYC ] = {
		.pme_name = "PM_SYNC_CYC",
		.pme_code = 0x920cc,
		.pme_short_desc = "Sync duration",
		.pme_long_desc = "Sync duration",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_SYNC_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_SYNC_CYC]
	},
	[ POWER6_PME_PM_FAB_SYS_PUMP ] = {
		.pme_name = "PM_FAB_SYS_PUMP",
		.pme_code = 0x50180,
		.pme_short_desc = "System pump operation",
		.pme_long_desc = " locally mastered",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_SYS_PUMP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_SYS_PUMP]
	},
	[ POWER6_PME_PM_IC_PREF_REQ ] = {
		.pme_name = "PM_IC_PREF_REQ",
		.pme_code = 0x4008c,
		.pme_short_desc = "Instruction prefetch requests",
		.pme_long_desc = "Asserted when a non-canceled prefetch is made to the cache interface unit (CIU).",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IC_PREF_REQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IC_PREF_REQ]
	},
	[ POWER6_PME_PM_MEM0_DP_RQ_GLOB_LOC ] = {
		.pme_name = "PM_MEM0_DP_RQ_GLOB_LOC",
		.pme_code = 0x50280,
		.pme_short_desc = "Memory read queue marking cache line double pump state from global to local side 0",
		.pme_long_desc = "Memory read queue marking cache line double pump state from global to local side 0",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM0_DP_RQ_GLOB_LOC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM0_DP_RQ_GLOB_LOC]
	},
	[ POWER6_PME_PM_FPU_ISSUE_0 ] = {
		.pme_name = "PM_FPU_ISSUE_0",
		.pme_code = 0x320c6,
		.pme_short_desc = "FPU issue 0 per cycle",
		.pme_long_desc = "FPU issue 0 per cycle",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_0],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_0]
	},
	[ POWER6_PME_PM_THRD_PRIO_2_CYC ] = {
		.pme_name = "PM_THRD_PRIO_2_CYC",
		.pme_code = 0x322040,
		.pme_short_desc = "Cycles thread running at priority level 2",
		.pme_long_desc = "Cycles thread running at priority level 2",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_2_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_2_CYC]
	},
	[ POWER6_PME_PM_VMX_SIMPLE_ISSUED ] = {
		.pme_name = "PM_VMX_SIMPLE_ISSUED",
		.pme_code = 0x70082,
		.pme_short_desc = "VMX instruction issued to simple",
		.pme_long_desc = "VMX instruction issued to simple",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_SIMPLE_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_SIMPLE_ISSUED]
	},
	[ POWER6_PME_PM_MRK_FPU1_FIN ] = {
		.pme_name = "PM_MRK_FPU1_FIN",
		.pme_code = 0xd008a,
		.pme_short_desc = "Marked instruction FPU1 processing finished",
		.pme_long_desc = "Marked instruction FPU1 processing finished",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_FPU1_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_FPU1_FIN]
	},
	[ POWER6_PME_PM_DPU_HELD_CW ] = {
		.pme_name = "PM_DPU_HELD_CW",
		.pme_code = 0x20084,
		.pme_short_desc = "DISP unit held due to cache writes ",
		.pme_long_desc = "DISP unit held due to cache writes ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_CW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_CW]
	},
	[ POWER6_PME_PM_L3SA_REF ] = {
		.pme_name = "PM_L3SA_REF",
		.pme_code = 0x50080,
		.pme_short_desc = "L3 slice A references",
		.pme_long_desc = "L3 slice A references",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L3SA_REF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L3SA_REF]
	},
	[ POWER6_PME_PM_STCX ] = {
		.pme_name = "PM_STCX",
		.pme_code = 0x830e6,
		.pme_short_desc = "STCX executed",
		.pme_long_desc = "STCX executed",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_STCX],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_STCX]
	},
	[ POWER6_PME_PM_L2SB_MISS ] = {
		.pme_name = "PM_L2SB_MISS",
		.pme_code = 0x5058c,
		.pme_short_desc = "L2 slice B misses",
		.pme_long_desc = "L2 slice B misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_MISS]
	},
	[ POWER6_PME_PM_LSU0_REJECT ] = {
		.pme_name = "PM_LSU0_REJECT",
		.pme_code = 0xa10a6,
		.pme_short_desc = "LSU0 reject",
		.pme_long_desc = "LSU0 reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT]
	},
	[ POWER6_PME_PM_TB_BIT_TRANS ] = {
		.pme_name = "PM_TB_BIT_TRANS",
		.pme_code = 0x100026,
		.pme_short_desc = "Time Base bit transition",
		.pme_long_desc = "When the selected time base bit (as specified in MMCR0[TBSEL])transitions from 0 to 1 ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_TB_BIT_TRANS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_TB_BIT_TRANS]
	},
	[ POWER6_PME_PM_THERMAL_MAX ] = {
		.pme_name = "PM_THERMAL_MAX",
		.pme_code = 0x30002a,
		.pme_short_desc = "Processor in thermal MAX",
		.pme_long_desc = "Processor in thermal MAX",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THERMAL_MAX],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THERMAL_MAX]
	},
	[ POWER6_PME_PM_FPU0_STF ] = {
		.pme_name = "PM_FPU0_STF",
		.pme_code = 0xc10a4,
		.pme_short_desc = "FPU0 executed store instruction",
		.pme_long_desc = "This signal is active for one cycle when fp0 is executing a store instruction.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_STF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_STF]
	},
	[ POWER6_PME_PM_FPU1_FMA ] = {
		.pme_name = "PM_FPU1_FMA",
		.pme_code = 0xc008a,
		.pme_short_desc = "FPU1 executed multiply-add instruction",
		.pme_long_desc = "This signal is active for one cycle when fp1 is executing multiply-add kind of instruction. This could be fmadd*, fnmadd*, fmsub*, fnmsub* where XYZ* means XYZ, XYZs, XYZ., XYZs.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FMA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FMA]
	},
	[ POWER6_PME_PM_LSU1_REJECT_LHS ] = {
		.pme_name = "PM_LSU1_REJECT_LHS",
		.pme_code = 0x9008e,
		.pme_short_desc = "LSU1 load hit store reject",
		.pme_long_desc = "LSU1 load hit store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_LHS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_LHS]
	},
	[ POWER6_PME_PM_DPU_HELD_INT ] = {
		.pme_name = "PM_DPU_HELD_INT",
		.pme_code = 0x310a8,
		.pme_short_desc = "DISP unit held due to exception",
		.pme_long_desc = "DISP unit held due to exception",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_INT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_INT]
	},
	[ POWER6_PME_PM_THRD_LLA_BOTH_CYC ] = {
		.pme_name = "PM_THRD_LLA_BOTH_CYC",
		.pme_code = 0x400008,
		.pme_short_desc = "Both threads in Load Look Ahead",
		.pme_long_desc = "Both threads in Load Look Ahead",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_LLA_BOTH_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_LLA_BOTH_CYC]
	},
	[ POWER6_PME_PM_DPU_HELD_THERMAL_COUNT ] = {
		.pme_name = "PM_DPU_HELD_THERMAL_COUNT",
		.pme_code = 0x10002b,
		.pme_short_desc = "Periods DISP unit held due to thermal condition",
		.pme_long_desc = "Periods DISP unit held due to thermal condition",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_THERMAL_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_THERMAL_COUNT]
	},
	[ POWER6_PME_PM_PMC4_REWIND ] = {
		.pme_name = "PM_PMC4_REWIND",
		.pme_code = 0x100020,
		.pme_short_desc = "PMC4 rewind event",
		.pme_long_desc = "PMC4 rewind event",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC4_REWIND],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC4_REWIND]
	},
	[ POWER6_PME_PM_DERAT_REF_16M ] = {
		.pme_name = "PM_DERAT_REF_16M",
		.pme_code = 0x382070,
		.pme_short_desc = "DERAT reference for 16M page",
		.pme_long_desc = "DERAT reference for 16M page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_REF_16M],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_REF_16M]
	},
	[ POWER6_PME_PM_FPU0_FCONV ] = {
		.pme_name = "PM_FPU0_FCONV",
		.pme_code = 0xd10a0,
		.pme_short_desc = "FPU0 executed FCONV instruction",
		.pme_long_desc = "FPU0 executed FCONV instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FCONV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FCONV]
	},
	[ POWER6_PME_PM_L2SA_LD_REQ_DATA ] = {
		.pme_name = "PM_L2SA_LD_REQ_DATA",
		.pme_code = 0x50480,
		.pme_short_desc = "L2 slice A data load requests",
		.pme_long_desc = "L2 slice A data load requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_LD_REQ_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_LD_REQ_DATA]
	},
	[ POWER6_PME_PM_DATA_FROM_MEM_DP ] = {
		.pme_name = "PM_DATA_FROM_MEM_DP",
		.pme_code = 0x10005e,
		.pme_short_desc = "Data loaded from double pump memory",
		.pme_long_desc = "Data loaded from double pump memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_MEM_DP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_MEM_DP]
	},
	[ POWER6_PME_PM_MRK_VMX_FLOAT_ISSUED ] = {
		.pme_name = "PM_MRK_VMX_FLOAT_ISSUED",
		.pme_code = 0x70088,
		.pme_short_desc = "Marked VMX instruction issued to float",
		.pme_long_desc = "Marked VMX instruction issued to float",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_VMX_FLOAT_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_VMX_FLOAT_ISSUED]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L2MISS ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L2MISS",
		.pme_code = 0x412054,
		.pme_short_desc = "Marked PTEG loaded from L2 miss",
		.pme_long_desc = "Marked PTEG loaded from L2 miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L2MISS]
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_1or2_CYC ] = {
		.pme_name = "PM_THRD_PRIO_DIFF_1or2_CYC",
		.pme_code = 0x223040,
		.pme_short_desc = "Cycles thread priority difference is 1 or 2",
		.pme_long_desc = "Cycles thread priority difference is 1 or 2",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_DIFF_1or2_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_DIFF_1or2_CYC]
	},
	[ POWER6_PME_PM_VMX0_STALL ] = {
		.pme_name = "PM_VMX0_STALL",
		.pme_code = 0xb0084,
		.pme_short_desc = "VMX0 stall",
		.pme_long_desc = "VMX0 stall",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX0_STALL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX0_STALL]
	},
	[ POWER6_PME_PM_IC_DEMAND_L2_BHT_REDIRECT ] = {
		.pme_name = "PM_IC_DEMAND_L2_BHT_REDIRECT",
		.pme_code = 0x420ca,
		.pme_short_desc = "L2 I cache demand request due to BHT redirect",
		.pme_long_desc = "L2 I cache demand request due to BHT redirect",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IC_DEMAND_L2_BHT_REDIRECT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IC_DEMAND_L2_BHT_REDIRECT]
	},
	[ POWER6_PME_PM_LSU_DERAT_MISS ] = {
		.pme_name = "PM_LSU_DERAT_MISS",
		.pme_code = 0x20000e,
		.pme_short_desc = "DERAT misses",
		.pme_long_desc = "Total DERAT Misses (Unit 0 + Unit 1). Requests that miss the Derat are rejected and retried until the request hits in the Erat. This may result in multiple erat misses for the same instruction.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_DERAT_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_DERAT_MISS]
	},
	[ POWER6_PME_PM_FPU0_SINGLE ] = {
		.pme_name = "PM_FPU0_SINGLE",
		.pme_code = 0xc10a6,
		.pme_short_desc = "FPU0 executed single precision instruction",
		.pme_long_desc = "This signal is active for one cycle when fp0 is executing single precision instruction.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_SINGLE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_SINGLE]
	},
	[ POWER6_PME_PM_FPU_ISSUE_STEERING ] = {
		.pme_name = "PM_FPU_ISSUE_STEERING",
		.pme_code = 0x320c4,
		.pme_short_desc = "FPU issue steering",
		.pme_long_desc = "FPU issue steering",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_STEERING],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_STEERING]
	},
	[ POWER6_PME_PM_THRD_PRIO_1_CYC ] = {
		.pme_name = "PM_THRD_PRIO_1_CYC",
		.pme_code = 0x222040,
		.pme_short_desc = "Cycles thread running at priority level 1",
		.pme_long_desc = "Cycles thread running at priority level 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_1_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_1_CYC]
	},
	[ POWER6_PME_PM_VMX_COMPLEX_ISSUED ] = {
		.pme_name = "PM_VMX_COMPLEX_ISSUED",
		.pme_code = 0x70084,
		.pme_short_desc = "VMX instruction issued to complex",
		.pme_long_desc = "VMX instruction issued to complex",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_COMPLEX_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_COMPLEX_ISSUED]
	},
	[ POWER6_PME_PM_FPU_ISSUE_ST_FOLDED ] = {
		.pme_name = "PM_FPU_ISSUE_ST_FOLDED",
		.pme_code = 0x320c2,
		.pme_short_desc = "FPU issue a folded store",
		.pme_long_desc = "FPU issue a folded store",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_ST_FOLDED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_ST_FOLDED]
	},
	[ POWER6_PME_PM_DFU_FIN ] = {
		.pme_name = "PM_DFU_FIN",
		.pme_code = 0xe0080,
		.pme_short_desc = "DFU instruction finish",
		.pme_long_desc = "DFU instruction finish",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_FIN]
	},
	[ POWER6_PME_PM_BR_PRED_CCACHE ] = {
		.pme_name = "PM_BR_PRED_CCACHE",
		.pme_code = 0x410a4,
		.pme_short_desc = "Branch count cache prediction",
		.pme_long_desc = "Branch count cache prediction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_PRED_CCACHE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_PRED_CCACHE]
	},
	[ POWER6_PME_PM_MRK_ST_CMPL_INT ] = {
		.pme_name = "PM_MRK_ST_CMPL_INT",
		.pme_code = 0x300006,
		.pme_short_desc = "Marked store completed with intervention",
		.pme_long_desc = "A marked store previously sent to the memory subsystem completed (data home) after requiring intervention",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_ST_CMPL_INT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_ST_CMPL_INT]
	},
	[ POWER6_PME_PM_FAB_MMIO ] = {
		.pme_name = "PM_FAB_MMIO",
		.pme_code = 0x50186,
		.pme_short_desc = "MMIO operation",
		.pme_long_desc = " locally mastered",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_MMIO],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_MMIO]
	},
	[ POWER6_PME_PM_MRK_VMX_SIMPLE_ISSUED ] = {
		.pme_name = "PM_MRK_VMX_SIMPLE_ISSUED",
		.pme_code = 0x7008a,
		.pme_short_desc = "Marked VMX instruction issued to simple",
		.pme_long_desc = "Marked VMX instruction issued to simple",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_VMX_SIMPLE_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_VMX_SIMPLE_ISSUED]
	},
	[ POWER6_PME_PM_FPU_STF ] = {
		.pme_name = "PM_FPU_STF",
		.pme_code = 0x3c1030,
		.pme_short_desc = "FPU executed store instruction",
		.pme_long_desc = "FPU is executing a store instruction. Combined Unit 0 + Unit 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_STF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_STF]
	},
	[ POWER6_PME_PM_MEM1_DP_CL_WR_GLOB ] = {
		.pme_name = "PM_MEM1_DP_CL_WR_GLOB",
		.pme_code = 0x5028c,
		.pme_short_desc = "cacheline write setting dp to global side 1",
		.pme_long_desc = "cacheline write setting dp to global side 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM1_DP_CL_WR_GLOB],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM1_DP_CL_WR_GLOB]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L3MISS ] = {
		.pme_name = "PM_MRK_DATA_FROM_L3MISS",
		.pme_code = 0x303028,
		.pme_short_desc = "Marked data loaded from L3 miss",
		.pme_long_desc = "Marked data loaded from L3 miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L3MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L3MISS]
	},
	[ POWER6_PME_PM_GCT_NOSLOT_CYC ] = {
		.pme_name = "PM_GCT_NOSLOT_CYC",
		.pme_code = 0x100008,
		.pme_short_desc = "Cycles no GCT slot allocated",
		.pme_long_desc = "Cycles this thread does not have any slots allocated in the GCT.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GCT_NOSLOT_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GCT_NOSLOT_CYC]
	},
	[ POWER6_PME_PM_L2_ST_REQ_DATA ] = {
		.pme_name = "PM_L2_ST_REQ_DATA",
		.pme_code = 0x250432,
		.pme_short_desc = "L2 data store requests",
		.pme_long_desc = "L2 data store requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_ST_REQ_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_ST_REQ_DATA]
	},
	[ POWER6_PME_PM_INST_TABLEWALK_COUNT ] = {
		.pme_name = "PM_INST_TABLEWALK_COUNT",
		.pme_code = 0x920cb,
		.pme_short_desc = "Periods doing instruction tablewalks",
		.pme_long_desc = "Periods doing instruction tablewalks",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_TABLEWALK_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_TABLEWALK_COUNT]
	},
	[ POWER6_PME_PM_PTEG_FROM_L35_SHR ] = {
		.pme_name = "PM_PTEG_FROM_L35_SHR",
		.pme_code = 0x21304e,
		.pme_short_desc = "PTEG loaded from L3.5 shared",
		.pme_long_desc = "PTEG loaded from L3.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L35_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L35_SHR]
	},
	[ POWER6_PME_PM_DPU_HELD_ISYNC ] = {
		.pme_name = "PM_DPU_HELD_ISYNC",
		.pme_code = 0x2008a,
		.pme_short_desc = "DISP unit held due to ISYNC ",
		.pme_long_desc = "DISP unit held due to ISYNC ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_ISYNC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_ISYNC]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L25_SHR ] = {
		.pme_name = "PM_MRK_DATA_FROM_L25_SHR",
		.pme_code = 0x40304e,
		.pme_short_desc = "Marked data loaded from L2.5 shared",
		.pme_long_desc = "DL1 was reloaded with shared (T or SL) data from the L2 of a chip on this MCM due to a marked demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L25_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L25_SHR]
	},
	[ POWER6_PME_PM_L3SA_HIT ] = {
		.pme_name = "PM_L3SA_HIT",
		.pme_code = 0x50082,
		.pme_short_desc = "L3 slice A hits",
		.pme_long_desc = "L3 slice A hits",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L3SA_HIT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L3SA_HIT]
	},
	[ POWER6_PME_PM_DERAT_MISS_16G ] = {
		.pme_name = "PM_DERAT_MISS_16G",
		.pme_code = 0x492070,
		.pme_short_desc = "DERAT misses for 16G page",
		.pme_long_desc = "A data request (load or store) missed the ERAT for 16G page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_MISS_16G],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_MISS_16G]
	},
	[ POWER6_PME_PM_DATA_PTEG_2ND_HALF ] = {
		.pme_name = "PM_DATA_PTEG_2ND_HALF",
		.pme_code = 0x910a2,
		.pme_short_desc = "Data table walk matched in second half pri­mary PTEG",
		.pme_long_desc = "Data table walk matched in second half pri­mary PTEG",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_PTEG_2ND_HALF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_PTEG_2ND_HALF]
	},
	[ POWER6_PME_PM_L2SA_ST_REQ ] = {
		.pme_name = "PM_L2SA_ST_REQ",
		.pme_code = 0x50484,
		.pme_short_desc = "L2 slice A store requests",
		.pme_long_desc = "A store request as seen at the L2 directory has been made from the core. Stores are counted after gathering in the L2 store queues. The event is provided on each of the three slices A,B,and C.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_ST_REQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_ST_REQ]
	},
	[ POWER6_PME_PM_INST_FROM_LMEM ] = {
		.pme_name = "PM_INST_FROM_LMEM",
		.pme_code = 0x442042,
		.pme_short_desc = "Instruction fetched from local memory",
		.pme_long_desc = "Instruction fetched from local memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_LMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_LMEM]
	},
	[ POWER6_PME_PM_IC_DEMAND_L2_BR_REDIRECT ] = {
		.pme_name = "PM_IC_DEMAND_L2_BR_REDIRECT",
		.pme_code = 0x420cc,
		.pme_short_desc = "L2 I cache demand request due to branch redirect",
		.pme_long_desc = "L2 I cache demand request due to branch redirect",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IC_DEMAND_L2_BR_REDIRECT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IC_DEMAND_L2_BR_REDIRECT]
	},
	[ POWER6_PME_PM_PTEG_FROM_L2 ] = {
		.pme_name = "PM_PTEG_FROM_L2",
		.pme_code = 0x113048,
		.pme_short_desc = "PTEG loaded from L2",
		.pme_long_desc = "PTEG loaded from L2",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L2]
	},
	[ POWER6_PME_PM_DATA_PTEG_1ST_HALF ] = {
		.pme_name = "PM_DATA_PTEG_1ST_HALF",
		.pme_code = 0x910a0,
		.pme_short_desc = "Data table walk matched in first half primary PTEG",
		.pme_long_desc = "Data table walk matched in first half primary PTEG",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_PTEG_1ST_HALF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_PTEG_1ST_HALF]
	},
	[ POWER6_PME_PM_BR_MPRED_COUNT ] = {
		.pme_name = "PM_BR_MPRED_COUNT",
		.pme_code = 0x410aa,
		.pme_short_desc = "Branch misprediction due to count prediction",
		.pme_long_desc = "Branch misprediction due to count prediction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_MPRED_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_MPRED_COUNT]
	},
	[ POWER6_PME_PM_IERAT_MISS_4K ] = {
		.pme_name = "PM_IERAT_MISS_4K",
		.pme_code = 0x492076,
		.pme_short_desc = "IERAT misses for 4K page",
		.pme_long_desc = "IERAT misses for 4K page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IERAT_MISS_4K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IERAT_MISS_4K]
	},
	[ POWER6_PME_PM_THRD_BOTH_RUN_COUNT ] = {
		.pme_name = "PM_THRD_BOTH_RUN_COUNT",
		.pme_code = 0x400005,
		.pme_short_desc = "Periods both threads in run cycles",
		.pme_long_desc = "Periods both threads in run cycles",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_BOTH_RUN_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_BOTH_RUN_COUNT]
	},
	[ POWER6_PME_PM_LSU_REJECT_ULD ] = {
		.pme_name = "PM_LSU_REJECT_ULD",
		.pme_code = 0x190030,
		.pme_short_desc = "Unaligned load reject",
		.pme_long_desc = "Unaligned load reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_ULD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_ULD]
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_MOD_CYC ] = {
		.pme_name = "PM_DATA_FROM_DL2L3_MOD_CYC",
		.pme_code = 0x40002a,
		.pme_short_desc = "Load latency from distant L2 or L3 modified",
		.pme_long_desc = "Load latency from distant L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_DL2L3_MOD_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_DL2L3_MOD_CYC]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_MOD ] = {
		.pme_name = "PM_MRK_PTEG_FROM_RL2L3_MOD",
		.pme_code = 0x112044,
		.pme_short_desc = "Marked PTEG loaded from remote L2 or L3 modified",
		.pme_long_desc = "Marked PTEG loaded from remote L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_MOD]
	},
	[ POWER6_PME_PM_FPU0_FLOP ] = {
		.pme_name = "PM_FPU0_FLOP",
		.pme_code = 0xc0086,
		.pme_short_desc = "FPU0 executed 1FLOP",
		.pme_long_desc = " FMA",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FLOP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FLOP]
	},
	[ POWER6_PME_PM_FPU0_FEST ] = {
		.pme_name = "PM_FPU0_FEST",
		.pme_code = 0xd10a6,
		.pme_short_desc = "FPU0 executed FEST instruction",
		.pme_long_desc = "This signal is active for one cycle when fp0 is executing one of the estimate instructions. This could be fres* or frsqrte* where XYZ* means XYZ or XYZ. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FEST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FEST]
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_LHS ] = {
		.pme_name = "PM_MRK_LSU0_REJECT_LHS",
		.pme_code = 0x930e6,
		.pme_short_desc = "LSU0 marked load hit store reject",
		.pme_long_desc = "LSU0 marked load hit store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU0_REJECT_LHS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU0_REJECT_LHS]
	},
	[ POWER6_PME_PM_VMX_RESULT_SAT_1 ] = {
		.pme_name = "PM_VMX_RESULT_SAT_1",
		.pme_code = 0xb0086,
		.pme_short_desc = "VMX valid result with sat=1",
		.pme_long_desc = "VMX valid result with sat=1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_RESULT_SAT_1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_RESULT_SAT_1]
	},
	[ POWER6_PME_PM_NO_ITAG_CYC ] = {
		.pme_name = "PM_NO_ITAG_CYC",
		.pme_code = 0x40088,
		.pme_short_desc = "Cyles no ITAG available",
		.pme_long_desc = "Cyles no ITAG available",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_NO_ITAG_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_NO_ITAG_CYC]
	},
	[ POWER6_PME_PM_LSU1_REJECT_NO_SCRATCH ] = {
		.pme_name = "PM_LSU1_REJECT_NO_SCRATCH",
		.pme_code = 0xa10aa,
		.pme_short_desc = "LSU1 reject due to scratch register not available",
		.pme_long_desc = "LSU1 reject due to scratch register not available",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_NO_SCRATCH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_NO_SCRATCH]
	},
	[ POWER6_PME_PM_0INST_FETCH ] = {
		.pme_name = "PM_0INST_FETCH",
		.pme_code = 0x40080,
		.pme_short_desc = "No instructions fetched",
		.pme_long_desc = "No instructions were fetched this cycles (due to IFU hold, redirect, or icache miss)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_0INST_FETCH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_0INST_FETCH]
	},
	[ POWER6_PME_PM_DPU_WT_BR_MPRED ] = {
		.pme_name = "PM_DPU_WT_BR_MPRED",
		.pme_code = 0x40000c,
		.pme_short_desc = "Cycles DISP unit is stalled due to branch misprediction",
		.pme_long_desc = "Cycles DISP unit is stalled due to branch misprediction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_WT_BR_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_WT_BR_MPRED]
	},
	[ POWER6_PME_PM_L1_PREF ] = {
		.pme_name = "PM_L1_PREF",
		.pme_code = 0x810a4,
		.pme_short_desc = "L1 cache data prefetches",
		.pme_long_desc = "A request to prefetch data into the L1 was made",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L1_PREF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L1_PREF]
	},
	[ POWER6_PME_PM_VMX_FLOAT_MULTICYCLE ] = {
		.pme_name = "PM_VMX_FLOAT_MULTICYCLE",
		.pme_code = 0xb0082,
		.pme_short_desc = "VMX multi-cycle floating point instruction issued",
		.pme_long_desc = "VMX multi-cycle floating point instruction issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_FLOAT_MULTICYCLE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_FLOAT_MULTICYCLE]
	},
	[ POWER6_PME_PM_DATA_FROM_L25_SHR_CYC ] = {
		.pme_name = "PM_DATA_FROM_L25_SHR_CYC",
		.pme_code = 0x200024,
		.pme_short_desc = "Load latency from L2.5 shared",
		.pme_long_desc = "Load latency from L2.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L25_SHR_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L25_SHR_CYC]
	},
	[ POWER6_PME_PM_DATA_FROM_L3 ] = {
		.pme_name = "PM_DATA_FROM_L3",
		.pme_code = 0x300058,
		.pme_short_desc = "Data loaded from L3",
		.pme_long_desc = "DL1 was reloaded from the local L3 due to a demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L3],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L3]
	},
	[ POWER6_PME_PM_PMC2_OVERFLOW ] = {
		.pme_name = "PM_PMC2_OVERFLOW",
		.pme_code = 0x300014,
		.pme_short_desc = "PMC2 Overflow",
		.pme_long_desc = "PMC2 Overflow",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC2_OVERFLOW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC2_OVERFLOW]
	},
	[ POWER6_PME_PM_VMX0_LD_WRBACK ] = {
		.pme_name = "PM_VMX0_LD_WRBACK",
		.pme_code = 0x60084,
		.pme_short_desc = "VMX0 load writeback valid",
		.pme_long_desc = "VMX0 load writeback valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX0_LD_WRBACK],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX0_LD_WRBACK]
	},
	[ POWER6_PME_PM_FPU0_DENORM ] = {
		.pme_name = "PM_FPU0_DENORM",
		.pme_code = 0xc10a2,
		.pme_short_desc = "FPU0 received denormalized data",
		.pme_long_desc = "This signal is active for one cycle when one of the operands is denormalized.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_DENORM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_DENORM]
	},
	[ POWER6_PME_PM_INST_FETCH_CYC ] = {
		.pme_name = "PM_INST_FETCH_CYC",
		.pme_code = 0x420c8,
		.pme_short_desc = "Cycles at least 1 instruction fetched",
		.pme_long_desc = "Asserted each cycle when the IFU sends at least one instruction to the IDU. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FETCH_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FETCH_CYC]
	},
	[ POWER6_PME_PM_LSU_LDF ] = {
		.pme_name = "PM_LSU_LDF",
		.pme_code = 0x280032,
		.pme_short_desc = "LSU executed Floating Point load instruction",
		.pme_long_desc = "LSU executed Floating Point load instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_LDF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_LDF]
	},
	[ POWER6_PME_PM_LSU_REJECT_L2_CORR ] = {
		.pme_name = "PM_LSU_REJECT_L2_CORR",
		.pme_code = 0x1a1034,
		.pme_short_desc = "LSU reject due to L2 correctable error",
		.pme_long_desc = "LSU reject due to L2 correctable error",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_L2_CORR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_L2_CORR]
	},
	[ POWER6_PME_PM_DERAT_REF_64K ] = {
		.pme_name = "PM_DERAT_REF_64K",
		.pme_code = 0x282070,
		.pme_short_desc = "DERAT reference for 64K page",
		.pme_long_desc = "DERAT reference for 64K page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_REF_64K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_REF_64K]
	},
	[ POWER6_PME_PM_THRD_PRIO_3_CYC ] = {
		.pme_name = "PM_THRD_PRIO_3_CYC",
		.pme_code = 0x422040,
		.pme_short_desc = "Cycles thread running at priority level 3",
		.pme_long_desc = "Cycles thread running at priority level 3",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_3_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_3_CYC]
	},
	[ POWER6_PME_PM_FPU_FMA ] = {
		.pme_name = "PM_FPU_FMA",
		.pme_code = 0x2c0030,
		.pme_short_desc = "FPU executed multiply-add instruction",
		.pme_long_desc = "This signal is active for one cycle when FPU is executing multiply-add kind of instruction. This could be fmadd*, fnmadd*, fmsub*, fnmsub* where XYZ* means XYZ, XYZs, XYZ., XYZs. Combined Unit 0 + Unit 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FMA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FMA]
	},
	[ POWER6_PME_PM_INST_FROM_L35_MOD ] = {
		.pme_name = "PM_INST_FROM_L35_MOD",
		.pme_code = 0x142046,
		.pme_short_desc = "Instruction fetched from L3.5 modified",
		.pme_long_desc = "Instruction fetched from L3.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L35_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L35_MOD]
	},
	[ POWER6_PME_PM_DFU_CONV ] = {
		.pme_name = "PM_DFU_CONV",
		.pme_code = 0xe008e,
		.pme_short_desc = "DFU convert from fixed op",
		.pme_long_desc = "DFU convert from fixed op",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_CONV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_CONV]
	},
	[ POWER6_PME_PM_INST_FROM_L25_MOD ] = {
		.pme_name = "PM_INST_FROM_L25_MOD",
		.pme_code = 0x342046,
		.pme_short_desc = "Instruction fetched from L2.5 modified",
		.pme_long_desc = "Instruction fetched from L2.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L25_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L25_MOD]
	},
	[ POWER6_PME_PM_PTEG_FROM_L35_MOD ] = {
		.pme_name = "PM_PTEG_FROM_L35_MOD",
		.pme_code = 0x11304e,
		.pme_short_desc = "PTEG loaded from L3.5 modified",
		.pme_long_desc = "PTEG loaded from L3.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L35_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L35_MOD]
	},
	[ POWER6_PME_PM_MRK_VMX_ST_ISSUED ] = {
		.pme_name = "PM_MRK_VMX_ST_ISSUED",
		.pme_code = 0xb0088,
		.pme_short_desc = "Marked VMX store issued",
		.pme_long_desc = "Marked VMX store issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_VMX_ST_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_VMX_ST_ISSUED]
	},
	[ POWER6_PME_PM_VMX_FLOAT_ISSUED ] = {
		.pme_name = "PM_VMX_FLOAT_ISSUED",
		.pme_code = 0x70080,
		.pme_short_desc = "VMX instruction issued to float",
		.pme_long_desc = "VMX instruction issued to float",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_FLOAT_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_FLOAT_ISSUED]
	},
	[ POWER6_PME_PM_LSU0_REJECT_L2_CORR ] = {
		.pme_name = "PM_LSU0_REJECT_L2_CORR",
		.pme_code = 0xa10a0,
		.pme_short_desc = "LSU0 reject due to L2 correctable error",
		.pme_long_desc = "LSU0 reject due to L2 correctable error",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_L2_CORR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_L2_CORR]
	},
	[ POWER6_PME_PM_THRD_L2MISS ] = {
		.pme_name = "PM_THRD_L2MISS",
		.pme_code = 0x310a0,
		.pme_short_desc = "Thread in L2 miss",
		.pme_long_desc = "Thread in L2 miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_L2MISS]
	},
	[ POWER6_PME_PM_FPU_FCONV ] = {
		.pme_name = "PM_FPU_FCONV",
		.pme_code = 0x1d1034,
		.pme_short_desc = "FPU executed FCONV instruction",
		.pme_long_desc = "FPU executed FCONV instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FCONV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FCONV]
	},
	[ POWER6_PME_PM_FPU_FXMULT ] = {
		.pme_name = "PM_FPU_FXMULT",
		.pme_code = 0x1d0032,
		.pme_short_desc = "FPU executed fixed point multiplication",
		.pme_long_desc = "FPU executed fixed point multiplication",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FXMULT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FXMULT]
	},
	[ POWER6_PME_PM_FPU1_FRSP ] = {
		.pme_name = "PM_FPU1_FRSP",
		.pme_code = 0xd10aa,
		.pme_short_desc = "FPU1 executed FRSP instruction",
		.pme_long_desc = "FPU1 executed FRSP instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FRSP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FRSP]
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_16M ] = {
		.pme_name = "PM_MRK_DERAT_REF_16M",
		.pme_code = 0x382044,
		.pme_short_desc = "Marked DERAT reference for 16M page",
		.pme_long_desc = "Marked DERAT reference for 16M page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_REF_16M],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_REF_16M]
	},
	[ POWER6_PME_PM_L2SB_CASTOUT_SHR ] = {
		.pme_name = "PM_L2SB_CASTOUT_SHR",
		.pme_code = 0x5068a,
		.pme_short_desc = "L2 slice B castouts - Shared",
		.pme_long_desc = "L2 slice B castouts - Shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_CASTOUT_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_CASTOUT_SHR]
	},
	[ POWER6_PME_PM_THRD_ONE_RUN_COUNT ] = {
		.pme_name = "PM_THRD_ONE_RUN_COUNT",
		.pme_code = 0x1000fb,
		.pme_short_desc = "Periods one of the threads in run cycles",
		.pme_long_desc = "Periods one of the threads in run cycles",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_ONE_RUN_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_ONE_RUN_COUNT]
	},
	[ POWER6_PME_PM_INST_FROM_RMEM ] = {
		.pme_name = "PM_INST_FROM_RMEM",
		.pme_code = 0x342042,
		.pme_short_desc = "Instruction fetched from remote memory",
		.pme_long_desc = "Instruction fetched from remote memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_RMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_RMEM]
	},
	[ POWER6_PME_PM_LSU_BOTH_BUS ] = {
		.pme_name = "PM_LSU_BOTH_BUS",
		.pme_code = 0x810aa,
		.pme_short_desc = "Both data return buses busy simultaneously",
		.pme_long_desc = "Both data return buses busy simultaneously",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_BOTH_BUS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_BOTH_BUS]
	},
	[ POWER6_PME_PM_FPU1_FSQRT_FDIV ] = {
		.pme_name = "PM_FPU1_FSQRT_FDIV",
		.pme_code = 0xc008c,
		.pme_short_desc = "FPU1 executed FSQRT or FDIV instruction",
		.pme_long_desc = "FPU1 executed FSQRT or FDIV instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FSQRT_FDIV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FSQRT_FDIV]
	},
	[ POWER6_PME_PM_L2_LD_REQ_INST ] = {
		.pme_name = "PM_L2_LD_REQ_INST",
		.pme_code = 0x150530,
		.pme_short_desc = "L2 instruction load requests",
		.pme_long_desc = "L2 instruction load requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_LD_REQ_INST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_LD_REQ_INST]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L35_SHR ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L35_SHR",
		.pme_code = 0x212046,
		.pme_short_desc = "Marked PTEG loaded from L3.5 shared",
		.pme_long_desc = "Marked PTEG loaded from L3.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L35_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L35_SHR]
	},
	[ POWER6_PME_PM_BR_PRED_CR ] = {
		.pme_name = "PM_BR_PRED_CR",
		.pme_code = 0x410a2,
		.pme_short_desc = "A conditional branch was predicted",
		.pme_long_desc = " CR prediction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_PRED_CR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_PRED_CR]
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_ULD ] = {
		.pme_name = "PM_MRK_LSU0_REJECT_ULD",
		.pme_code = 0x930e0,
		.pme_short_desc = "LSU0 marked unaligned load reject",
		.pme_long_desc = "LSU0 marked unaligned load reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU0_REJECT_ULD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU0_REJECT_ULD]
	},
	[ POWER6_PME_PM_LSU_REJECT ] = {
		.pme_name = "PM_LSU_REJECT",
		.pme_code = 0x4a1030,
		.pme_short_desc = "LSU reject",
		.pme_long_desc = "LSU reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT]
	},
	[ POWER6_PME_PM_LSU_REJECT_LHS_BOTH ] = {
		.pme_name = "PM_LSU_REJECT_LHS_BOTH",
		.pme_code = 0x290038,
		.pme_short_desc = "Load hit store reject both units",
		.pme_long_desc = "Load hit store reject both units",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_LHS_BOTH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_LHS_BOTH]
	},
	[ POWER6_PME_PM_GXO_ADDR_CYC_BUSY ] = {
		.pme_name = "PM_GXO_ADDR_CYC_BUSY",
		.pme_code = 0x50382,
		.pme_short_desc = "Outbound GX address utilization (# of cycles address out is valid)",
		.pme_long_desc = "Outbound GX address utilization (# of cycles address out is valid)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GXO_ADDR_CYC_BUSY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GXO_ADDR_CYC_BUSY]
	},
	[ POWER6_PME_PM_LSU_SRQ_EMPTY_COUNT ] = {
		.pme_name = "PM_LSU_SRQ_EMPTY_COUNT",
		.pme_code = 0x40001d,
		.pme_short_desc = "Periods SRQ empty",
		.pme_long_desc = "The Store Request Queue is empty",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_SRQ_EMPTY_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_SRQ_EMPTY_COUNT]
	},
	[ POWER6_PME_PM_PTEG_FROM_L3 ] = {
		.pme_name = "PM_PTEG_FROM_L3",
		.pme_code = 0x313048,
		.pme_short_desc = "PTEG loaded from L3",
		.pme_long_desc = "PTEG loaded from L3",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L3],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L3]
	},
	[ POWER6_PME_PM_VMX0_LD_ISSUED ] = {
		.pme_name = "PM_VMX0_LD_ISSUED",
		.pme_code = 0x60082,
		.pme_short_desc = "VMX0 load issued",
		.pme_long_desc = "VMX0 load issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX0_LD_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX0_LD_ISSUED]
	},
	[ POWER6_PME_PM_FXU_PIPELINED_MULT_DIV ] = {
		.pme_name = "PM_FXU_PIPELINED_MULT_DIV",
		.pme_code = 0x210ae,
		.pme_short_desc = "Fix point multiply/divide pipelined",
		.pme_long_desc = "Fix point multiply/divide pipelined",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FXU_PIPELINED_MULT_DIV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FXU_PIPELINED_MULT_DIV]
	},
	[ POWER6_PME_PM_FPU1_STF ] = {
		.pme_name = "PM_FPU1_STF",
		.pme_code = 0xc10ac,
		.pme_short_desc = "FPU1 executed store instruction",
		.pme_long_desc = "This signal is active for one cycle when fp1 is executing a store instruction.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_STF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_STF]
	},
	[ POWER6_PME_PM_DFU_ADD ] = {
		.pme_name = "PM_DFU_ADD",
		.pme_code = 0xe008c,
		.pme_short_desc = "DFU add type instruction",
		.pme_long_desc = "DFU add type instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_ADD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_ADD]
	},
	[ POWER6_PME_PM_MEM_DP_CL_WR_GLOB ] = {
		.pme_name = "PM_MEM_DP_CL_WR_GLOB",
		.pme_code = 0x250232,
		.pme_short_desc = "cache line write setting double pump state to global",
		.pme_long_desc = "cache line write setting double pump state to global",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM_DP_CL_WR_GLOB],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM_DP_CL_WR_GLOB]
	},
	[ POWER6_PME_PM_MRK_LSU1_REJECT_ULD ] = {
		.pme_name = "PM_MRK_LSU1_REJECT_ULD",
		.pme_code = 0x930e8,
		.pme_short_desc = "LSU1 marked unaligned load reject",
		.pme_long_desc = "LSU1 marked unaligned load reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU1_REJECT_ULD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU1_REJECT_ULD]
	},
	[ POWER6_PME_PM_ITLB_REF ] = {
		.pme_name = "PM_ITLB_REF",
		.pme_code = 0x920c2,
		.pme_short_desc = "Instruction TLB reference",
		.pme_long_desc = "Instruction TLB reference",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ITLB_REF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ITLB_REF]
	},
	[ POWER6_PME_PM_LSU0_REJECT_L2MISS ] = {
		.pme_name = "PM_LSU0_REJECT_L2MISS",
		.pme_code = 0x90084,
		.pme_short_desc = "LSU0 L2 miss reject",
		.pme_long_desc = "LSU0 L2 miss reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_L2MISS]
	},
	[ POWER6_PME_PM_DATA_FROM_L35_SHR ] = {
		.pme_name = "PM_DATA_FROM_L35_SHR",
		.pme_code = 0x20005a,
		.pme_short_desc = "Data loaded from L3.5 shared",
		.pme_long_desc = "Data loaded from L3.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L35_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L35_SHR]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_RL2L3_MOD ] = {
		.pme_name = "PM_MRK_DATA_FROM_RL2L3_MOD",
		.pme_code = 0x10304c,
		.pme_short_desc = "Marked data loaded from remote L2 or L3 modified",
		.pme_long_desc = "Marked data loaded from remote L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_RL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_RL2L3_MOD]
	},
	[ POWER6_PME_PM_FPU0_FPSCR ] = {
		.pme_name = "PM_FPU0_FPSCR",
		.pme_code = 0xd0084,
		.pme_short_desc = "FPU0 executed FPSCR instruction",
		.pme_long_desc = "This signal is active for one cycle when fp0 is executing fpscr move related instruction. This could be mtfsfi*, mtfsb0*, mtfsb1*. mffs*, mtfsf*, mcrsf* where XYZ* means XYZ, XYZs, XYZ., XYZs",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FPSCR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FPSCR]
	},
	[ POWER6_PME_PM_DATA_FROM_L2 ] = {
		.pme_name = "PM_DATA_FROM_L2",
		.pme_code = 0x100058,
		.pme_short_desc = "Data loaded from L2",
		.pme_long_desc = "DL1 was reloaded from the local L2 due to a demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L2]
	},
	[ POWER6_PME_PM_DPU_HELD_XER ] = {
		.pme_name = "PM_DPU_HELD_XER",
		.pme_code = 0x20088,
		.pme_short_desc = "DISP unit held due to XER dependency",
		.pme_long_desc = "DISP unit held due to XER dependency",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_XER],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_XER]
	},
	[ POWER6_PME_PM_FAB_NODE_PUMP ] = {
		.pme_name = "PM_FAB_NODE_PUMP",
		.pme_code = 0x50188,
		.pme_short_desc = "Node pump operation",
		.pme_long_desc = " locally mastered",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_NODE_PUMP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_NODE_PUMP]
	},
	[ POWER6_PME_PM_VMX_RESULT_SAT_0_1 ] = {
		.pme_name = "PM_VMX_RESULT_SAT_0_1",
		.pme_code = 0xb008e,
		.pme_short_desc = "VMX valid result with sat bit is set (0->1)",
		.pme_long_desc = "VMX valid result with sat bit is set (0->1)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_RESULT_SAT_0_1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_RESULT_SAT_0_1]
	},
	[ POWER6_PME_PM_LD_REF_L1 ] = {
		.pme_name = "PM_LD_REF_L1",
		.pme_code = 0x80082,
		.pme_short_desc = "L1 D cache load references",
		.pme_long_desc = "Total DL1 Load references",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LD_REF_L1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LD_REF_L1]
	},
	[ POWER6_PME_PM_TLB_REF ] = {
		.pme_name = "PM_TLB_REF",
		.pme_code = 0x920c8,
		.pme_short_desc = "TLB reference",
		.pme_long_desc = "TLB reference",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_TLB_REF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_TLB_REF]
	},
	[ POWER6_PME_PM_DC_PREF_OUT_OF_STREAMS ] = {
		.pme_name = "PM_DC_PREF_OUT_OF_STREAMS",
		.pme_code = 0x810a0,
		.pme_short_desc = "D cache out of streams",
		.pme_long_desc = "out of streams",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DC_PREF_OUT_OF_STREAMS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DC_PREF_OUT_OF_STREAMS]
	},
	[ POWER6_PME_PM_FLUSH_FPU ] = {
		.pme_name = "PM_FLUSH_FPU",
		.pme_code = 0x230ec,
		.pme_short_desc = "Flush caused by FPU exception",
		.pme_long_desc = "Flush caused by FPU exception",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FLUSH_FPU],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FLUSH_FPU]
	},
	[ POWER6_PME_PM_MEM1_DP_CL_WR_LOC ] = {
		.pme_name = "PM_MEM1_DP_CL_WR_LOC",
		.pme_code = 0x5028e,
		.pme_short_desc = "cacheline write setting dp to local side 1",
		.pme_long_desc = "cacheline write setting dp to local side 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM1_DP_CL_WR_LOC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM1_DP_CL_WR_LOC]
	},
	[ POWER6_PME_PM_L2SB_LD_HIT ] = {
		.pme_name = "PM_L2SB_LD_HIT",
		.pme_code = 0x5078a,
		.pme_short_desc = "L2 slice B load hits",
		.pme_long_desc = "L2 slice B load hits",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_LD_HIT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_LD_HIT]
	},
	[ POWER6_PME_PM_FAB_DCLAIM ] = {
		.pme_name = "PM_FAB_DCLAIM",
		.pme_code = 0x50184,
		.pme_short_desc = "Dclaim operation",
		.pme_long_desc = " locally mastered",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_DCLAIM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_DCLAIM]
	},
	[ POWER6_PME_PM_MEM_DP_CL_WR_LOC ] = {
		.pme_name = "PM_MEM_DP_CL_WR_LOC",
		.pme_code = 0x150232,
		.pme_short_desc = "cache line write setting double pump state to local",
		.pme_long_desc = "cache line write setting double pump state to local",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM_DP_CL_WR_LOC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM_DP_CL_WR_LOC]
	},
	[ POWER6_PME_PM_BR_MPRED_CR ] = {
		.pme_name = "PM_BR_MPRED_CR",
		.pme_code = 0x410a8,
		.pme_short_desc = "Branch mispredictions due to CR bit setting",
		.pme_long_desc = "This signal is asserted when the branch execution unit detects a branch mispredict because the CR value is opposite of the predicted value. This signal is asserted after a branch issue event and will result in a branch redirect flush if not overridden by a flush of an older instruction.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_MPRED_CR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_MPRED_CR]
	},
	[ POWER6_PME_PM_LSU_REJECT_EXTERN ] = {
		.pme_name = "PM_LSU_REJECT_EXTERN",
		.pme_code = 0x3a1030,
		.pme_short_desc = "LSU external reject request ",
		.pme_long_desc = "LSU external reject request ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_EXTERN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_EXTERN]
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_MOD ] = {
		.pme_name = "PM_DATA_FROM_RL2L3_MOD",
		.pme_code = 0x10005c,
		.pme_short_desc = "Data loaded from remote L2 or L3 modified",
		.pme_long_desc = "Data loaded from remote L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_RL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_RL2L3_MOD]
	},
	[ POWER6_PME_PM_DPU_HELD_RU_WQ ] = {
		.pme_name = "PM_DPU_HELD_RU_WQ",
		.pme_code = 0x2008e,
		.pme_short_desc = "DISP unit held due to RU FXU write queue full",
		.pme_long_desc = "DISP unit held due to RU FXU write queue full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_RU_WQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_RU_WQ]
	},
	[ POWER6_PME_PM_LD_MISS_L1 ] = {
		.pme_name = "PM_LD_MISS_L1",
		.pme_code = 0x80080,
		.pme_short_desc = "L1 D cache load misses",
		.pme_long_desc = "Total DL1 Load references that miss the DL1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LD_MISS_L1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LD_MISS_L1]
	},
	[ POWER6_PME_PM_DC_INV_L2 ] = {
		.pme_name = "PM_DC_INV_L2",
		.pme_code = 0x150632,
		.pme_short_desc = "L1 D cache entries invalidated from L2",
		.pme_long_desc = "A dcache invalidated was received from the L2 because a line in L2 was castout.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DC_INV_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DC_INV_L2]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_RMEM ] = {
		.pme_name = "PM_MRK_PTEG_FROM_RMEM",
		.pme_code = 0x312042,
		.pme_short_desc = "Marked PTEG loaded from remote memory",
		.pme_long_desc = "Marked PTEG loaded from remote memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_RMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_RMEM]
	},
	[ POWER6_PME_PM_FPU_FIN ] = {
		.pme_name = "PM_FPU_FIN",
		.pme_code = 0x1d0030,
		.pme_short_desc = "FPU produced a result",
		.pme_long_desc = "FPU finished, produced a result This only indicates finish, not completion. Combined Unit 0 + Unit 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FIN]
	},
	[ POWER6_PME_PM_FXU0_FIN ] = {
		.pme_name = "PM_FXU0_FIN",
		.pme_code = 0x300016,
		.pme_short_desc = "FXU0 produced a result",
		.pme_long_desc = "The Fixed Point unit 0 finished an instruction and produced a result",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FXU0_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FXU0_FIN]
	},
	[ POWER6_PME_PM_DPU_HELD_FPQ ] = {
		.pme_name = "PM_DPU_HELD_FPQ",
		.pme_code = 0x20086,
		.pme_short_desc = "DISP unit held due to FPU issue queue full",
		.pme_long_desc = "DISP unit held due to FPU issue queue full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_FPQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_FPQ]
	},
	[ POWER6_PME_PM_GX_DMA_READ ] = {
		.pme_name = "PM_GX_DMA_READ",
		.pme_code = 0x5038c,
		.pme_short_desc = "DMA Read Request",
		.pme_long_desc = "DMA Read Request",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GX_DMA_READ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GX_DMA_READ]
	},
	[ POWER6_PME_PM_LSU1_REJECT_PARTIAL_SECTOR ] = {
		.pme_name = "PM_LSU1_REJECT_PARTIAL_SECTOR",
		.pme_code = 0xa008e,
		.pme_short_desc = "LSU1 reject due to partial sector valid",
		.pme_long_desc = "LSU1 reject due to partial sector valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_PARTIAL_SECTOR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_PARTIAL_SECTOR]
	},
	[ POWER6_PME_PM_0INST_FETCH_COUNT ] = {
		.pme_name = "PM_0INST_FETCH_COUNT",
		.pme_code = 0x40081,
		.pme_short_desc = "Periods with no instructions fetched",
		.pme_long_desc = "No instructions were fetched this periods (due to IFU hold, redirect, or icache miss)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_0INST_FETCH_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_0INST_FETCH_COUNT]
	},
	[ POWER6_PME_PM_PMC5_OVERFLOW ] = {
		.pme_name = "PM_PMC5_OVERFLOW",
		.pme_code = 0x100024,
		.pme_short_desc = "PMC5 Overflow",
		.pme_long_desc = "PMC5 Overflow",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC5_OVERFLOW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC5_OVERFLOW]
	},
	[ POWER6_PME_PM_L2SB_LD_REQ ] = {
		.pme_name = "PM_L2SB_LD_REQ",
		.pme_code = 0x50788,
		.pme_short_desc = "L2 slice B load requests ",
		.pme_long_desc = "L2 slice B load requests ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_LD_REQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_LD_REQ]
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_0_CYC ] = {
		.pme_name = "PM_THRD_PRIO_DIFF_0_CYC",
		.pme_code = 0x123040,
		.pme_short_desc = "Cycles no thread priority difference",
		.pme_long_desc = "Cycles no thread priority difference",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_DIFF_0_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_DIFF_0_CYC]
	},
	[ POWER6_PME_PM_DATA_FROM_RMEM ] = {
		.pme_name = "PM_DATA_FROM_RMEM",
		.pme_code = 0x30005e,
		.pme_short_desc = "Data loaded from remote memory",
		.pme_long_desc = "Data loaded from remote memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_RMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_RMEM]
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_CYC ] = {
		.pme_name = "PM_LSU_LMQ_SRQ_EMPTY_BOTH_CYC",
		.pme_code = 0x30001c,
		.pme_short_desc = "Cycles both threads LMQ and SRQ empty",
		.pme_long_desc = "Cycles both threads LMQ and SRQ empty",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_CYC]
	},
	[ POWER6_PME_PM_ST_REF_L1_BOTH ] = {
		.pme_name = "PM_ST_REF_L1_BOTH",
		.pme_code = 0x280038,
		.pme_short_desc = "Both units L1 D cache store reference",
		.pme_long_desc = "Both units L1 D cache store reference",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ST_REF_L1_BOTH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ST_REF_L1_BOTH]
	},
	[ POWER6_PME_PM_VMX_PERMUTE_ISSUED ] = {
		.pme_name = "PM_VMX_PERMUTE_ISSUED",
		.pme_code = 0x70086,
		.pme_short_desc = "VMX instruction issued to permute",
		.pme_long_desc = "VMX instruction issued to permute",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_PERMUTE_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_PERMUTE_ISSUED]
	},
	[ POWER6_PME_PM_BR_TAKEN ] = {
		.pme_name = "PM_BR_TAKEN",
		.pme_code = 0x200052,
		.pme_short_desc = "Branches taken",
		.pme_long_desc = "Branches taken",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_TAKEN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_TAKEN]
	},
	[ POWER6_PME_PM_FAB_DMA ] = {
		.pme_name = "PM_FAB_DMA",
		.pme_code = 0x5018c,
		.pme_short_desc = "DMA operation",
		.pme_long_desc = " locally mastered",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_DMA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_DMA]
	},
	[ POWER6_PME_PM_GCT_EMPTY_COUNT ] = {
		.pme_name = "PM_GCT_EMPTY_COUNT",
		.pme_code = 0x200009,
		.pme_short_desc = "Periods GCT empty",
		.pme_long_desc = "The Global Completion Table is completely empty.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GCT_EMPTY_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GCT_EMPTY_COUNT]
	},
	[ POWER6_PME_PM_FPU1_SINGLE ] = {
		.pme_name = "PM_FPU1_SINGLE",
		.pme_code = 0xc10ae,
		.pme_short_desc = "FPU1 executed single precision instruction",
		.pme_long_desc = "This signal is active for one cycle when fp1 is executing single precision instruction.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_SINGLE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_SINGLE]
	},
	[ POWER6_PME_PM_L2SA_CASTOUT_SHR ] = {
		.pme_name = "PM_L2SA_CASTOUT_SHR",
		.pme_code = 0x50682,
		.pme_short_desc = "L2 slice A castouts - Shared",
		.pme_long_desc = "L2 slice A castouts - Shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_CASTOUT_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_CASTOUT_SHR]
	},
	[ POWER6_PME_PM_L3SB_REF ] = {
		.pme_name = "PM_L3SB_REF",
		.pme_code = 0x50088,
		.pme_short_desc = "L3 slice B references",
		.pme_long_desc = "L3 slice B references",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L3SB_REF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L3SB_REF]
	},
	[ POWER6_PME_PM_FPU0_FRSP ] = {
		.pme_name = "PM_FPU0_FRSP",
		.pme_code = 0xd10a2,
		.pme_short_desc = "FPU0 executed FRSP instruction",
		.pme_long_desc = "FPU0 executed FRSP instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FRSP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FRSP]
	},
	[ POWER6_PME_PM_PMC4_SAVED ] = {
		.pme_name = "PM_PMC4_SAVED",
		.pme_code = 0x300022,
		.pme_short_desc = "PMC4 rewind value saved",
		.pme_long_desc = "PMC4 rewind value saved",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC4_SAVED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC4_SAVED]
	},
	[ POWER6_PME_PM_L2SA_DC_INV ] = {
		.pme_name = "PM_L2SA_DC_INV",
		.pme_code = 0x50686,
		.pme_short_desc = "L2 slice A D cache invalidate",
		.pme_long_desc = "L2 slice A D cache invalidate",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_DC_INV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_DC_INV]
	},
	[ POWER6_PME_PM_GXI_ADDR_CYC_BUSY ] = {
		.pme_name = "PM_GXI_ADDR_CYC_BUSY",
		.pme_code = 0x50388,
		.pme_short_desc = "Inbound GX address utilization (# of cycle address is in valid)",
		.pme_long_desc = "Inbound GX address utilization (# of cycle address is in valid)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GXI_ADDR_CYC_BUSY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GXI_ADDR_CYC_BUSY]
	},
	[ POWER6_PME_PM_FPU0_FMA ] = {
		.pme_name = "PM_FPU0_FMA",
		.pme_code = 0xc0082,
		.pme_short_desc = "FPU0 executed multiply-add instruction",
		.pme_long_desc = "This signal is active for one cycle when fp0 is executing multiply-add kind of instruction. This could be fmadd*, fnmadd*, fmsub*, fnmsub* where XYZ* means XYZ, XYZs, XYZ., XYZs.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FMA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FMA]
	},
	[ POWER6_PME_PM_SLB_MISS ] = {
		.pme_name = "PM_SLB_MISS",
		.pme_code = 0x183034,
		.pme_short_desc = "SLB misses",
		.pme_long_desc = "SLB misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_SLB_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_SLB_MISS]
	},
	[ POWER6_PME_PM_MRK_ST_GPS ] = {
		.pme_name = "PM_MRK_ST_GPS",
		.pme_code = 0x200006,
		.pme_short_desc = "Marked store sent to GPS",
		.pme_long_desc = "A sampled store has been sent to the memory subsystem",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_ST_GPS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_ST_GPS]
	},
	[ POWER6_PME_PM_DERAT_REF_4K ] = {
		.pme_name = "PM_DERAT_REF_4K",
		.pme_code = 0x182070,
		.pme_short_desc = "DERAT reference for 4K page",
		.pme_long_desc = "DERAT reference for 4K page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_REF_4K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_REF_4K]
	},
	[ POWER6_PME_PM_L2_CASTOUT_SHR ] = {
		.pme_name = "PM_L2_CASTOUT_SHR",
		.pme_code = 0x250630,
		.pme_short_desc = "L2 castouts - Shared (T",
		.pme_long_desc = " Te",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_CASTOUT_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_CASTOUT_SHR]
	},
	[ POWER6_PME_PM_DPU_HELD_STCX_CR ] = {
		.pme_name = "PM_DPU_HELD_STCX_CR",
		.pme_code = 0x2008c,
		.pme_short_desc = "DISP unit held due to STCX updating CR ",
		.pme_long_desc = "DISP unit held due to STCX updating CR ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_STCX_CR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_STCX_CR]
	},
	[ POWER6_PME_PM_FPU0_ST_FOLDED ] = {
		.pme_name = "PM_FPU0_ST_FOLDED",
		.pme_code = 0xd10a4,
		.pme_short_desc = "FPU0 folded store",
		.pme_long_desc = "FPU0 folded store",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_ST_FOLDED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_ST_FOLDED]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L21 ] = {
		.pme_name = "PM_MRK_DATA_FROM_L21",
		.pme_code = 0x203048,
		.pme_short_desc = "Marked data loaded from private L2 other core",
		.pme_long_desc = "Marked data loaded from private L2 other core",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L21],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L21]
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus3or4_CYC ] = {
		.pme_name = "PM_THRD_PRIO_DIFF_minus3or4_CYC",
		.pme_code = 0x323046,
		.pme_short_desc = "Cycles thread priority difference is -3 or -4",
		.pme_long_desc = "Cycles thread priority difference is -3 or -4",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_DIFF_minus3or4_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_DIFF_minus3or4_CYC]
	},
	[ POWER6_PME_PM_DATA_FROM_L35_MOD ] = {
		.pme_name = "PM_DATA_FROM_L35_MOD",
		.pme_code = 0x10005a,
		.pme_short_desc = "Data loaded from L3.5 modified",
		.pme_long_desc = "Data loaded from L3.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L35_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L35_MOD]
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_SHR ] = {
		.pme_name = "PM_DATA_FROM_DL2L3_SHR",
		.pme_code = 0x30005c,
		.pme_short_desc = "Data loaded from distant L2 or L3 shared",
		.pme_long_desc = "Data loaded from distant L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_DL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_DL2L3_SHR]
	},
	[ POWER6_PME_PM_GXI_DATA_CYC_BUSY ] = {
		.pme_name = "PM_GXI_DATA_CYC_BUSY",
		.pme_code = 0x5038a,
		.pme_short_desc = "Inbound GX Data utilization (# of cycle data in is valid)",
		.pme_long_desc = "Inbound GX Data utilization (# of cycle data in is valid)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GXI_DATA_CYC_BUSY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GXI_DATA_CYC_BUSY]
	},
	[ POWER6_PME_PM_LSU_REJECT_STEAL ] = {
		.pme_name = "PM_LSU_REJECT_STEAL",
		.pme_code = 0x9008c,
		.pme_short_desc = "LSU reject due to steal",
		.pme_long_desc = "LSU reject due to steal",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_STEAL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_STEAL]
	},
	[ POWER6_PME_PM_ST_FIN ] = {
		.pme_name = "PM_ST_FIN",
		.pme_code = 0x100054,
		.pme_short_desc = "Store instructions finished",
		.pme_long_desc = "Store instructions finished",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ST_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ST_FIN]
	},
	[ POWER6_PME_PM_DPU_HELD_CR_LOGICAL ] = {
		.pme_name = "PM_DPU_HELD_CR_LOGICAL",
		.pme_code = 0x3008e,
		.pme_short_desc = "DISP unit held due to CR",
		.pme_long_desc = " LR or CTR updated by CR logical",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_CR_LOGICAL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_CR_LOGICAL]
	},
	[ POWER6_PME_PM_THRD_SEL_T0 ] = {
		.pme_name = "PM_THRD_SEL_T0",
		.pme_code = 0x310a6,
		.pme_short_desc = "Decode selected thread 0",
		.pme_long_desc = "Decode selected thread 0",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_SEL_T0],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_SEL_T0]
	},
	[ POWER6_PME_PM_PTEG_RELOAD_VALID ] = {
		.pme_name = "PM_PTEG_RELOAD_VALID",
		.pme_code = 0x130e8,
		.pme_short_desc = "TLB reload valid",
		.pme_long_desc = "TLB reload valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_RELOAD_VALID],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_RELOAD_VALID]
	},
	[ POWER6_PME_PM_L2_PREF_ST ] = {
		.pme_name = "PM_L2_PREF_ST",
		.pme_code = 0x810a8,
		.pme_short_desc = "L2 cache prefetches",
		.pme_long_desc = "L2 cache prefetches",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_PREF_ST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_PREF_ST]
	},
	[ POWER6_PME_PM_MRK_STCX_FAIL ] = {
		.pme_name = "PM_MRK_STCX_FAIL",
		.pme_code = 0x830e4,
		.pme_short_desc = "Marked STCX failed",
		.pme_long_desc = "A marked stcx (stwcx or stdcx) failed",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_STCX_FAIL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_STCX_FAIL]
	},
	[ POWER6_PME_PM_LSU0_REJECT_LHS ] = {
		.pme_name = "PM_LSU0_REJECT_LHS",
		.pme_code = 0x90086,
		.pme_short_desc = "LSU0 load hit store reject",
		.pme_long_desc = "LSU0 load hit store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_LHS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_LHS]
	},
	[ POWER6_PME_PM_DFU_EXP_EQ ] = {
		.pme_name = "PM_DFU_EXP_EQ",
		.pme_code = 0xe0084,
		.pme_short_desc = "DFU operand exponents are equal for add type",
		.pme_long_desc = "DFU operand exponents are equal for add type",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_EXP_EQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_EXP_EQ]
	},
	[ POWER6_PME_PM_DPU_HELD_FP_FX_MULT ] = {
		.pme_name = "PM_DPU_HELD_FP_FX_MULT",
		.pme_code = 0x210a8,
		.pme_short_desc = "DISP unit held due to non fixed multiple/divide after fixed multiply/divide",
		.pme_long_desc = "DISP unit held due to non fixed multiple/divide after fixed multiply/divide",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_FP_FX_MULT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_FP_FX_MULT]
	},
	[ POWER6_PME_PM_L2_LD_MISS_DATA ] = {
		.pme_name = "PM_L2_LD_MISS_DATA",
		.pme_code = 0x250430,
		.pme_short_desc = "L2 data load misses",
		.pme_long_desc = "L2 data load misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_LD_MISS_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_LD_MISS_DATA]
	},
	[ POWER6_PME_PM_DATA_FROM_L35_MOD_CYC ] = {
		.pme_name = "PM_DATA_FROM_L35_MOD_CYC",
		.pme_code = 0x400026,
		.pme_short_desc = "Load latency from L3.5 modified",
		.pme_long_desc = "Load latency from L3.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L35_MOD_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L35_MOD_CYC]
	},
	[ POWER6_PME_PM_FLUSH_FXU ] = {
		.pme_name = "PM_FLUSH_FXU",
		.pme_code = 0x230ea,
		.pme_short_desc = "Flush caused by FXU exception",
		.pme_long_desc = "Flush caused by FXU exception",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FLUSH_FXU],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FLUSH_FXU]
	},
	[ POWER6_PME_PM_FPU_ISSUE_1 ] = {
		.pme_name = "PM_FPU_ISSUE_1",
		.pme_code = 0x320c8,
		.pme_short_desc = "FPU issue 1 per cycle",
		.pme_long_desc = "FPU issue 1 per cycle",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_1]
	},
	[ POWER6_PME_PM_DATA_FROM_LMEM_CYC ] = {
		.pme_name = "PM_DATA_FROM_LMEM_CYC",
		.pme_code = 0x20002c,
		.pme_short_desc = "Load latency from local memory",
		.pme_long_desc = "Load latency from local memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_LMEM_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_LMEM_CYC]
	},
	[ POWER6_PME_PM_DPU_HELD_LSU_SOPS ] = {
		.pme_name = "PM_DPU_HELD_LSU_SOPS",
		.pme_code = 0x30080,
		.pme_short_desc = "DISP unit held due to LSU slow ops (sync",
		.pme_long_desc = " tlbie",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_LSU_SOPS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_LSU_SOPS]
	},
	[ POWER6_PME_PM_INST_PTEG_2ND_HALF ] = {
		.pme_name = "PM_INST_PTEG_2ND_HALF",
		.pme_code = 0x910aa,
		.pme_short_desc = "Instruction table walk matched in second half primary PTEG",
		.pme_long_desc = "Instruction table walk matched in second half primary PTEG",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_PTEG_2ND_HALF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_PTEG_2ND_HALF]
	},
	[ POWER6_PME_PM_THRESH_TIMEO ] = {
		.pme_name = "PM_THRESH_TIMEO",
		.pme_code = 0x300018,
		.pme_short_desc = "Threshold timeout",
		.pme_long_desc = "The threshold timer expired",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRESH_TIMEO],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRESH_TIMEO]
	},
	[ POWER6_PME_PM_LSU_REJECT_UST_BOTH ] = {
		.pme_name = "PM_LSU_REJECT_UST_BOTH",
		.pme_code = 0x190036,
		.pme_short_desc = "Unaligned store reject both units",
		.pme_long_desc = "Unaligned store reject both units",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_UST_BOTH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_UST_BOTH]
	},
	[ POWER6_PME_PM_LSU_REJECT_FAST ] = {
		.pme_name = "PM_LSU_REJECT_FAST",
		.pme_code = 0x30003e,
		.pme_short_desc = "LSU fast reject",
		.pme_long_desc = "LSU fast reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_FAST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_FAST]
	},
	[ POWER6_PME_PM_DPU_HELD_THRD_PRIO ] = {
		.pme_name = "PM_DPU_HELD_THRD_PRIO",
		.pme_code = 0x3008a,
		.pme_short_desc = "DISP unit held due to lower priority thread",
		.pme_long_desc = "DISP unit held due to lower priority thread",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_THRD_PRIO],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_THRD_PRIO]
	},
	[ POWER6_PME_PM_L2_PREF_LD ] = {
		.pme_name = "PM_L2_PREF_LD",
		.pme_code = 0x810a6,
		.pme_short_desc = "L2 cache prefetches",
		.pme_long_desc = "L2 cache prefetches",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_PREF_LD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_PREF_LD]
	},
	[ POWER6_PME_PM_FPU_FEST ] = {
		.pme_name = "PM_FPU_FEST",
		.pme_code = 0x4d1030,
		.pme_short_desc = "FPU executed FEST instruction",
		.pme_long_desc = "This signal is active for one cycle when executing one of the estimate instructions. This could be fres* or frsqrte* where XYZ* means XYZ or XYZ. Combined Unit 0 + Unit 1.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FEST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FEST]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_RMEM ] = {
		.pme_name = "PM_MRK_DATA_FROM_RMEM",
		.pme_code = 0x30304a,
		.pme_short_desc = "Marked data loaded from remote memory",
		.pme_long_desc = "Marked data loaded from remote memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_RMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_RMEM]
	},
	[ POWER6_PME_PM_LD_MISS_L1_CYC ] = {
		.pme_name = "PM_LD_MISS_L1_CYC",
		.pme_code = 0x10000c,
		.pme_short_desc = "L1 data load miss cycles",
		.pme_long_desc = "L1 data load miss cycles",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LD_MISS_L1_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LD_MISS_L1_CYC]
	},
	[ POWER6_PME_PM_DERAT_MISS_4K ] = {
		.pme_name = "PM_DERAT_MISS_4K",
		.pme_code = 0x192070,
		.pme_short_desc = "DERAT misses for 4K page",
		.pme_long_desc = "A data request (load or store) missed the ERAT for 4K page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_MISS_4K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_MISS_4K]
	},
	[ POWER6_PME_PM_DPU_HELD_COMPLETION ] = {
		.pme_name = "PM_DPU_HELD_COMPLETION",
		.pme_code = 0x210ac,
		.pme_short_desc = "DISP unit held due to completion holding dispatch ",
		.pme_long_desc = "DISP unit held due to completion holding dispatch ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_COMPLETION],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_COMPLETION]
	},
	[ POWER6_PME_PM_FPU_ISSUE_STALL_ST ] = {
		.pme_name = "PM_FPU_ISSUE_STALL_ST",
		.pme_code = 0x320ce,
		.pme_short_desc = "FPU issue stalled due to store",
		.pme_long_desc = "FPU issue stalled due to store",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_STALL_ST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_STALL_ST]
	},
	[ POWER6_PME_PM_L2SB_DC_INV ] = {
		.pme_name = "PM_L2SB_DC_INV",
		.pme_code = 0x5068e,
		.pme_short_desc = "L2 slice B D cache invalidate",
		.pme_long_desc = "L2 slice B D cache invalidate",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_DC_INV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_DC_INV]
	},
	[ POWER6_PME_PM_PTEG_FROM_L25_SHR ] = {
		.pme_name = "PM_PTEG_FROM_L25_SHR",
		.pme_code = 0x41304e,
		.pme_short_desc = "PTEG loaded from L2.5 shared",
		.pme_long_desc = "PTEG loaded from L2.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L25_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L25_SHR]
	},
	[ POWER6_PME_PM_PTEG_FROM_DL2L3_MOD ] = {
		.pme_name = "PM_PTEG_FROM_DL2L3_MOD",
		.pme_code = 0x41304c,
		.pme_short_desc = "PTEG loaded from distant L2 or L3 modified",
		.pme_long_desc = "PTEG loaded from distant L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_DL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_DL2L3_MOD]
	},
	[ POWER6_PME_PM_FAB_CMD_RETRIED ] = {
		.pme_name = "PM_FAB_CMD_RETRIED",
		.pme_code = 0x250130,
		.pme_short_desc = "Fabric command retried",
		.pme_long_desc = "Fabric command retried",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_CMD_RETRIED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_CMD_RETRIED]
	},
	[ POWER6_PME_PM_BR_PRED_LSTACK ] = {
		.pme_name = "PM_BR_PRED_LSTACK",
		.pme_code = 0x410a6,
		.pme_short_desc = "A conditional branch was predicted",
		.pme_long_desc = " link stack",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_PRED_LSTACK],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_PRED_LSTACK]
	},
	[ POWER6_PME_PM_GXO_DATA_CYC_BUSY ] = {
		.pme_name = "PM_GXO_DATA_CYC_BUSY",
		.pme_code = 0x50384,
		.pme_short_desc = "Outbound GX Data utilization (# of cycles data out is valid)",
		.pme_long_desc = "Outbound GX Data utilization (# of cycles data out is valid)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GXO_DATA_CYC_BUSY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GXO_DATA_CYC_BUSY]
	},
	[ POWER6_PME_PM_DFU_SUBNORM ] = {
		.pme_name = "PM_DFU_SUBNORM",
		.pme_code = 0xe0086,
		.pme_short_desc = "DFU result is a subnormal",
		.pme_long_desc = "DFU result is a subnormal",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_SUBNORM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_SUBNORM]
	},
	[ POWER6_PME_PM_FPU_ISSUE_OOO ] = {
		.pme_name = "PM_FPU_ISSUE_OOO",
		.pme_code = 0x320c0,
		.pme_short_desc = "FPU issue out-of-order",
		.pme_long_desc = "FPU issue out-of-order",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_OOO],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_OOO]
	},
	[ POWER6_PME_PM_LSU_REJECT_ULD_BOTH ] = {
		.pme_name = "PM_LSU_REJECT_ULD_BOTH",
		.pme_code = 0x290036,
		.pme_short_desc = "Unaligned load reject both units",
		.pme_long_desc = "Unaligned load reject both units",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_ULD_BOTH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_ULD_BOTH]
	},
	[ POWER6_PME_PM_L2SB_ST_MISS ] = {
		.pme_name = "PM_L2SB_ST_MISS",
		.pme_code = 0x5048e,
		.pme_short_desc = "L2 slice B store misses",
		.pme_long_desc = "L2 slice B store misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_ST_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_ST_MISS]
	},
	[ POWER6_PME_PM_DATA_FROM_L25_MOD_CYC ] = {
		.pme_name = "PM_DATA_FROM_L25_MOD_CYC",
		.pme_code = 0x400024,
		.pme_short_desc = "Load latency from L2.5 modified",
		.pme_long_desc = "Load latency from L2.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L25_MOD_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L25_MOD_CYC]
	},
	[ POWER6_PME_PM_INST_PTEG_1ST_HALF ] = {
		.pme_name = "PM_INST_PTEG_1ST_HALF",
		.pme_code = 0x910a8,
		.pme_short_desc = "Instruction table walk matched in first half primary PTEG",
		.pme_long_desc = "Instruction table walk matched in first half primary PTEG",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_PTEG_1ST_HALF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_PTEG_1ST_HALF]
	},
	[ POWER6_PME_PM_DERAT_MISS_16M ] = {
		.pme_name = "PM_DERAT_MISS_16M",
		.pme_code = 0x392070,
		.pme_short_desc = "DERAT misses for 16M page",
		.pme_long_desc = "A data request (load or store) missed the ERAT for 16M page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_MISS_16M],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_MISS_16M]
	},
	[ POWER6_PME_PM_GX_DMA_WRITE ] = {
		.pme_name = "PM_GX_DMA_WRITE",
		.pme_code = 0x5038e,
		.pme_short_desc = "All DMA Write Requests (including dma wrt lgcy)",
		.pme_long_desc = "All DMA Write Requests (including dma wrt lgcy)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GX_DMA_WRITE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GX_DMA_WRITE]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_MOD ] = {
		.pme_name = "PM_MRK_PTEG_FROM_DL2L3_MOD",
		.pme_code = 0x412044,
		.pme_short_desc = "Marked PTEG loaded from distant L2 or L3 modified",
		.pme_long_desc = "Marked PTEG loaded from distant L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_MOD]
	},
	[ POWER6_PME_PM_MEM1_DP_RQ_GLOB_LOC ] = {
		.pme_name = "PM_MEM1_DP_RQ_GLOB_LOC",
		.pme_code = 0x50288,
		.pme_short_desc = "Memory read queue marking cache line double pump state from global to local side 1",
		.pme_long_desc = "Memory read queue marking cache line double pump state from global to local side 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM1_DP_RQ_GLOB_LOC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM1_DP_RQ_GLOB_LOC]
	},
	[ POWER6_PME_PM_L2SB_LD_REQ_DATA ] = {
		.pme_name = "PM_L2SB_LD_REQ_DATA",
		.pme_code = 0x50488,
		.pme_short_desc = "L2 slice B data load requests",
		.pme_long_desc = "L2 slice B data load requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_LD_REQ_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_LD_REQ_DATA]
	},
	[ POWER6_PME_PM_L2SA_LD_MISS_INST ] = {
		.pme_name = "PM_L2SA_LD_MISS_INST",
		.pme_code = 0x50582,
		.pme_short_desc = "L2 slice A instruction load misses",
		.pme_long_desc = "L2 slice A instruction load misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_LD_MISS_INST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_LD_MISS_INST]
	},
	[ POWER6_PME_PM_MRK_LSU0_REJECT_L2MISS ] = {
		.pme_name = "PM_MRK_LSU0_REJECT_L2MISS",
		.pme_code = 0x930e4,
		.pme_short_desc = "LSU0 marked L2 miss reject",
		.pme_long_desc = "LSU0 marked L2 miss reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU0_REJECT_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU0_REJECT_L2MISS]
	},
	[ POWER6_PME_PM_MRK_IFU_FIN ] = {
		.pme_name = "PM_MRK_IFU_FIN",
		.pme_code = 0x20000a,
		.pme_short_desc = "Marked instruction IFU processing finished",
		.pme_long_desc = "Marked instruction IFU processing finished",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_IFU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_IFU_FIN]
	},
	[ POWER6_PME_PM_INST_FROM_L3 ] = {
		.pme_name = "PM_INST_FROM_L3",
		.pme_code = 0x342040,
		.pme_short_desc = "Instruction fetched from L3",
		.pme_long_desc = "An instruction fetch group was fetched from L3. Fetch Groups can contain up to 8 instructions",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L3],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L3]
	},
	[ POWER6_PME_PM_FXU1_FIN ] = {
		.pme_name = "PM_FXU1_FIN",
		.pme_code = 0x400016,
		.pme_short_desc = "FXU1 produced a result",
		.pme_long_desc = "The Fixed Point unit 1 finished an instruction and produced a result",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FXU1_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FXU1_FIN]
	},
	[ POWER6_PME_PM_THRD_PRIO_4_CYC ] = {
		.pme_name = "PM_THRD_PRIO_4_CYC",
		.pme_code = 0x422046,
		.pme_short_desc = "Cycles thread running at priority level 4",
		.pme_long_desc = "Cycles thread running at priority level 4",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_4_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_4_CYC]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L35_MOD ] = {
		.pme_name = "PM_MRK_DATA_FROM_L35_MOD",
		.pme_code = 0x10304e,
		.pme_short_desc = "Marked data loaded from L3.5 modified",
		.pme_long_desc = "Marked data loaded from L3.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L35_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L35_MOD]
	},
	[ POWER6_PME_PM_LSU_REJECT_SET_MPRED ] = {
		.pme_name = "PM_LSU_REJECT_SET_MPRED",
		.pme_code = 0x2a0032,
		.pme_short_desc = "LSU reject due to mispredicted set",
		.pme_long_desc = "LSU reject due to mispredicted set",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_SET_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_SET_MPRED]
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_16G ] = {
		.pme_name = "PM_MRK_DERAT_MISS_16G",
		.pme_code = 0x492044,
		.pme_short_desc = "Marked DERAT misses for 16G page",
		.pme_long_desc = "A marked data request (load or store) missed the ERAT for 16G page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_MISS_16G],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_MISS_16G]
	},
	[ POWER6_PME_PM_FPU0_FXDIV ] = {
		.pme_name = "PM_FPU0_FXDIV",
		.pme_code = 0xc10a0,
		.pme_short_desc = "FPU0 executed fixed point division",
		.pme_long_desc = "FPU0 executed fixed point division",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FXDIV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FXDIV]
	},
	[ POWER6_PME_PM_MRK_LSU1_REJECT_UST ] = {
		.pme_name = "PM_MRK_LSU1_REJECT_UST",
		.pme_code = 0x930ea,
		.pme_short_desc = "LSU1 marked unaligned store reject",
		.pme_long_desc = "LSU1 marked unaligned store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU1_REJECT_UST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU1_REJECT_UST]
	},
	[ POWER6_PME_PM_FPU_ISSUE_DIV_SQRT_OVERLAP ] = {
		.pme_name = "PM_FPU_ISSUE_DIV_SQRT_OVERLAP",
		.pme_code = 0x320cc,
		.pme_short_desc = "FPU divide/sqrt overlapped with other divide/sqrt",
		.pme_long_desc = "FPU divide/sqrt overlapped with other divide/sqrt",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_DIV_SQRT_OVERLAP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_DIV_SQRT_OVERLAP]
	},
	[ POWER6_PME_PM_INST_FROM_L35_SHR ] = {
		.pme_name = "PM_INST_FROM_L35_SHR",
		.pme_code = 0x242046,
		.pme_short_desc = "Instruction fetched from L3.5 shared",
		.pme_long_desc = "Instruction fetched from L3.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L35_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L35_SHR]
	},
	[ POWER6_PME_PM_MRK_LSU_REJECT_LHS ] = {
		.pme_name = "PM_MRK_LSU_REJECT_LHS",
		.pme_code = 0x493030,
		.pme_short_desc = "Marked load hit store reject",
		.pme_long_desc = "Marked load hit store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU_REJECT_LHS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU_REJECT_LHS]
	},
	[ POWER6_PME_PM_LSU_LMQ_FULL_CYC ] = {
		.pme_name = "PM_LSU_LMQ_FULL_CYC",
		.pme_code = 0x810ac,
		.pme_short_desc = "Cycles LMQ full",
		.pme_long_desc = "The LMQ was full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_LMQ_FULL_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_LMQ_FULL_CYC]
	},
	[ POWER6_PME_PM_SYNC_COUNT ] = {
		.pme_name = "PM_SYNC_COUNT",
		.pme_code = 0x920cd,
		.pme_short_desc = "SYNC instructions completed",
		.pme_long_desc = "SYNC instructions completed",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_SYNC_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_SYNC_COUNT]
	},
	[ POWER6_PME_PM_MEM0_DP_RQ_LOC_GLOB ] = {
		.pme_name = "PM_MEM0_DP_RQ_LOC_GLOB",
		.pme_code = 0x50282,
		.pme_short_desc = "Memory read queue marking cache line double pump state from local to global side 0",
		.pme_long_desc = "Memory read queue marking cache line double pump state from local to global side 0",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM0_DP_RQ_LOC_GLOB],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM0_DP_RQ_LOC_GLOB]
	},
	[ POWER6_PME_PM_L2SA_CASTOUT_MOD ] = {
		.pme_name = "PM_L2SA_CASTOUT_MOD",
		.pme_code = 0x50680,
		.pme_short_desc = "L2 slice A castouts - Modified",
		.pme_long_desc = "L2 slice A castouts - Modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_CASTOUT_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_CASTOUT_MOD]
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_COUNT ] = {
		.pme_name = "PM_LSU_LMQ_SRQ_EMPTY_BOTH_COUNT",
		.pme_code = 0x30001d,
		.pme_short_desc = "Periods both threads LMQ and SRQ empty",
		.pme_long_desc = "Periods both threads LMQ and SRQ empty",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_BOTH_COUNT]
	},
	[ POWER6_PME_PM_PTEG_FROM_MEM_DP ] = {
		.pme_name = "PM_PTEG_FROM_MEM_DP",
		.pme_code = 0x11304a,
		.pme_short_desc = "PTEG loaded from double pump memory",
		.pme_long_desc = "PTEG loaded from double pump memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_MEM_DP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_MEM_DP]
	},
	[ POWER6_PME_PM_LSU_REJECT_SLOW ] = {
		.pme_name = "PM_LSU_REJECT_SLOW",
		.pme_code = 0x20003e,
		.pme_short_desc = "LSU slow reject",
		.pme_long_desc = "LSU slow reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_SLOW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_SLOW]
	},
	[ POWER6_PME_PM_PTEG_FROM_L25_MOD ] = {
		.pme_name = "PM_PTEG_FROM_L25_MOD",
		.pme_code = 0x31304e,
		.pme_short_desc = "PTEG loaded from L2.5 modified",
		.pme_long_desc = "PTEG loaded from L2.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L25_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L25_MOD]
	},
	[ POWER6_PME_PM_THRD_PRIO_7_CYC ] = {
		.pme_name = "PM_THRD_PRIO_7_CYC",
		.pme_code = 0x122046,
		.pme_short_desc = "Cycles thread running at priority level 7",
		.pme_long_desc = "Cycles thread running at priority level 7",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_7_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_7_CYC]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_SHR ] = {
		.pme_name = "PM_MRK_PTEG_FROM_RL2L3_SHR",
		.pme_code = 0x212044,
		.pme_short_desc = "Marked PTEG loaded from remote L2 or L3 shared",
		.pme_long_desc = "Marked PTEG loaded from remote L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_RL2L3_SHR]
	},
	[ POWER6_PME_PM_ST_REQ_L2 ] = {
		.pme_name = "PM_ST_REQ_L2",
		.pme_code = 0x250732,
		.pme_short_desc = "L2 store requests",
		.pme_long_desc = "L2 store requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ST_REQ_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ST_REQ_L2]
	},
	[ POWER6_PME_PM_ST_REF_L1 ] = {
		.pme_name = "PM_ST_REF_L1",
		.pme_code = 0x80086,
		.pme_short_desc = "L1 D cache store references",
		.pme_long_desc = "Total DL1 Store references",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ST_REF_L1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ST_REF_L1]
	},
	[ POWER6_PME_PM_FPU_ISSUE_STALL_THRD ] = {
		.pme_name = "PM_FPU_ISSUE_STALL_THRD",
		.pme_code = 0x330e0,
		.pme_short_desc = "FPU issue stalled due to thread resource conflict",
		.pme_long_desc = "FPU issue stalled due to thread resource conflict",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_STALL_THRD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_STALL_THRD]
	},
	[ POWER6_PME_PM_RUN_COUNT ] = {
		.pme_name = "PM_RUN_COUNT",
		.pme_code = 0x10000b,
		.pme_short_desc = "Run Periods",
		.pme_long_desc = "Processor Periods gated by the run latch",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_RUN_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_RUN_COUNT]
	},
	[ POWER6_PME_PM_RUN_CYC ] = {
		.pme_name = "PM_RUN_CYC",
		.pme_code = 0x10000a,
		.pme_short_desc = "Run cycles",
		.pme_long_desc = "Processor Cycles gated by the run latch",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_RUN_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_RUN_CYC]
	},
	[ POWER6_PME_PM_PTEG_FROM_RMEM ] = {
		.pme_name = "PM_PTEG_FROM_RMEM",
		.pme_code = 0x31304a,
		.pme_short_desc = "PTEG loaded from remote memory",
		.pme_long_desc = "PTEG loaded from remote memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_RMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_RMEM]
	},
	[ POWER6_PME_PM_LSU0_LDF ] = {
		.pme_name = "PM_LSU0_LDF",
		.pme_code = 0x80084,
		.pme_short_desc = "LSU0 executed Floating Point load instruction",
		.pme_long_desc = "A floating point load was executed from LSU unit 0",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_LDF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_LDF]
	},
	[ POWER6_PME_PM_ST_MISS_L1 ] = {
		.pme_name = "PM_ST_MISS_L1",
		.pme_code = 0x80088,
		.pme_short_desc = "L1 D cache store misses",
		.pme_long_desc = "A store missed the dcache",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_ST_MISS_L1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_ST_MISS_L1]
	},
	[ POWER6_PME_PM_INST_FROM_DL2L3_SHR ] = {
		.pme_name = "PM_INST_FROM_DL2L3_SHR",
		.pme_code = 0x342044,
		.pme_short_desc = "Instruction fetched from distant L2 or L3 shared",
		.pme_long_desc = "Instruction fetched from distant L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_DL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_DL2L3_SHR]
	},
	[ POWER6_PME_PM_L2SA_IC_INV ] = {
		.pme_name = "PM_L2SA_IC_INV",
		.pme_code = 0x50684,
		.pme_short_desc = "L2 slice A I cache invalidate",
		.pme_long_desc = "L2 slice A I cache invalidate",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_IC_INV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_IC_INV]
	},
	[ POWER6_PME_PM_THRD_ONE_RUN_CYC ] = {
		.pme_name = "PM_THRD_ONE_RUN_CYC",
		.pme_code = 0x100016,
		.pme_short_desc = "One of the threads in run cycles",
		.pme_long_desc = "One of the threads in run cycles",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_ONE_RUN_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_ONE_RUN_CYC]
	},
	[ POWER6_PME_PM_L2SB_LD_REQ_INST ] = {
		.pme_name = "PM_L2SB_LD_REQ_INST",
		.pme_code = 0x50588,
		.pme_short_desc = "L2 slice B instruction load requests",
		.pme_long_desc = "L2 slice B instruction load requests",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_LD_REQ_INST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_LD_REQ_INST]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L25_MOD ] = {
		.pme_name = "PM_MRK_DATA_FROM_L25_MOD",
		.pme_code = 0x30304e,
		.pme_short_desc = "Marked data loaded from L2.5 modified",
		.pme_long_desc = "DL1 was reloaded with modified (M) data from the L2 of a chip on this MCM due to a marked demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L25_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L25_MOD]
	},
	[ POWER6_PME_PM_DPU_HELD_XTHRD ] = {
		.pme_name = "PM_DPU_HELD_XTHRD",
		.pme_code = 0x30082,
		.pme_short_desc = "DISP unit held due to cross thread resource conflicts",
		.pme_long_desc = "DISP unit held due to cross thread resource conflicts",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_XTHRD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_XTHRD]
	},
	[ POWER6_PME_PM_L2SB_ST_REQ ] = {
		.pme_name = "PM_L2SB_ST_REQ",
		.pme_code = 0x5048c,
		.pme_short_desc = "L2 slice B store requests",
		.pme_long_desc = "A store request as seen at the L2 directory has been made from the core. Stores are counted after gathering in the L2 store queues. The event is provided on each of the three slices A,B,and C.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_ST_REQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_ST_REQ]
	},
	[ POWER6_PME_PM_INST_FROM_L21 ] = {
		.pme_name = "PM_INST_FROM_L21",
		.pme_code = 0x242040,
		.pme_short_desc = "Instruction fetched from private L2 other core",
		.pme_long_desc = "Instruction fetched from private L2 other core",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L21],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L21]
	},
	[ POWER6_PME_PM_INST_FROM_L3MISS ] = {
		.pme_name = "PM_INST_FROM_L3MISS",
		.pme_code = 0x342054,
		.pme_short_desc = "Instruction fetched missed L3",
		.pme_long_desc = "Instruction fetched missed L3",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L3MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L3MISS]
	},
	[ POWER6_PME_PM_L3SB_HIT ] = {
		.pme_name = "PM_L3SB_HIT",
		.pme_code = 0x5008a,
		.pme_short_desc = "L3 slice B hits",
		.pme_long_desc = "L3 slice B hits",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L3SB_HIT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L3SB_HIT]
	},
	[ POWER6_PME_PM_EE_OFF_EXT_INT ] = {
		.pme_name = "PM_EE_OFF_EXT_INT",
		.pme_code = 0x230ee,
		.pme_short_desc = "Cycles MSR(EE) bit off and external interrupt pending",
		.pme_long_desc = "Cycles MSR(EE) bit off and external interrupt pending",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_EE_OFF_EXT_INT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_EE_OFF_EXT_INT]
	},
	[ POWER6_PME_PM_INST_FROM_DL2L3_MOD ] = {
		.pme_name = "PM_INST_FROM_DL2L3_MOD",
		.pme_code = 0x442044,
		.pme_short_desc = "Instruction fetched from distant L2 or L3 modified",
		.pme_long_desc = "Instruction fetched from distant L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_DL2L3_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_DL2L3_MOD]
	},
	[ POWER6_PME_PM_PMC6_OVERFLOW ] = {
		.pme_name = "PM_PMC6_OVERFLOW",
		.pme_code = 0x300024,
		.pme_short_desc = "PMC6 Overflow",
		.pme_long_desc = "PMC6 Overflow",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC6_OVERFLOW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC6_OVERFLOW]
	},
	[ POWER6_PME_PM_FPU_FLOP ] = {
		.pme_name = "PM_FPU_FLOP",
		.pme_code = 0x1c0032,
		.pme_short_desc = "FPU executed 1FLOP",
		.pme_long_desc = " FMA",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FLOP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FLOP]
	},
	[ POWER6_PME_PM_FXU_BUSY ] = {
		.pme_name = "PM_FXU_BUSY",
		.pme_code = 0x200050,
		.pme_short_desc = "FXU busy",
		.pme_long_desc = "FXU0 and FXU1 are both busy",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FXU_BUSY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FXU_BUSY]
	},
	[ POWER6_PME_PM_FPU1_FLOP ] = {
		.pme_name = "PM_FPU1_FLOP",
		.pme_code = 0xc008e,
		.pme_short_desc = "FPU1 executed 1FLOP",
		.pme_long_desc = " FMA",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FLOP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FLOP]
	},
	[ POWER6_PME_PM_IC_RELOAD_SHR ] = {
		.pme_name = "PM_IC_RELOAD_SHR",
		.pme_code = 0x4008e,
		.pme_short_desc = "I cache line reloading to be shared by threads",
		.pme_long_desc = "I cache line reloading to be shared by threads",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IC_RELOAD_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IC_RELOAD_SHR]
	},
	[ POWER6_PME_PM_INST_TABLEWALK_CYC ] = {
		.pme_name = "PM_INST_TABLEWALK_CYC",
		.pme_code = 0x920ca,
		.pme_short_desc = "Cycles doing instruction tablewalks",
		.pme_long_desc = "Cycles doing instruction tablewalks",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_TABLEWALK_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_TABLEWALK_CYC]
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_MOD_CYC ] = {
		.pme_name = "PM_DATA_FROM_RL2L3_MOD_CYC",
		.pme_code = 0x400028,
		.pme_short_desc = "Load latency from remote L2 or L3 modified",
		.pme_long_desc = "Load latency from remote L2 or L3 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_RL2L3_MOD_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_RL2L3_MOD_CYC]
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_5or6_CYC ] = {
		.pme_name = "PM_THRD_PRIO_DIFF_5or6_CYC",
		.pme_code = 0x423040,
		.pme_short_desc = "Cycles thread priority difference is 5 or 6",
		.pme_long_desc = "Cycles thread priority difference is 5 or 6",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_DIFF_5or6_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_DIFF_5or6_CYC]
	},
	[ POWER6_PME_PM_IBUF_FULL_CYC ] = {
		.pme_name = "PM_IBUF_FULL_CYC",
		.pme_code = 0x40084,
		.pme_short_desc = "Cycles instruction buffer full",
		.pme_long_desc = "Cycles instruction buffer full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IBUF_FULL_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IBUF_FULL_CYC]
	},
	[ POWER6_PME_PM_L2SA_LD_REQ ] = {
		.pme_name = "PM_L2SA_LD_REQ",
		.pme_code = 0x50780,
		.pme_short_desc = "L2 slice A load requests ",
		.pme_long_desc = "L2 slice A load requests ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_LD_REQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_LD_REQ]
	},
	[ POWER6_PME_PM_VMX1_LD_WRBACK ] = {
		.pme_name = "PM_VMX1_LD_WRBACK",
		.pme_code = 0x6008c,
		.pme_short_desc = "VMX1 load writeback valid",
		.pme_long_desc = "VMX1 load writeback valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX1_LD_WRBACK],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX1_LD_WRBACK]
	},
	[ POWER6_PME_PM_MRK_FPU_FIN ] = {
		.pme_name = "PM_MRK_FPU_FIN",
		.pme_code = 0x2d0030,
		.pme_short_desc = "Marked instruction FPU processing finished",
		.pme_long_desc = "One of the Floating Point Units finished a marked instruction. Instructions that finish may not necessary complete",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_FPU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_FPU_FIN]
	},
	[ POWER6_PME_PM_THRD_PRIO_5_CYC ] = {
		.pme_name = "PM_THRD_PRIO_5_CYC",
		.pme_code = 0x322046,
		.pme_short_desc = "Cycles thread running at priority level 5",
		.pme_long_desc = "Cycles thread running at priority level 5",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_5_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_5_CYC]
	},
	[ POWER6_PME_PM_DFU_BACK2BACK ] = {
		.pme_name = "PM_DFU_BACK2BACK",
		.pme_code = 0xe0082,
		.pme_short_desc = "DFU back to back operations executed",
		.pme_long_desc = "DFU back to back operations executed",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_BACK2BACK],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_BACK2BACK]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_LMEM ] = {
		.pme_name = "PM_MRK_DATA_FROM_LMEM",
		.pme_code = 0x40304a,
		.pme_short_desc = "Marked data loaded from local memory",
		.pme_long_desc = "Marked data loaded from local memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_LMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_LMEM]
	},
	[ POWER6_PME_PM_LSU_REJECT_LHS ] = {
		.pme_name = "PM_LSU_REJECT_LHS",
		.pme_code = 0x190032,
		.pme_short_desc = "Load hit store reject",
		.pme_long_desc = "Load hit store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_LHS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_LHS]
	},
	[ POWER6_PME_PM_DPU_HELD_SPR ] = {
		.pme_name = "PM_DPU_HELD_SPR",
		.pme_code = 0x3008c,
		.pme_short_desc = "DISP unit held due to MTSPR/MFSPR",
		.pme_long_desc = "DISP unit held due to MTSPR/MFSPR",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_SPR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_SPR]
	},
	[ POWER6_PME_PM_FREQ_DOWN ] = {
		.pme_name = "PM_FREQ_DOWN",
		.pme_code = 0x30003c,
		.pme_short_desc = "Frequency is being slewed down due to Power Management",
		.pme_long_desc = "Frequency is being slewed down due to Power Management",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FREQ_DOWN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FREQ_DOWN]
	},
	[ POWER6_PME_PM_DFU_ENC_BCD_DPD ] = {
		.pme_name = "PM_DFU_ENC_BCD_DPD",
		.pme_code = 0xe008a,
		.pme_short_desc = "DFU Encode BCD to DPD",
		.pme_long_desc = "DFU Encode BCD to DPD",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_ENC_BCD_DPD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_ENC_BCD_DPD]
	},
	[ POWER6_PME_PM_DPU_HELD_GPR ] = {
		.pme_name = "PM_DPU_HELD_GPR",
		.pme_code = 0x20080,
		.pme_short_desc = "DISP unit held due to GPR dependencies",
		.pme_long_desc = "DISP unit held due to GPR dependencies",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_GPR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_GPR]
	},
	[ POWER6_PME_PM_LSU0_NCST ] = {
		.pme_name = "PM_LSU0_NCST",
		.pme_code = 0x820cc,
		.pme_short_desc = "LSU0 non-cachable stores",
		.pme_long_desc = "LSU0 non-cachable stores",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_NCST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_NCST]
	},
	[ POWER6_PME_PM_MRK_INST_ISSUED ] = {
		.pme_name = "PM_MRK_INST_ISSUED",
		.pme_code = 0x10001c,
		.pme_short_desc = "Marked instruction issued",
		.pme_long_desc = "Marked instruction issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_INST_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_INST_ISSUED]
	},
	[ POWER6_PME_PM_INST_FROM_RL2L3_SHR ] = {
		.pme_name = "PM_INST_FROM_RL2L3_SHR",
		.pme_code = 0x242044,
		.pme_short_desc = "Instruction fetched from remote L2 or L3 shared",
		.pme_long_desc = "Instruction fetched from remote L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_RL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_RL2L3_SHR]
	},
	[ POWER6_PME_PM_FPU_DENORM ] = {
		.pme_name = "PM_FPU_DENORM",
		.pme_code = 0x2c1034,
		.pme_short_desc = "FPU received denormalized data",
		.pme_long_desc = "This signal is active for one cycle when one of the operands is denormalized. Combined Unit 0 + Unit 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_DENORM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_DENORM]
	},
	[ POWER6_PME_PM_PTEG_FROM_L3MISS ] = {
		.pme_name = "PM_PTEG_FROM_L3MISS",
		.pme_code = 0x313028,
		.pme_short_desc = "PTEG loaded from L3 miss",
		.pme_long_desc = "PTEG loaded from L3 miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_L3MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_L3MISS]
	},
	[ POWER6_PME_PM_RUN_PURR ] = {
		.pme_name = "PM_RUN_PURR",
		.pme_code = 0x4000f4,
		.pme_short_desc = "Run PURR Event",
		.pme_long_desc = "Run PURR Event",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_RUN_PURR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_RUN_PURR]
	},
	[ POWER6_PME_PM_MRK_VMX0_LD_WRBACK ] = {
		.pme_name = "PM_MRK_VMX0_LD_WRBACK",
		.pme_code = 0x60086,
		.pme_short_desc = "Marked VMX0 load writeback valid",
		.pme_long_desc = "Marked VMX0 load writeback valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_VMX0_LD_WRBACK],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_VMX0_LD_WRBACK]
	},
	[ POWER6_PME_PM_L2_MISS ] = {
		.pme_name = "PM_L2_MISS",
		.pme_code = 0x250532,
		.pme_short_desc = "L2 cache misses",
		.pme_long_desc = "L2 cache misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_MISS]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L3 ] = {
		.pme_name = "PM_MRK_DATA_FROM_L3",
		.pme_code = 0x303048,
		.pme_short_desc = "Marked data loaded from L3",
		.pme_long_desc = "DL1 was reloaded from the local L3 due to a marked demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L3],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L3]
	},
	[ POWER6_PME_PM_MRK_LSU1_REJECT_LHS ] = {
		.pme_name = "PM_MRK_LSU1_REJECT_LHS",
		.pme_code = 0x930ee,
		.pme_short_desc = "LSU1 marked load hit store reject",
		.pme_long_desc = "LSU1 marked load hit store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU1_REJECT_LHS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU1_REJECT_LHS]
	},
	[ POWER6_PME_PM_L2SB_LD_MISS_INST ] = {
		.pme_name = "PM_L2SB_LD_MISS_INST",
		.pme_code = 0x5058a,
		.pme_short_desc = "L2 slice B instruction load misses",
		.pme_long_desc = "L2 slice B instruction load misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_LD_MISS_INST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_LD_MISS_INST]
	},
	[ POWER6_PME_PM_PTEG_FROM_RL2L3_SHR ] = {
		.pme_name = "PM_PTEG_FROM_RL2L3_SHR",
		.pme_code = 0x21304c,
		.pme_short_desc = "PTEG loaded from remote L2 or L3 shared",
		.pme_long_desc = "PTEG loaded from remote L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PTEG_FROM_RL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PTEG_FROM_RL2L3_SHR]
	},
	[ POWER6_PME_PM_MRK_DERAT_MISS_64K ] = {
		.pme_name = "PM_MRK_DERAT_MISS_64K",
		.pme_code = 0x192044,
		.pme_short_desc = "Marked DERAT misses for 64K page",
		.pme_long_desc = "A marked data request (load or store) missed the ERAT for 64K page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_MISS_64K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_MISS_64K]
	},
	[ POWER6_PME_PM_LWSYNC ] = {
		.pme_name = "PM_LWSYNC",
		.pme_code = 0x810ae,
		.pme_short_desc = "Isync instruction completed",
		.pme_long_desc = "Isync instruction completed",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LWSYNC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LWSYNC]
	},
	[ POWER6_PME_PM_FPU1_FXMULT ] = {
		.pme_name = "PM_FPU1_FXMULT",
		.pme_code = 0xd008e,
		.pme_short_desc = "FPU1 executed fixed point multiplication",
		.pme_long_desc = "FPU1 executed fixed point multiplication",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FXMULT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FXMULT]
	},
	[ POWER6_PME_PM_MEM0_DP_CL_WR_GLOB ] = {
		.pme_name = "PM_MEM0_DP_CL_WR_GLOB",
		.pme_code = 0x50284,
		.pme_short_desc = "cacheline write setting dp to global side 0",
		.pme_long_desc = "cacheline write setting dp to global side 0",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM0_DP_CL_WR_GLOB],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM0_DP_CL_WR_GLOB]
	},
	[ POWER6_PME_PM_LSU0_REJECT_PARTIAL_SECTOR ] = {
		.pme_name = "PM_LSU0_REJECT_PARTIAL_SECTOR",
		.pme_code = 0xa0086,
		.pme_short_desc = "LSU0 reject due to partial sector valid",
		.pme_long_desc = "LSU0 reject due to partial sector valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_PARTIAL_SECTOR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_PARTIAL_SECTOR]
	},
	[ POWER6_PME_PM_INST_IMC_MATCH_CMPL ] = {
		.pme_name = "PM_INST_IMC_MATCH_CMPL",
		.pme_code = 0x1000f0,
		.pme_short_desc = "IMC matched instructions completed",
		.pme_long_desc = "Number of instructions resulting from the marked instructions expansion that completed.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_IMC_MATCH_CMPL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_IMC_MATCH_CMPL]
	},
	[ POWER6_PME_PM_DPU_HELD_THERMAL ] = {
		.pme_name = "PM_DPU_HELD_THERMAL",
		.pme_code = 0x10002a,
		.pme_short_desc = "DISP unit held due to thermal condition",
		.pme_long_desc = "DISP unit held due to thermal condition",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_THERMAL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_THERMAL]
	},
	[ POWER6_PME_PM_FPU_FRSP ] = {
		.pme_name = "PM_FPU_FRSP",
		.pme_code = 0x2d1034,
		.pme_short_desc = "FPU executed FRSP instruction",
		.pme_long_desc = "FPU executed FRSP instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FRSP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FRSP]
	},
	[ POWER6_PME_PM_MRK_INST_FIN ] = {
		.pme_name = "PM_MRK_INST_FIN",
		.pme_code = 0x30000a,
		.pme_short_desc = "Marked instruction finished",
		.pme_long_desc = "One of the execution units finished a marked instruction. Instructions that finish may not necessary complete",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_INST_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_INST_FIN]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_SHR ] = {
		.pme_name = "PM_MRK_PTEG_FROM_DL2L3_SHR",
		.pme_code = 0x312044,
		.pme_short_desc = "Marked PTEG loaded from distant L2 or L3 shared",
		.pme_long_desc = "Marked PTEG loaded from distant L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_DL2L3_SHR]
	},
	[ POWER6_PME_PM_MRK_DTLB_REF ] = {
		.pme_name = "PM_MRK_DTLB_REF",
		.pme_code = 0x920c0,
		.pme_short_desc = "Marked Data TLB reference",
		.pme_long_desc = "Marked Data TLB reference",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DTLB_REF],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DTLB_REF]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L25_SHR ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L25_SHR",
		.pme_code = 0x412046,
		.pme_short_desc = "Marked PTEG loaded from L2.5 shared",
		.pme_long_desc = "Marked PTEG loaded from L2.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L25_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L25_SHR]
	},
	[ POWER6_PME_PM_DPU_HELD_LSU ] = {
		.pme_name = "PM_DPU_HELD_LSU",
		.pme_code = 0x210a2,
		.pme_short_desc = "DISP unit held due to LSU move or invalidate SLB and SR",
		.pme_long_desc = "DISP unit held due to LSU move or invalidate SLB and SR",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_LSU],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_LSU]
	},
	[ POWER6_PME_PM_FPU_FSQRT_FDIV ] = {
		.pme_name = "PM_FPU_FSQRT_FDIV",
		.pme_code = 0x2c0032,
		.pme_short_desc = "FPU executed FSQRT or FDIV instruction",
		.pme_long_desc = "FPU executed FSQRT or FDIV instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_FSQRT_FDIV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_FSQRT_FDIV]
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_COUNT ] = {
		.pme_name = "PM_LSU_LMQ_SRQ_EMPTY_COUNT",
		.pme_code = 0x20001d,
		.pme_short_desc = "Periods LMQ and SRQ empty",
		.pme_long_desc = "Periods when both the LMQ and SRQ are empty (LSU is idle)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_COUNT]
	},
	[ POWER6_PME_PM_DATA_PTEG_SECONDARY ] = {
		.pme_name = "PM_DATA_PTEG_SECONDARY",
		.pme_code = 0x910a4,
		.pme_short_desc = "Data table walk matched in secondary PTEG",
		.pme_long_desc = "Data table walk matched in secondary PTEG",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_PTEG_SECONDARY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_PTEG_SECONDARY]
	},
	[ POWER6_PME_PM_FPU1_FEST ] = {
		.pme_name = "PM_FPU1_FEST",
		.pme_code = 0xd10ae,
		.pme_short_desc = "FPU1 executed FEST instruction",
		.pme_long_desc = "This signal is active for one cycle when fp1 is executing one of the estimate instructions. This could be fres* or frsqrte* where XYZ* means XYZ or XYZ. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FEST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FEST]
	},
	[ POWER6_PME_PM_L2SA_LD_HIT ] = {
		.pme_name = "PM_L2SA_LD_HIT",
		.pme_code = 0x50782,
		.pme_short_desc = "L2 slice A load hits",
		.pme_long_desc = "L2 slice A load hits",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SA_LD_HIT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SA_LD_HIT]
	},
	[ POWER6_PME_PM_DATA_FROM_MEM_DP_CYC ] = {
		.pme_name = "PM_DATA_FROM_MEM_DP_CYC",
		.pme_code = 0x40002e,
		.pme_short_desc = "Load latency from double pump memory",
		.pme_long_desc = "Load latency from double pump memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_MEM_DP_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_MEM_DP_CYC]
	},
	[ POWER6_PME_PM_BR_MPRED_CCACHE ] = {
		.pme_name = "PM_BR_MPRED_CCACHE",
		.pme_code = 0x410ae,
		.pme_short_desc = "Branch misprediction due to count cache prediction",
		.pme_long_desc = "Branch misprediction due to count cache prediction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_MPRED_CCACHE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_MPRED_CCACHE]
	},
	[ POWER6_PME_PM_DPU_HELD_COUNT ] = {
		.pme_name = "PM_DPU_HELD_COUNT",
		.pme_code = 0x200005,
		.pme_short_desc = "Periods DISP unit held",
		.pme_long_desc = "Dispatch unit held",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_COUNT]
	},
	[ POWER6_PME_PM_LSU1_REJECT_SET_MPRED ] = {
		.pme_name = "PM_LSU1_REJECT_SET_MPRED",
		.pme_code = 0xa008c,
		.pme_short_desc = "LSU1 reject due to mispredicted set",
		.pme_long_desc = "LSU1 reject due to mispredicted set",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_SET_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_SET_MPRED]
	},
	[ POWER6_PME_PM_FPU_ISSUE_2 ] = {
		.pme_name = "PM_FPU_ISSUE_2",
		.pme_code = 0x320ca,
		.pme_short_desc = "FPU issue 2 per cycle",
		.pme_long_desc = "FPU issue 2 per cycle",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_2]
	},
	[ POWER6_PME_PM_LSU1_REJECT_L2_CORR ] = {
		.pme_name = "PM_LSU1_REJECT_L2_CORR",
		.pme_code = 0xa10a8,
		.pme_short_desc = "LSU1 reject due to L2 correctable error",
		.pme_long_desc = "LSU1 reject due to L2 correctable error",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_L2_CORR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_L2_CORR]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_DMEM ] = {
		.pme_name = "PM_MRK_PTEG_FROM_DMEM",
		.pme_code = 0x212042,
		.pme_short_desc = "Marked PTEG loaded from distant memory",
		.pme_long_desc = "Marked PTEG loaded from distant memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_DMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_DMEM]
	},
	[ POWER6_PME_PM_MEM1_DP_RQ_LOC_GLOB ] = {
		.pme_name = "PM_MEM1_DP_RQ_LOC_GLOB",
		.pme_code = 0x5028a,
		.pme_short_desc = "Memory read queue marking cache line double pump state from local to global side 1",
		.pme_long_desc = "Memory read queue marking cache line double pump state from local to global side 1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM1_DP_RQ_LOC_GLOB],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM1_DP_RQ_LOC_GLOB]
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus1or2_CYC ] = {
		.pme_name = "PM_THRD_PRIO_DIFF_minus1or2_CYC",
		.pme_code = 0x223046,
		.pme_short_desc = "Cycles thread priority difference is -1 or -2",
		.pme_long_desc = "Cycles thread priority difference is -1 or -2",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_DIFF_minus1or2_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_DIFF_minus1or2_CYC]
	},
	[ POWER6_PME_PM_THRD_PRIO_0_CYC ] = {
		.pme_name = "PM_THRD_PRIO_0_CYC",
		.pme_code = 0x122040,
		.pme_short_desc = "Cycles thread running at priority level 0",
		.pme_long_desc = "Cycles thread running at priority level 0",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_0_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_0_CYC]
	},
	[ POWER6_PME_PM_FXU0_BUSY_FXU1_IDLE ] = {
		.pme_name = "PM_FXU0_BUSY_FXU1_IDLE",
		.pme_code = 0x300050,
		.pme_short_desc = "FXU0 busy FXU1 idle",
		.pme_long_desc = "FXU0 is busy while FXU1 was idle",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FXU0_BUSY_FXU1_IDLE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FXU0_BUSY_FXU1_IDLE]
	},
	[ POWER6_PME_PM_LSU1_REJECT_DERAT_MPRED ] = {
		.pme_name = "PM_LSU1_REJECT_DERAT_MPRED",
		.pme_code = 0xa008a,
		.pme_short_desc = "LSU1 reject due to mispredicted DERAT",
		.pme_long_desc = "LSU1 reject due to mispredicted DERAT",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_DERAT_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_DERAT_MPRED]
	},
	[ POWER6_PME_PM_MRK_VMX1_LD_WRBACK ] = {
		.pme_name = "PM_MRK_VMX1_LD_WRBACK",
		.pme_code = 0x6008e,
		.pme_short_desc = "Marked VMX1 load writeback valid",
		.pme_long_desc = "Marked VMX1 load writeback valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_VMX1_LD_WRBACK],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_VMX1_LD_WRBACK]
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_SHR_CYC ] = {
		.pme_name = "PM_DATA_FROM_RL2L3_SHR_CYC",
		.pme_code = 0x200028,
		.pme_short_desc = "Load latency from remote L2 or L3 shared",
		.pme_long_desc = "Load latency from remote L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_RL2L3_SHR_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_RL2L3_SHR_CYC]
	},
	[ POWER6_PME_PM_IERAT_MISS_16M ] = {
		.pme_name = "PM_IERAT_MISS_16M",
		.pme_code = 0x292076,
		.pme_short_desc = "IERAT misses for 16M page",
		.pme_long_desc = "IERAT misses for 16M page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IERAT_MISS_16M],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IERAT_MISS_16M]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_MEM_DP ] = {
		.pme_name = "PM_MRK_DATA_FROM_MEM_DP",
		.pme_code = 0x10304a,
		.pme_short_desc = "Marked data loaded from double pump memory",
		.pme_long_desc = "Marked data loaded from double pump memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_MEM_DP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_MEM_DP]
	},
	[ POWER6_PME_PM_LARX_L1HIT ] = {
		.pme_name = "PM_LARX_L1HIT",
		.pme_code = 0x830e2,
		.pme_short_desc = "larx hits in L1",
		.pme_long_desc = "larx hits in L1",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LARX_L1HIT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LARX_L1HIT]
	},
	[ POWER6_PME_PM_L2_ST_MISS_DATA ] = {
		.pme_name = "PM_L2_ST_MISS_DATA",
		.pme_code = 0x150432,
		.pme_short_desc = "L2 data store misses",
		.pme_long_desc = "L2 data store misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2_ST_MISS_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2_ST_MISS_DATA]
	},
	[ POWER6_PME_PM_FPU_ST_FOLDED ] = {
		.pme_name = "PM_FPU_ST_FOLDED",
		.pme_code = 0x3d1030,
		.pme_short_desc = "FPU folded store",
		.pme_long_desc = "FPU folded store",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ST_FOLDED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ST_FOLDED]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L35_SHR ] = {
		.pme_name = "PM_MRK_DATA_FROM_L35_SHR",
		.pme_code = 0x20304e,
		.pme_short_desc = "Marked data loaded from L3.5 shared",
		.pme_long_desc = "Marked data loaded from L3.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L35_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L35_SHR]
	},
	[ POWER6_PME_PM_DPU_HELD_MULT_GPR ] = {
		.pme_name = "PM_DPU_HELD_MULT_GPR",
		.pme_code = 0x210aa,
		.pme_short_desc = "DISP unit held due to multiple/divide multiply/divide GPR dependencies",
		.pme_long_desc = "DISP unit held due to multiple/divide multiply/divide GPR dependencies",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_MULT_GPR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_MULT_GPR]
	},
	[ POWER6_PME_PM_FPU0_1FLOP ] = {
		.pme_name = "PM_FPU0_1FLOP",
		.pme_code = 0xc0080,
		.pme_short_desc = "FPU0 executed add",
		.pme_long_desc = " mult",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_1FLOP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_1FLOP]
	},
	[ POWER6_PME_PM_IERAT_MISS_16G ] = {
		.pme_name = "PM_IERAT_MISS_16G",
		.pme_code = 0x192076,
		.pme_short_desc = "IERAT misses for 16G page",
		.pme_long_desc = "IERAT misses for 16G page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IERAT_MISS_16G],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IERAT_MISS_16G]
	},
	[ POWER6_PME_PM_IC_PREF_WRITE ] = {
		.pme_name = "PM_IC_PREF_WRITE",
		.pme_code = 0x430e0,
		.pme_short_desc = "Instruction prefetch written into I cache",
		.pme_long_desc = "Instruction prefetch written into I cache",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IC_PREF_WRITE],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IC_PREF_WRITE]
	},
	[ POWER6_PME_PM_THRD_PRIO_DIFF_minus5or6_CYC ] = {
		.pme_name = "PM_THRD_PRIO_DIFF_minus5or6_CYC",
		.pme_code = 0x423046,
		.pme_short_desc = "Cycles thread priority difference is -5 or -6",
		.pme_long_desc = "Cycles thread priority difference is -5 or -6",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_PRIO_DIFF_minus5or6_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_PRIO_DIFF_minus5or6_CYC]
	},
	[ POWER6_PME_PM_FPU0_FIN ] = {
		.pme_name = "PM_FPU0_FIN",
		.pme_code = 0xd0080,
		.pme_short_desc = "FPU0 produced a result",
		.pme_long_desc = "fp0 finished, produced a result This only indicates finish, not completion. ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FIN]
	},
	[ POWER6_PME_PM_DATA_FROM_L2_CYC ] = {
		.pme_name = "PM_DATA_FROM_L2_CYC",
		.pme_code = 0x200020,
		.pme_short_desc = "Load latency from L2",
		.pme_long_desc = "Load latency from L2",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L2_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L2_CYC]
	},
	[ POWER6_PME_PM_DERAT_REF_16G ] = {
		.pme_name = "PM_DERAT_REF_16G",
		.pme_code = 0x482070,
		.pme_short_desc = "DERAT reference for 16G page",
		.pme_long_desc = "DERAT reference for 16G page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_REF_16G],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_REF_16G]
	},
	[ POWER6_PME_PM_BR_PRED ] = {
		.pme_name = "PM_BR_PRED",
		.pme_code = 0x410a0,
		.pme_short_desc = "A conditional branch was predicted",
		.pme_long_desc = "A conditional branch was predicted",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_PRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_PRED]
	},
	[ POWER6_PME_PM_VMX1_LD_ISSUED ] = {
		.pme_name = "PM_VMX1_LD_ISSUED",
		.pme_code = 0x6008a,
		.pme_short_desc = "VMX1 load issued",
		.pme_long_desc = "VMX1 load issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX1_LD_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX1_LD_ISSUED]
	},
	[ POWER6_PME_PM_L2SB_CASTOUT_MOD ] = {
		.pme_name = "PM_L2SB_CASTOUT_MOD",
		.pme_code = 0x50688,
		.pme_short_desc = "L2 slice B castouts - Modified",
		.pme_long_desc = "L2 slice B castouts - Modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_CASTOUT_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_CASTOUT_MOD]
	},
	[ POWER6_PME_PM_INST_FROM_DMEM ] = {
		.pme_name = "PM_INST_FROM_DMEM",
		.pme_code = 0x242042,
		.pme_short_desc = "Instruction fetched from distant memory",
		.pme_long_desc = "Instruction fetched from distant memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_DMEM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_DMEM]
	},
	[ POWER6_PME_PM_DATA_FROM_L35_SHR_CYC ] = {
		.pme_name = "PM_DATA_FROM_L35_SHR_CYC",
		.pme_code = 0x200026,
		.pme_short_desc = "Load latency from L3.5 shared",
		.pme_long_desc = "Load latency from L3.5 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L35_SHR_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L35_SHR_CYC]
	},
	[ POWER6_PME_PM_LSU0_NCLD ] = {
		.pme_name = "PM_LSU0_NCLD",
		.pme_code = 0x820ca,
		.pme_short_desc = "LSU0 non-cacheable loads",
		.pme_long_desc = "LSU0 non-cacheable loads",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_NCLD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_NCLD]
	},
	[ POWER6_PME_PM_FAB_RETRY_NODE_PUMP ] = {
		.pme_name = "PM_FAB_RETRY_NODE_PUMP",
		.pme_code = 0x5018a,
		.pme_short_desc = "Retry of a node pump",
		.pme_long_desc = " locally mastered",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_RETRY_NODE_PUMP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_RETRY_NODE_PUMP]
	},
	[ POWER6_PME_PM_VMX0_INST_ISSUED ] = {
		.pme_name = "PM_VMX0_INST_ISSUED",
		.pme_code = 0x60080,
		.pme_short_desc = "VMX0 instruction issued",
		.pme_long_desc = "VMX0 instruction issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX0_INST_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX0_INST_ISSUED]
	},
	[ POWER6_PME_PM_DATA_FROM_L25_MOD ] = {
		.pme_name = "PM_DATA_FROM_L25_MOD",
		.pme_code = 0x30005a,
		.pme_short_desc = "Data loaded from L2.5 modified",
		.pme_long_desc = "DL1 was reloaded with modified (M) data from the L2 of a chip on this MCM due to a demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L25_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L25_MOD]
	},
	[ POWER6_PME_PM_DPU_HELD_ITLB_ISLB ] = {
		.pme_name = "PM_DPU_HELD_ITLB_ISLB",
		.pme_code = 0x210a4,
		.pme_short_desc = "DISP unit held due to SLB or TLB invalidates ",
		.pme_long_desc = "DISP unit held due to SLB or TLB invalidates ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_ITLB_ISLB],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_ITLB_ISLB]
	},
	[ POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_CYC ] = {
		.pme_name = "PM_LSU_LMQ_SRQ_EMPTY_CYC",
		.pme_code = 0x20001c,
		.pme_short_desc = "Cycles LMQ and SRQ empty",
		.pme_long_desc = "Cycles when both the LMQ and SRQ are empty (LSU is idle)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_LMQ_SRQ_EMPTY_CYC]
	},
	[ POWER6_PME_PM_THRD_CONC_RUN_INST ] = {
		.pme_name = "PM_THRD_CONC_RUN_INST",
		.pme_code = 0x300026,
		.pme_short_desc = "Concurrent run instructions",
		.pme_long_desc = "Concurrent run instructions",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_CONC_RUN_INST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_CONC_RUN_INST]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L2 ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L2",
		.pme_code = 0x112040,
		.pme_short_desc = "Marked PTEG loaded from L2.5 modified",
		.pme_long_desc = "Marked PTEG loaded from L2.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L2]
	},
	[ POWER6_PME_PM_PURR ] = {
		.pme_name = "PM_PURR",
		.pme_code = 0x10000e,
		.pme_short_desc = "PURR Event",
		.pme_long_desc = "PURR Event",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PURR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PURR]
	},
	[ POWER6_PME_PM_DERAT_MISS_64K ] = {
		.pme_name = "PM_DERAT_MISS_64K",
		.pme_code = 0x292070,
		.pme_short_desc = "DERAT misses for 64K page",
		.pme_long_desc = "A data request (load or store) missed the ERAT for 64K page and resulted in an ERAT reload.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DERAT_MISS_64K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DERAT_MISS_64K]
	},
	[ POWER6_PME_PM_PMC2_REWIND ] = {
		.pme_name = "PM_PMC2_REWIND",
		.pme_code = 0x300020,
		.pme_short_desc = "PMC2 rewind event",
		.pme_long_desc = "PMC2 rewind event",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC2_REWIND],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC2_REWIND]
	},
	[ POWER6_PME_PM_INST_FROM_L2 ] = {
		.pme_name = "PM_INST_FROM_L2",
		.pme_code = 0x142040,
		.pme_short_desc = "Instructions fetched from L2",
		.pme_long_desc = "An instruction fetch group was fetched from L2. Fetch Groups can contain up to 8 instructions",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L2]
	},
	[ POWER6_PME_PM_INST_DISP ] = {
		.pme_name = "PM_INST_DISP",
		.pme_code = 0x200012,
		.pme_short_desc = "Instructions dispatched",
		.pme_long_desc = "The ISU sends the number of instructions dispatched.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_DISP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_DISP]
	},
	[ POWER6_PME_PM_DATA_FROM_L25_SHR ] = {
		.pme_name = "PM_DATA_FROM_L25_SHR",
		.pme_code = 0x40005a,
		.pme_short_desc = "Data loaded from L2.5 shared",
		.pme_long_desc = "DL1 was reloaded with shared (T or SL) data from the L2 of a chip on this MCM due to a demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L25_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L25_SHR]
	},
	[ POWER6_PME_PM_L1_DCACHE_RELOAD_VALID ] = {
		.pme_name = "PM_L1_DCACHE_RELOAD_VALID",
		.pme_code = 0x3000f6,
		.pme_short_desc = "L1 reload data source valid",
		.pme_long_desc = "The data source information is valid",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L1_DCACHE_RELOAD_VALID],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L1_DCACHE_RELOAD_VALID]
	},
	[ POWER6_PME_PM_LSU1_REJECT_UST ] = {
		.pme_name = "PM_LSU1_REJECT_UST",
		.pme_code = 0x9008a,
		.pme_short_desc = "LSU1 unaligned store reject",
		.pme_long_desc = "LSU1 unaligned store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_UST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_UST]
	},
	[ POWER6_PME_PM_FAB_ADDR_COLLISION ] = {
		.pme_name = "PM_FAB_ADDR_COLLISION",
		.pme_code = 0x5018e,
		.pme_short_desc = "local node launch collision with off-node address   ",
		.pme_long_desc = "local node launch collision with off-node address   ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FAB_ADDR_COLLISION],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FAB_ADDR_COLLISION]
	},
	[ POWER6_PME_PM_MRK_FXU_FIN ] = {
		.pme_name = "PM_MRK_FXU_FIN",
		.pme_code = 0x20001a,
		.pme_short_desc = "Marked instruction FXU processing finished",
		.pme_long_desc = "The fixed point units (Unit 0 + Unit 1) finished a marked instruction. Instructions that finish may not necessary complete.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_FXU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_FXU_FIN]
	},
	[ POWER6_PME_PM_LSU0_REJECT_UST ] = {
		.pme_name = "PM_LSU0_REJECT_UST",
		.pme_code = 0x90082,
		.pme_short_desc = "LSU0 unaligned store reject",
		.pme_long_desc = "LSU0 unaligned store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_UST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_UST]
	},
	[ POWER6_PME_PM_PMC4_OVERFLOW ] = {
		.pme_name = "PM_PMC4_OVERFLOW",
		.pme_code = 0x100014,
		.pme_short_desc = "PMC4 Overflow",
		.pme_long_desc = "PMC4 Overflow",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_PMC4_OVERFLOW],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_PMC4_OVERFLOW]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L3 ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L3",
		.pme_code = 0x312040,
		.pme_short_desc = "Marked PTEG loaded from L3",
		.pme_long_desc = "Marked PTEG loaded from L3",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L3],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L3]
	},
	[ POWER6_PME_PM_INST_FROM_L2MISS ] = {
		.pme_name = "PM_INST_FROM_L2MISS",
		.pme_code = 0x442054,
		.pme_short_desc = "Instructions fetched missed L2",
		.pme_long_desc = "An instruction fetch group was fetched from beyond L2.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L2MISS]
	},
	[ POWER6_PME_PM_L2SB_ST_HIT ] = {
		.pme_name = "PM_L2SB_ST_HIT",
		.pme_code = 0x5078e,
		.pme_short_desc = "L2 slice B store hits",
		.pme_long_desc = "A store request made from the core hit in the L2 directory.  This event is provided on each of the three L2 slices A,B, and C.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_ST_HIT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_ST_HIT]
	},
	[ POWER6_PME_PM_DPU_WT_IC_MISS_COUNT ] = {
		.pme_name = "PM_DPU_WT_IC_MISS_COUNT",
		.pme_code = 0x20000d,
		.pme_short_desc = "Periods DISP unit is stalled due to I cache miss",
		.pme_long_desc = "Periods DISP unit is stalled due to I cache miss",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_WT_IC_MISS_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_WT_IC_MISS_COUNT]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_DL2L3_SHR ] = {
		.pme_name = "PM_MRK_DATA_FROM_DL2L3_SHR",
		.pme_code = 0x30304c,
		.pme_short_desc = "Marked data loaded from distant L2 or L3 shared",
		.pme_long_desc = "Marked data loaded from distant L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_DL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_DL2L3_SHR]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L35_MOD ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L35_MOD",
		.pme_code = 0x112046,
		.pme_short_desc = "Marked PTEG loaded from L3.5 modified",
		.pme_long_desc = "Marked PTEG loaded from L3.5 modified",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L35_MOD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L35_MOD]
	},
	[ POWER6_PME_PM_FPU1_FPSCR ] = {
		.pme_name = "PM_FPU1_FPSCR",
		.pme_code = 0xd008c,
		.pme_short_desc = "FPU1 executed FPSCR instruction",
		.pme_long_desc = "FPU1 executed FPSCR instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_FPSCR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_FPSCR]
	},
	[ POWER6_PME_PM_LSU_REJECT_UST ] = {
		.pme_name = "PM_LSU_REJECT_UST",
		.pme_code = 0x290030,
		.pme_short_desc = "Unaligned store reject",
		.pme_long_desc = "Unaligned store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_UST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_UST]
	},
	[ POWER6_PME_PM_LSU0_DERAT_MISS ] = {
		.pme_name = "PM_LSU0_DERAT_MISS",
		.pme_code = 0x910a6,
		.pme_short_desc = "LSU0 DERAT misses",
		.pme_long_desc = "A data request (load or store) from LSU Unit 0 missed the ERAT and resulted in an ERAT reload. Multiple instructions may miss the ERAT entry for the same 4K page, but only one reload will occur.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_DERAT_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_DERAT_MISS]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_MEM_DP ] = {
		.pme_name = "PM_MRK_PTEG_FROM_MEM_DP",
		.pme_code = 0x112042,
		.pme_short_desc = "Marked PTEG loaded from double pump memory",
		.pme_long_desc = "Marked PTEG loaded from double pump memory",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_MEM_DP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_MEM_DP]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L2 ] = {
		.pme_name = "PM_MRK_DATA_FROM_L2",
		.pme_code = 0x103048,
		.pme_short_desc = "Marked data loaded from L2",
		.pme_long_desc = "DL1 was reloaded from the local L2 due to a marked demand load",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L2]
	},
	[ POWER6_PME_PM_FPU0_FSQRT_FDIV ] = {
		.pme_name = "PM_FPU0_FSQRT_FDIV",
		.pme_code = 0xc0084,
		.pme_short_desc = "FPU0 executed FSQRT or FDIV instruction",
		.pme_long_desc = "FPU0 executed FSQRT or FDIV instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU0_FSQRT_FDIV],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU0_FSQRT_FDIV]
	},
	[ POWER6_PME_PM_DPU_HELD_FXU_SOPS ] = {
		.pme_name = "PM_DPU_HELD_FXU_SOPS",
		.pme_code = 0x30088,
		.pme_short_desc = "DISP unit held due to FXU slow ops (mtmsr",
		.pme_long_desc = " scv",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_FXU_SOPS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_FXU_SOPS]
	},
	[ POWER6_PME_PM_MRK_FPU0_FIN ] = {
		.pme_name = "PM_MRK_FPU0_FIN",
		.pme_code = 0xd0082,
		.pme_short_desc = "Marked instruction FPU0 processing finished",
		.pme_long_desc = "Marked instruction FPU0 processing finished",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_FPU0_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_FPU0_FIN]
	},
	[ POWER6_PME_PM_L2SB_LD_MISS_DATA ] = {
		.pme_name = "PM_L2SB_LD_MISS_DATA",
		.pme_code = 0x5048a,
		.pme_short_desc = "L2 slice B data load misses",
		.pme_long_desc = "L2 slice B data load misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L2SB_LD_MISS_DATA],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L2SB_LD_MISS_DATA]
	},
	[ POWER6_PME_PM_LSU_SRQ_EMPTY_CYC ] = {
		.pme_name = "PM_LSU_SRQ_EMPTY_CYC",
		.pme_code = 0x40001c,
		.pme_short_desc = "Cycles SRQ empty",
		.pme_long_desc = "The Store Request Queue is empty",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_SRQ_EMPTY_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_SRQ_EMPTY_CYC]
	},
	[ POWER6_PME_PM_1PLUS_PPC_DISP ] = {
		.pme_name = "PM_1PLUS_PPC_DISP",
		.pme_code = 0x100012,
		.pme_short_desc = "Cycles at least one instruction dispatched",
		.pme_long_desc = "Cycles at least one instruction dispatched",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_1PLUS_PPC_DISP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_1PLUS_PPC_DISP]
	},
	[ POWER6_PME_PM_VMX_ST_ISSUED ] = {
		.pme_name = "PM_VMX_ST_ISSUED",
		.pme_code = 0xb0080,
		.pme_short_desc = "VMX store issued",
		.pme_long_desc = "VMX store issued",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_VMX_ST_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_VMX_ST_ISSUED]
	},
	[ POWER6_PME_PM_DATA_FROM_L2MISS ] = {
		.pme_name = "PM_DATA_FROM_L2MISS",
		.pme_code = 0x2000fe,
		.pme_short_desc = "Data loaded missed L2",
		.pme_long_desc = "DL1 was reloaded from beyond L2.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L2MISS]
	},
	[ POWER6_PME_PM_LSU0_REJECT_ULD ] = {
		.pme_name = "PM_LSU0_REJECT_ULD",
		.pme_code = 0x90080,
		.pme_short_desc = "LSU0 unaligned load reject",
		.pme_long_desc = "LSU0 unaligned load reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_ULD],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_ULD]
	},
	[ POWER6_PME_PM_SUSPENDED ] = {
		.pme_name = "PM_SUSPENDED",
		.pme_code = 0x0,
		.pme_short_desc = "Suspended",
		.pme_long_desc = "Suspended",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_SUSPENDED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_SUSPENDED]
	},
	[ POWER6_PME_PM_DFU_ADD_SHIFTED_BOTH ] = {
		.pme_name = "PM_DFU_ADD_SHIFTED_BOTH",
		.pme_code = 0xe0088,
		.pme_short_desc = "DFU add type with both operands shifted",
		.pme_long_desc = "DFU add type with both operands shifted",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DFU_ADD_SHIFTED_BOTH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DFU_ADD_SHIFTED_BOTH]
	},
	[ POWER6_PME_PM_LSU_REJECT_NO_SCRATCH ] = {
		.pme_name = "PM_LSU_REJECT_NO_SCRATCH",
		.pme_code = 0x2a1034,
		.pme_short_desc = "LSU reject due to scratch register not available",
		.pme_long_desc = "LSU reject due to scratch register not available",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU_REJECT_NO_SCRATCH],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU_REJECT_NO_SCRATCH]
	},
	[ POWER6_PME_PM_STCX_FAIL ] = {
		.pme_name = "PM_STCX_FAIL",
		.pme_code = 0x830ee,
		.pme_short_desc = "STCX failed",
		.pme_long_desc = "A stcx (stwcx or stdcx) failed",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_STCX_FAIL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_STCX_FAIL]
	},
	[ POWER6_PME_PM_FPU1_DENORM ] = {
		.pme_name = "PM_FPU1_DENORM",
		.pme_code = 0xc10aa,
		.pme_short_desc = "FPU1 received denormalized data",
		.pme_long_desc = "This signal is active for one cycle when one of the operands is denormalized.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU1_DENORM],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU1_DENORM]
	},
	[ POWER6_PME_PM_GCT_NOSLOT_COUNT ] = {
		.pme_name = "PM_GCT_NOSLOT_COUNT",
		.pme_code = 0x100009,
		.pme_short_desc = "Periods no GCT slot allocated",
		.pme_long_desc = "Periods this thread does not have any slots allocated in the GCT.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GCT_NOSLOT_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GCT_NOSLOT_COUNT]
	},
	[ POWER6_PME_PM_DATA_FROM_DL2L3_SHR_CYC ] = {
		.pme_name = "PM_DATA_FROM_DL2L3_SHR_CYC",
		.pme_code = 0x20002a,
		.pme_short_desc = "Load latency from distant L2 or L3 shared",
		.pme_long_desc = "Load latency from distant L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_DL2L3_SHR_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_DL2L3_SHR_CYC]
	},
	[ POWER6_PME_PM_DATA_FROM_L21 ] = {
		.pme_name = "PM_DATA_FROM_L21",
		.pme_code = 0x200058,
		.pme_short_desc = "Data loaded from private L2 other core",
		.pme_long_desc = "Data loaded from private L2 other core",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L21],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L21]
	},
	[ POWER6_PME_PM_FPU_1FLOP ] = {
		.pme_name = "PM_FPU_1FLOP",
		.pme_code = 0x1c0030,
		.pme_short_desc = "FPU executed one flop instruction ",
		.pme_long_desc = "This event counts the number of one flop instructions. These could be fadd*, fmul*, fsub*, fneg+, fabs+, fnabs+, fres+, frsqrte+, fcmp**, or fsel where XYZ* means XYZ, XYZs, XYZ., XYZs., XYZ+ means XYZ, XYZ., and XYZ** means XYZu, XYZo.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_1FLOP],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_1FLOP]
	},
	[ POWER6_PME_PM_LSU1_REJECT ] = {
		.pme_name = "PM_LSU1_REJECT",
		.pme_code = 0xa10ae,
		.pme_short_desc = "LSU1 reject",
		.pme_long_desc = "LSU1 reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT]
	},
	[ POWER6_PME_PM_IC_REQ ] = {
		.pme_name = "PM_IC_REQ",
		.pme_code = 0x4008a,
		.pme_short_desc = "I cache demand of prefetch request",
		.pme_long_desc = "I cache demand of prefetch request",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IC_REQ],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IC_REQ]
	},
	[ POWER6_PME_PM_MRK_DFU_FIN ] = {
		.pme_name = "PM_MRK_DFU_FIN",
		.pme_code = 0x300008,
		.pme_short_desc = "DFU marked instruction finish",
		.pme_long_desc = "DFU marked instruction finish",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DFU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DFU_FIN]
	},
	[ POWER6_PME_PM_NOT_LLA_CYC ] = {
		.pme_name = "PM_NOT_LLA_CYC",
		.pme_code = 0x401e,
		.pme_short_desc = "Load Look Ahead not Active",
		.pme_long_desc = "Load Look Ahead not Active",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_NOT_LLA_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_NOT_LLA_CYC]
	},
	[ POWER6_PME_PM_INST_FROM_L1 ] = {
		.pme_name = "PM_INST_FROM_L1",
		.pme_code = 0x40082,
		.pme_short_desc = "Instruction fetched from L1",
		.pme_long_desc = "An instruction fetch group was fetched from L1. Fetch Groups can contain up to 8 instructions",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_INST_FROM_L1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_INST_FROM_L1]
	},
	[ POWER6_PME_PM_MRK_VMX_COMPLEX_ISSUED ] = {
		.pme_name = "PM_MRK_VMX_COMPLEX_ISSUED",
		.pme_code = 0x7008c,
		.pme_short_desc = "Marked VMX instruction issued to complex",
		.pme_long_desc = "Marked VMX instruction issued to complex",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_VMX_COMPLEX_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_VMX_COMPLEX_ISSUED]
	},
	[ POWER6_PME_PM_BRU_FIN ] = {
		.pme_name = "PM_BRU_FIN",
		.pme_code = 0x430e6,
		.pme_short_desc = "BRU produced a result",
		.pme_long_desc = "BRU produced a result",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BRU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BRU_FIN]
	},
	[ POWER6_PME_PM_LSU1_REJECT_EXTERN ] = {
		.pme_name = "PM_LSU1_REJECT_EXTERN",
		.pme_code = 0xa10ac,
		.pme_short_desc = "LSU1 external reject request ",
		.pme_long_desc = "LSU1 external reject request ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_REJECT_EXTERN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_REJECT_EXTERN]
	},
	[ POWER6_PME_PM_DATA_FROM_L21_CYC ] = {
		.pme_name = "PM_DATA_FROM_L21_CYC",
		.pme_code = 0x400020,
		.pme_short_desc = "Load latency from private L2 other core",
		.pme_long_desc = "Load latency from private L2 other core",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_L21_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_L21_CYC]
	},
	[ POWER6_PME_PM_GXI_CYC_BUSY ] = {
		.pme_name = "PM_GXI_CYC_BUSY",
		.pme_code = 0x50386,
		.pme_short_desc = "Inbound GX bus utilizations (# of cycles in use)",
		.pme_long_desc = "Inbound GX bus utilizations (# of cycles in use)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GXI_CYC_BUSY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GXI_CYC_BUSY]
	},
	[ POWER6_PME_PM_MRK_LD_MISS_L1 ] = {
		.pme_name = "PM_MRK_LD_MISS_L1",
		.pme_code = 0x200056,
		.pme_short_desc = "Marked L1 D cache load misses",
		.pme_long_desc = "Marked L1 D cache load misses",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LD_MISS_L1],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LD_MISS_L1]
	},
	[ POWER6_PME_PM_L1_WRITE_CYC ] = {
		.pme_name = "PM_L1_WRITE_CYC",
		.pme_code = 0x430e2,
		.pme_short_desc = "Cycles writing to instruction L1",
		.pme_long_desc = "This signal is asserted each cycle a cache write is active.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_L1_WRITE_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_L1_WRITE_CYC]
	},
	[ POWER6_PME_PM_LLA_CYC ] = {
		.pme_name = "PM_LLA_CYC",
		.pme_code = 0xc01e,
		.pme_short_desc = "Load Look Ahead Active",
		.pme_long_desc = "Load Look Ahead Active",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LLA_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LLA_CYC]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_L2MISS ] = {
		.pme_name = "PM_MRK_DATA_FROM_L2MISS",
		.pme_code = 0x103028,
		.pme_short_desc = "Marked data loaded missed L2",
		.pme_long_desc = "DL1 was reloaded from beyond L2 due to a marked demand load.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_L2MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_L2MISS]
	},
	[ POWER6_PME_PM_GCT_FULL_COUNT ] = {
		.pme_name = "PM_GCT_FULL_COUNT",
		.pme_code = 0x40087,
		.pme_short_desc = "Periods GCT full",
		.pme_long_desc = "The ISU sends a signal indicating the gct is full.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GCT_FULL_COUNT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GCT_FULL_COUNT]
	},
	[ POWER6_PME_PM_MEM_DP_RQ_LOC_GLOB ] = {
		.pme_name = "PM_MEM_DP_RQ_LOC_GLOB",
		.pme_code = 0x250230,
		.pme_short_desc = "Memory read queue marking cache line double pump state from local to global",
		.pme_long_desc = "Memory read queue marking cache line double pump state from local to global",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MEM_DP_RQ_LOC_GLOB],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MEM_DP_RQ_LOC_GLOB]
	},
	[ POWER6_PME_PM_DATA_FROM_RL2L3_SHR ] = {
		.pme_name = "PM_DATA_FROM_RL2L3_SHR",
		.pme_code = 0x20005c,
		.pme_short_desc = "Data loaded from remote L2 or L3 shared",
		.pme_long_desc = "Data loaded from remote L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DATA_FROM_RL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DATA_FROM_RL2L3_SHR]
	},
	[ POWER6_PME_PM_MRK_LSU_REJECT_UST ] = {
		.pme_name = "PM_MRK_LSU_REJECT_UST",
		.pme_code = 0x293034,
		.pme_short_desc = "Marked unaligned store reject",
		.pme_long_desc = "Marked unaligned store reject",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU_REJECT_UST],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU_REJECT_UST]
	},
	[ POWER6_PME_PM_MRK_VMX_PERMUTE_ISSUED ] = {
		.pme_name = "PM_MRK_VMX_PERMUTE_ISSUED",
		.pme_code = 0x7008e,
		.pme_short_desc = "Marked VMX instruction issued to permute",
		.pme_long_desc = "Marked VMX instruction issued to permute",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_VMX_PERMUTE_ISSUED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_VMX_PERMUTE_ISSUED]
	},
	[ POWER6_PME_PM_MRK_PTEG_FROM_L21 ] = {
		.pme_name = "PM_MRK_PTEG_FROM_L21",
		.pme_code = 0x212040,
		.pme_short_desc = "Marked PTEG loaded from private L2 other core",
		.pme_long_desc = "Marked PTEG loaded from private L2 other core",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_PTEG_FROM_L21],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_PTEG_FROM_L21]
	},
	[ POWER6_PME_PM_THRD_GRP_CMPL_BOTH_CYC ] = {
		.pme_name = "PM_THRD_GRP_CMPL_BOTH_CYC",
		.pme_code = 0x200018,
		.pme_short_desc = "Cycles group completed by both threads",
		.pme_long_desc = "Cycles group completed by both threads",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_THRD_GRP_CMPL_BOTH_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_THRD_GRP_CMPL_BOTH_CYC]
	},
	[ POWER6_PME_PM_BR_MPRED ] = {
		.pme_name = "PM_BR_MPRED",
		.pme_code = 0x400052,
		.pme_short_desc = "Branches incorrectly predicted",
		.pme_long_desc = "Branches incorrectly predicted",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_BR_MPRED],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_BR_MPRED]
	},
	[ POWER6_PME_PM_LD_REQ_L2 ] = {
		.pme_name = "PM_LD_REQ_L2",
		.pme_code = 0x150730,
		.pme_short_desc = "L2 load requests ",
		.pme_long_desc = "L2 load requests ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LD_REQ_L2],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LD_REQ_L2]
	},
	[ POWER6_PME_PM_FLUSH_ASYNC ] = {
		.pme_name = "PM_FLUSH_ASYNC",
		.pme_code = 0x220ca,
		.pme_short_desc = "Flush caused by asynchronous exception",
		.pme_long_desc = "Flush caused by asynchronous exception",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FLUSH_ASYNC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FLUSH_ASYNC]
	},
	[ POWER6_PME_PM_HV_CYC ] = {
		.pme_name = "PM_HV_CYC",
		.pme_code = 0x200016,
		.pme_short_desc = "Hypervisor Cycles",
		.pme_long_desc = "Cycles when the processor is executing in Hypervisor (MSR[HV] = 1 and MSR[PR]=0)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_HV_CYC],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_HV_CYC]
	},
	[ POWER6_PME_PM_LSU1_DERAT_MISS ] = {
		.pme_name = "PM_LSU1_DERAT_MISS",
		.pme_code = 0x910ae,
		.pme_short_desc = "LSU1 DERAT misses",
		.pme_long_desc = "A data request (load or store) from LSU Unit 1 missed the ERAT and resulted in an ERAT reload. Multiple instructions may miss the ERAT entry for the same 4K page, but only one reload will occur.",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU1_DERAT_MISS],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU1_DERAT_MISS]
	},
	[ POWER6_PME_PM_DPU_HELD_SMT ] = {
		.pme_name = "PM_DPU_HELD_SMT",
		.pme_code = 0x20082,
		.pme_short_desc = "DISP unit held due to SMT conflicts ",
		.pme_long_desc = "DISP unit held due to SMT conflicts ",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_DPU_HELD_SMT],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_DPU_HELD_SMT]
	},
	[ POWER6_PME_PM_MRK_LSU_FIN ] = {
		.pme_name = "PM_MRK_LSU_FIN",
		.pme_code = 0x40001a,
		.pme_short_desc = "Marked instruction LSU processing finished",
		.pme_long_desc = "One of the Load/Store Units finished a marked instruction. Instructions that finish may not necessary complete",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_LSU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_LSU_FIN]
	},
	[ POWER6_PME_PM_MRK_DATA_FROM_RL2L3_SHR ] = {
		.pme_name = "PM_MRK_DATA_FROM_RL2L3_SHR",
		.pme_code = 0x20304c,
		.pme_short_desc = "Marked data loaded from remote L2 or L3 shared",
		.pme_long_desc = "Marked data loaded from remote L2 or L3 shared",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DATA_FROM_RL2L3_SHR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DATA_FROM_RL2L3_SHR]
	},
	[ POWER6_PME_PM_LSU0_REJECT_STQ_FULL ] = {
		.pme_name = "PM_LSU0_REJECT_STQ_FULL",
		.pme_code = 0xa0080,
		.pme_short_desc = "LSU0 reject due to store queue full",
		.pme_long_desc = "LSU0 reject due to store queue full",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_LSU0_REJECT_STQ_FULL],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_LSU0_REJECT_STQ_FULL]
	},
	[ POWER6_PME_PM_MRK_DERAT_REF_4K ] = {
		.pme_name = "PM_MRK_DERAT_REF_4K",
		.pme_code = 0x282044,
		.pme_short_desc = "Marked DERAT reference for 4K page",
		.pme_long_desc = "Marked DERAT reference for 4K page",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_MRK_DERAT_REF_4K],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_MRK_DERAT_REF_4K]
	},
	[ POWER6_PME_PM_FPU_ISSUE_STALL_FPR ] = {
		.pme_name = "PM_FPU_ISSUE_STALL_FPR",
		.pme_code = 0x330e2,
		.pme_short_desc = "FPU issue stalled due to FPR dependencies",
		.pme_long_desc = "FPU issue stalled due to FPR dependencies",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_FPU_ISSUE_STALL_FPR],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_FPU_ISSUE_STALL_FPR]
	},
	[ POWER6_PME_PM_IFU_FIN ] = {
		.pme_name = "PM_IFU_FIN",
		.pme_code = 0x430e4,
		.pme_short_desc = "IFU finished an instruction",
		.pme_long_desc = "IFU finished an instruction",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_IFU_FIN],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_IFU_FIN]
	},
	[ POWER6_PME_PM_GXO_CYC_BUSY ] = {
		.pme_name = "PM_GXO_CYC_BUSY",
		.pme_code = 0x50380,
		.pme_short_desc = "Outbound GX bus utilizations (# of cycles in use)",
		.pme_long_desc = "Outbound GX bus utilizations (# of cycles in use)",
		.pme_event_ids = power6_event_ids[POWER6_PME_PM_GXO_CYC_BUSY],
		.pme_group_vector = power6_group_vecs[POWER6_PME_PM_GXO_CYC_BUSY]
	}
};
#define POWER6_PME_EVENT_COUNT 553

static const int power6_group_event_ids[][POWER6_NUM_EVENT_COUNTERS] = {
	[ 0 ] = { 302, 148, 139, 12, 0, 0 },
	[ 1 ] = { 315, 11, 306, 347, 0, 0 },
	[ 2 ] = { 10, 4, 6, 5, 0, 0 },
	[ 3 ] = { 9, 9, 2, 7, 0, 0 },
	[ 4 ] = { 8, 8, 7, 11, 0, 0 },
	[ 5 ] = { 6, 3, 5, 4, 0, 0 },
	[ 6 ] = { 8, 10, 1, 3, 0, 0 },
	[ 7 ] = { 13, 15, 13, 19, 0, 0 },
	[ 8 ] = { 14, 19, 14, 19, 0, 0 },
	[ 9 ] = { 14, 19, 12, 16, 0, 0 },
	[ 10 ] = { 16, 23, 11, 13, 0, 0 },
	[ 11 ] = { 15, 13, 16, 20, 0, 0 },
	[ 12 ] = { 200, 24, 10, 17, 0, 0 },
	[ 13 ] = { 139, 22, 10, 14, 0, 0 },
	[ 14 ] = { 139, 14, 16, 23, 0, 0 },
	[ 15 ] = { 14, 12, 11, 18, 0, 0 },
	[ 16 ] = { 16, 21, 14, 22, 0, 0 },
	[ 17 ] = { 15, 16, 12, 21, 0, 0 },
	[ 18 ] = { 13, 18, 139, 160, 0, 0 },
	[ 19 ] = { 67, 15, 10, 15, 0, 0 },
	[ 20 ] = { 2, 22, 139, 20, 0, 0 },
	[ 21 ] = { 14, 20, 10, 18, 0, 0 },
	[ 22 ] = { 143, 154, 144, 151, 0, 0 },
	[ 23 ] = { 144, 155, 145, 150, 0, 0 },
	[ 24 ] = { 146, 156, 142, 148, 0, 0 },
	[ 25 ] = { 145, 152, 147, 152, 0, 0 },
	[ 26 ] = { 143, 154, 146, 151, 0, 0 },
	[ 27 ] = { 295, 305, 291, 290, 0, 0 },
	[ 28 ] = { 296, 305, 292, 289, 0, 0 },
	[ 29 ] = { 297, 306, 293, 291, 0, 0 },
	[ 30 ] = { 298, 304, 294, 291, 0, 0 },
	[ 31 ] = { 299, 307, 290, 292, 0, 0 },
	[ 32 ] = { 17, 26, 19, 312, 0, 0 },
	[ 33 ] = { 148, 158, 150, 157, 0, 0 },
	[ 34 ] = { 52, 55, 115, 123, 0, 0 },
	[ 35 ] = { 154, 64, 54, 62, 0, 0 },
	[ 36 ] = { 52, 55, 305, 56, 0, 0 },
	[ 37 ] = { 39, 38, 32, 41, 0, 0 },
	[ 38 ] = { 54, 49, 50, 55, 0, 0 },
	[ 39 ] = { 35, 52, 41, 44, 0, 0 },
	[ 40 ] = { 36, 54, 30, 46, 0, 0 },
	[ 41 ] = { 40, 63, 42, 54, 0, 0 },
	[ 42 ] = { 38, 61, 49, 39, 0, 0 },
	[ 43 ] = { 41, 60, 47, 42, 0, 0 },
	[ 44 ] = { 41, 43, 45, 38, 0, 0 },
	[ 45 ] = { 247, 11, 303, 52, 0, 0 },
	[ 46 ] = { 202, 212, 300, 301, 0, 0 },
	[ 47 ] = { 310, 212, 299, 202, 0, 0 },
	[ 48 ] = { 21, 29, 155, 136, 0, 0 },
	[ 49 ] = { 67, 77, 67, 76, 0, 0 },
	[ 50 ] = { 136, 142, 134, 140, 0, 0 },
	[ 51 ] = { 304, 311, 298, 200, 0, 0 },
	[ 52 ] = { 197, 208, 296, 297, 0, 0 },
	[ 53 ] = { 315, 320, 306, 304, 0, 0 },
	[ 54 ] = { 1, 137, 313, 306, 0, 0 },
	[ 55 ] = { 57, 67, 304, 0, 0, 0 },
	[ 56 ] = { 307, 321, 10, 145, 0, 0 },
	[ 57 ] = { 152, 65, 314, 159, 0, 0 },
	[ 58 ] = { 152, 65, 136, 294, 0, 0 },
	[ 59 ] = { 239, 249, 236, 238, 0, 0 },
	[ 60 ] = { 240, 250, 235, 243, 0, 0 },
	[ 61 ] = { 243, 253, 237, 242, 0, 0 },
	[ 62 ] = { 245, 256, 214, 232, 0, 0 },
	[ 63 ] = { 244, 255, 213, 231, 0, 0 },
	[ 64 ] = { 216, 238, 238, 244, 0, 0 },
	[ 65 ] = { 218, 241, 214, 231, 0, 0 },
	[ 66 ] = { 208, 218, 217, 227, 0, 0 },
	[ 67 ] = { 210, 221, 219, 225, 0, 0 },
	[ 68 ] = { 214, 224, 222, 228, 0, 0 },
	[ 69 ] = { 213, 213, 221, 220, 0, 0 },
	[ 70 ] = { 217, 217, 225, 222, 0, 0 },
	[ 71 ] = { 209, 213, 218, 220, 0, 0 },
	[ 72 ] = { 237, 246, 200, 221, 0, 0 },
	[ 73 ] = { 206, 216, 240, 233, 0, 0 },
	[ 74 ] = { 238, 248, 234, 211, 0, 0 },
	[ 75 ] = { 234, 243, 229, 236, 0, 0 },
	[ 76 ] = { 140, 51, 139, 306, 0, 0 },
	[ 77 ] = { 121, 127, 119, 147, 0, 0 },
	[ 78 ] = { 316, 322, 308, 307, 0, 0 },
	[ 79 ] = { 317, 323, 309, 308, 0, 0 },
	[ 80 ] = { 318, 324, 310, 309, 0, 0 },
	[ 81 ] = { 319, 325, 311, 310, 0, 0 },
	[ 82 ] = { 117, 125, 116, 124, 0, 0 },
	[ 83 ] = { 118, 147, 117, 125, 0, 0 },
	[ 84 ] = { 329, 337, 329, 324, 0, 0 },
	[ 85 ] = { 321, 332, 316, 318, 0, 0 },
	[ 86 ] = { 322, 330, 320, 319, 0, 0 },
	[ 87 ] = { 331, 340, 328, 328, 0, 0 },
	[ 88 ] = { 336, 331, 322, 323, 0, 0 },
	[ 89 ] = { 23, 31, 24, 33, 0, 0 },
	[ 90 ] = { 27, 35, 28, 37, 0, 0 },
	[ 91 ] = { 59, 69, 59, 67, 0, 0 },
	[ 92 ] = { 63, 74, 64, 72, 0, 0 },
	[ 93 ] = { 59, 69, 58, 68, 0, 0 },
	[ 94 ] = { 257, 268, 250, 255, 0, 0 },
	[ 95 ] = { 250, 262, 242, 248, 0, 0 },
	[ 96 ] = { 254, 266, 246, 252, 0, 0 },
	[ 97 ] = { 126, 132, 125, 129, 0, 0 },
	[ 98 ] = { 123, 129, 122, 132, 0, 0 },
	[ 99 ] = { 126, 130, 126, 135, 0, 0 },
	[ 100 ] = { 142, 165, 285, 153, 0, 0 },
	[ 101 ] = { 186, 195, 188, 193, 0, 0 },
	[ 102 ] = { 187, 196, 185, 191, 0, 0 },
	[ 103 ] = { 185, 194, 131, 27, 0, 0 },
	[ 104 ] = { 203, 209, 301, 298, 0, 0 },
	[ 105 ] = { 165, 171, 179, 182, 0, 0 },
	[ 106 ] = { 166, 172, 180, 183, 0, 0 },
	[ 107 ] = { 170, 178, 184, 189, 0, 0 },
	[ 108 ] = { 167, 197, 13, 187, 0, 0 },
	[ 109 ] = { 157, 167, 171, 178, 0, 0 },
	[ 110 ] = { 160, 168, 174, 179, 0, 0 },
	[ 111 ] = { 164, 170, 178, 181, 0, 0 },
	[ 112 ] = { 170, 177, 184, 188, 0, 0 },
	[ 113 ] = { 131, 140, 187, 191, 0, 0 },
	[ 114 ] = { 193, 201, 14, 195, 0, 0 },
	[ 115 ] = { 196, 204, 14, 198, 0, 0 },
	[ 116 ] = { 107, 116, 105, 120, 0, 0 },
	[ 117 ] = { 111, 124, 106, 118, 0, 0 },
	[ 118 ] = { 114, 120, 106, 118, 0, 0 },
	[ 119 ] = { 71, 86, 78, 90, 0, 0 },
	[ 120 ] = { 76, 90, 70, 89, 0, 0 },
	[ 121 ] = { 75, 83, 76, 88, 0, 0 },
	[ 122 ] = { 73, 88, 83, 80, 0, 0 },
	[ 123 ] = { 86, 101, 93, 105, 0, 0 },
	[ 124 ] = { 91, 105, 85, 104, 0, 0 },
	[ 125 ] = { 90, 98, 91, 103, 0, 0 },
	[ 126 ] = { 88, 103, 98, 95, 0, 0 },
	[ 127 ] = { 101, 111, 102, 109, 0, 0 },
	[ 128 ] = { 103, 113, 100, 111, 0, 0 },
	[ 129 ] = { 105, 110, 113, 122, 0, 0 },
	[ 130 ] = { 102, 113, 114, 108, 0, 0 },
	[ 131 ] = { 301, 309, 10, 145, 0, 0 },
	[ 132 ] = { 311, 11, 303, 145, 0, 0 },
	[ 133 ] = { 200, 243, 197, 234, 0, 0 },
	[ 134 ] = { 351, 361, 343, 343, 0, 0 },
	[ 135 ] = { 350, 360, 342, 342, 0, 0 },
	[ 136 ] = { 352, 362, 344, 344, 0, 0 },
	[ 137 ] = { 258, 280, 251, 145, 0, 0 },
	[ 138 ] = { 139, 269, 252, 256, 0, 0 },
	[ 139 ] = { 259, 270, 253, 145, 0, 0 },
	[ 140 ] = { 260, 148, 254, 257, 0, 0 },
	[ 141 ] = { 261, 271, 255, 145, 0, 0 },
	[ 142 ] = { 262, 272, 138, 258, 0, 0 },
	[ 143 ] = { 263, 272, 256, 145, 0, 0 },
	[ 144 ] = { 276, 288, 138, 273, 0, 0 },
	[ 145 ] = { 270, 283, 266, 145, 0, 0 },
	[ 146 ] = { 273, 286, 269, 145, 0, 0 },
	[ 147 ] = { 268, 279, 262, 145, 0, 0 },
	[ 148 ] = { 265, 275, 261, 145, 0, 0 },
	[ 149 ] = { 276, 277, 257, 145, 0, 0 },
	[ 150 ] = { 281, 278, 138, 263, 0, 0 },
	[ 151 ] = { 289, 281, 138, 271, 0, 0 },
	[ 152 ] = { 12, 11, 138, 272, 0, 0 },
	[ 153 ] = { 282, 294, 277, 145, 0, 0 },
	[ 154 ] = { 277, 289, 271, 145, 0, 0 },
	[ 155 ] = { 139, 290, 272, 275, 0, 0 },
	[ 156 ] = { 278, 291, 138, 276, 0, 0 },
	[ 157 ] = { 279, 148, 273, 277, 0, 0 },
	[ 158 ] = { 280, 148, 274, 278, 0, 0 },
	[ 159 ] = { 12, 292, 275, 145, 0, 0 },
	[ 160 ] = { 285, 298, 282, 145, 0, 0 },
	[ 161 ] = { 283, 296, 258, 145, 0, 0 },
	[ 162 ] = { 288, 342, 10, 145, 0, 0 },
	[ 163 ] = { 265, 276, 260, 145, 0, 0 },
	[ 164 ] = { 353, 363, 345, 145, 0, 0 },
	[ 165 ] = { 354, 364, 346, 145, 0, 0 },
	[ 166 ] = { 199, 243, 197, 335, 0, 0 },
	[ 167 ] = { 238, 359, 341, 211, 0, 0 },
	[ 168 ] = { 349, 358, 333, 147, 0, 0 },
	[ 169 ] = { 348, 356, 115, 123, 0, 0 },
	[ 170 ] = { 154, 357, 340, 340, 0, 0 },
	[ 171 ] = { 199, 64, 339, 337, 0, 0 },
	[ 172 ] = { 337, 0, 332, 136, 0, 0 },
	[ 173 ] = { 343, 309, 335, 157, 0, 0 },
	[ 174 ] = { 339, 128, 334, 287, 0, 0 },
	[ 175 ] = { 344, 351, 336, 303, 0, 0 },
	[ 176 ] = { 315, 137, 336, 303, 0, 0 },
	[ 177 ] = { 348, 355, 340, 340, 0, 0 },
	[ 178 ] = { 52, 38, 54, 62, 0, 0 },
	[ 179 ] = { 349, 358, 341, 341, 0, 0 },
	[ 180 ] = { 121, 127, 234, 244, 0, 0 },
	[ 181 ] = { 342, 359, 303, 335, 0, 0 },
	[ 182 ] = { 312, 248, 336, 234, 0, 0 },
	[ 183 ] = { 309, 357, 197, 203, 0, 0 },
	[ 184 ] = { 31, 356, 31, 336, 0, 0 },
	[ 185 ] = { 31, 55, 31, 304, 0, 0 },
	[ 186 ] = { 101, 111, 102, 12, 0, 0 },
	[ 187 ] = { 139, 246, 113, 12, 0, 0 },
	[ 188 ] = { 12, 210, 299, 145, 0, 0 },
	[ 189 ] = { 139, 149, 198, 300, 0, 0 },
	[ 190 ] = { 103, 11, 117, 125, 0, 0 },
	[ 191 ] = { 13, 15, 12, 16, 0, 0 },
	[ 192 ] = { 14, 19, 14, 12, 0, 0 },
	[ 193 ] = { 101, 111, 113, 202, 0, 0 },
	[ 194 ] = { 199, 11, 231, 299, 0, 0 },
	[ 195 ] = { 139, 197, 146, 19, 0, 0 },
	[ 196 ] = { 353, 363, 138, 345, 0, 0 },
	[ 197 ] = { 354, 364, 138, 346, 0, 0 }
};

static const pmg_power_group_t power6_groups[] = {
	[ 0 ] = {
		.pmg_name = "pm_utilization",
		.pmg_desc = "CPI and utilization data",
		.pmg_event_ids = power6_group_event_ids[0],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000000a02121eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 1 ] = {
		.pmg_name = "pm_utilization_capacity",
		.pmg_desc = "CPU utilization and capacity",
		.pmg_event_ids = power6_group_event_ids[1],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00000000fa1ef4f4ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 2 ] = {
		.pmg_name = "pm_branch",
		.pmg_desc = "Branch operations",
		.pmg_event_ids = power6_group_event_ids[2],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x04000000a2a8808aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 3 ] = {
		.pmg_name = "pm_branch2",
		.pmg_desc = "Branch operations",
		.pmg_event_ids = power6_group_event_ids[3],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x04000000a4a68e8cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 4 ] = {
		.pmg_name = "pm_branch3",
		.pmg_desc = "Branch operations",
		.pmg_event_ids = power6_group_event_ids[4],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x04000000a0a28486ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 5 ] = {
		.pmg_name = "pm_branch4",
		.pmg_desc = "Branch operations",
		.pmg_event_ids = power6_group_event_ids[5],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x04000000a8aa8c8eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 6 ] = {
		.pmg_name = "pm_branch5",
		.pmg_desc = "Branch operations",
		.pmg_event_ids = power6_group_event_ids[6],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x04040000a052c652ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 7 ] = {
		.pmg_name = "pm_dsource",
		.pmg_desc = "Data source",
		.pmg_event_ids = power6_group_event_ids[7],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000058585656ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 8 ] = {
		.pmg_name = "pm_dsource2",
		.pmg_desc = "Data sources",
		.pmg_event_ids = power6_group_event_ids[8],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005a5a5856ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 9 ] = {
		.pmg_name = "pm_dsource3",
		.pmg_desc = "Data sources",
		.pmg_event_ids = power6_group_event_ids[9],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005a5a5a5aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 10 ] = {
		.pmg_name = "pm_dsource4",
		.pmg_desc = "Data sources",
		.pmg_event_ids = power6_group_event_ids[10],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005c5c5c5cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 11 ] = {
		.pmg_name = "pm_dsource5",
		.pmg_desc = "Data sources",
		.pmg_event_ids = power6_group_event_ids[11],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005e5e5e5eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 12 ] = {
		.pmg_name = "pm_dlatencies",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[12],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000000c281e24ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 13 ] = {
		.pmg_name = "pm_dlatencies2",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[13],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00000000022c1e2aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 14 ] = {
		.pmg_name = "pm_dlatencies3",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[14],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00000000022e5e2cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 15 ] = {
		.pmg_name = "pm_dlatencies4",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[15],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005a2a5c26ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 16 ] = {
		.pmg_name = "pm_dlatencies5",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[16],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005c225828ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 17 ] = {
		.pmg_name = "pm_dlatencies6",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[17],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005e245a2eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 18 ] = {
		.pmg_name = "pm_dlatencies7",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[18],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005820120eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 19 ] = {
		.pmg_name = "pm_dlatencies8",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[19],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000010581e20ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 20 ] = {
		.pmg_name = "pm_dlatencies9",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[20],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00000000122c125eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 21 ] = {
		.pmg_name = "pm_dlatencies10",
		.pmg_desc = "Data latencies",
		.pmg_event_ids = power6_group_event_ids[21],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005a261e26ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 22 ] = {
		.pmg_name = "pm_isource",
		.pmg_desc = "Instruction sources",
		.pmg_event_ids = power6_group_event_ids[22],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0040000040404654ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 23 ] = {
		.pmg_name = "pm_isource2",
		.pmg_desc = "Instruction sources",
		.pmg_event_ids = power6_group_event_ids[23],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0040000046464046ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 24 ] = {
		.pmg_name = "pm_isource3",
		.pmg_desc = "Instruction sources",
		.pmg_event_ids = power6_group_event_ids[24],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0040000044444444ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 25 ] = {
		.pmg_name = "pm_isource4",
		.pmg_desc = "Instruction sources",
		.pmg_event_ids = power6_group_event_ids[25],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0040000042424242ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 26 ] = {
		.pmg_name = "pm_isource5",
		.pmg_desc = "Instruction sources",
		.pmg_event_ids = power6_group_event_ids[26],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0040000040405454ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 27 ] = {
		.pmg_name = "pm_pteg",
		.pmg_desc = "PTEG sources",
		.pmg_event_ids = power6_group_event_ids[27],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0001000048484e4eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 28 ] = {
		.pmg_name = "pm_pteg2",
		.pmg_desc = "PTEG sources",
		.pmg_event_ids = power6_group_event_ids[28],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000100002848484cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 29 ] = {
		.pmg_name = "pm_pteg3",
		.pmg_desc = "PTEG sources",
		.pmg_event_ids = power6_group_event_ids[29],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000100004e4e284aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 30 ] = {
		.pmg_name = "pm_pteg4",
		.pmg_desc = "PTEG sources",
		.pmg_event_ids = power6_group_event_ids[30],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000100004a4a4a4aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 31 ] = {
		.pmg_name = "pm_pteg5",
		.pmg_desc = "PTEG sources",
		.pmg_event_ids = power6_group_event_ids[31],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000100004c4c4cc8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 32 ] = {
		.pmg_name = "pm_data_tablewalk",
		.pmg_desc = "Data tablewalks",
		.pmg_event_ids = power6_group_event_ids[32],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x09900000a0a284e8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 33 ] = {
		.pmg_name = "pm_inst_tablewalk",
		.pmg_desc = "Instruction tablewalks",
		.pmg_event_ids = power6_group_event_ids[33],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x09900000a8aa8ceaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 34 ] = {
		.pmg_name = "pm_freq",
		.pmg_desc = "Frequency events",
		.pmg_event_ids = power6_group_event_ids[34],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000002a3c3c3cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 35 ] = {
		.pmg_name = "pm_disp_wait",
		.pmg_desc = "Dispatch stalls",
		.pmg_event_ids = power6_group_event_ids[35],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00000000560c040cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 36 ] = {
		.pmg_name = "pm_disp_held",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[36],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x200000002a3c2aa2ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 37 ] = {
		.pmg_name = "pm_disp_held2",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[37],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x200000008004a4a6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 38 ] = {
		.pmg_name = "pm_disp_held3",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[38],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x20000000888aacaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 39 ] = {
		.pmg_name = "pm_disp_held4",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[39],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x02000000a0a28486ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 40 ] = {
		.pmg_name = "pm_disp_held5",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[40],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x22000000a8aa8ca0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 41 ] = {
		.pmg_name = "pm_disp_held6",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[41],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x33000000a882a4a6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 42 ] = {
		.pmg_name = "pm_disp_held7",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[42],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x30000000888aacaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 43 ] = {
		.pmg_name = "pm_disp_held8",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[43],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x220000008a8cae80ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 44 ] = {
		.pmg_name = "pm_disp_held9",
		.pmg_desc = "Dispatch held conditions",
		.pmg_event_ids = power6_group_event_ids[44],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x220000008aa08a8cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 45 ] = {
		.pmg_name = "pm_sync",
		.pmg_desc = "Sync events",
		.pmg_event_ids = power6_group_event_ids[45],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x38900000ae1eeca0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 46 ] = {
		.pmg_name = "pm_L1_ref",
		.pmg_desc = "L1 references",
		.pmg_event_ids = power6_group_event_ids[46],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x80000000368aa63aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 47 ] = {
		.pmg_name = "pm_L1_ldst",
		.pmg_desc = "L1 load/store ref/miss",
		.pmg_event_ids = power6_group_event_ids[47],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x800000003230a8a0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 48 ] = {
		.pmg_name = "pm_streams",
		.pmg_desc = "Streams",
		.pmg_event_ids = power6_group_event_ids[48],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x48000000a0a284a4ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 49 ] = {
		.pmg_name = "pm_flush",
		.pmg_desc = "Flushes",
		.pmg_event_ids = power6_group_event_ids[49],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0022000010cacccaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 50 ] = {
		.pmg_name = "pm_prefetch",
		.pmg_desc = "I cache Prefetches",
		.pmg_event_ids = power6_group_event_ids[50],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x400400008a8caec0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 51 ] = {
		.pmg_name = "pm_stcx",
		.pmg_desc = "STCX",
		.pmg_event_ids = power6_group_event_ids[51],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00080000e6eccecaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 52 ] = {
		.pmg_name = "pm_larx",
		.pmg_desc = "LARX",
		.pmg_event_ids = power6_group_event_ids[52],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00080000eae2c6ceULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 53 ] = {
		.pmg_name = "pm_thread_cyc",
		.pmg_desc = "Thread cycles",
		.pmg_event_ids = power6_group_event_ids[53],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000016182604ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 54 ] = {
		.pmg_name = "pm_misc",
		.pmg_desc = "Misc",
		.pmg_event_ids = power6_group_event_ids[54],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000004161808ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 55 ] = {
		.pmg_name = "pm_misc2",
		.pmg_desc = "Misc",
		.pmg_event_ids = power6_group_event_ids[55],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x40020000eef8f8a0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 56 ] = {
		.pmg_name = "pm_misc3",
		.pmg_desc = "Misc",
		.pmg_event_ids = power6_group_event_ids[56],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0300000054a01e02ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 57 ] = {
		.pmg_name = "pm_tlb_slb",
		.pmg_desc = "TLB and SLB events",
		.pmg_event_ids = power6_group_event_ids[57],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00980000e0e8e8e2ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 58 ] = {
		.pmg_name = "pm_slb_miss",
		.pmg_desc = "SLB Misses",
		.pmg_event_ids = power6_group_event_ids[58],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00480001e0e8ee32ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 59 ] = {
		.pmg_name = "pm_rejects",
		.pmg_desc = "Reject events",
		.pmg_event_ids = power6_group_event_ids[59],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xaa00000034303e30ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 60 ] = {
		.pmg_name = "pm_rejects2",
		.pmg_desc = "Reject events",
		.pmg_event_ids = power6_group_event_ids[60],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x9a000000323830acULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 61 ] = {
		.pmg_name = "pm_rejects3",
		.pmg_desc = "Reject events",
		.pmg_event_ids = power6_group_event_ids[61],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xaa000000303e3234ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 62 ] = {
		.pmg_name = "pm_rejects4",
		.pmg_desc = "Unaligned store rejects",
		.pmg_event_ids = power6_group_event_ids[62],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x900000003630a2aaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 63 ] = {
		.pmg_name = "pm_rejects5",
		.pmg_desc = "Unaligned load rejects",
		.pmg_event_ids = power6_group_event_ids[63],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x900000003036a0a8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 64 ] = {
		.pmg_name = "pm_rejects6",
		.pmg_desc = "Set mispredictions rejects",
		.pmg_event_ids = power6_group_event_ids[64],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xa0000000848c341cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 65 ] = {
		.pmg_name = "pm_rejects_unit",
		.pmg_desc = "Unaligned reject events by unit",
		.pmg_event_ids = power6_group_event_ids[65],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x90000000808aa2a8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 66 ] = {
		.pmg_name = "pm_rejects_unit2",
		.pmg_desc = "Reject events by unit",
		.pmg_event_ids = power6_group_event_ids[66],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xaa000000a6828e8aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 67 ] = {
		.pmg_name = "pm_rejects_unit3",
		.pmg_desc = "Reject events by unit",
		.pmg_event_ids = power6_group_event_ids[67],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0a000000a4a08c88ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 68 ] = {
		.pmg_name = "pm_rejects_unit4",
		.pmg_desc = "Reject events by unit",
		.pmg_event_ids = power6_group_event_ids[68],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xaa000000a2868aaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 69 ] = {
		.pmg_name = "pm_rejects_unit5",
		.pmg_desc = "Reject events by unit",
		.pmg_event_ids = power6_group_event_ids[69],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x9900000086a6ae8eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 70 ] = {
		.pmg_name = "pm_rejects_unit6",
		.pmg_desc = "Reject events by unit",
		.pmg_event_ids = power6_group_event_ids[70],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xaa00000080a6a88eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 71 ] = {
		.pmg_name = "pm_rejects_unit7",
		.pmg_desc = "Reject events by unit",
		.pmg_event_ids = power6_group_event_ids[71],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xa900000082a6aa8eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 72 ] = {
		.pmg_name = "pm_ldf",
		.pmg_desc = "Floating Point loads",
		.pmg_event_ids = power6_group_event_ids[72],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x800000003832a4acULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 73 ] = {
		.pmg_name = "pm_lsu_misc",
		.pmg_desc = "LSU events",
		.pmg_event_ids = power6_group_event_ids[73],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x08800000caccee8aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 74 ] = {
		.pmg_name = "pm_lsu_lmq",
		.pmg_desc = "LSU LMQ events",
		.pmg_event_ids = power6_group_event_ids[74],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x98000000ac1c1ca4ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 75 ] = {
		.pmg_name = "pm_lsu_flush_derat_miss",
		.pmg_desc = "LSU flush and DERAT misses",
		.pmg_event_ids = power6_group_event_ids[75],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00200000fc0eeceeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 76 ] = {
		.pmg_name = "pm_lla",
		.pmg_desc = "Look Load Ahead events",
		.pmg_event_ids = power6_group_event_ids[76],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x33000000a2841208ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 77 ] = {
		.pmg_name = "pm_gct",
		.pmg_desc = "GCT events",
		.pmg_event_ids = power6_group_event_ids[77],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x404000000808a6e8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 78 ] = {
		.pmg_name = "pm_smt_priorities",
		.pmg_desc = "Thread priority events",
		.pmg_event_ids = power6_group_event_ids[78],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0020000040404040ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 79 ] = {
		.pmg_name = "pm_smt_priorities2",
		.pmg_desc = "Thread priority events",
		.pmg_event_ids = power6_group_event_ids[79],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0020000046464646ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 80 ] = {
		.pmg_name = "pm_smt_priorities3",
		.pmg_desc = "Thread priority differences events",
		.pmg_event_ids = power6_group_event_ids[80],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0002000040404040ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 81 ] = {
		.pmg_name = "pm_smt_priorities4",
		.pmg_desc = "Thread priority differences events",
		.pmg_event_ids = power6_group_event_ids[81],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x03020000a6464646ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 82 ] = {
		.pmg_name = "pm_fxu",
		.pmg_desc = "FXU events",
		.pmg_event_ids = power6_group_event_ids[82],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000050505050ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 83 ] = {
		.pmg_name = "pm_fxu2",
		.pmg_desc = "FXU events",
		.pmg_event_ids = power6_group_event_ids[83],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x02040000aee41616ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 84 ] = {
		.pmg_name = "pm_vmx",
		.pmg_desc = "VMX events",
		.pmg_event_ids = power6_group_event_ids[84],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x700000008480a2a6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 85 ] = {
		.pmg_name = "pm_vmx2",
		.pmg_desc = "VMX events",
		.pmg_event_ids = power6_group_event_ids[85],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x600000008088a2aaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 86 ] = {
		.pmg_name = "pm_vmx3",
		.pmg_desc = "VMX events",
		.pmg_event_ids = power6_group_event_ids[86],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x600000008284aaacULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 87 ] = {
		.pmg_name = "pm_vmx4",
		.pmg_desc = "VMX events",
		.pmg_event_ids = power6_group_event_ids[87],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xb0000000828ea6a0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 88 ] = {
		.pmg_name = "pm_vmx5",
		.pmg_desc = "VMX events",
		.pmg_event_ids = power6_group_event_ids[88],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xb00000008084aca2ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 89 ] = {
		.pmg_name = "pm_dfu",
		.pmg_desc = "DFU events",
		.pmg_event_ids = power6_group_event_ids[89],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xe00000008c88a2aeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 90 ] = {
		.pmg_name = "pm_dfu2",
		.pmg_desc = "DFU events",
		.pmg_event_ids = power6_group_event_ids[90],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xe00000008a84a0a6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 91 ] = {
		.pmg_name = "pm_fab",
		.pmg_desc = "Fabric events",
		.pmg_event_ids = power6_group_event_ids[91],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500020003030a4acULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 92 ] = {
		.pmg_name = "pm_fab2",
		.pmg_desc = "Fabric events",
		.pmg_event_ids = power6_group_event_ids[92],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x50002000888aa2a0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 93 ] = {
		.pmg_name = "pm_fab3",
		.pmg_desc = "Fabric events",
		.pmg_event_ids = power6_group_event_ids[93],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500020003030aea6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 94 ] = {
		.pmg_name = "pm_mem_dblpump",
		.pmg_desc = "Double pump",
		.pmg_event_ids = power6_group_event_ids[94],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000400030303434ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 95 ] = {
		.pmg_name = "pm_mem0_dblpump",
		.pmg_desc = "MCS0 Double pump",
		.pmg_event_ids = power6_group_event_ids[95],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500040008082a4a6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 96 ] = {
		.pmg_name = "pm_mem1_dblpump",
		.pmg_desc = "MCS1 Double pump",
		.pmg_event_ids = power6_group_event_ids[96],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x50004000888aacaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 97 ] = {
		.pmg_name = "pm_gxo",
		.pmg_desc = "GX outbound",
		.pmg_event_ids = power6_group_event_ids[97],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500060008082a4a6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 98 ] = {
		.pmg_name = "pm_gxi",
		.pmg_desc = "GX inbound",
		.pmg_event_ids = power6_group_event_ids[98],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500060008688aaa0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 99 ] = {
		.pmg_name = "pm_gx_dma",
		.pmg_desc = "DMA events",
		.pmg_event_ids = power6_group_event_ids[99],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500060008086acaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 100 ] = {
		.pmg_name = "pm_L1_misc",
		.pmg_desc = "L1 misc events",
		.pmg_event_ids = power6_group_event_ids[100],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x4004000082e2a80aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 101 ] = {
		.pmg_name = "pm_L2_data",
		.pmg_desc = "L2 load and store data",
		.pmg_event_ids = power6_group_event_ids[101],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000800030303434ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 102 ] = {
		.pmg_name = "pm_L2_ld_inst",
		.pmg_desc = "L2 Load instructions",
		.pmg_event_ids = power6_group_event_ids[102],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5800a00030303486ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 103 ] = {
		.pmg_name = "pm_L2_castout_invalidate",
		.pmg_desc = "L2 castout and invalidate events",
		.pmg_event_ids = power6_group_event_ids[103],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000c00030303434ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 104 ] = {
		.pmg_name = "pm_L2_ldst_reqhit",
		.pmg_desc = "L2 load and store requests and hits",
		.pmg_event_ids = power6_group_event_ids[104],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000e00030303434ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 105 ] = {
		.pmg_name = "pm_L2_ld_data_slice",
		.pmg_desc = "L2 data loads by slice",
		.pmg_event_ids = power6_group_event_ids[105],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500080008082a8aaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 106 ] = {
		.pmg_name = "pm_L2_ld_inst_slice",
		.pmg_desc = "L2 instruction loads by slice",
		.pmg_event_ids = power6_group_event_ids[106],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000a0008082a8aaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 107 ] = {
		.pmg_name = "pm_L2_st_slice",
		.pmg_desc = "L2 slice stores by slice",
		.pmg_event_ids = power6_group_event_ids[107],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x500080008486acaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 108 ] = {
		.pmg_name = "pm_L2miss_slice",
		.pmg_desc = "L2 misses by slice",
		.pmg_event_ids = power6_group_event_ids[108],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000a000843256acULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 109 ] = {
		.pmg_name = "pm_L2_castout_slice",
		.pmg_desc = "L2 castouts by slice",
		.pmg_event_ids = power6_group_event_ids[109],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000c0008082a8aaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 110 ] = {
		.pmg_name = "pm_L2_invalidate_slice",
		.pmg_desc = "L2 invalidate by slice",
		.pmg_event_ids = power6_group_event_ids[110],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000c0008486acaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 111 ] = {
		.pmg_name = "pm_L2_ld_reqhit_slice",
		.pmg_desc = "L2 load requests and hist by slice",
		.pmg_event_ids = power6_group_event_ids[111],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000e0008082a8aaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 112 ] = {
		.pmg_name = "pm_L2_st_reqhit_slice",
		.pmg_desc = "L2 store requests and hist by slice",
		.pmg_event_ids = power6_group_event_ids[112],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5000e0008486acaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 113 ] = {
		.pmg_name = "pm_L2_redir_pref",
		.pmg_desc = "L2 redirect and prefetch",
		.pmg_event_ids = power6_group_event_ids[113],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x08400000cacc8886ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 114 ] = {
		.pmg_name = "pm_L3_SliceA",
		.pmg_desc = "L3 slice A events",
		.pmg_event_ids = power6_group_event_ids[114],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x50000000303058a4ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 115 ] = {
		.pmg_name = "pm_L3_SliceB",
		.pmg_desc = "L3 slice B events",
		.pmg_event_ids = power6_group_event_ids[115],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x50000000888a58acULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 116 ] = {
		.pmg_name = "pm_fpu_issue",
		.pmg_desc = "FPU issue events",
		.pmg_event_ids = power6_group_event_ids[116],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00300000c6c8eae4ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 117 ] = {
		.pmg_name = "pm_fpu_issue2",
		.pmg_desc = "FPU issue events",
		.pmg_event_ids = power6_group_event_ids[117],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00300000c0c2eceeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 118 ] = {
		.pmg_name = "pm_fpu_issue3",
		.pmg_desc = "FPU issue events",
		.pmg_event_ids = power6_group_event_ids[118],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00330000e0e2eceeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 119 ] = {
		.pmg_name = "pm_fpu0_flop",
		.pmg_desc = "FPU0 flop events",
		.pmg_event_ids = power6_group_event_ids[119],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xcc0000008082a484ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 120 ] = {
		.pmg_name = "pm_fpu0_misc",
		.pmg_desc = "FPU0 events",
		.pmg_event_ids = power6_group_event_ids[120],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xcc00000086a08286ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 121 ] = {
		.pmg_name = "pm_fpu0_misc2",
		.pmg_desc = "FPU0 events",
		.pmg_event_ids = power6_group_event_ids[121],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xdd00000080a6a4a6ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 122 ] = {
		.pmg_name = "pm_fpu0_misc3",
		.pmg_desc = "FPU0 events",
		.pmg_event_ids = power6_group_event_ids[122],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0d000000a0a28486ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 123 ] = {
		.pmg_name = "pm_fpu1_flop",
		.pmg_desc = "FPU1 flop events",
		.pmg_event_ids = power6_group_event_ids[123],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xcc000000888aac8cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 124 ] = {
		.pmg_name = "pm_fpu1_misc",
		.pmg_desc = "FPU1 events",
		.pmg_event_ids = power6_group_event_ids[124],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xcc0000008ea88a8eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 125 ] = {
		.pmg_name = "pm_fpu1_misc2",
		.pmg_desc = "FPU1 events",
		.pmg_event_ids = power6_group_event_ids[125],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xdd00000088aeacaeULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 126 ] = {
		.pmg_name = "pm_fpu1_misc3",
		.pmg_desc = "FPU1 events",
		.pmg_event_ids = power6_group_event_ids[126],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0d000000a8aa8c8eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 127 ] = {
		.pmg_name = "pm_fpu_flop",
		.pmg_desc = "FPU flop events",
		.pmg_event_ids = power6_group_event_ids[127],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xc000000030303434ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 128 ] = {
		.pmg_name = "pm_fpu_misc",
		.pmg_desc = "FPU events",
		.pmg_event_ids = power6_group_event_ids[128],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xdd00000030343434ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 129 ] = {
		.pmg_name = "pm_fpu_misc2",
		.pmg_desc = "FPU events",
		.pmg_event_ids = power6_group_event_ids[129],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0c00000034343030ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 130 ] = {
		.pmg_name = "pm_fpu_misc3",
		.pmg_desc = "FPU events",
		.pmg_event_ids = power6_group_event_ids[130],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0d00000034343030ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 131 ] = {
		.pmg_name = "pm_purr",
		.pmg_desc = "PURR events",
		.pmg_event_ids = power6_group_event_ids[131],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000000ef41e02ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 132 ] = {
		.pmg_name = "pm_suspend",
		.pmg_desc = "SUSPENDED events",
		.pmg_event_ids = power6_group_event_ids[132],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00900000001eec02ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 133 ] = {
		.pmg_name = "pm_dcache",
		.pmg_desc = "D cache",
		.pmg_event_ids = power6_group_event_ids[133],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000000c0e0c06ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 134 ] = {
		.pmg_name = "pm_derat_miss",
		.pmg_desc = "DERAT miss",
		.pmg_event_ids = power6_group_event_ids[134],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0090000f40404040ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 135 ] = {
		.pmg_name = "pm_derat_ref",
		.pmg_desc = "DERAT ref",
		.pmg_event_ids = power6_group_event_ids[135],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0080000f40404040ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 136 ] = {
		.pmg_name = "pm_ierat_miss",
		.pmg_desc = "IERAT miss",
		.pmg_event_ids = power6_group_event_ids[136],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0090000f46464646ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 137 ] = {
		.pmg_name = "pm_mrk_br",
		.pmg_desc = "Marked Branch events",
		.pmg_event_ids = power6_group_event_ids[137],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000052565202ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 138 ] = {
		.pmg_name = "pm_mrk_dsource",
		.pmg_desc = "Marked data sources",
		.pmg_event_ids = power6_group_event_ids[138],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00000000024a4c4cULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 139 ] = {
		.pmg_name = "pm_mrk_dsource2",
		.pmg_desc = "Marked data sources",
		.pmg_event_ids = power6_group_event_ids[139],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000048484e02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 140 ] = {
		.pmg_name = "pm_mrk_dsource3",
		.pmg_desc = "Marked data sources",
		.pmg_event_ids = power6_group_event_ids[140],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000002802484eULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 141 ] = {
		.pmg_name = "pm_mrk_dsource4",
		.pmg_desc = "Marked data sources",
		.pmg_event_ids = power6_group_event_ids[141],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000004e4e2802ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 142 ] = {
		.pmg_name = "pm_mrk_dsource5",
		.pmg_desc = "Marked data sources",
		.pmg_event_ids = power6_group_event_ids[142],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000004a4c024aULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 143 ] = {
		.pmg_name = "pm_mrk_dsource6",
		.pmg_desc = "Marked data sources",
		.pmg_event_ids = power6_group_event_ids[143],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000004c4c4a02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 144 ] = {
		.pmg_name = "pm_mrk_rejects",
		.pmg_desc = "Marked rejects",
		.pmg_event_ids = power6_group_event_ids[144],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0009000d34340230ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 145 ] = {
		.pmg_name = "pm_mrk_rejects2",
		.pmg_desc = "Marked rejects LSU0",
		.pmg_event_ids = power6_group_event_ids[145],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00090000e6e0c202ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 146 ] = {
		.pmg_name = "pm_mrk_rejects3",
		.pmg_desc = "Marked rejects LSU1",
		.pmg_event_ids = power6_group_event_ids[146],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00090000eee8ca02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 147 ] = {
		.pmg_name = "pm_mrk_inst",
		.pmg_desc = "Marked instruction events",
		.pmg_event_ids = power6_group_event_ids[147],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000001c100a02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 148 ] = {
		.pmg_name = "pm_mrk_fpu_fin",
		.pmg_desc = "Marked Floating Point instructions finished",
		.pmg_event_ids = power6_group_event_ids[148],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xd0000000828a1a02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 149 ] = {
		.pmg_name = "pm_mrk_misc",
		.pmg_desc = "Marked misc events",
		.pmg_event_ids = power6_group_event_ids[149],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00090008341a0802ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 150 ] = {
		.pmg_name = "pm_mrk_misc2",
		.pmg_desc = "Marked misc events",
		.pmg_event_ids = power6_group_event_ids[150],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00080000e40a023eULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 151 ] = {
		.pmg_name = "pm_mrk_misc3",
		.pmg_desc = "Marked misc events",
		.pmg_event_ids = power6_group_event_ids[151],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xb009000088e40212ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 152 ] = {
		.pmg_name = "pm_mrk_misc4",
		.pmg_desc = "Marked misc events",
		.pmg_event_ids = power6_group_event_ids[152],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000001e1e021aULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 153 ] = {
		.pmg_name = "pm_mrk_st",
		.pmg_desc = "Marked stores events",
		.pmg_event_ids = power6_group_event_ids[153],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000006060602ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 154 ] = {
		.pmg_name = "pm_mrk_pteg",
		.pmg_desc = "Marked PTEG",
		.pmg_event_ids = power6_group_event_ids[154],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0010000040424402ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 155 ] = {
		.pmg_name = "pm_mrk_pteg2",
		.pmg_desc = "Marked PTEG",
		.pmg_event_ids = power6_group_event_ids[155],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0010000002404644ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 156 ] = {
		.pmg_name = "pm_mrk_pteg3",
		.pmg_desc = "Marked PTEG",
		.pmg_event_ids = power6_group_event_ids[156],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0010000046460246ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 157 ] = {
		.pmg_name = "pm_mrk_pteg4",
		.pmg_desc = "Marked PTEG",
		.pmg_event_ids = power6_group_event_ids[157],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0010000042024054ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 158 ] = {
		.pmg_name = "pm_mrk_pteg5",
		.pmg_desc = "Marked PTEG",
		.pmg_event_ids = power6_group_event_ids[158],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0010000044025442ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 159 ] = {
		.pmg_name = "pm_mrk_pteg6",
		.pmg_desc = "Marked PTEG",
		.pmg_event_ids = power6_group_event_ids[159],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x001000001e444202ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 160 ] = {
		.pmg_name = "pm_mrk_vmx",
		.pmg_desc = "Marked VMX",
		.pmg_event_ids = power6_group_event_ids[160],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x700000008c88ae02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 161 ] = {
		.pmg_name = "pm_mrk_vmx2",
		.pmg_desc = "Marked VMX",
		.pmg_event_ids = power6_group_event_ids[161],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x60900000868ee002ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 162 ] = {
		.pmg_name = "pm_mrk_vmx3",
		.pmg_desc = "Marked VMX",
		.pmg_event_ids = power6_group_event_ids[162],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x700000008a821e02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 163 ] = {
		.pmg_name = "pm_mrk_fp",
		.pmg_desc = "Marked FP events",
		.pmg_event_ids = power6_group_event_ids[163],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xd00000008230aa02ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 164 ] = {
		.pmg_name = "pm_mrk_derat_ref",
		.pmg_desc = "Marked DERAT ref",
		.pmg_event_ids = power6_group_event_ids[164],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0080000044444402ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 165 ] = {
		.pmg_name = "pm_mrk_derat_miss",
		.pmg_desc = "Marked DERAT miss",
		.pmg_event_ids = power6_group_event_ids[165],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0090000044444402ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 166 ] = {
		.pmg_name = "pm_dcache_edge",
		.pmg_desc = "D cache - edge",
		.pmg_event_ids = power6_group_event_ids[166],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000000d0e0c07ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 167 ] = {
		.pmg_name = "pm_lsu_lmq_edge",
		.pmg_desc = "LSU LMQ events - edge",
		.pmg_event_ids = power6_group_event_ids[167],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x98000000ac1d1da4ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 168 ] = {
		.pmg_name = "pm_gct_edge",
		.pmg_desc = "GCT events - edge",
		.pmg_event_ids = power6_group_event_ids[168],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x404000000909a7e8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 169 ] = {
		.pmg_name = "pm_freq_edge",
		.pmg_desc = "Frequency events - edge",
		.pmg_event_ids = power6_group_event_ids[169],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000002b3d3c3cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 170 ] = {
		.pmg_name = "pm_disp_wait_edge",
		.pmg_desc = "Dispatch stalls - edge",
		.pmg_event_ids = power6_group_event_ids[170],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00000000560d050dULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 171 ] = {
		.pmg_name = "pm_edge1",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[171],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000006300d0c1f1eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 172 ] = {
		.pmg_name = "pm_edge2",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[172],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x400000008180a5a4ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 173 ] = {
		.pmg_name = "pm_edge3",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[173],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x009000000bf4ebeaULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 174 ] = {
		.pmg_name = "pm_edge4",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[174],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x400000008786a9a8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 175 ] = {
		.pmg_name = "pm_edge5",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[175],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00900000fb17edecULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 176 ] = {
		.pmg_name = "pm_noedge5",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[176],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00900000fa16edecULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 177 ] = {
		.pmg_name = "pm_edge6",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[177],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000002b05050dULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 178 ] = {
		.pmg_name = "pm_noedge6",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[178],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000002a04040cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 179 ] = {
		.pmg_name = "pm_edge7",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[179],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000009091d1dULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 180 ] = {
		.pmg_name = "pm_noedge7",
		.pmg_desc = "NOEDGE event group",
		.pmg_event_ids = power6_group_event_ids[180],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000008081c1cULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 181 ] = {
		.pmg_name = "pm_edge8",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[181],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00900000cd1dec07ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 182 ] = {
		.pmg_name = "pm_noedge8",
		.pmg_desc = "NOEDGE event group",
		.pmg_event_ids = power6_group_event_ids[182],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x00900000cc1ced06ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 183 ] = {
		.pmg_name = "pm_edge9",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[183],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x80000000880d0ca2ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 184 ] = {
		.pmg_name = "pm_edge10",
		.pmg_desc = "EDGE event group",
		.pmg_event_ids = power6_group_event_ids[184],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x32000000ac3dae05ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 185 ] = {
		.pmg_name = "pm_noedge10",
		.pmg_desc = "NOEDGE event group",
		.pmg_event_ids = power6_group_event_ids[185],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x32000000ac3cae04ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 186 ] = {
		.pmg_name = "pm_hpm1",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[186],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xc00000003030341eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 187 ] = {
		.pmg_name = "pm_hpm2",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[187],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x8c0000000232301eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 188 ] = {
		.pmg_name = "pm_hpm3",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[188],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x800000001e80f002ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 189 ] = {
		.pmg_name = "pm_hpm4",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[189],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x800000000212a234ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 190 ] = {
		.pmg_name = "pm_hpm5",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[190],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xd0000000301e1616ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 191 ] = {
		.pmg_name = "pm_hpm6",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[191],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0000000058585a5aULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 192 ] = {
		.pmg_name = "pm_hpm7",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[192],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x000000005a5a581eULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 193 ] = {
		.pmg_name = "pm_hpm8",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[193],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0xcc000000303030f0ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 194 ] = {
		.pmg_name = "pm_hpm9",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[194],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x80000000801e34a8ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 195 ] = {
		.pmg_name = "pm_hpm10",
		.pmg_desc = "HPM group",
		.pmg_event_ids = power6_group_event_ids[195],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x5040a00002325456ULL,
		.pmg_mmcra = 0x0000000000000000ULL
	},
	[ 196 ] = {
		.pmg_name = "pm_mrk_derat_ref2",
		.pmg_desc = "Marked DERAT ref",
		.pmg_event_ids = power6_group_event_ids[196],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0080000044440244ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	},
	[ 197 ] = {
		.pmg_name = "pm_mrk_derat_miss2",
		.pmg_desc = "Marked DERAT miss",
		.pmg_event_ids = power6_group_event_ids[197],
		.pmg_mmcr0 = 0x0000000000000000ULL,
		.pmg_mmcr1 = 0x0090000044440244ULL,
		.pmg_mmcra = 0x0000000000000001ULL
	}
};

#endif

